#!/bin/bash

# Variveis Globais

ERR_LOGFILE_NOT_FOUND=128

# Funes
# 
# filter -> recebe como argumento 
#

filter()
{
	echo "Begin - filter"
	LOGTOBEFILTERED=$1
	LOGALL="$TMPDIR/logall.log"
	NUMBEREDLOG="$TMPDIR/peerlogfullnumbered.log"
	RESULTNOT="$TMPDIR/resultnot.log"
	RESULTUP="$TMPDIR/resultup.log"
	RESULTDOWN="$TMPDIR/resultdown.log"
	RESULTNEED="$TMPDIR/resultneed.log"
	RESULTPEERUP="$TMPDIR/resultpeerup.log"
	RESULTPEERDOWN="$TMPDIR/resultpeerdown.log"
	RESULT="$TMPDIR/result.log"
	FILTEREDLOG="$TMPDIR/filtered.log"

	
	echo "Filtering peer log file (this may take a few minutes)..."

	# Adicionando nmero de linhas ao arquivo
	cat -n $LOGTOBEFILTERED > $NUMBEREDLOG

	# Extraindo eventos relevantes do arquivo 

	cat $NUMBEREDLOG | grep not\ connect | grep UAServerConnector.getUserAgentServer | awk '{print $1" "$2" "$3" "$13" not connected" }' > $RESULTNOT
  
	cat $NUMBEREDLOG | grep set\ up | awk '{print $1" "$2" "$3" "$8" up"}' > $RESULTUP

	cat $NUMBEREDLOG | grep set\ down | awk '{print $1" "$2" "$3" "$8" down"}' > $RESULTDOWN

   	cat $NUMBEREDLOG | grep  CommunityObtainer.wannaGuMs | grep Local\ Peer\ needs | awk '{print $1" "$2" "$3" Peer needs "$10}' > $RESULTNEED

 	cat $NUMBEREDLOG | grep  ourgrid.peer.main.Main.main | grep Starting\ doctor\ engine | awk '{print $1" "$2" "$3" Peer started!"}' >> $RESULTPEERUP

   	cat $NUMBEREDLOG | grep  ourgrid.peer.main.Main.main | grep Starting\ doctor\ engine | awk '{print $1}' >> $RESULTPEERDOWN
  

	for i in $( cat $RESULTPEERDOWN ); do
		POS=`expr $i - 1`
		if [ "$POS" != "0" ]; then
			cat $NUMBEREDLOG | sed -n ''$POS'p' | awk '{print $1" "$2" "$3" Peer down!"}' >> $LOGALL
		fi
	done
													  
	sort -m $RESULTDOWN $RESULTUP $RESULTNOT $RESULTNEED $RESULTPEERUP >> $LOGALL

	sort -n $LOGALL > $RESULT

	cat $RESULT | awk '{print $2" "$3" "$4" "$5" "$6}' > $FILTEREDLOG

	tail -n 1 $NUMBEREDLOG | awk '{print $2" "$3" Peer Down!"}' >> $FILTEREDLOG

	echo "End - filter"

}



find_log_files()
{
	echo "Begin - find_log_files"
	SRCDIR=$1
	PEERFILENAME="Peer*.log"
	LOGFILES="$TMPDIR/peerlogs.log"
	
	# Localiza todos os arquivos - a partir de SRCDIR - cujo nome
	# comea com "Peer" e terminam com ".log"

	find $SRCDIR/$PEERFILENAME > $LOGFILES
#	find $PEERFILENAME > $LOGFILES
	
	# testa se o arquivo existe e  vazio
	if [ ! -s $LOGFILES ]; then
		echo "$(basename $0): Peer log files don't exist in '$SRCDIR' directory"
		exit $ERR_LOGFILES_NOT_FOUND
	else
		echo "Using the following files..."
		cat $LOGFILES
	fi
	echo "End - find_log_files"

}


create_full_log()
{
	echo "Begin - create_full_log"
	LOGFILES=$1
	FULLLOG="$TMPDIR/peerfulllog.log"
	
	for i in $( cat $LOGFILES ); do
		cat $i >> $FULLLOG
	done
	echo "End - create_full_log"

}



generate_events_by_entities()
{
	echo "Begin - generate_events_by_entities"
	FILTEREDLOG=$1
	WORKLOADDIR="$TMPDIR/workload"

	mkdir -p $WORKLOADDIR

	while read line
	do
		id=$(echo $line | awk '{print $3}')
		echo $line >> $WORKLOADDIR/$id.log
	done < $FILTEREDLOG
	echo "End - generate_events_by_entities"
 
}


workload()
{
	echo "Begin - workload"
	STATISTICSFILE="peerstatistics.log" # Arquivo que vai ter todos os eventos organizados por GuMS. Inicialmente com os eventos do PEER.
	PEERLOGFILE="Peer.log"
	FINALSTATISTICS="finalstatistics.log" # Arquivo que vai ter a verso final do workload

	echo "Starting workload"
	rm -rf $STATISTICSFILE

	echo ":: Peer Begin"          >> $STATISTICSFILE
	cat $WORKLOADDIR/$PEERLOGFILE >> $STATISTICSFILE
	echo ":: Peer End"            >> $STATISTICSFILE
	

	for i in $(ls $WORKLOADDIR | grep -v $PEERLOGFILE | sort)
	do
		echo ":::: GuM Begin $i" >> $STATISTICSFILE
		cat $WORKLOADDIR/$i      >> $STATISTICSFILE
		echo ":::: GuM End $i"   >> $STATISTICSFILE
	done

	cat $STATISTICSFILE | grep -v "not connect" > $FINALSTATISTICS
	
	echo "End - workload"

}


# Main 
#
# Este script  invocado com 1 argumento opcional
#

red='\E[31;47m'
green='\E[32;47m'

nargs=$#
TMPDIR="/tmp/peer-$$"

# Cria um diretrio temporrio onde os logs sero armazenados
mkdir -p $TMPDIR


# Testa o nmero de argumentos e descobre a partir de onde vai
# procurar os arquivos de log. 

if [ ${nargs} -eq 0 ]
then
	SRCDIR=.
else
	if [ ${nargs} -eq 1 ]
	then
		SRCDIR=$1
	else
		usage
	fi
fi

echo "Initializing Workload..."
# Seta o valor da variavel $LOGFILES
find_log_files $SRCDIR

# Cria um unico arquivo de log concatenado todos os logs listados
# em LOGFILES
create_full_log $LOGFILES

# Filtra o arquivo gerado acima ($FULLLOG), gerando um arquivo
# menor ($FILTEREDLOG)
filter $FULLLOG


# Gera um arquivo por maquina em '$WORKLOAD' dir contendo todos os eventos de cada
# uma.
generate_events_by_entities $FILTEREDLOG


# Gera o workload do peer e mostra na tela
workload $FILTEREDLOG

# remove todos os arquivos temporrios gerados para o
# processamento dos logs.
#rm -rf $TMPDIR
