/*   Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 *   Permission is hereby granted, free of charge, to any person
 *   obtaining a copy of this software and associated documentation
 *   files (the "Software"), to deal with the Software without
 *   restriction, including no limitations in the rights to use, copy,
 *   modify, merge, publish, distribute, sublicense, and/or sell copies
 *   of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions: The above
 *   copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.  THE SOFTWARE
 *   IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *   NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITIES,
 *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 *   IMPORTANT:
 *
 *   There are some packages that are distributed with MyGrid:
 *   activation.jar, xmlParserAPIs.jar, xercesImpl.jar, ogsa.jar,
 *   cog-jglobus.jar, mjs.jar and AbsoluteLayout.jar.  Their license
 *   terms are included in MyGrid documentation, read them before
 *   changing and/or distributing any content of these packages.
*/

import java.rmi.RemoteException;

import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.mygrid.ui.*;

/**
 * This class adds job and set grid to MyGrid instead of create a Job description file
 * and Grid description file.
 * 
 * @author Leonardo de Assis - leoassis@dsc.ufcg.edu.br
*/
public class FactoryOnGrid {

	public static void main(String args[]) {

		MyGridStarter starter = new MyGridStarter();
		
		try {
			starter.startMyGrid();
		} catch (Exception e1) {
			System.err.println("ERROR! Unable to start MyGrid. Cause: "
					+ e1.getMessage());
		}

		if (! ConcreteUIServices.getInstance().isHomeMachineUp()) {
			System.err.println("ERROR! MyGrid is not running.");
			System.exit(-1);
		}

//		Config.getProperties();
		JobAdder jobAdder = new JobAdder();
		try {
			jobAdder.addJob( "simplejob.jdf");
		} catch (RemoteException e) {
			System.err.println("ERROR! Unable to add job in MyGrid. Cause: "
					+ e.getMessage());
			System.exit(-1);
		} catch (CompilerException e) {
			System.err.println("ERROR! Unable to add job in MyGrid. Cause: "
					+ e.getMessage());
			System.exit(-1);
		} catch (CouldNotAddJobException e) {
			System.err.println("ERROR! Unable to add job in MyGrid. Cause: "
					+ e.getMessage());
			System.exit(-1);
		}
		
		MyGridSetter gridSetter = new MyGridSetter();
		try {
			gridSetter.setGrid("../resources/example.gdf");
		} catch (RemoteException e2) {
			System.err.println("ERROR! Unable to set grid in MyGrid. Cause: "
					+ e2.getMessage());
			System.exit(-1);
		} catch (CompilerException e2) {
			System.err.println("ERROR! Unable to set grid in MyGrid. Cause: "
					+ e2.getMessage());
			System.exit(-1);
		}
		
	}

}
