/*   Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 *   Permission is hereby granted, free of charge, to any person
 *   obtaining a copy of this software and associated documentation
 *   files (the "Software"), to deal with the Software without
 *   restriction, including no limitations in the rights to use, copy,
 *   modify, merge, publish, distribute, sublicense, and/or sell copies
 *   of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions: The above
 *   copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.  THE SOFTWARE
 *   IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *   NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITIES,
 *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 *   IMPORTANT:
 *
 *   There are some packages that are distributed with MyGrid:
 *   activation.jar, xmlParserAPIs.jar, xercesImpl.jar, ogsa.jar,
 *   cog-jglobus.jar, mjs.jar and AbsoluteLayout.jar.  Their license
 *   terms are included in MyGrid documentation, read them before
 *   changing and/or distributing any content of these packages.
*/
/**
 * $Date: 2005/04/22 13:53:42 $
 * $Revision: 1.1 $
 * (BID116-FIXED)
 */

import java.io.File;
import java.io.IOException;

/**
 * This class appends a write into the file "fileName" with the description of
 * tasks that can be executed by the grid machines.
 * Each task created by this class has the form:
 *          task:
 *          init=  ..../sample/init
 *          remote= java mygrid.sample.Fat min max number output-$TASK
 *          final= ..../sample/collect
 *          processor= linux
 *          playpensize= 0  
 *          cost = 1
 *
 * The number of task descriptions added to the file "tasks" is equal to the
 * parameter passed to the main method.
 */

public class Factory {

	private static final File inputFile = (new File("Fat.class"));
	private static final File outputFile = (new File("output/output-"));
	private static final String fileName = (new String("Job.jdf"));
	
	/**
	 * An auxiliar method that performs the write of one
	 * task into the "Job" file.
	 * @param remote the remote field of the task to be added
	 * @throws IOException if can't write into "tasks" file
	 */
	private static void writeTask(String remote) throws IOException {
		StringBuffer taskDesc=  new StringBuffer();
		taskDesc.append("task :\n");

		taskDesc.append("	remote : ");
		taskDesc.append( remote );
		taskDesc.append("\n");
		taskDesc.append("	final : get "+ outputFile.getName()+"$TASK output/" + outputFile.getName()+"$TASK \n\n");
		FileUtil.append(fileName, taskDesc.toString());
	}

	/**
	 * Wtrite n task descriptions into the "fileName" file, where n is
	 * the number passed as argummet. The remote field of each task 
	 * is:
	 *
	 *      java mygrid.sample.Fat min max number  output-$TASK
	 *
	 * where the min field in a task is a identical integer to 
	 * the max field in its precedent task. In the first task min is
	 * 3 and in the last, max is iquals to the number field.
	 */
	public static void main(String args[]) {

		if (args.length != 1) {
			System.err.println("Fatora usage: Fatora <num-of-tasks>");
		}

		int quantIntvs = Integer.parseInt(args[0]);

		long number = (long) (Math.random() * (Long.MAX_VALUE - 1));
		if (number % 2 == 0) {
			number--;
		}

		try {
			File f = new File(fileName);
			if (f.exists()) {
				if (!f.delete()) {
					System.err.println("Could not remove file " + fileName);
					System.exit(1);
				}
			}
			FileUtil.append(fileName, "Job : \n");
			FileUtil.append(fileName, "	label : Fat\n");
			FileUtil.append(fileName, "	init : put ../resources/" + inputFile.getName() + " " + inputFile.getName() + "\n\n"); 
			

			double sqrt = Math.sqrt((double) number);
			long intv = (long)Math.ceil(sqrt / quantIntvs); //Perhaps it will do a little bit more than necessary. No problem.
			System.out.println("Interval = " + intv + " | Root = " + sqrt + " | BigNumber = " + number);

			
			for (long i = 3; i < sqrt; i += intv) { 
				writeTask("java -cp . Fat " + i + " " + (i + intv) + " " + number + " output-$TASK");
			}			

		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			System.exit(1);
		}

	}
}
