/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

import java.text.MessageFormat;

import org.ourgrid.common.exception.OurgridException;

/**
 * Used to signalize errors related to the MyGrid properties (see MG.properties)
 * and its uses.
 */
public class ConfigException extends OurgridException {

	private static final long serialVersionUID = 33L;

	private String parameter = "";


	public ConfigException( Throwable detail ) {

		super( detail );
	}


	/**
	 * Constructor that receives a string that defines the MyGrid property
	 * related to the exception.
	 * 
	 * @param wrongParameter the MyGrid property related to the exception.
	 */
	public ConfigException( String wrongParameter ) {

		super( wrongParameter );
		this.parameter = wrongParameter;
	}


	public ConfigException( String wrongParameter, Throwable detail ) {

		super( detail );
		this.parameter = wrongParameter;
	}


	/**
	 * @return the property that is related with the exception.
	 */
	public String getParameter() {

		return parameter;
	}


	/**
	 * @see java.lang.Throwable#getMessage()
	 */
	public String getMessage() {

		return MessageFormat.format( super.getMessage(), getParameter() );
	}
}
