/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

/**
 * Corepeer configuration properties.
 */
public class CorePeerConfiguration extends Configuration {

	/** Corepeer configuration rootdir. */
	public static final String ROOT_DIR = getConfDir();

	/** Corepeer configuration file. */
	public static final String PROPERTIES_FILENAME = ROOT_DIR + "/corepeer.properties";

	/** Delay between peer tests. */
	public static long UPDATE_DELAY = 1000 * 60 * 30;

	/** Corepeer property of delay between peer tests. */
	public static final String PROP_UPDATE_DELAY = "updatedelay";

	/** Peer property prefix. */
	public static final String PREFIX = "corepeer.";


	/**
	 * Default constructor
	 */
	public CorePeerConfiguration() {

		setProperty( PROP_PORT, "3087" );
		setProperty( PROP_NAME, getHostname() );
		setProperty( PROP_UPDATE_DELAY, Long.toString( UPDATE_DELAY ) );
		setProperty( PROP_LOG_PROPERTIES_FILE, "corepeer.log.properties" );

		loadPropertiesFromFile( PROPERTIES_FILENAME );

		if ( getProperty( PROP_EXTERNAL_NAME ) == null )
			setProperty( PROP_EXTERNAL_NAME, getProperty( PROP_NAME ) );
	}


	/**
	 * Returns the configuration directory.
	 * 
	 * @return The directory.
	 */
	protected static String getConfDir() {

		String prop = System.getProperty( "OGROOT" );

		if ( prop == null || prop.equals( "" ) ) {
			return System.getProperty( "MGROOT" );
		}
		return prop;
	}


	protected String getPrefix() {

		return PREFIX;
	}


	public String getRootDir() {

		return ROOT_DIR;
	}


	public String getLogPath() {

		String logfile = getProperty( PROP_LOGFILE );

		if ( logfile == null ) {
			String peerName = getProperty( Configuration.PROP_NAME );
			String peerPort = getProperty( Configuration.PROP_PORT );
			logfile = getRootDir() + "/log/" + peerName + "-" + peerPort + ".log";

		}

		return logfile;
	}

}
