/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

import java.io.File;
import java.rmi.registry.Registry;

/**
 * Description: MyGrid configuration properties
 * 
 * @version 1.0 Date14/09/2004
 */
public class MyGridConfiguration extends Configuration {

	public static final String SETGRID_COMMAND = "setgrid";

	public static final String ROOT_DIR = getConfDir() + File.separator + ".mygrid";

	/**
	 * MyGrid properties file
	 */
	public static final String PROPERTIES_FILENAME = ROOT_DIR + File.separator + "mg.properties";

	/**
	 * The property key for the Maximum Number of Failed Replicas to consider
	 * the Task as Failed
	 */
	public static final String PROP_MAX_FAILS = "maxfails";

	/** The property key for the Heuristic type */
	public static final String PROP_HEURISTIC = "heuristic";

	/** The property key for the Maximum Number of Replicas */
	public static final String PROP_MAX_REPLICAS = "maxreplicas";

	public static final String PROP_PERSISTJOBID = "persist.jobid";

	public static final String PROP_JOBCOUNTERFILE = ROOT_DIR + File.separator + "mg.jobid";

	/**
	 * The property key that sets the default GDF (Setting this property, the
	 * user avoid to do setGrid manually)
	 */
	public static final String PROP_DEFAULT_GDF = "default.gdf";

	/**
	 * Added by Hunter Hagewood 09/03/2005 The property key for the
	 * DataDiscovery cache
	 */
	public static final String PROP_CACHE_DDFILE = "cache.ddfile";

	/** MyGrid property prefix */
	public static final String PREFIX = "mg.";

	/**
	 * MyGrid log properties file
	 */
	public static final String LOG_PROPERTIES_FILENAME = "mygrid.log.properties";

	/**
	 * MyGrid log properties file destination (in .mygrid folder)
	 */
	public static final String LOG_PROPERTIES_FILENAME_DESTINATION = ROOT_DIR + File.separator
			+ LOG_PROPERTIES_FILENAME;

	/**
	 * Max number of fails of an given machine executing a given job to enter in
	 * the black list.
	 */
	public static final String PROP_MAX_BL_FAILS = "max.blacklist.fails";

	/**
	 * The default value to the property PROP_MAX_BL_FAILS
	 */
	public static final String DEFAULT_MAX_BL_FAILS = "10";

	/** Port used Mygrid GUI to publish its listeners */
	public static final String PROP_GUI_PORT = "gui.port";

	private static final String DEFAULT_GUI_PORT = "4040";

	public static final String PROP_GUI_NAME = "gui.name";


	/**
	 * Default constructor
	 */
	public MyGridConfiguration() {

		setProperty( PROP_NAME, getHostname() );
		setProperty( PROP_PORT, Integer.toString( Registry.REGISTRY_PORT ) );
		setProperty( PROP_MAX_FAILS, "3" );
		setProperty( PROP_HEURISTIC, "workqueue" );
		setProperty( PROP_MAX_REPLICAS, "1" );
		setProperty( PROP_CACHE_DDFILE, "no" );
		setProperty( PROP_SECURE_PORT, "0" );
		setProperty( PROP_PERSISTJOBID, "yes" );
		setProperty( PROP_LOGFILE, "mygrid.log" );
		setProperty( PROP_LOG_PROPERTIES_FILE, LOG_PROPERTIES_FILENAME );
		setProperty( PROP_MAX_BL_FAILS, DEFAULT_MAX_BL_FAILS );
		setProperty( PROP_GUI_PORT, DEFAULT_GUI_PORT );
		setProperty( PROP_GUI_NAME, getHostname() );

		loadPropertiesFromFile( PROPERTIES_FILENAME );

		if ( getProperty( PROP_EXTERNAL_NAME ) == null )
			setProperty( PROP_EXTERNAL_NAME, getProperty( PROP_NAME ) );

	}


	/**
	 * Returns the configuration directory.
	 * 
	 * @return The directory.
	 */
	protected static String getConfDir() {

		String prop = System.getProperty( "OGROOT" );

		if ( prop == null || prop.equals( "" ) ) {
			prop = System.getProperty( "user.home" );
		}
		return prop;
	}


	@Override
	protected String getPrefix() {

		return PREFIX;
	}


	@Override
	public String getRootDir() {

		return ROOT_DIR;
	}

}
