/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;

/**
 * This test is used to verify if the OGROOT property is been loaded correctly.
 */
public class OGROOTTest extends TestCase implements CommonTest {

	/*
	 * Initialization of the test
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

	}


	/*
	 * Finishes the test
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

	}


	/**
	 * Tests the <code>getRootDir()</code> method when OGROOT property isn't
	 * defined.
	 */
	public void testWhenOGROOTIsNotDefined() {

		// check the root dir (used to save the configuration files)
		Configuration.reset();
		assertEquals( System.getProperty( "user.home" ), MyGridConfiguration.getConfDir() );
		Configuration.reset();
		assertEquals( System.getProperty( "MGROOT" ), PeerConfiguration.getConfDir() );

		// sets the OGROOT empty
		System.setProperty( "OGROOT", "" );

		// check the root dir
		Configuration.reset();
		assertEquals( System.getProperty( "user.home" ), MyGridConfiguration.getConfDir() );
		Configuration.reset();
		assertEquals( System.getProperty( "MGROOT" ), PeerConfiguration.getConfDir() );

	}


	/**
	 * Tests the <code>getRootDir()</code> method when OGROOT property is
	 * defined.
	 */
	public void testWhenOGROOTIsDefined() {

		// setting the $OGROOT value
		System.setProperty( "OGROOT", "tmp_dir" );

		// check the root dir (used to save the configuration files)
		Configuration.reset();
		assertEquals( "tmp_dir", MyGridConfiguration.getConfDir() );
		Configuration.reset();
		assertEquals( "tmp_dir", PeerConfiguration.getConfDir() );

	}

}
