/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

public class PeerConfiguration extends Configuration {

	public static final String SETGUMS_COMMAND = "setgums";

	public static final String ROOT_DIR = getConfDir();

	public static final String PROPERTIES_FILENAME = ROOT_DIR + "/peer.properties";

	public static final String PROP_IDFILE = "peeridfile";

	public static final String PROP_LABEL = "label";

	public static final String PROP_RANKINGFILE = "rankingfile";

	public static final String PROP_JOIN_COMMUNITY = "joincommunity";

	public static final String PROP_COREPEER_NAME = "corepeername";

	public static final String PROP_COREPEER_PORT = "corepeerport";

	public static final String PROP_EMAIL = "email";

	public static final String PROP_DESCRIPTION = "description";

	public static final String PROP_SAVING_INTERVAL = "interval.saving";

	public static final String PROP_REDISTRIBUTING_INTERVAL = "interval.redistribute";

	public static final String PROP_PING_INTERVAL = "interval.ping";

	public static final String PROP_LATITUDE = "latitude";

	public static final String PROP_LONGITUDE = "longitude";

	public static final String PREFIX = "peer.";

	// The default SDF file where the gums to be loaded are listed
	public static final String PROP_DEFAULT_SDF = "default.sdf";


	// Constructor
	public PeerConfiguration() {

		setProperty( PROP_PORT, "3081" );
		setProperty( PROP_NAME, getHostname() );
		setProperty( PROP_LABEL, getHostname() );
		setProperty( PROP_RANKINGFILE, getRootDir() + "/rankings.dat" );

		setProperty( PROP_LOG_PROPERTIES_FILE, "peer.log.properties" );
		setProperty( PROP_JOIN_COMMUNITY, "yes" );
		setProperty( PROP_COREPEER_NAME, "corepeer.lsd.ufcg.edu.br" );
		setProperty( PROP_COREPEER_PORT, "3087" );
		setProperty( PROP_SECURE_PORT, "0" );
		setProperty( PROP_EMAIL, "no e-mail available" );
		setProperty( PROP_DESCRIPTION, "no description available" );

		// times in milliseconds
		setProperty( PROP_SAVING_INTERVAL, "1800000" );
		setProperty( PROP_PING_INTERVAL, "600000" );
		setProperty( PROP_REDISTRIBUTING_INTERVAL, "1800000" );
		setProperty( PROP_LATITUDE, "0" );
		setProperty( PROP_LONGITUDE, "0" );

		loadPropertiesFromFile( PROPERTIES_FILENAME );

		setProperty( PROP_IDFILE, ROOT_DIR + "/.peerid-" + this.getProperty( Configuration.PROP_NAME ) + ":"
				+ this.getProperty( Configuration.PROP_PORT ) );

		if ( getProperty( PROP_EXTERNAL_NAME ) == null )
			setProperty( PROP_EXTERNAL_NAME, getProperty( PROP_NAME ) );

	}


	/**
	 * Returns the configuration directory.
	 * 
	 * @return The directory.
	 */
	protected static String getConfDir() {

		String prop = System.getProperty( "OGROOT" );

		if ( prop == null || prop.equals( "" ) ) {
			return System.getProperty( "MGROOT" );
		}
		return prop;
	}


	protected String getPrefix() {

		return PREFIX;
	}


	public String getRootDir() {

		return ROOT_DIR;
	}


	public String getLogPath() {

		String logfile = getProperty( PROP_LOGFILE );

		if ( logfile == null ) {
			String peerName = getProperty( Configuration.PROP_NAME );
			String peerPort = getProperty( Configuration.PROP_PORT );
			logfile = getRootDir() + "/log/" + peerName + "-" + peerPort + ".log";

		}

		return logfile;
	}

}
