/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.config;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

public class UserAgentConfiguration extends Configuration {

	/**
	 * The UA property prefix
	 */
	public static final String PREFIX = "ua.";

	private static final String ROOT_DIR = System.getProperty( "user.dir" );

	public static final String PROPERTIES_FILENAME = ROOT_DIR + File.separator + "ua.properties";

	public static final String PROP_STORAGE_DIR = "storagedir";

	public static final String PROP_STORAGE_SIZE = "storagesize";

	public static final String PROP_STORAGE_SHARED = "storageshared";

	public static final String PROP_PLAYPEN_ROOT = "playpenroot";

	public static final String PROP_PLAYPEN_SIZE = "playpensize";

	public static final String PROP_IDLENESS_DETECTOR = "idlenessdetector";

	public static final String PROP_IDLENESS_TIME = "idlenesstime";

	public static final String ATT_TYPE = "type";

	public static final String ATT_REM_EXEC = "remexec";

	public static final String ATT_COPY_FROM = "copyfrom";

	public static final String ATT_COPY_TO = "copyto";

	public static final String ATT_PLAYPEN_ROOT = "playpenroot";

	public static final String ATT_PLAYPEN_SIZE = "playpensize";

	public static final String ATT_STORAGE_DIR = "storagedir";

	public static final String ATT_STORAGE_SIZE = "storagesize";

	public static final String ATT_OS = "os";

	public static final String ATT_MEM = "mem";

	public static final String ATT_SITE = "site";

	public static final String ATT_STORAGE_SHARED = "storageshared";

	public static final String TYPE_UA_LINUX = "ualinux";

	public static final String TYPE_UA_WINDOWS = "uawindows";

	public static final String TYPE_UA_SWAN = "uaswan";

	public static final String OS_LINUX = "linux";

	public static final String OS_WINDOWS = "windows";

	public static final String OS_LINUX_XEN = "/proc/xen";

	public static final int QUERY_FREQ = 120000;

	/**
	 * ua.properties default values
	 */
	private static final String DEF_PROP_PORT = "3080";

	private static final String DEF_PROP_SECURITY = "no";

	private static final String DEF_PROP_STORAGE_DIR = ".mgstorage";

	private static final String DEF_PROP_STORAGE_SIZE = "0";

	private static final String DEF_PROP_STORAGE_SHARED = "no";

	private static final String DEF_PROP_PLAYPEN_ROOT = File.separator + "tmp";

	private static final String DEF_PROP_PLAYPEN_SIZE = "0";

	private static final String DEF_PROP_NAME = "localhost";

	private static final String DEF_PROP_LOGFILE = "useragent.log";

	private static final String DEF_PROP_IDLENESS_DETECTOR = "no";

	private static final String DEF_PROP_IDLENESS_TIME = "1200"; // 20
																	// minutes

	private static final String DEF_PROP_SECURE_PORT = "0";

	private static final String DEF_PROP_LOG_PROPERTIES_FILE = "ua.log.properties";


	/**
	 * Default constructor
	 */
	public UserAgentConfiguration() {

		setProperty( PROP_PORT, DEF_PROP_PORT );
		setProperty( PROP_SECURITY, DEF_PROP_SECURITY );
		setProperty( PROP_STORAGE_DIR, DEF_PROP_STORAGE_DIR );
		setProperty( PROP_STORAGE_SIZE, DEF_PROP_STORAGE_SIZE );
		setProperty( PROP_STORAGE_SHARED, DEF_PROP_STORAGE_SHARED );
		setProperty( PROP_PLAYPEN_ROOT, DEF_PROP_PLAYPEN_ROOT );
		setProperty( PROP_PLAYPEN_SIZE, DEF_PROP_PLAYPEN_SIZE );
		setProperty( PROP_NAME, DEF_PROP_NAME );
		setProperty( PROP_LOGFILE, DEF_PROP_LOGFILE );
		setProperty( PROP_LOG_PROPERTIES_FILE, DEF_PROP_LOG_PROPERTIES_FILE );
		setProperty( PROP_IDLENESS_DETECTOR, DEF_PROP_IDLENESS_DETECTOR );
		setProperty( PROP_IDLENESS_TIME, DEF_PROP_IDLENESS_TIME ); // 20
		// minutes
		// in
		// seconds
		setProperty( PROP_SECURE_PORT, DEF_PROP_SECURE_PORT );

		loadPropertiesFromFile( PROPERTIES_FILENAME );

		if ( getProperty( PROP_EXTERNAL_NAME ) == null )
			setProperty( PROP_EXTERNAL_NAME, getProperty( PROP_NAME ) );

	}


	protected String getPrefix() {

		return PREFIX;
	}


	public String getRootDir() {

		return ROOT_DIR;
	}


	/**
	 * Gets default values in a Map object.
	 * 
	 * @return A Map object that holds default values
	 */
	public static Map<String,String> getDefaults() {

		Map<String,String> defaults;

		defaults = new HashMap<String,String>();

		defaults.put( PREFIX + Configuration.PROP_PORT, DEF_PROP_PORT );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_PLAYPEN_ROOT, DEF_PROP_PLAYPEN_ROOT );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_PLAYPEN_SIZE, DEF_PROP_PLAYPEN_SIZE );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_STORAGE_DIR, DEF_PROP_STORAGE_DIR );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_STORAGE_SIZE, DEF_PROP_STORAGE_SIZE );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_STORAGE_SHARED, DEF_PROP_STORAGE_SHARED );
		defaults.put( PREFIX + Configuration.PROP_SECURITY, DEF_PROP_SECURITY );
		defaults.put( PREFIX + Configuration.PROP_LOGFILE, DEF_PROP_LOGFILE );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_IDLENESS_DETECTOR, DEF_PROP_IDLENESS_DETECTOR );
		defaults.put( PREFIX + UserAgentConfiguration.PROP_IDLENESS_TIME, DEF_PROP_IDLENESS_TIME );
		defaults.put( PREFIX + Configuration.PROP_SECURE_PORT, DEF_PROP_SECURE_PORT );

		return defaults;
	}

}
