/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.exception;

import java.text.MessageFormat;

/**
 * This class represents a signal that some problem during the generation of a
 * key has occurred.
 */
public class CouldNotGenerateKeyException extends OurgridException {

	private static final long serialVersionUID = 33L;

	/** Description for the exception cause */
	String exceptionCause;


	/**
	 * Constructs a new Exception to signal that a problem with the key
	 * generation has occurred.
	 */
	public CouldNotGenerateKeyException() {

		super();
	}


	/**
	 * Cosntructs a new Exception to signal that a problem with the key
	 * generation has occured.
	 * 
	 * @param cause An string describing briefly the cause of the exception.
	 * @param detail The exception that is the origin of the problem.
	 */
	public CouldNotGenerateKeyException( String cause, Throwable detail ) {

		super( detail );
		this.exceptionCause = cause;
	}


	/**
	 * Constructs a new Exception to signal that a problem with the key
	 * generation has occured.
	 * 
	 * @param cause An string describing briefly the cause of the exception.
	 */
	public CouldNotGenerateKeyException( String cause ) {

		super();
		this.exceptionCause = cause;
	}


	/**
	 * Returns the exception message.
	 * 
	 * @return The message
	 */
	public String getMessage() {

		Object params[] = { exceptionCause };
		return MessageFormat.format( super.getMessage(), params );
	}

}
