/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.exception;

import java.io.PrintStream;
import java.io.PrintWriter;

import org.ourgrid.common.util.ArrayUtil;

/**
 * This class is the Mother-Exception of the MyGrid. The idea is to use its
 * subclasses to signal that some problem with MyGrid-semantic has occured. For
 * instance, problems when creating Playpen or Storage, since both are concepts
 * defined by MyGrid.
 */
public class OurgridException extends Exception {

	static final long serialVersionUID = 33L;


	/**
	 * Constructs a new MyGridException object
	 */
	public OurgridException() {

		super( null, null );
	}


	/**
	 * Constructs a new MyGridException object
	 * 
	 * @param newDetail an object that specifies the exception details
	 */
	public OurgridException( Throwable newDetail ) {

		super( null, newDetail );

	}


	/**
	 * Constructs a new MyGridException object
	 * 
	 * @param message a string representing the exception message
	 */
	public OurgridException( String message ) {

		super( message, null );

	}


	/**
	 * Constructs a new MyGridException object
	 * 
	 * @param message a string representing the exception message
	 * @param newDetail an object that specifies the exception details
	 */
	public OurgridException( String message, Throwable newDetail ) {

		super( message, newDetail );
	}


	/**
	 * 
	 */
	public StackTraceElement[ ] getStackTrace() {

		Throwable detail = getCause();

		if ( detail != null ) {
			StackTraceElement[ ] stackTrace = new StackTraceElement[ detail.getStackTrace().length
					+ super.getStackTrace().length ];
			ArrayUtil.concat( detail.getStackTrace(), super.getStackTrace(), stackTrace );
			return stackTrace;
		}

		return super.getStackTrace();
	}


	public void printStackTrace() {

		printStackTrace( System.out );
	}


	public void printStackTrace( PrintWriter pw ) {

		Throwable detail = getCause();

		if ( detail != null ) {
			detail.printStackTrace( pw );
		}
		super.printStackTrace( pw );
	}


	public void printStackTrace( PrintStream ps ) {

		Throwable detail = getCause();

		if ( detail != null ) {
			detail.printStackTrace( ps );
		}
		super.printStackTrace( ps );
	}


	public String getMessage() {

		StringBuffer message = new StringBuffer();

		message.append( "Exception: " + getClass().getName() );
		message.append( "\nAdditional info: " + super.getMessage() );

		if ( getCause() != null ) {
			message.append( "\nNested exception: " + getCause().getClass().getName() + ":" + getCause().getMessage() );
		}
		return message.toString();

	}
}
