/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.exception;

import org.ourgrid.gridmachine.exception.FailCauseException;

/**
 * Signals that the creation of Playpen was not possible due to some problem.
 * Playpen is an area reserved to store temporary file used during the execution
 * of a Task at a Grid Machine. Possible problems related to the creation of
 * Playpens are: <br>
 * <ul>
 * <li> Permission denied
 * <li> Disk space unavailable
 * <li> Since "/tmp" is assumed like parent dir by default, in some particualr
 * circunstances this directory may not exist.
 * </ul>
 */
public class UnableToCreatePlaypenException extends FailCauseException {

	private static final long serialVersionUID = 33L;


	public UnableToCreatePlaypenException() {

		super();
	}


	public UnableToCreatePlaypenException( String message, Throwable newDetail ) {

		super( message, newDetail );
	}


	public UnableToCreatePlaypenException( String message ) {

		super( message );
	}


	public UnableToCreatePlaypenException( Throwable newDetail ) {

		super( newDetail );
	}


	@Override
	public String getSimpleMessage() {

		return "Unable to create Playpen, it is likely that the playpen directory is full on the" + "Grid Machine.";
	}
}
