/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.exception;

import org.ourgrid.gridmachine.exception.FailCauseException;

/**
 * Signals that the creation of Storage Directory was not possible due to some
 * problem. The Storage Directory is supposed to be a shared area among the
 * Tasks and Jobs. The idea is to allow Jobs to reuse data used/generated for
 * Jobs which ran before. Possible problems related to the creation of Storage
 * Directory are: <br>
 * <ul>
 * <li> Permission denied
 * <li> Disk space unavailable
 * </ul>
 */
public class UnableToCreateStorageException extends FailCauseException {

	private static final long serialVersionUID = 33L;


	/**
	 * Constructs an instance of UnableToCreateStorageException without any
	 * special description.
	 */
	public UnableToCreateStorageException() {

		super();
	}


	/**
	 * Constructs an instance of UnableCreateStorageException using the
	 * descriptions provided by the Throwable object.
	 * 
	 * @param newDetail An object of an implementor of Throwable interface that
	 *        contains details about the exception.
	 */
	public UnableToCreateStorageException( Throwable newDetail ) {

		super( newDetail );
	}


	/**
	 * Constructs an instance of UnableCreateStorageException using the
	 * descriptions provided by the Throwable object and the additional message.
	 * 
	 * @param message The additional message about the problem occured during
	 *        the Storage creation.
	 * @param newDetail The object that contains the details about the
	 *        exception.
	 */
	public UnableToCreateStorageException( String message, Throwable newDetail ) {

		super( message, newDetail );
	}


	@Override
	public String getSimpleMessage() {

		return "Unable to use Storage, it is likely that the storage directory is full on the" + "Grid Machine.";
	}

}
