/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.exception;

import java.text.MessageFormat;

/**
 * This class represents an Exception thrown when it is not possible to get a
 * digest representation (hash) from a particular file
 */
public class UnableToDigestFileException extends OurgridException {

	private static final long serialVersionUID = 33L;

	/** The path to the file where the digest process failed */
	String filePath = new String();


	/**
	 * Return the file path used during the digest process
	 * 
	 * @return the string representation of file path used during the digest
	 *         process
	 */
	public String getFile() {

		return filePath;
	}


	/**
	 * Constructs a new exception that represents a problem during the digest
	 * representation calculation of a given file.
	 * 
	 * @param filePath the file path used during the digest process.
	 */
	public UnableToDigestFileException( String filePath ) {

		super();
		this.filePath = filePath;
	}


	/**
	 * Constructs a new exception that represents a problem during the digest
	 * representation calculation of a given file. This constructor encapsulates
	 * another throwable class that possibly caused the failure.
	 * 
	 * @param filePath the file path used during the digest process.
	 * @param detail the throwable class possibly caused the failure.
	 */
	public UnableToDigestFileException( String filePath, Throwable detail ) {

		super( detail );
		this.filePath = filePath;
	}


	/**
	 * Gives the message detailing the exception occured during the file digest
	 * process.
	 * 
	 * @return the message associated to the digestion
	 */
	public String getMessage() {

		Object params[] = { this.filePath };
		return MessageFormat.format( super.getMessage(), params );
	}

}
