/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.executor;

/**
 * This class define an abstraction for the operating system dependent part of
 * executing commands. The concrete implementors of this class must provide the
 * correct behavior for executing a command, killing a process and changing the
 * permissions of files.
 */
public abstract class AbstractExecutor implements Executor {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;


	/**
	 * Creates and returns a instance of ExcecutorResult that will contain the
	 * informations about the finished process.
	 * 
	 * @param process The process that is supposed to be finished and the
	 *        streams must be caught and the exit value recovered.
	 * @return An instance of ExecutorResult describing the result of <i>process</i>
	 *         execution.
	 * @throws InterruptedException If the thread that is catching the result is
	 *         interrupted.
	 */
	protected ExecutorResult catchOutput( Process process ) throws InterruptedException {

		/* create an instance of Executor result information class */
		ExecutorResult result = new ExecutorResult();

		OutputCatcher stdOutput = new OutputCatcher( process.getInputStream() );
		OutputCatcher stdErr = new OutputCatcher( process.getErrorStream() );

		result.setExitValue( process.waitFor() );

		result.setStdout( stdOutput.getResult() );
		result.setStderr( stdErr.getResult() );

		return result;
	}

}
