/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.executor;

import java.io.File;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.UserAgentConfiguration;

/**
 * This class is responsible for obtaining an Executor instace according to the
 * Operating System being used.
 */
public class ExecutorFactory {

	/** The sigleton instance of a concrete executor */
	private static Executor instanceOnly = null;


	/**
	 * This is a factory method that returns a Executor depending on the
	 * platform the software is running.
	 * 
	 * @return a concrete implementation of Executor depending on the OS
	 */
	public static synchronized Executor getExecutorInstance() {

		/* Get the operating system platform */
		String os = System.getProperty( "os.name" ).toLowerCase();

		if ( instanceOnly == null ) {

			if ( os.equalsIgnoreCase( UserAgentConfiguration.OS_LINUX ) ) {
				/*
				 * If the OS is linux it should check if it is running on SWAN,
				 * which uses xeno.
				 */
				File tmpXen = new File( UserAgentConfiguration.OS_LINUX_XEN );
				if ( tmpXen.exists() && Configuration.getInstance() instanceof UserAgentConfiguration ) {
					instanceOnly = new SWANExecutor();
				} else {
					instanceOnly = new LinuxExecutor();
				}

			} else if ( os.indexOf( UserAgentConfiguration.OS_WINDOWS ) != -1 ) {
				instanceOnly = new Win32Executor();
			}
		}

		return instanceOnly;
	}

}
