/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.executor;

import junit.framework.TestCase;

import org.ourgrid.test.ReplicaExecutorFastTest;
import org.ourgrid.test.threadServices.ThreadServices;

/**
 * This class is the test suite for the ExecutorFactory class.
 * 
 * @author Elizeu Santos-Neto, elizeu@dsc.ufcg.edu.br
 */
public class ExecutorFactoryTest extends TestCase implements ReplicaExecutorFastTest {

	/**
	 * This method constructs a new instance of ExecutorFactoryTest suite class.
	 * 
	 * @param name The name of class file that corresponds to this test suite.
	 */
	public ExecutorFactoryTest( String name ) {

		super( name );
	}


	/**
	 * Set the environment before starting the tests!
	 */
	protected void setUp() throws Exception {

		super.setUp();
	}


	/**
	 * Just clean the evironment before dying!
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	/**
	 * This method test the get instance method. In order to guarantee the
	 * instance is the same. A simple but necessary test!
	 */
	public void testGetInstance() throws Exception {

		/** Executor instances (singletons) obtained by the Executor Factory */
		Executor executor1 = ExecutorFactory.getExecutorInstance();
		Executor executor2 = ExecutorFactory.getExecutorInstance();

		assertEquals( "the command must be successful", executor1, executor2 );
	}


	/**
	 * This method test the get instance method. In order to guarantee the
	 * instance is the same. A simple but necessary test!
	 */
	public void testGetInstanceMultiThreaded() throws Exception {

		/** Create the Threads */
		ExecutorThread thread1 = new ExecutorThread();
		ExecutorThread thread2 = new ExecutorThread();
		ExecutorThread thread3 = new ExecutorThread();

		/* Verifiy before constructing the Executor singleton object */
		assertEquals( "The Executor Reference should be the same. T1 [" + thread1 + "] vs. T2 [" + thread2 + "].",
				thread1.getExecutorReference(), thread2.getExecutorReference() );

		assertEquals( "The Executor Reference should be the same. T1 [" + thread1 + "] vs. T3 [" + thread3 + "].",
				thread1.getExecutorReference(), thread3.getExecutorReference() );

		assertEquals( "The Executor Reference should be the same. T2 [" + thread2 + "] vs. T3 [" + thread3 + "].",
				thread2.getExecutorReference(), thread3.getExecutorReference() );

		/** Start the inner threads */
		thread1.start();
		thread2.start();
		thread3.start();

		/** Using aspects to perform the assertions on a consistent environment. */
		ThreadServices.waitUntilThreadHasFinished( thread1 );
		ThreadServices.waitUntilThreadHasFinished( thread2 );
		ThreadServices.waitUntilThreadHasFinished( thread3 );

		/* Verify after Cosntructing the Executor Singleton object */
		assertEquals( "The Executor Reference should be the same (after). T1 [" + thread1 + "] vs. T2 [" + thread2
				+ "].", thread1.getExecutorReference(), thread2.getExecutorReference() );

		assertEquals( "The Executor Reference should be the same (after). T1 [" + thread1 + "] vs. T3 [" + thread3
				+ "].", thread1.getExecutorReference(), thread3.getExecutorReference() );

		assertEquals( "The Executor Reference should be the same (after). T2 [" + thread2 + "] vs. T3 [" + thread3
				+ "].", thread2.getExecutorReference(), thread3.getExecutorReference() );

	}

}


/* Inner class just used to test the */
class ExecutorThread extends Thread {

	/* Private reference to an instance of executor */
	private Executor privateExecutor = null;


	public void run() {

		this.privateExecutor = ExecutorFactory.getExecutorInstance();
	}


	public Executor getExecutorReference() {

		return this.privateExecutor;
	}

}
