/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.ourgrid.common.fd;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

/**
 * Fake class used to test. At constructor, the user can set a
 * <code>Monitorable</code> that every call to
 * <code>getSynchronizedRemoteReference</code> returns this reference.
 */
public class FakeMonitoringUnit extends MonitoringUnit {

	private Monitorable monitorable;


	/**
	 * Constructor
	 * 
	 * @param initialState The <code>MonitoringUnit</code> initial state
	 * @param remoteFailureDetectorAddress The address The key
	 */
	public FakeMonitoringUnit( GenericState initialState, String remoteFailureDetectorAddress,
								String monitorableServiceIndex ) {

		this( initialState, remoteFailureDetectorAddress, monitorableServiceIndex, null );
	}


	/**
	 * Constructor
	 * 
	 * @param initialState The <code>MonitoringUnit</code> initial state
	 * @param monitorableAddress The address The key
	 * @param monitorable The monitorable reference to be returned at every call
	 *        to <code>getSynchronizedRemoteReference</code> method.
	 */
	public FakeMonitoringUnit( GenericState initialState, String monitorableAddress, String monitorableServiceIndex,
								Monitorable monitorable ) {

		super( initialState, monitorableAddress, monitorableServiceIndex );
		this.monitorable = monitorable;
	}


	@Override
	protected Monitorable getSynchronizedRemoteReference() throws RemoteException, MalformedURLException,
		NotBoundException {

		if ( this.monitorable == null ) {
			return new FakeMonitorable( getLastState(), getMonitorableIdentifier() );
		}
		return this.monitorable;
	}

}
