/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.fd;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * <b>Title:</b> <i>Failure Spotter - Research Line 2</i> <b>Copyright:</b>
 * <i>Copyright (c) 2004</i> <b>Company:</b> <i>UFCG-DSC/HP</i> The
 * Monitorable interface allows a monitor to probe a monitorable entity.
 * <p>
 * To recover session information about the <code>Monitorable</code> session
 * we are using the encarnation number. This number must be generated at
 * <code>Monitorable</code> construction, and is used to identify when a
 * <code>Monitorable</code> changes quickly it state, from from up to down and
 * then do up again (between two pings), so the <code>FailureDetector</code>
 * could not realize.
 * <p>
 * Note: By default, a monitorable should have two states:
 * <ol>
 * <li>DOWN : To represent that the <code>Monitorable</code> is down.</li>
 * <li>UP : To represent that the <code>Monitorable</code> is ok.</li>
 * </ol>
 * In application level one may use as many states as desired.
 * 
 * @see org.ourgrid.common.gum.GumStatus
 */
public interface Monitorable extends Remote {

	/**
	 * Retrieves the monitorable current status.
	 * 
	 * @return the <code>Monitorable</code> state.
	 * @throws RemoteException if anything wrong occurs in communication.
	 */
	public GenericState getCurrentState() throws RemoteException;


	/**
	 * Return an unique identifier of the <code>Monitorable</code> session.
	 * 
	 * @return a unique identifier of the <code>Monitorable</code> session.
	 * @throws RemoteException if anything wrong occurs in communication.
	 */
	public double getEncarnationNumber() throws RemoteException;
}
