/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.fd;

/**
 * A <b>Factory</b> of <code>MonitoringUnit</code> classes.
 */
public interface MonitoringUnitFactory {

	/**
	 * The creates a new instance of <code>MonitoringUnit</code> class (or
	 * subclass)
	 * 
	 * @param initialState The <code>MonitoringUnit</code> initial state
	 * @param monitorableAddress The address of the <code>MonitoringUnit</code>
	 *        to be created
	 * @param monitorableIdentifier An index name given by the
	 *        <code>Notifier</code> when register it interests to the
	 *        <code>Monitorable</code>. It will be used to notify that a stat
	 *        has changed.
	 * @return A new instance of <code>MonitoringUnit</code> class (or
	 *         subclass)
	 */
	public MonitoringUnit createMonitoringUnit( GenericState initialState, String monitorableAddress,
												String monitorableIdentifier );

}
