/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.fd;

/**
 * This interface must be implemented by entities that needs to be aware about
 * Grid Machines state changes. <br>
 * <br>
 * <br>
 * In <b>OurGrid</b> case the <code>Notifiable</code>s will be the entities
 * that will make use of <b>FailureDetector</b> use directly, and at least at
 * <b>3.3 version</b> will always be at the same <b>JVM</b>. <br>
 * <br>
 * With this design we simplifies the architecture, send notifying messages only
 * localy, and the interesteds entities knows how to deal with the advices.
 */
public interface Notifiable {

	/**
	 * This method is invoked when the Grid Machine state changes.
	 * 
	 * @param monitorableName The name of the Grid Machine
	 * @param monitorableState The new state
	 */
	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState );


	/**
	 * @return The name of the <code>Notifiable</code> entity.
	 */
	public String getName();
}
