/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.ourgrid.common.fd;

import java.rmi.RemoteException;
import java.util.concurrent.Executors;

import org.ourgrid.test.threadServices.ThreadPoolServices;
import org.ourgrid.test.threadServices.ThreadServices;

/**
 * 
 */
public class SimpleTestFailureDetector extends AbstractFailureDetector {

	private static final long serialVersionUID = 33L;


	/**
	 * @param factory The <code>MonitoringUnitFactory</code> to be used.
	 * @throws RemoteException If anything wrong occurs.
	 */
	public SimpleTestFailureDetector( MonitoringUnitFactory factory ) throws RemoteException {

		super( factory );
	}


	/**
	 * Constructor
	 * 
	 * @param queryFrequency The verification frequency.
	 * @throws RemoteException If anything wrong occurs.
	 */
	public SimpleTestFailureDetector( int queryFrequency ) throws RemoteException {

		super( queryFrequency );
	}


	@Override
	public GenericState getCurrentState() throws RemoteException {

		return new GenericState( State.UP );
	}


	/**
	 * This is a <b>ThreadSafe</b> implementation of
	 * <code>startMonitoring</code> method. This method makes
	 * <code>numberOfVerfications</code> verifications respecting the thread
	 * order. The second verification only will starts when the first terminates
	 * and so on. The method only finishes after all threads of all iterations
	 * have finished. <br>
	 * <br>
	 * Notice that this method does not take the time in count. The
	 * <code>QueryFrequency</code> is ignored here.
	 * 
	 * @param numberOfVerfications The number of verification iterations.
	 * @throws RemoteException If anything wrong occurs.
	 * @see ThreadServices
	 * @see ThreadServices#waitUntilWorkIsDone()
	 */
	public void startMonitoring( int numberOfVerfications ) throws RemoteException {

		for ( int i = 0; i < numberOfVerfications; i++ ) {
			Thread thread = new Thread( new MachinesPinger( this, Executors.newCachedThreadPool() ) );
			thread.start();
			ThreadServices.waitUntilWorkIsDone();
			ThreadPoolServices.waitUntilWorkIsDone();
		}
	}

}
