/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.gum;

import java.io.File;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import org.ourgrid.common.exception.UnavailableGumException;
import org.ourgrid.common.executor.ExecutorHandle;
import org.ourgrid.common.executor.ExecutorResult;
import org.ourgrid.common.executor.IntegerExecutorHandle;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.gridmachine.useragent.UserAgentClient;
import org.ourgrid.gridmachine.useragent.test.FakeUserAgentServer;

public class FakeGumClient extends UserAgentClient {

	private static final long serialVersionUID = 33L;

	private GumID gumID;

	int TIMETOSLEEP = 100;

	private GumStatus gumStatus;

	private Map<File,Long> filesAndSizes;


	public FakeGumClient( GumSpec gumSpec ) throws RemoteException {

		this( gumSpec.getGumID(), gumSpec );
	}


	public FakeGumClient( GumSpec gumSpec, File[ ] files, long[ ] sizes ) throws RemoteException {

		this( gumSpec.getGumID(), gumSpec );
		filesAndSizes = new HashMap<File,Long>();
		for ( int i = 0; i < files.length; i++ ) {
			filesAndSizes.put( files[i], new Long( sizes[i] ) );
		}
	}


	public FakeGumClient( GumID gumID, GumSpec spec ) throws RemoteException {

		super( (new FakeUserAgentServer( gumID, spec )), spec );
		this.gumID = gumID;
		this.gumStatus = GumStatus.UP;
	}


	public String createPlaypen( int size ) {

		return "Created";
	}


	public String createStorageDirectory() {

		return "Storage Directory Created";
	}


	public String getStorageDirectory() {

		return ".mgstorage";
	}


	public void getFile( String filePath, String localDir ) {

	}


	public void storeFile( String localFile, String remoteFile ) {

	}


	public long getRemoteFileLastModification( File remoteFile, boolean normalized ) {

		return 1;
	}


	public boolean fileExists( File file ) {

		return filesAndSizes.get( file ) != null;
	}


	public long getRemoteFileLength( File file ) {

		Long size = filesAndSizes.get( file );
		if ( size == null )
			return 0;
		return size.longValue();

	}


	/** put a file on remoteDir at grid machine */
	public void putFile( String filePath, String remoteDir ) {

	}


	/**
	 * @see org.ourgrid.common.gum.Gum#changeFilePermissions(java.io.File, int)
	 */
	public void changeFilePermissions( File file, int permissions ) {

	}


	/**
	 * Executes a remote command in a specific remoteDir
	 * 
	 * @throws UnavailableGumException
	 * @see org.ourgrid.common.gum.Gum#remoteExecute(String, String, Map)
	 */
	public ExecutorHandle remoteExecute( String remoteDir, String command, Map envVars ) throws UnavailableGumException {

		try {
			Thread.sleep( TIMETOSLEEP );
		} catch ( InterruptedException e ) {
			throw new UnavailableGumException( getID().toString(), e );
		}

		ExecutorHandle handle = new IntegerExecutorHandle( 1 );

		return handle;
	}


	public GumID getID() {

		return this.gumID;
	}


	public ExecutorResult getResult( ExecutorHandle handle ) {

		ExecutorResult result = new ExecutorResult();
		result.setExitValue( 0 );
		result.setStderr( "" );
		result.setStdout( "SUCESSO!" );

		return result;
	}


	public Gum getUAServer() throws RemoteException {

		return super.getUAServer();
	}


	public void setStatus( GumStatus status ) {

		this.gumStatus = status;
	}


	public GumStatus getStatus() {

		try {
			Thread.sleep( 10 );
		} catch ( InterruptedException e ) {
			// Waiting a little before returning...
		}
		return this.gumStatus;
	}
}
