/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.gum;

import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.State;

public class GumStateFD extends GenericState {

	private static final long serialVersionUID = 33L;

	private final GumStatus gumState;


	/**
	 * Default constructor
	 * 
	 * @param gumState the enumeration representing the gum state
	 */
	public GumStateFD( GumStatus gumState ) {

		super( gumState.basicState() );
		this.gumState = gumState;
	}


	/**
	 * A constructor to allow the creation a <code>GumStateFD</code> from a
	 * generic state
	 * 
	 * @param basicState the generic FD state (UP or DOWN)
	 */
	public GumStateFD( State basicState ) {

		super( basicState );
		switch ( basicState ) {
			case UP:
				gumState = GumStatus.UP;
				break;
			default:
				gumState = GumStatus.CONTACTING;
				break;
		}
	}


	public GumStatus gumState() {

		return gumState;
	}


	@Override
	public boolean equals( Object obj ) {

		if ( this == obj )
			return true;
		if ( !(obj instanceof GumStateFD) )
			return false;
		return super.equals( obj ) && ((GumStateFD) obj).gumState().equals( this.gumState );
	}


	@Override
	public String toString() {

		return gumState.toString() + "(" + super.toString() + ")";
	}

}
