package org.ourgrid.common.gump;

import java.rmi.Naming;
import java.rmi.registry.LocateRegistry;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.mygrid.scheduler.gridmanager.GumResponseEntry;
import org.ourgrid.test.PeerFastTest;
import org.ourgrid.test.TestUtils;

public class BindableMockGumpClientWrapperTest extends TestCase implements PeerFastTest {

	private TestUtils testUtils;

	private int registryPort;

	private GumpClient mock;

	private ObjectID ID;


	protected void setUp() throws Exception {

		this.testUtils = new TestUtils();
		this.testUtils.load();
		this.testUtils.createOGProperties();

		this.registryPort = Integer.parseInt( testUtils.getProperty( TestUtils.RMI_PORT ) );

		try {
			LocateRegistry.createRegistry( registryPort );
		} catch ( Exception e ) {
			LocateRegistry.getRegistry( registryPort );
		}

		Configuration.getInstance( Configuration.PEER );

		ID = new ObjectID( "MockGumClient", (long) 1, "rmi://localhost:" + registryPort + "/MockGumpClient" );

		mock = EasyMock.createMock( GumpClient.class );

		BindableMockGumpClientWrapper bindable = new BindableMockGumpClientWrapper( mock );

		Naming.rebind( ID.getObjectURL(), bindable );
	}


	protected void tearDown() throws Exception {

		super.tearDown();
	}


	/*
	 * Test method for
	 * 'org.ourgrid.common.gump.BindableMockGumpClientWrapper.hereIsGum(GumResponseEntry)'
	 */
	public void testHereIsGum() throws Exception {

		GumSpec spec = new GumSpec();
		spec.putAttribute( "NAME", "PROP" );

		ObjectID providerID = new ObjectID( "name", 1L, "url" );

		GumResponseEntry gumResp = new GumResponseEntry( null, spec, 1, providerID );

		mock.hereIsGum( EasyMock.eq( gumResp ) );
		EasyMock.expectLastCall().once();

		EasyMock.replay( mock );

		GumpClient client = (GumpClient) Naming.lookup( ID.getObjectURL() );

		client.hereIsGum( gumResp );

	}


	/*
	 * Test method for
	 * 'org.ourgrid.common.gump.BindableMockGumpClientWrapper.getObjectID()'
	 */
	public void testGetObjectID() throws Exception {

		mock.getObjectID();
		EasyMock.expectLastCall().andReturn( ID ).anyTimes();

		EasyMock.replay( mock );

		GumpClient client = (GumpClient) Naming.lookup( ID.getObjectURL() );

		assertEquals( client.getObjectID(), this.ID );
	}

}
