/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.gump;

import java.rmi.RemoteException;

import org.ourgrid.common.exception.RequestAlreadyExistsException;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.spec.RequestSpec;

/**
 * Used in tests to allow that a mock <code>GumpClient</code> can be exported
 * as a RMI object. This is done to allow that <code>Naming#lookup</code>
 * returns a mock object (actually the wrapper) when invoked.
 */
public class BindableMockGumpWrapper extends OurgridUnicastRemoteObject implements Gump {

	private static final long serialVersionUID = 33L;

	private final Gump gump;


	/**
	 * Default constructor
	 * 
	 * @param gump A mock to be wrapped.
	 * @throws RemoteException
	 */
	public BindableMockGumpWrapper( Gump gump ) throws RemoteException {

		super();
		this.gump = gump;
	}


	public void requestGums( RequestSpec requestSpec ) throws RemoteException, RequestAlreadyExistsException {

		gump.requestGums( requestSpec );
	}


	public void finishRequestGums( long requestID ) throws RemoteException {

		gump.finishRequestGums( requestID );
	}


	public void disposeGum( long requestID, GumID gumID ) throws RemoteException {

		gump.disposeGum( requestID, gumID );
	}


	public void pauseRequestGums( long requestID ) throws RemoteException {

		gump.pauseRequestGums( requestID );
	}


	public void resumeRequestGums( long requestID ) throws RemoteException {

		gump.resumeRequestGums( requestID );
	}


	public void unwantedGum( long requestID, GumID gumID ) throws RemoteException {

		gump.unwantedGum( requestID, gumID );
	}


	public ObjectID getObjectID() throws RemoteException {

		return gump.getObjectID();
	}
}
