/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.gump;

import java.rmi.Naming;

import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.mygrid.scheduler.gridmanager.GumResponseEntry;
import org.ourgrid.peer.community.nof.AccountingManager;
import org.ourgrid.peer.manager.EBPeerManagerFacade;

/**
 * This thread delivers a <code>GridMachine</code> for an external
 * <code>GridMachineConsumer</code>, eg.: MyGrid Broker.
 */
class DelivererThread implements Runnable {

	/**
	 * Logger to store log messages.
	 */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( DelivererThread.class );

	private EBPeerManagerFacade ebPeerManagerFacade;

	private GumResponseEntry gumResponseEntry;

	private ObjectID gumClientID;


	/**
	 * Default constructor.
	 * 
	 * @param gumClientID id of the consumer for which the gum will be delivered
	 * @param gumResponseEntry an object containing information about the gum
	 *        being delivered, including a reference to a <code>Gum</code>
	 *        type
	 * @param ebPeerManagerFacade
	 */
	public DelivererThread( ObjectID gumClientID, GumResponseEntry gumResponseEntry,
							EBPeerManagerFacade ebPeerManagerFacade ) {

		this.gumClientID = gumClientID;
		this.gumResponseEntry = gumResponseEntry;
		this.ebPeerManagerFacade = ebPeerManagerFacade;
	}


	public void run() {

		GumID gumID = gumResponseEntry.getGumSpec().getGumID();

		long requestID = gumResponseEntry.getRequestID();
		try {
			AccountingManager.getInstance().startAccounting( gumID, requestID );
			GumpClient gumpClient = (GumpClient) Naming.lookup( gumClientID.getObjectURL() );
			gumpClient.hereIsGum( gumResponseEntry );
			LOG.debug( "Gum " + gumID + " has been delivered to " + gumClientID.getObjectURL() );
		} catch ( Exception e ) {
			try {
				LOG.error( "Error delivering the Gum <" + gumID + "> for request <" + requestID + "> (client: + "
						+ gumClientID.getObjectURL() + ") Cause: " + e.getClass().getName() + ", " + e.getMessage() );
				ebPeerManagerFacade.finishRequestGums( requestID );

				// if the consumer couldn't be contacted, dispose the machine.
				Gump gump = (Gump) Naming.lookup( gumResponseEntry.getObjectID().getObjectURL() );
				gump.disposeGum( requestID, gumID );
			} catch ( Exception e1 ) {
				LOG.error( "Error disposing a Gum for request <" + requestID + ">." );
			}
		}
	}
}
