/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.gump;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.mygrid.scheduler.gridmanager.GumResponseEntry;

public class RemoteFakeGumpClient extends OurgridUnicastRemoteObject implements GumpClient {

	private static final long serialVersionUID = 33L;

	protected int expectedHereIsGuM = 0;

	protected int hereIsGuMCount = 0;

	private ObjectID objectID;

	private CountDownLatch verficationCountDownLatch;

	private List<GumResponseEntry> gumResponseList;


	public RemoteFakeGumpClient( String objectIdUrl ) throws RemoteException {

		double random = 1000 * Math.random();
		this.objectID = new ObjectID( objectIdUrl, Math.round( random ), objectIdUrl );

	}


	public RemoteFakeGumpClient( ObjectID objectid ) throws RemoteException {

		this( objectid, null );
	}


	public RemoteFakeGumpClient( ObjectID objectid, CountDownLatch verficationCountDownLatch ) throws RemoteException {

		gumResponseList = new LinkedList<GumResponseEntry>();
		objectID = objectid;
		this.verficationCountDownLatch = verficationCountDownLatch;
	}


	public void setExpectedHereIsGum( int expectedHereIsGuM, Object... args ) {

		this.expectedHereIsGuM = expectedHereIsGuM;
		this.verficationCountDownLatch = new CountDownLatch( expectedHereIsGuM );

	}


	public void hereIsGum( GumResponseEntry gumResponseEntry ) throws RemoteException {

		hereIsGuMCount++;
		gumResponseList.add( gumResponseEntry );
		if ( this.verficationCountDownLatch != null ) {
			this.verficationCountDownLatch.countDown();
		}
	}


	public CountDownLatch getVerficationCountDownLatch() {

		return this.verficationCountDownLatch;
	}


	public void verify() {

		if ( hereIsGuMCount != expectedHereIsGuM ) {
			throw new RuntimeException( "hereIsGuM() was expected to be called " + expectedHereIsGuM
					+ " times but was " + hereIsGuMCount );
		}

	}


	public ObjectID getObjectID() {

		return objectID;
	}


	public boolean equals( Object o ) {

		RemoteFakeGumpClient gumpClient;

		if ( o instanceof GumpClient ) {
			gumpClient = (RemoteFakeGumpClient) o;
			return this.objectID.equals( gumpClient.getObjectID() );
		}
		return false;
	}


	public String toString() {

		return "" + objectID;
	}


	public List<GumResponseEntry> getGumResponseList() {

		return gumResponseList;
	}

}
