/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.id;

import java.io.Serializable;

import org.ourgrid.common.spec.GumSpec;

public class GumID implements Serializable {

	private static final long serialVersionUID = 33L;

	private String gumName;

	private String gumPort;

	private String peerName;

	private String peerPort;

	private String id;

	private String url;


	public GumID( String gumName, String gumPort, String peerName, String peerPort ) {

		this.gumName = gumName;
		this.gumPort = gumPort;
		this.peerName = peerName;
		this.peerPort = peerPort;
		this.id = gumName + ":" + gumPort + "@" + peerName + ":" + peerPort;
		this.url = "rmi://" + gumName + ":" + gumPort;
	}


	public GumID( GumSpec gumSpec ) {

		this.gumName = gumSpec.getAttribute( GumSpec.ATT_NAME );
		this.gumPort = gumSpec.getAttribute( GumSpec.ATT_PORT );
		this.peerName = gumSpec.getAttribute( GumSpec.ATT_PEERNAME );
		this.peerPort = gumSpec.getAttribute( GumSpec.ATT_PEERPORT );
		this.id = gumName + ":" + gumPort + "@" + peerName + ":" + peerPort;
		this.url = "rmi://" + gumName + ":" + gumPort;

	}


	public String getGumName() {

		return gumName;
	}


	/**
	 * Gets the port of this Gum.
	 * 
	 * @return The port of this Gum.
	 */
	public String getGumPort() {

		return gumPort;
	}


	/**
	 * Gets the peer name that have this Gum.
	 * 
	 * @return The peer name that have this Gum.
	 */
	public String getPeerName() {

		return peerName;
	}


	/**
	 * Assembles a rmi url for this gum
	 */
	public String getUrl() {

		return url;
	}


	/**
	 * Gets the peer port of the peer that have this Gum.
	 * 
	 * @return The peer port of the peer that have this Gum.
	 */
	public String getPeerPort() {

		return peerPort;
	}


	@Override
	public String toString() {

		return id;
	}


	@Override
	public int hashCode() {

		return id.hashCode();
	}


	@Override
	public boolean equals( Object o ) {

		if ( this == o )
			return true;

		GumID gumID;

		if ( o instanceof GumID ) {
			gumID = (GumID) o;
			return this.id.equals( gumID.id );
		}
		return false;
	}
}
