/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.id;

import java.io.Serializable;

/**
 * Description: Consumer Identification.
 */
public class ObjectID implements Serializable {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;

	/**
	 * Unique id value.
	 */
	private Long idValue;

	/**
	 * Remote consumer url.
	 */
	private String URL;

	/**
	 * Consumer name
	 */
	private String name;


	/**
	 * Creates a new peer id.
	 * 
	 * @param name The peer name
	 * @param idValue Unique peer identification.
	 * @param identifierURL Remote URL associated with this id.
	 */
	public ObjectID( String name, Long idValue, String identifierURL ) {

		this.name = name;
		this.idValue = idValue;
		this.URL = identifierURL;
	}


	/**
	 * @return The peer remote URL.
	 */
	public String getObjectURL() {

		return URL;
	}


	/**
	 * Sets the peer URL.
	 * 
	 * @param peerURL A RMI URL to the peer.
	 */
	public void setObjectURL( String peerURL ) {

		this.URL = peerURL;
	}


	/**
	 * @return the peer name.
	 */
	public String getName() {

		return name;
	}


	/**
	 * Sets the peer name.
	 * 
	 * @param name Peer name.
	 */
	public void setName( String name ) {

		this.name = name;
	}


	/**
	 * @return return peer unique identification.
	 */
	public Long getIdValue() {

		return idValue;
	}


	/**
	 * Set the peer unique identification
	 * 
	 * @param idValue Peer unique identification.
	 */
	public void setIdValue( Long idValue ) {

		this.idValue = idValue;
	}


	/**
	 * Returns true if two objectids have the same identification.
	 */
	@Override
	public boolean equals( Object o ) {

		if ( o != null && o instanceof ObjectID ) {
			ObjectID otherID = (ObjectID) o;
			if ( getIdValue() != null )
				return getIdValue().equals( otherID.getIdValue() );
			return otherID.getIdValue() == null;
		}

		return false;
	}


	/**
	 * Objects hashcode.
	 */
	@Override
	public int hashCode() {

		return idValue.hashCode();
	}


	@Override
	public String toString() {

		return name;
	}
}
