/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.logger;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ourgrid.common.config.Configuration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

/**
 * Class used to parse the log in XML and translate it to HTML.
 */
public class LogParser {

	/**
	 * String representing the log in HTML.
	 */
	private StringBuffer logBuffer;

	/**
	 * Filter to be used.
	 */
	private String[ ] logFilter;

	/**
	 * MyGrid root dir.
	 */
	private String MGRoot;


	/**
	 * The constructor.
	 */
	public LogParser() {

		logBuffer = new StringBuffer();
		MGRoot = System.getProperty( "MGROOT" );
	}


	/**
	 * Method that translates the XML log to HTML.
	 * 
	 * @param xml XML log piece
	 * @param filter Filter to be used
	 * @return A string representing the log in XML
	 */
	public String getHtml( String xml, String filter ) {

		initHtml();
		setFilter( filter );
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		StringBuffer rootedXML = new StringBuffer();
		rootedXML.append( "<LOGROOT>" );
		rootedXML.append( xml );
		rootedXML.append( "</LOGROOT>" );
		try {
			Configuration.getInstance( Configuration.MYGRID );
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document log = db.parse( new ByteArrayInputStream( rootedXML.toString().getBytes() ) );
			String iconTag = "";
			Element element = log.getDocumentElement();
			NodeList nodeList = element.getElementsByTagName( "ENTRY" );
			boolean interesting = false;
			for ( int x = 0; x < nodeList.getLength(); x++ ) {
				NamedNodeMap nnm = nodeList.item( x ).getAttributes();
				for ( int y = nnm.getLength() - 1; y >= 0; y-- ) {
					if ( nnm.item( y ).getNodeName().equals( "Type" ) ) {
						interesting = isInteresting( nnm.item( y ).getNodeValue() );
						if ( interesting ) {
							if ( nnm.item( y ).getNodeValue().equals( "INFO" ) ) {
								iconTag = "<img src=file://" + MGRoot + "/resources/images/info.gif>";
							} else if ( nnm.item( y ).getNodeValue().equals( "DEBUG" ) ) {
								iconTag = "<img src=file://" + MGRoot + "/resources/images/debug.gif>";
							} else if ( nnm.item( y ).getNodeValue().equals( "ERROR" ) ) {
								iconTag = "<img src=file://" + MGRoot + "/resources/images/error.gif>";
							} else if ( nnm.item( y ).getNodeValue().equals( "FATAL" ) ) {
								iconTag = "<img src=file://" + MGRoot + "/resources/images/fatal.gif>";
							} else if ( nnm.item( y ).getNodeValue().equals( "USER" ) ) {
								iconTag = "<img src=file://" + MGRoot + "/resources/images/user.gif>";
							}
						} else {
							break;
						}
					} else if ( nnm.item( y ).getNodeName().equals( "Time" ) ) {
						logBuffer.append( iconTag );
						logBuffer.append( "\n" );
						logBuffer.append( "&nbsp;&nbsp;\n" );
						logBuffer.append( "<FONT COLOR='green'>" );
						logBuffer.append( nnm.item( y ).getNodeValue() );
						logBuffer.append( "</FONT>\n" );
						logBuffer.append( "<BR>\n" );
					} else if ( nnm.item( y ).getNodeName().equals( "Reference" ) ) {
						logBuffer.append( "<FONT COLOR='blue'>&nbsp;&nbsp;&nbsp;Source</FONT> : " );
						logBuffer.append( nnm.item( y ).getNodeValue() );
						logBuffer.append( "<BR>\n" );
					}
				}
				if ( interesting ) {
					NodeList childNodeList = nodeList.item( x ).getChildNodes();
					logBuffer.append( "<FONT COLOR='blue'>&nbsp;&nbsp;&nbsp;Message</FONT> : " );
					logBuffer.append( childNodeList.item( 0 ).getNodeValue().replaceAll( "<", "&lt;" ).replaceAll( ">",
							"&gt;" ) );
					logBuffer.append( "\n" );
					logBuffer.append( "<BR><BR>\n" );
				}
			}
		} catch ( Exception e ) {
			return null;
		}

		closeHtml();

		return logBuffer.toString();
	}


	/**
	 * HTML initial tags.
	 */
	private void initHtml() {

		logBuffer.append( "<HTML>\n" );
		logBuffer.append( "<BODY>\n" );
	}


	/**
	 * HTML final tags.
	 */
	private void closeHtml() {

		logBuffer.append( "</BODY>\n" );
		logBuffer.append( "</HTML>\n" );
	}


	/**
	 * Sets the filter.
	 * 
	 * @param filter The filter options
	 */
	private void setFilter( String filter ) {

		logFilter = filter.split( "," );
	}


	/**
	 * Verifies if an event is interesting according to the filter.
	 * 
	 * @param type The event type
	 */
	private boolean isInteresting( String type ) {

		boolean interesting = false;
		for ( int x = 0; x < logFilter.length; x++ ) {
			if ( logFilter[x].equals( type ) ) {
				interesting = true;
				break;
			}
		}

		return interesting;
	}

}
