/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.matcher;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.ourgrid.common.exception.SyntacticalErrorAtExpressionException;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.test.CommonTest;

public class MatcherTest extends TestCase implements CommonTest {

	private Map<String,String> machine1 = new HashMap<String,String>();

	private Map<String,String> machine2 = new HashMap<String,String>();

	private Map<String,String> machine3 = new HashMap<String,String>();


	/*
	 * Constructor
	 */
	public MatcherTest( String s ) {

		super( s );
	}


	/*
	 * Initialization of the test
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {

		machine1.put( GumSpec.ATT_NAME, "machine1" );
		machine1.put( "os", "linux" );
		machine1.put( "site", "lsd" );
		machine1.put( "ram", "200" );
		machine1.put( "SO", "bla" );

		machine2.put( GumSpec.ATT_NAME, "machineWithoutAttributes" );

		machine3.put( GumSpec.ATT_NAME, "machine3" );
		machine3.put( "os", "windows" );
		machine3.put( "site", "puc" );
		machine3.put( "mem", "50" );
	}


	/*
	 * Finishes the test
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	public void testMatchesWithNotDefinedAttributes() throws SyntacticalErrorAtExpressionException {

		String jobRequirement = "(site != lsd.ufcg.edu.br)";
		boolean matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(site != lsd.ufcg.edu.br) && (os == linux)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "( site != lsd.ufcg.edu.br ) AND ( os == windows )";
		matched = Matcher.match( jobRequirement, machine2 );
		assertFalse( matched );

		jobRequirement = "(site != lsd.ufcg.edu.br) || (os == windows)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(site != lsd.ufcg.edu.br) OR ( os == windows)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "! (site != lsd.ufcg.edu.br)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem < 200)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem <= 200)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem > 200)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem >= 200)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "! (site != lsd.ufcg.edu.br) || (mem >= 200)";
		matched = Matcher.match( jobRequirement, machine2 );
		System.out.println( "test > " + jobRequirement + " matches " + machine2 + " ? ==> " + matched );
		assertFalse( matched );
	}


	public void testMatchesWithDefinedAndNotDefinedAttributes() throws SyntacticalErrorAtExpressionException {

		String jobRequirement = "(site != lsd.ufcg.edu.br)";
		boolean matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );

		jobRequirement = "(site != lsd.ufcg.edu.br) && (so == linux)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "( site == puc ) AND ( os == windows )";
		matched = Matcher.match( jobRequirement, machine3 );
		assertTrue( matched );

		jobRequirement = "(site == puc) || (os == linux)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );

		jobRequirement = "(site == lsd) OR ( os == windows)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );

		jobRequirement = "! (caca != lsd)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem < 200) || ( caca = lsd)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );

		jobRequirement = "(mem <= 200)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );

		jobRequirement = "(mem > 200)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "(mem >= 200)";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertFalse( matched );

		jobRequirement = "! (site != lsd.ufcg.edu.br) || (mem >= 200) || ( os == windows ) || ( caca == xpto )";
		matched = Matcher.match( jobRequirement, machine3 );
		System.out.println( "test > " + jobRequirement + " matches " + machine3 + " ? ==> " + matched );
		assertTrue( matched );
	}


	public void testGoodMatches() throws SyntacticalErrorAtExpressionException {

		String jobRequirement = "os == linux";
		boolean matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "site == lsd";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( site == lsd && os == linux )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( site == xpto || os == linux )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( site != xpto && os == linux )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ( site == ucsd || site == lsd ) && os == linux )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( os == linux &&  ! ( site == ucsd ) )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "ram > 100";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "((site == ucsd|| site== lsd ) && ram >100 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ram < 300 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( (site == ucsd || site == lsd )&& ( ram > 100 && ram< 300 ))";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ( site = ucsd || site = lsd ) && ram >= 200 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ( site == ucsd || site == lsd ) && ram <= 200 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "not ( ram > 300 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ( site == ucsd OR site == lsd ) AND ram <= 200 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( ( site = ucsd OR site = lsd ) AND ram <= 200 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );

		jobRequirement = "( SO = bla )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertTrue( matched );
	}


	public void testBadMatches() throws SyntacticalErrorAtExpressionException {

		String jobRequirement = "!(site == lsd )";
		boolean matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( site == xpto && os = linux )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( site != xpto && os = windows )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( ram == 400 || os == windows )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( os == windows && ( site != ucsd ) )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( os == windows && ( site == noname ) )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( ram < 300 && site == noname )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( XPTO == bla )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( XPTO == bla ) && ( XPTO2 != ble )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

		jobRequirement = "( XPTO3 < 9876543210 )";
		matched = Matcher.match( jobRequirement, machine1 );
		assertFalse( matched );

	}


	public void testExceptionsMatches() {

		String jobRequirement;

		jobRequirement = "( ram >= linux && ( site == ucsd ) )";
		assertFalse( Matcher.match( jobRequirement, machine1 ) );
	}

}
