/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.rmi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.security.MGSecureClientSocketFactory;
import org.ourgrid.common.security.MGSecureServerSocketFactory;

/**
 * This class defines a default constructor that must be used by every remote
 * object in OurGrid.
 * 
 * @author Flavio Vinicius Diniz de Figueiredo - flaviov@lsd.ufcg.edu.br
 */
public class OurgridUnicastRemoteObject extends UnicastRemoteObject {

	/**
	 * Serial UID.
	 */
	private static final long serialVersionUID = 33L;


	/**
	 * Builds a new <code>OurgridUnicastRemoteObject</code>.
	 * 
	 * @throws RemoteException If failed to export object
	 */
	protected OurgridUnicastRemoteObject() throws RemoteException {

		super( Integer.parseInt( Configuration.getInstance().getProperty( Configuration.PROP_PORT ) ) );
	}


	/**
	 * Builds a new <code>OurgridUnicastRemoteObject</code> for secure
	 * communications.
	 * 
	 * @throws RemoteException If failed to export object
	 */
	public OurgridUnicastRemoteObject( int securePort, MGSecureClientSocketFactory secureClientSocketFactory,
										MGSecureServerSocketFactory secureServerSocketFactory ) throws RemoteException {

		super( securePort, secureClientSocketFactory, secureServerSocketFactory );
	}


	public OurgridUnicastRemoteObject( int port ) throws RemoteException {

		super( port );
	}
}
