/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.security;

import java.io.File;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.test.CommonTest;

/**
 * This class implemetns the test of KeyToolWrapper.
 */
public class KeyToolWrapperTest extends TestCase implements CommonTest {

	/** The tool */
	KeyToolWrapper keytool;


	/**
	 * This method constructs a new instance of this TestCase.
	 * 
	 * @param s The name of the TestCase class.
	 */
	public KeyToolWrapperTest( String s ) {

		super( s );
	}


	/*
	 * Initialization of the test
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		Configuration.getInstance( Configuration.MYGRID );
		keytool = new KeyToolWrapper();
	}


	/*
	 * Finishes the test
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		File keystore = new File( "/tmp/keystore" );

		if ( keystore.exists() ) {
			keystore.delete();
		}

		Configuration.reset();
	}


	/**
	 * This method test the Key Generation functionality passing a negative
	 * validity
	 */
	public void testGenerateKeyNegativeValidity() {

		assertTrue( keytool.generateKey( "username", "UserName", "/tmp/keystore", 512, -1 ) );
	}


	/**
	 * This method test the Key Generation functionality passing a validity
	 * equals to zero
	 */
	public void testGenerateKeyZeroValidity() {

		assertTrue( keytool.generateKey( "username", "UserName", "/tmp/keystore", 512, 0 ) );
	}


	/**
	 * This method test the Key Generation functionality passing a validity
	 * equals to zero and a oversized unsupported keysize
	 */
	public void testGenerateKeyOverSize() {

		assertTrue( keytool.generateKey( "username", "UserName", "/tmp/keystore", 2048, 0 ) );
	}


	/**
	 * This method test the Key Generation functionality passing a validity
	 * equals to zero and a negative keysize
	 */
	public void testGenerateKeyNegativeSize() {

		assertTrue( keytool.generateKey( "username", "UserName", "/tmp/keystore", -1024, 0 ) );
	}


	/**
	 * This method test the Key Generation functionality passing a random
	 * keysize
	 */
	public void testGenerateKeyRandomSize() {

		assertTrue( keytool.generateKey( "username", "UserName", "/tmp/keystore", 7, 0 ) );
	}


	/**
	 * The main method to bring the test runner up.
	 * 
	 * @param args
	 */
	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( KeyToolWrapperTest.class );
	}

}
