/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.security;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.url.CorePeerURLProvider;
import org.ourgrid.corepeer.CorePeerImpl;
import org.ourgrid.test.SecurityTest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

/**
 * Executes MyGrid x Peer x UA using the security scheme.
 */
public class SecurityExecutionTest extends TestCase implements SecurityTest, SpecialCareTest {

	private static final String SH = "sh";

	private static final String OUTPUT = "test" + File.separator + "resources" + File.separator + "output-1.1";

	private static final String UA_PROPERTIES = "ua.properties";

	private static final String UA_ = "ua.";

	private static final String SETGUMS = "setgums";

	private static final String EXPECTED_RESULT = "47";

	private static final String BIN = "bin" + File.separator;

	private static final String NO = "no";

	private static final String YES = "yes";

	private static final String PEER_JOINCOMMUNITY = "peer.joincommunity";

	private static final String WAITFORJOB = "waitforjob";

	private static final String ADDJOB = "addjob";

	private static final String SETGRID = "setgrid";

	private static final String MG_ = "mg.";

	private static final String PEER_ = "peer.";

	private static final String STOP = "stop";

	private TestUtils testUtils;

	private static final String UA_ROOT = System.getProperty( "user.home" ) + File.separator + ".ourgrid/localhost";

	private static final String START = "start";

	private static final String MYGRID_PEER_I_KEYSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "mygrid_peer_I_keystore";

	private static final String MYGRID_PEER_I_KEYSTORE_PASSWORD = "mygrid_peer_I";

	private static final String MYGRID_PEER_I_TRUSTSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "mygrid_peer_I_truststore";

	private static final String MYGRID_PEER_I_TRUSTSTORE_PASSWORD = "mygrid_peer_I";

	private static final String PEER_I_MYGRID_PEER_II_KEYSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "peer_I_mygrid_peer_II_keystore";

	private static final String PEER_I_MYGRID_PEER_II_KEYSTORE_PASSWORD = "peer_I_mygrid_peer_II";

	private static final String PEER_I_MYGRID_PEER_II_TRUSTSTORE = "test" + File.separator + "resources"
			+ File.separator + "keystores" + File.separator + "peer_I_mygrid_peer_II_truststore";

	private static final String PEER_I_MYGRID_PEER_II_TRUSTSTORE_PASSWORD = "peer_I_mygrid_peer_II";

	private static final String UA_PEER_II_KEYSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "ua_peer_II_keystore";

	private static final String UA_PEER_II_KEYSTORE_PASSWORD = "ua_peer_II";

	private static final String UA_PEER_II_TRUSTSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "ua_peer_II_truststore";

	private static final String UA_PEER_II_TRUSTSTORE_PASSWORD = "ua_peer_II";

	private static final String PEER_II_UA_PEER_I_KEYSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "peer_II_ua_peer_I_keystore";

	private static final String PEER_II_UA_PEER_I_KEYSTORE_PASSWORD = "peer_II_ua_peer_I";

	private static final String PEER_II_UA_PEER_I_TRUSTSTORE = "test" + File.separator + "resources" + File.separator
			+ "keystores" + File.separator + "peer_II_ua_peer_I_truststore";

	private static final String PEER_II_UA_PEER_I_TRUSTSTORE_PASSWORD = "peer_II_ua_peer_I";

	private static final String PEER_PROPERTIES_II = "peer.properties.II";

	private static final String PEER_PROPERTIES_I = "peer.properties.I";

	private static final String PEER_PROPERTIES = "peer.properties";

	private static final String UA_COMMAND = "useragent";

	private static final String PEER_COMMAND = BIN + "peer";

	private CorePeerImpl corePeer;

	private static final String PEER_ID = ".peerid";


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		testUtils = new TestUtils();
		testUtils.load();
		delete( OUTPUT );

	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		testUtils.reset();
		delete( OUTPUT );
	}


	/**
	 * Deletes a file.
	 * 
	 * @param filename The Filename.
	 * @throws Exception Case the file could not be deleted.
	 */
	private void delete( String filename ) throws Exception {

		File file = new File( filename );
		file.delete();
	}


	/**
	 * Check the results of the job.
	 * 
	 * @throws Exception Case the result cannot be checked or the resul was
	 *         wrong.
	 */
	private void checkResult( String expected ) throws Exception {

		File resultFile = new File( OUTPUT );
		BufferedReader bufferedReader = new BufferedReader( new FileReader( resultFile ) );
		String line = bufferedReader.readLine();
		String result = "";
		while ( line != null ) {
			result += line;
			line = bufferedReader.readLine();
		}
		assertEquals( expected, result );
	}


	/**
	 * Executes the mygrid command.
	 * 
	 * @param args The arguments of the mygrid command.
	 * @throws Exception Case the command cannot be executed.
	 */
	private void executeMyGrid( String[ ] args ) throws Exception {

		if ( args[0].equals( START ) ) {
			Configuration.reset();
			Configuration.getInstance( Configuration.MYGRID );
			TestUtils.executeCommand( TestUtils.MYGRID_COMMAND, args );
			Configuration.reset();
		} else {
			TestUtils.executeCommand( TestUtils.MYGRID_COMMAND, args );
		}
	}


	/**
	 * Executes a command with its arguments.
	 * 
	 * @param command The command to be executed.
	 * @param args Its arguments.
	 * @throws Exception Case the command cannot be executed.
	 */
	private void executeCommand( String command, String[ ] args ) throws Exception {

		String[ ] commandArray = new String[ args.length + 2 ];
		commandArray[0] = SH;
		commandArray[1] = command;
		for ( int k = 0; k < args.length; k++ ) {
			commandArray[k + 2] = args[k];
		}
		TestUtils.runCommand( commandArray, 0 );
	}


	/**
	 * Executes UA command.
	 * 
	 * @param args The arguments of the useragent command.
	 * @throws Exception Case The command cannot be executed.
	 */

	private void executeUA( String command, String port ) throws Exception {

		String fullCommand = "sh " + UA_ROOT + "_" + port + File.separator + UA_COMMAND + " " + command;
		Runtime runtime = Runtime.getRuntime();
		Process process = runtime.exec( fullCommand );
		if ( process.waitFor() != 0 ) {
			fail( "useragent command failed!" );
		}
	}


	/**
	 * Adds the attributes of the keystores to the TestUtils specified.
	 * 
	 * @param prefix The prefix of the attributes.
	 * @param keystore_property A keystore.
	 * @param keystore_password_property The password for the keystore
	 *        specified.
	 * @param truststore_property A truststore.
	 * @param truststore_password_property The password for the truststore
	 *        specified.
	 * @param testUtils The testUtils
	 */
	private void addKeystoresToTestUtils( String prefix, String keystore_property, String keystore_password_property,
											String truststore_property, String truststore_password_property,
											TestUtils testUtils ) {

		String[ ] keys = { prefix + Configuration.PROP_KEYSTORE_FILENAME,
							prefix + Configuration.PROP_KEYSTORE_PASSWORD,
							prefix + Configuration.PROP_TRUSTSTORE_FILENAME,
							prefix + Configuration.PROP_TRUSTSTORE_PASSWORD };
		String[ ] values = { keystore_property, keystore_password_property, truststore_property,
							truststore_password_property };
		for ( int k = 0; k < keys.length; k++ ) {
			testUtils.setProperty( keys[k], values[k] );
		}
	}


	/**
	 * Creates a ua.properties on the directory specified.
	 * 
	 * @param directory The directory where the ua.properties is to be created.
	 * @param attribs The attributes of the ua.properties.
	 * @return A String containing the filename of the ua.properties recently
	 *         created.
	 * @throws Exception Case something goes wrong.
	 */
	private String createUAProperties( String directory, Map attribs ) throws Exception {

		String uaPropertiesFilename = directory + File.separator + UA_PROPERTIES;
		File uaPropertiesFile = new File( uaPropertiesFilename );
		new File( directory ).mkdirs();
		uaPropertiesFile.createNewFile();
		Properties prop = new Properties();
		Iterator it = attribs.entrySet().iterator();
		while ( it.hasNext() ) {
			Map.Entry entry = (Map.Entry) it.next();
			prop.put( entry.getKey(), entry.getValue() );
		}
		FileOutputStream fileOutputStream = new FileOutputStream( uaPropertiesFile );
		prop.store( fileOutputStream, "ua.properties created in SecurityExecutionTest" );
		fileOutputStream.close();

		return uaPropertiesFilename;
	}


	/**
	 * Puts the keystores properties on the Map specified.
	 * 
	 * @param ua_keystore The UA keystore.
	 * @param ua_keystore_password The UA keystore password.
	 * @param ua_truststore The UA truststore.
	 * @param ua_truststore_password The UA truststore password.
	 */
	private void addUAKeystoresToAttribs( String ua_keystore, String ua_keystore_password, String ua_truststore,
											String ua_truststore_password ) {

		String prefix = UA_;
		String[ ] keys = { prefix + Configuration.PROP_KEYSTORE_FILENAME,
							prefix + Configuration.PROP_KEYSTORE_PASSWORD,
							prefix + Configuration.PROP_TRUSTSTORE_FILENAME,
							prefix + Configuration.PROP_TRUSTSTORE_PASSWORD };
		String[ ] values = { ua_keystore, ua_keystore_password, ua_truststore, ua_truststore_password };
		for ( int k = 0; k < keys.length; k++ ) {
			testUtils.setProperty( keys[k], values[k] );
		}
	}


	/**
	 * Generates a Map to be written as a ua.properties.
	 * 
	 * @param attributes The Attributes of the GuM.
	 * @return A Map formatted to be written as a ua.properties.
	 */
	private Map generateUAProperties( Map<String,String> attributes ) {

		Iterator<Entry<String,String>> it = attributes.entrySet().iterator();
		Map<String,String> map = new HashMap<String,String>();
		while ( it.hasNext() ) {
			Map.Entry<String,String> entry = it.next();
			String entryKey = entry.getKey();
			map.put( UA_ + entryKey, entry.getValue() );

		}
		return map;
	}


	/**
	 * Executes 'peer' command.
	 * 
	 * @param peerArgs The arguments of the peer command.
	 * @throws Exception Case the command cannot be performed.
	 */
	private void executePeer( String[ ] peerArgs ) throws Exception {

		if ( peerArgs[0].equals( START ) ) {
			Configuration.reset();
			Configuration.getInstance( Configuration.PEER );
			TestUtils.executeCommand( TestUtils.PEER_COMMAND, new String[ ] { "start" } );
			Configuration.reset();
		} else {
			executeCommand( PEER_COMMAND, peerArgs );
		}
	}


	/**
	 * Sets the join community property value.
	 * 
	 * @param b The value.
	 * @param testUtils The testUtils where the property will be changed.
	 */
	private void setJoinCommunity( boolean b, TestUtils testUtils ) {

		testUtils.setProperty( PEER_JOINCOMMUNITY, b ? YES : NO );
	}


	/**
	 * Starts the core peer.
	 * 
	 * @throws Exception Case the core peer cannot be started.
	 */
	private void initCorePeer() throws Exception {

		Configuration.getInstance( Configuration.COREPEER );

		int registryPort = Integer.parseInt( testUtils.getProperty( TestUtils.COREPEER_PORT ) );

		Configuration.getInstance().setProperty( Configuration.PROP_PORT, registryPort + "" );

		Registry r = null;

		try {

			r = LocateRegistry.createRegistry( registryPort );

		} catch ( Exception e ) {

			r = LocateRegistry.getRegistry( registryPort );

		}

		this.corePeer = new CorePeerImpl();
		r.rebind( CorePeerURLProvider.COREPEER_OBJECT_NAME, corePeer );
		Configuration.reset();
	}


	/**
	 * Stops the core peer.
	 * 
	 * @throws Exception Case the core peer cannot be stopped.
	 */
	private void shutdownCorePeer() throws Exception {

		int registryPort = Integer.parseInt( testUtils.getProperty( TestUtils.COREPEER_PORT ) );

		corePeer.shutdown();

		Registry r = null;

		try {

			r = LocateRegistry.createRegistry( registryPort );

		} catch ( Exception e ) {

			r = LocateRegistry.getRegistry( registryPort );

		}

		r.unbind( CorePeerURLProvider.COREPEER_OBJECT_NAME );
		OurgridUnicastRemoteObject.unexportObject( corePeer, true );
	}


	/**
	 * Follow these steps. Config UA environment (ua.properties, keystore,
	 * truststore) Config Peer I environment (peer.properties, keystore,
	 * truststore) Config Peer II environment (peer.properties, keystore,
	 * truststore) Config MyGrid environment (mg.properties, keystore,
	 * truststore) Exchange certificates Init Core Peer Init UA Init Peer I Init
	 * Peer II Init MyGrid Peer I SetGrid MyGrid SetGrid Peer II Addjob Wait for
	 * job Check results Shutdown everything
	 * 
	 * @throws Exception
	 */
	public void testExecution_MyGrid_Peer_Peer_UAs() throws Exception {

		addKeystoresToTestUtils( PEER_, PEER_II_UA_PEER_I_KEYSTORE, PEER_II_UA_PEER_I_KEYSTORE_PASSWORD,
				PEER_II_UA_PEER_I_TRUSTSTORE, PEER_II_UA_PEER_I_TRUSTSTORE_PASSWORD, testUtils );
		setJoinCommunity( true, testUtils );

		initCorePeer();

		testUtils.createOGProperties();

		delete( PEER_ID );

		String[ ] peerStart = { START };
		executePeer( peerStart );
		rename( PEER_PROPERTIES, PEER_PROPERTIES_II );

		addKeystoresToTestUtils( PEER_, PEER_I_MYGRID_PEER_II_KEYSTORE, PEER_I_MYGRID_PEER_II_KEYSTORE_PASSWORD,
				PEER_I_MYGRID_PEER_II_TRUSTSTORE, PEER_I_MYGRID_PEER_II_TRUSTSTORE_PASSWORD, testUtils );
		setJoinCommunity( true, testUtils );
		setNewPeerPort( testUtils );
		testUtils.createOGProperties();
		delete( PEER_ID );
		executePeer( peerStart );

		GumSpec guMSpec = testUtils.getLocalGuMSpec( TestUtils.TYPE_UALINUX );

		ArrayList<GumSpec> gumList = new ArrayList<GumSpec>();

		gumList.add( guMSpec );

		String sdfLocal = testUtils.createSDF( gumList );

		// Init UA
		// Deployer.main(uaadminInit);

		executeUaadmin( "restart", sdfLocal );
		executeUaadmin( "stop", sdfLocal );

		// ThreadServices.waitUntilWorkIsDone();

		Map attribs = generateUAProperties( guMSpec.getAttributes() );
		addUAKeystoresToAttribs( UA_PEER_II_KEYSTORE, UA_PEER_II_KEYSTORE_PASSWORD, UA_PEER_II_TRUSTSTORE,
				UA_PEER_II_TRUSTSTORE_PASSWORD );
		String uaPropertiesFilename = createUAProperties( UA_ROOT + "_"
				+ guMSpec.getAttribute( Configuration.PROP_PORT ), attribs );

		executeUA( "start", guMSpec.getAttribute( Configuration.PROP_PORT ) );

		addKeystoresToTestUtils( MG_, MYGRID_PEER_I_KEYSTORE, MYGRID_PEER_I_KEYSTORE_PASSWORD,
				MYGRID_PEER_I_TRUSTSTORE, MYGRID_PEER_I_TRUSTSTORE_PASSWORD, testUtils );
		testUtils.createMGProperties();
		String[ ] mygridStart = { START };
		executeMyGrid( mygridStart );

		String[ ] mygridSetGrid = { SETGRID, testUtils.createLocalPeerGDF() };
		executeMyGrid( mygridSetGrid );

		List<GumSpec> gums = new LinkedList<GumSpec>();
		gums.add( guMSpec );

		rename( PEER_PROPERTIES, PEER_PROPERTIES_I );
		rename( PEER_PROPERTIES_II, PEER_PROPERTIES );

		String[ ] peerSetGrid = { SETGUMS, testUtils.createSDF( gums ) };
		executePeer( peerSetGrid );
		String[ ] mygridAddjob = { ADDJOB, testUtils.getProperty( "job.simpletask" ) };
		executeMyGrid( mygridAddjob );

		String[ ] mygridWaitForJob = { WAITFORJOB, "1" };
		executeMyGrid( mygridWaitForJob );
		String[ ] mygridStop = { STOP };
		executeMyGrid( mygridStop );

		executeUA( "stop", guMSpec.getAttribute( Configuration.PROP_PORT ) );

		String[ ] peerStop = { STOP };
		executePeer( peerStop );

		rename( PEER_PROPERTIES_I, PEER_PROPERTIES );

		executePeer( peerStop );

		delete( uaPropertiesFilename );

		shutdownCorePeer();

		// Stop UA
		// executeUaadmin("stop", sdfLocal);
		// Deployer.main(uaadminStop);

		checkResult( EXPECTED_RESULT );

	}


	/**
	 * Starts the user agents
	 * 
	 * @throws Exception
	 */
	private void executeUaadmin( String command, String sdf ) throws Exception {

		String fullCommand = "bin/uaadmin " + command + " " + sdf;

		Runtime runtime = Runtime.getRuntime();
		Process process = runtime.exec( fullCommand );
		if ( process.waitFor() != 0 ) {
			fail( "uaadmin failed!" );
		}
	}


	/**
	 * Renames the filename1 to the filename2.
	 * 
	 * @param filename1 A filename.
	 * @param filename2 A filename.
	 */
	private void rename( String filename1, String filename2 ) {

		File file1 = new File( filename1 );
		File file2 = new File( filename2 );
		file1.renameTo( file2 );
	}


	/**
	 * Sets a new available port to start a new peer.
	 * 
	 * @param testUtils The TestUtils where the new port will be port.
	 */
	private void setNewPeerPort( TestUtils testUtils ) {

		testUtils.setProperty( TestUtils.OG_PORT, testUtils.getProperty( TestUtils.OG_PORT_II ) );
	}
}
