/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec;

import org.ourgrid.common.spec.grammar.Grammar;
import org.ourgrid.common.spec.lexical.CommonLexicalAnalyzer;
import org.ourgrid.common.spec.lexical.LexicalAnalyzer;
import org.ourgrid.common.spec.lexical.LexicalException;
import org.ourgrid.common.spec.semantic.CommonSemanticAnalyzer;
import org.ourgrid.common.spec.semantic.SemanticActions;
import org.ourgrid.common.spec.semantic.SemanticAnalyzer;
import org.ourgrid.common.spec.semantic.exception.SemanticException;
import org.ourgrid.common.spec.syntactical.CommonSyntacticalAnalyzer;
import org.ourgrid.common.spec.syntactical.SyntacticalAnalyzer;
import org.ourgrid.common.spec.syntactical.SyntacticalException;

/**
 * @see org.ourgrid.common.spec.CompilerModulesFactory
 */
public class CommonCModulesFactory implements CompilerModulesFactory {

	/**
	 * The constructor.
	 */
	public CommonCModulesFactory() {

		// do nothing...
	}


	/**
	 * @see org.ourgrid.common.spec.CompilerModulesFactory#createLexicalAnalyzer(String)
	 */
	public LexicalAnalyzer createLexicalAnalyzer( String sourceFile ) throws LexicalException {

		return new CommonLexicalAnalyzer( sourceFile );
	}


	/**
	 * @see org.ourgrid.common.spec.CompilerModulesFactory#createSyntacticalAnalyzer(org.ourgrid.common.spec.lexical.LexicalAnalyzer,
	 *      org.ourgrid.common.spec.grammar.Grammar,
	 *      org.ourgrid.common.spec.semantic.SemanticAnalyzer)
	 */
	public SyntacticalAnalyzer createSyntacticalAnalyzer( LexicalAnalyzer lexicalAnalyzer, Grammar languageGrammar,
															SemanticAnalyzer semantic ) throws SyntacticalException {

		return new CommonSyntacticalAnalyzer( lexicalAnalyzer, languageGrammar, semantic );
	}


	/**
	 * @see org.ourgrid.common.spec.CompilerModulesFactory#createSemanticAnalyzer(org.ourgrid.common.spec.semantic.SemanticActions)
	 */
	public SemanticAnalyzer createSemanticAnalyzer( SemanticActions actionsContainer ) throws SemanticException {

		CommonSemanticAnalyzer semantic = new CommonSemanticAnalyzer( actionsContainer );
		return semantic;
	}

}
