/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec;

import org.ourgrid.common.spec.grammar.Grammar;
import org.ourgrid.common.spec.lexical.LexicalAnalyzer;
import org.ourgrid.common.spec.lexical.LexicalException;
import org.ourgrid.common.spec.semantic.SemanticActions;
import org.ourgrid.common.spec.semantic.SemanticAnalyzer;
import org.ourgrid.common.spec.semantic.exception.SemanticException;
import org.ourgrid.common.spec.syntactical.SyntacticalAnalyzer;
import org.ourgrid.common.spec.syntactical.SyntacticalException;

/**
 * Is a central factory for the compiler modules.
 */
public interface CompilerModulesFactory {

	/**
	 * Makes an instance of the Lexical Module.
	 * 
	 * @param sourceFile The file constaining the source to be compiled.
	 * @return A instance of the LexicalAnalyzer entity.
	 * @throws LexicalException If happens some problem at the creations
	 *         process, for example, the file could not be found.
	 */
	public LexicalAnalyzer createLexicalAnalyzer( String sourceFile ) throws LexicalException;


	/**
	 * Makes an instance of the Syntactical Module.
	 * 
	 * @param lexicalAnalyzer The lexical module that will give tokens to this
	 *        module.
	 * @param languageGrammar The grammar entity that knows how is the language.
	 * @param semantic The semanic module that will execute with the
	 *        compilation. Can be null if there is no intention of run it.
	 * @return A instance of the SyntacticalAnalyzer entity.
	 * @throws SyntacticalException If happens some problem at the creations
	 *         process.
	 */
	public SyntacticalAnalyzer createSyntacticalAnalyzer( LexicalAnalyzer lexicalAnalyzer, Grammar languageGrammar,
															SemanticAnalyzer semantic ) throws SyntacticalException;


	/**
	 * Makes an instance of the Syntactical Module.
	 * 
	 * @param actionsContainer The class that has the actions to be executed by
	 *        introspection.
	 * @return A instance of the SemanticalAnalyzer entity.
	 * @throws SemanticException If happens some problem at the creations
	 *         process.
	 */
	public SemanticAnalyzer createSemanticAnalyzer( SemanticActions actionsContainer ) throws SemanticException;
}
