/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec;

import java.io.Serializable;

/**
 * Encapsulates informations about a input/output operation at the MyGrid
 * environment.
 * 
 * @version 1.0 Created on Jul 2, 2004
 */
public class IOEntry implements Serializable {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;

	private String sourceFile, command, destiny;


	/**
	 * An empty constructor.
	 */
	public IOEntry() {

	}


	/**
	 * Constructor.
	 * 
	 * @param command The command that will be executed at the tranfering time.
	 * @param sourceFile The path file that will be transfered.
	 * @param destiny The destiny's path of the file transfered.
	 */
	public IOEntry( String command, String sourceFile, String destiny ) {

		this.command = command;
		this.sourceFile = sourceFile;
		this.destiny = destiny;
	}


	/**
	 * @return Returns the command.
	 */
	public String getCommand() {

		return command;
	}


	/**
	 * @return Returns the destiny.
	 */
	public String getDestiny() {

		return destiny;
	}


	/**
	 * @return Returns the sourceFile.
	 */
	public String getSourceFile() {

		return sourceFile;
	}


	/**
	 * @param command The command to set.
	 */
	public void setCommand( String command ) {

		this.command = command;
	}


	/**
	 * @param destiny The destiny to set.
	 */
	public void setDestiny( String destiny ) {

		this.destiny = destiny;
	}


	/**
	 * @param sourceFile The sourceFile to set.
	 */
	public void setSourceFile( String sourceFile ) {

		this.sourceFile = sourceFile;
	}


	/**
	 * Returns an string representation of this IOEntry
	 */
	public String toString() {

		return command + " " + this.sourceFile + " to " + this.destiny;

	}

}
