/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.grammar.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.common.spec.grammar.CommonGrammar;
import org.ourgrid.common.spec.grammar.Rule;
import org.ourgrid.common.spec.grammar.Symbol;
import org.ourgrid.common.spec.grammar.exception.InvalidRuleException;
import org.ourgrid.test.CommonTest;

/**
 * Created on 24/05/2004
 */
public class GalsGrammarReaderTest extends TestCase implements CommonTest {

	GalsGrammarReader grammarReader;


	/**
	 * Constructor for GalsGrammarReaderTest.
	 * 
	 * @param arg0
	 */
	public GalsGrammarReaderTest( String arg0 ) {

		super( arg0 );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		this.grammarReader = new GalsGrammarReader();
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		this.grammarReader = null;
	}


	public void testReadWithNonExistentFile() throws MalformedGrammarException {

		File emptyFile = new File( "test/specs/grammar/nonExistentFile.gals" );
		try {
			grammarReader.read( emptyFile, null );
			fail( "The file does not exist. Should throw an exception!" );
		} catch ( IOException e ) {
		}
	}


	public void testReadEmptyFile() throws FileNotFoundException, IOException {

		File badGRammar = new File( "test/specs/grammar/emptyGrammarFile.gals" );
		try {
			grammarReader.read( badGRammar, null );
			fail( "The file is empty. Should throw an exception!" );
		} catch ( MalformedGrammarException mgex ) {
		}
	}


	public void testReadWrongRuleGrammar() throws FileNotFoundException, IOException {

		File nonExistent = new File( "test/specs/grammar/WrongRule_Grammar.gals" );
		CommonGrammar grammar = new CommonGrammar();
		try {
			grammarReader.read( nonExistent, grammar );
			fail( "The file has a wrong rule. Should throw an exception!" );
		} catch ( MalformedGrammarException mgex ) {
			assertTrue( mgex.getCause() instanceof InvalidRuleException );
		}
	}


	public void testReadincompleteBlocksGrammar() throws FileNotFoundException, IOException {

		File nonExistent = new File( "test/specs/grammar/IncompleteBlocksGrammar.gals" );
		CommonGrammar grammar = new CommonGrammar();
		try {
			grammarReader.read( nonExistent, grammar );
			fail( "The file has not all the blocks needed. Should throw an exception!" );
		} catch ( MalformedGrammarException mgex ) {
			assertTrue( mgex.getMessage().equals( "The grammar have not all the informations block necessaries." ) );
		}
	}


	public void testReadWrongRuleGrammar2() throws FileNotFoundException, IOException {

		File nonExistent = new File( "test/specs/grammar/WrongRule_Grammar2.gals" );
		CommonGrammar grammar = new CommonGrammar();
		try {
			grammarReader.read( nonExistent, grammar );
			fail( "The file has a wrong rule. Should throw an exception!" );
		} catch ( MalformedGrammarException mgex ) {
			assertTrue( mgex.getMessage().equals( "There is a rule without a head or the symbol \"::=\" was forgot." ) );
		}
	}


	public void testReaderWithGridGrammar_1() throws MalformedGrammarException, FileNotFoundException, IOException {

		File gridGrammar_1 = new File( "test/specs/grammar/GridGrammar.gals" );
		CommonGrammar grammar = null;
		grammar = new CommonGrammar();
		grammarReader.read( gridGrammar_1, grammar );

		assertNull( grammar.getSymbol( "NotExistent" ) );
		// From the terminal block
		assertEquals( "GUMP", grammar.getSymbol( "GUMP" ).getValue() );
		assertEquals( ";", grammar.getSymbol( ";" ).getValue() );

		// From the non-terminal block
		assertEquals( "<resource>", grammar.getSymbol( "<resource>" ).getValue() );
		assertEquals( "<boolean>", grammar.getSymbol( "<boolean>" ).getValue() );

		// From the rules block
		assertEquals( "<resource>", grammar.getRule( 1 ).getHead().getValue() );
		assertEquals( "<attributes>", grammar.getRule( 8 ).getHead().getValue() );
		assertEquals( "<boolean>", grammar.getRule( 15 ).getHead().getValue() );

		// Testing Semantic Action founded
		Rule auxRule = grammar.getRule( 8 );
		Symbol[ ] symbols = auxRule.getBody();
		assertTrue( symbols[2].isSemanticAction() );
		auxRule = grammar.getRule( 14 );
		symbols = auxRule.getBody();
		assertTrue( symbols[0].isSemanticAction() );
		auxRule = grammar.getRule( 15 );
		symbols = auxRule.getBody();
		assertTrue( symbols[0].isSemanticAction() );
	}

}
