/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.grammar.test;

import java.util.Iterator;

import junit.framework.TestCase;

import org.ourgrid.common.spec.grammar.CommonGrammar;
import org.ourgrid.common.spec.grammar.Rule;
import org.ourgrid.common.spec.grammar.Symbol;
import org.ourgrid.common.spec.grammar.exception.InvalidRuleException;
import org.ourgrid.test.CommonTest;

/**
 * Project: Caymman(DSC/UFCG) Description: This class tests the CommonGrammar
 * class.
 * 
 * @version 1.0 Created on Sep 25, 2003 Last update: Sep 25, 2003
 */

public class CommonGrammarTest extends TestCase implements CommonTest {

	/* Variables to be used by the test. */
	CommonGrammar grammar = null;


	/**
	 * Constructor for CommonGrammarTest.
	 * 
	 * @param message
	 */
	public CommonGrammarTest( String message ) {

		super( message );

	}


	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {

		super.setUp();
		grammar = new CommonGrammar();

		grammar
				.addRule( new Rule( 1, new Symbol( 1, "h1", Symbol.NON_TERMINAL ),
						new Symbol[ ] { new Symbol( 1, "b1", Symbol.NON_TERMINAL ),
										new Symbol( 2, "b2", Symbol.NON_TERMINAL ) } ) );
		grammar
				.addRule( new Rule( 2, new Symbol( 2, "h2", Symbol.NON_TERMINAL ),
						new Symbol[ ] { new Symbol( 3, "b3", Symbol.NON_TERMINAL ),
										new Symbol( 4, "b4", Symbol.NON_TERMINAL ) } ) );

		grammar.addSymbol( new Symbol( 1, "test1", Symbol.NON_TERMINAL ) );
		grammar.addSymbol( new Symbol( 2, "test2", Symbol.TERMINAL ) );
		grammar.addSymbol( new Symbol( 3, "test3", Symbol.SEMANTIC_ACTION ) );
	}


	public void testAddSymbol() {

		grammar.addSymbol( new Symbol( 4, "test", Symbol.TERMINAL ) );
		assertTrue( "test".equals( grammar.getSymbol( "test" ).getValue() ) );
	}


	public void testAddRule() throws InvalidRuleException {

		grammar.addRule( new Rule( 3, new Symbol( 4, "h2", Symbol.NON_TERMINAL ),
				new Symbol[ ] { new Symbol( 4, "b4", Symbol.TERMINAL ), new Symbol( 5, "b5", Symbol.NON_TERMINAL ) } ) );

		assertTrue( "h2".equals( grammar.getRule( 3 ).getHead().getValue() ) );
	}


	public void testGetSymbol() {

		assertTrue( grammar.getSymbol( "test1" ).getCode() == 1 );
		assertTrue( grammar.getSymbol( "test1" ).isNonTerminal() );
		assertTrue( grammar.getSymbol( "test2" ).getCode() == 2 );
		assertTrue( grammar.getSymbol( "test2" ).isTerminal() );
		assertTrue( grammar.getSymbol( "test3" ).getCode() == 3 );
		assertTrue( grammar.getSymbol( "test3" ).isSemanticAction() );
	}


	public void testGetRules() {

		Iterator<Rule> itRules = grammar.getRules();
		Rule rule;
		int quantity = 0;
		while ( itRules.hasNext() ) {
			rule = itRules.next();
			assertTrue( rule.getID() == 2 - quantity );
			assertTrue( ("h" + (2 - quantity)).equals( rule.getHead().getValue() ) );
			quantity++;
		}
		assertTrue( quantity == 2 );

	}


	public void testGetRule() {

		assertTrue( grammar.getRule( 1 ).getID() == 1 );
		assertTrue( "h1".equals( grammar.getRule( 1 ).getHead().getValue() ) );
		assertTrue( grammar.getRule( 2 ).getID() == 2 );
		assertTrue( "h2".equals( grammar.getRule( 2 ).getHead().getValue() ) );
	}

}
