/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.grammar.test;

import java.io.File;
import java.util.Vector;

import junit.framework.TestCase;

import org.ourgrid.common.spec.grammar.CommonGrammar;
import org.ourgrid.common.spec.grammar.Rule;
import org.ourgrid.common.spec.grammar.Symbol;
import org.ourgrid.common.spec.grammar.io.GalsGrammarReader;
import org.ourgrid.test.CommonTest;

/**
 * Created on Jun 8, 2004
 */
public class CommonGrammarTest2 extends TestCase implements CommonTest {

	private final String grammarFileName = "test/specs/grammar/JobGrammar.gals";

	private int NUMBER_OF_RULES = 58;

	private CommonGrammar grammar;

	private Vector<Rule> rules;


	/**
	 * Constructor for CommonGrammarTest2.
	 * 
	 * @param arg0
	 */
	public CommonGrammarTest2( String arg0 ) {

		super( arg0 );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		File grammarFile = new File( grammarFileName );
		grammar = new CommonGrammar();
		GalsGrammarReader reader = new GalsGrammarReader();
		reader.read( grammarFile, grammar );
		rules = new Vector<Rule>( NUMBER_OF_RULES + 1 );
		rules.add( 0, null );
		for ( int count = 0; count <= NUMBER_OF_RULES; ++count ) {
			rules.add( count, grammar.getRule( count ) );
		}
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	public void testFirstFollow() {

		grammar.loadSyntacticTable();
		grammar.loadSyntacticTable(); // Will do nothing because it was
										// already loaded

		// For NonTerminal <job>
		Symbol stackTop = grammar.getSymbol( "<job>" );
		Symbol fromSource = grammar.getSymbol( "JOB" );
		Rule fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 1 ), fromTable );

		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <defaultTask>
		stackTop = grammar.getSymbol( "<defaultTask>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "REQUIRES" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "INIT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "INPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 2 ), fromTable );

		fromSource = grammar.getSymbol( "IF" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <taskList>
		stackTop = grammar.getSymbol( "<taskList>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 3 ), fromTable );

		fromSource = grammar.getSymbol( "REQUIRES" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <definitions>
		stackTop = grammar.getSymbol( "<definitions>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "REQUIRES" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "INIT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "INPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 5 ), fromTable );

		fromSource = grammar.getSymbol( "THEN" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <requires>
		stackTop = grammar.getSymbol( "<requires>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "REQUIRES" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 6 ), fromTable );

		fromSource = grammar.getSymbol( "INIT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "INPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 7 ), fromTable );

		fromSource = grammar.getSymbol( "ELSE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <init>
		stackTop = grammar.getSymbol( "<init>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 9 ), fromTable );

		fromSource = grammar.getSymbol( "INIT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 8 ), fromTable );

		fromSource = grammar.getSymbol( "INPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 9 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 9 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 9 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 9 ), fromTable );

		fromSource = grammar.getSymbol( "PUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <input>
		stackTop = grammar.getSymbol( "<input>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 11 ), fromTable );

		fromSource = grammar.getSymbol( "INPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 10 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 11 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 11 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 11 ), fromTable );

		fromSource = grammar.getSymbol( "STORE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <remote>
		stackTop = grammar.getSymbol( "<remote>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 21 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 20 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 21 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 21 ), fromTable );

		fromSource = grammar.getSymbol( "GET" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <output>
		stackTop = grammar.getSymbol( "<output>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 23 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 22 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 23 ), fromTable );

		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <final>
		stackTop = grammar.getSymbol( "<final>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 32 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 31 ), fromTable );

		fromSource = grammar.getSymbol( "int" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <inputEntries>
		stackTop = grammar.getSymbol( "<inputEntries>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 14 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 14 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 14 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 14 ), fromTable );

		fromSource = grammar.getSymbol( "IF" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 12 ), fromTable );

		fromSource = grammar.getSymbol( "PUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 13 ), fromTable );

		fromSource = grammar.getSymbol( "STORE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 13 ), fromTable );

		fromSource = grammar.getSymbol( "real" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <inputElse>
		stackTop = grammar.getSymbol( "<inputElse>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "REMOTE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "OUTPUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "IF" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "ELSE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 15 ), fromTable );

		fromSource = grammar.getSymbol( "PUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "STORE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 16 ), fromTable );

		fromSource = grammar.getSymbol( "boolean" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <inputCommands>
		stackTop = grammar.getSymbol( "<inputCommands>" );
		fromSource = grammar.getSymbol( "PUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 17 ), fromTable );

		fromSource = grammar.getSymbol( "STORE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 17 ), fromTable );

		fromSource = grammar.getSymbol( "=" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <inCommand>
		stackTop = grammar.getSymbol( "<inCommand>" );
		fromSource = grammar.getSymbol( "PUT" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 18 ), fromTable );

		fromSource = grammar.getSymbol( "STORE" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 19 ), fromTable );

		fromSource = grammar.getSymbol( ":" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <outputEntries>
		stackTop = grammar.getSymbol( "<outputEntries>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 26 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 26 ), fromTable );

		fromSource = grammar.getSymbol( "IF" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 24 ), fromTable );

		fromSource = grammar.getSymbol( "GET" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 25 ), fromTable );

		fromSource = grammar.getSymbol( ";" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <outputElse>
		stackTop = grammar.getSymbol( "<outputElse>" );
		fromSource = grammar.getSymbol( "TASK" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 28 ), fromTable );

		fromSource = grammar.getSymbol( "FINAL" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 28 ), fromTable );

		fromSource = grammar.getSymbol( "IF" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 28 ), fromTable );

		fromSource = grammar.getSymbol( "GET" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 28 ), fromTable );

		fromSource = grammar.getSymbol( "{" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <outputCommands>
		stackTop = grammar.getSymbol( "<outputCommands>" );
		fromSource = grammar.getSymbol( "GET" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 29 ), fromTable );

		fromSource = grammar.getSymbol( "}" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <outCommand>
		stackTop = grammar.getSymbol( "<outCommand>" );

		fromSource = grammar.getSymbol( "GET" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 30 ), fromTable );

		fromSource = grammar.getSymbol( "(" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <line>
		stackTop = grammar.getSymbol( "<line>" );
		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 33 ), fromTable );

		fromSource = grammar.getSymbol( "int" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 34 ), fromTable );

		fromSource = grammar.getSymbol( "real" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 35 ), fromTable );

		fromSource = grammar.getSymbol( "boolean" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 36 ), fromTable );

		fromSource = grammar.getSymbol( ";" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 37 ), fromTable );

		fromSource = grammar.getSymbol( ")" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <expression>
		stackTop = grammar.getSymbol( "<expression>" );
		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "(" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "||" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "&&" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "!" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "IN_PATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "IN_CLASSPATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "VERSION" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 38 ), fromTable );

		fromSource = grammar.getSymbol( "==" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <expression2>
		stackTop = grammar.getSymbol( "<expression2>" );
		fromSource = grammar.getSymbol( "THEN" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( ";" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "(" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 40 ), fromTable );

		fromSource = grammar.getSymbol( ")" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "||" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "&&" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "!" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 39 ), fromTable );

		fromSource = grammar.getSymbol( "IN_PATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "IN_CLASSPATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "VERSION" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 41 ), fromTable );

		fromSource = grammar.getSymbol( "!=" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <and_Or>
		stackTop = grammar.getSymbol( "<and_Or>" );
		fromSource = grammar.getSymbol( "||" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 42 ), fromTable );

		fromSource = grammar.getSymbol( "&&" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 43 ), fromTable );

		fromSource = grammar.getSymbol( ">" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <expr>
		stackTop = grammar.getSymbol( "<expr>" );
		fromSource = grammar.getSymbol( "THEN" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 48 ), fromTable );

		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 47 ), fromTable );

		fromSource = grammar.getSymbol( ";" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 48 ), fromTable );

		fromSource = grammar.getSymbol( ")" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 48 ), fromTable );

		fromSource = grammar.getSymbol( "||" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 48 ), fromTable );

		fromSource = grammar.getSymbol( "&&" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 48 ), fromTable );

		fromSource = grammar.getSymbol( "IN_PATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 44 ), fromTable );

		fromSource = grammar.getSymbol( "IN_CLASSPATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 45 ), fromTable );

		fromSource = grammar.getSymbol( "VERSION" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 46 ), fromTable );

		fromSource = grammar.getSymbol( "<" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <expr2>
		stackTop = grammar.getSymbol( "<expr2>" );
		fromSource = grammar.getSymbol( "==" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 49 ), fromTable );

		fromSource = grammar.getSymbol( "!=" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 50 ), fromTable );

		fromSource = grammar.getSymbol( ">" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 52 ), fromTable );

		fromSource = grammar.getSymbol( "<" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 51 ), fromTable );

		fromSource = grammar.getSymbol( "<=" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 53 ), fromTable );

		fromSource = grammar.getSymbol( ">=" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 54 ), fromTable );

		fromSource = grammar.getSymbol( "IN_PATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

		// For NonTerminal <value>
		stackTop = grammar.getSymbol( "<value>" );
		fromSource = grammar.getSymbol( "string" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 55 ), fromTable );

		fromSource = grammar.getSymbol( "int" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 56 ), fromTable );

		fromSource = grammar.getSymbol( "real" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 57 ), fromTable );

		fromSource = grammar.getSymbol( "boolean" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( rules.elementAt( 58 ), fromTable );

		fromSource = grammar.getSymbol( "IN_CLASSPATH" );
		fromTable = grammar.getRule( stackTop, fromSource );
		assertEquals( null, fromTable );

	}

}
