/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.grammar.test;

import junit.framework.TestCase;

import org.ourgrid.common.spec.grammar.Rule;
import org.ourgrid.common.spec.grammar.Symbol;
import org.ourgrid.common.spec.grammar.exception.InvalidRuleException;
import org.ourgrid.test.CommonTest;

/**
 * Project: Caymman(DSC/UFCG) Description: This class tests the Rule class.
 * 
 * @version 1.0 Created on Sep 29, 2003 Last update: Sep 29, 2003
 */

public class RuleTest extends TestCase implements CommonTest {

	// Variables to test
	private Rule rule1, rule2, rule3 = null;


	/**
	 * Constructor for RuleTest.
	 * 
	 * @param message
	 */
	public RuleTest( String message ) {

		super( message );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		rule1 = new Rule( 1, new Symbol( 1, "function_main", Symbol.NON_TERMINAL ), new Symbol[ ] { new Symbol( 2,
				"main", Symbol.TERMINAL ) } );
		rule2 = new Rule( 2, new Symbol( 1, "expression", Symbol.NON_TERMINAL ),
				new Symbol[ ] { new Symbol( 2, "term", Symbol.NON_TERMINAL ),
								new Symbol( 3, "expression'", Symbol.NON_TERMINAL ) } );
		rule3 = new Rule( 3, new Symbol( 1, "empty", Symbol.NON_TERMINAL ), new Symbol[ 0 ] );
	}


	public void testRule() throws InvalidRuleException {

		new Rule( 1, new Symbol( 1, "function_main", Symbol.NON_TERMINAL ), new Symbol[ ] { new Symbol( 2, "main",
				Symbol.TERMINAL ) } );

		try {
			new Rule( 1, new Symbol( 1, "function_main", Symbol.TERMINAL ), new Symbol[ ] { new Symbol( 2, "main",
					Symbol.TERMINAL ) } );
			fail( "Must throw a exception because the rule is not valid." );
		} catch ( InvalidRuleException e ) {
		}

		try {
			new Rule( 1, new Symbol( 1, "function_main", Symbol.SEMANTIC_ACTION ), new Symbol[ ] { new Symbol( 2,
					"main", Symbol.TERMINAL ) } );
			fail( "Must throw a exception because the rule is not valid." );
		} catch ( InvalidRuleException e ) {
		}

	}


	public void testGetBody() {

		assertTrue( "main".equals( rule1.getBody()[0].getValue() ) );
		assertTrue( "term".equals( rule2.getBody()[0].getValue() ) );
		assertTrue( "expression'".equals( rule2.getBody()[1].getValue() ) );
	}


	public void testGetHead() {

		assertTrue( "function_main".equals( rule1.getHead().getValue() ) );
	}


	public void testGetID() {

		assertTrue( 1 == rule1.getID() );
		assertTrue( 2 == rule2.getID() );
	}


	public void testToString() {

		assertEquals( "1\tfunction_main ::= main", rule1.toString() );
		assertEquals( "2\texpression ::= term expression'", rule2.toString() );
		assertEquals( "3\tempty ::= &", rule3.toString() );
	}

}
