/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.grammar.test;

import junit.framework.TestCase;

import org.ourgrid.common.spec.grammar.Symbol;
import org.ourgrid.test.CommonTest;

/**
 * Project: Caymman(DSC/UFCG) Description: This class tests the Symbol class.
 * 
 * @version 1.0 Created on Sep 29, 2003 Last update: Sep 29, 2003
 */

public class SymbolTest extends TestCase implements CommonTest {

	// Variables to test
	private Symbol symbol1, symbol2, symbol3, symbol4 = null;


	/**
	 * Constructor for SymbolTest.
	 * 
	 * @param message
	 */
	public SymbolTest( String message ) {

		super( message );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		symbol1 = new Symbol( 1, "test1", Symbol.TERMINAL );
		symbol2 = new Symbol( 1, "test2", Symbol.NON_TERMINAL );
		symbol3 = new Symbol( 1, "test3", Symbol.SEMANTIC_ACTION );
		symbol4 = new Symbol();
	}


	/*
	 * Test for void Symbol(int, String, int)
	 */
	public void testSymbolIntStringInt() {

		assertTrue( "test1".equals( symbol1.getValue() ) );
		assertTrue( symbol1.getCode() == 1 );
		assertTrue( symbol1.isTerminal() );
		assertTrue( !symbol1.isNonTerminal() );
		assertTrue( !symbol1.isSemanticAction() );
	}


	/*
	 * Test for void Symbol()
	 */
	public void testSymbol() {

		assertTrue( "".equals( symbol4.getValue() ) );
		assertTrue( symbol4.getCode() == 0 );
		assertTrue( symbol1.isTerminal() );
		assertTrue( !symbol1.isNonTerminal() );
		assertTrue( !symbol1.isSemanticAction() );
	}


	public void testGetCode() {

		assertTrue( symbol2.getCode() == 1 );
	}


	public void testGetValue() {

		assertTrue( "test2".equals( symbol2.getValue() ) );
	}


	public void testIsTerminal() {

		assertTrue( symbol1.isTerminal() );
		assertTrue( !symbol1.isNonTerminal() );
		assertTrue( !symbol1.isSemanticAction() );
	}


	public void testIsNonTerminal() {

		assertTrue( !symbol1.isNonTerminal() );
		assertTrue( symbol2.isNonTerminal() );
		assertTrue( !symbol3.isNonTerminal() );
	}


	public void testIsSemanticAction() {

		assertTrue( !symbol1.isSemanticAction() );
		assertTrue( !symbol2.isSemanticAction() );
		assertTrue( symbol3.isSemanticAction() );
	}


	/*
	 * Test for boolean equals(Object)
	 */
	public void testEqualsObject() {

		Symbol newSymbol = new Symbol( 2, "", Symbol.NON_TERMINAL );
		assertTrue( !symbol3.equals( newSymbol ) );
		Symbol newSymbol2 = new Symbol( 2, "", Symbol.NON_TERMINAL );
		assertTrue( newSymbol2.equals( newSymbol ) );
		Object noSence = new Object();
		assertFalse( newSymbol2.equals( noSence ) );
	}


	public void testToString() {

		assertEquals( "Code: 1 - Value: test1 - Type: 0", symbol1.toString() );
		assertEquals( "Code: 0 - Value:  - Type: 0", symbol4.toString() );
	}
}
