/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.io;

import java.io.FileNotFoundException;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;

public class CharReaderTest extends TestCase implements CommonTest {

	private final String TEST_FILE = "test/specs/io/charReaderTestFile";

	private CharReader cr;


	/**
	 * Constructor for CharReaderTest.
	 * 
	 * @param arg0
	 */
	public CharReaderTest( String arg0 ) {

		super( arg0 );
	}


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

		cr = new CharReader( TEST_FILE );
	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		cr.closeStreams();
	}


	// ///////////////////////////////////////////////////////////////////////////////
	// TESTS STARTS HERE
	// ///////////////////////////////////////////////////////////////////////////////

	public void testCharReaderWithNonExistentFile() throws Exception {

		try {
			new CharReader( "nonExistentFile" );
			fail( "Should not create a file reader because the given file does not exist!!" );
		} catch ( Exception e ) {
			assertTrue( "Do I received a FileNotFoundException?", e instanceof FileNotFoundException );
		}
	}


	public void testReadNonBlankChar() throws Exception {

		// this test consists in to read all the source file, char by char but
		// only the non blank chars, in the right order
		assertEquals( '<', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		assertEquals( '?', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		assertEquals( 'S', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'g', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'C', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'z', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'C', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'z', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '=', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'H', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'l', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'l', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 4, cr.getActualLine() );
		assertEquals( '/', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '/', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'f', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '?', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '?', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'f', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'c', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 's', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'C', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( '(', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( ')', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( '{', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'p', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '(', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 's', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'c', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ')', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '}', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 8, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 8, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 9, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'f', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '(', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'I', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'd', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '<', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '1', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '5', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ')', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '{', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '/', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '/', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'c', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'p', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '(', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'S', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'a', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '!', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( ')', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '}', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 12, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 12, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 13, cr.getActualLine() );
		assertEquals( 'w', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'h', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'l', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '(', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'c', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '_', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'x', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '<', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '=', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '2', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '0', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ')', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '{', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '}', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'm', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '=', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '2', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '0', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '*', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'c', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'o', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'u', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '_', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'x', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '+', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '2', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '\n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 16, cr.getActualLine() );
		assertEquals( 'S', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'i', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'n', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'g', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 's', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '3', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '4', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 's', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '3', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '4', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '=', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'w', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'e', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '1', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '2', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '\'', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ';', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '?', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '>', cr.readNonBlankChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
	}


	public void testReadChar() throws Exception {

		// this test consists in to read all the source file, char by char, in
		// the right order
		assertEquals( '<', cr.readChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		assertEquals( '?', cr.readChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 1, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'S', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'g', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'C', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'z', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 2, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'C', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'z', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '=', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'H', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'l', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'l', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 3, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 4, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '/', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '/', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'f', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '?', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( '?', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 5, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'f', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'c', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 's', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'C', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( '(', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( ')', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		assertEquals( '{', cr.readChar() );
		assertEquals( "must be in the right line", 6, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'p', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '(', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 's', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'c', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ')', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 7, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 8, cr.getActualLine() );
		assertEquals( '}', cr.readChar() );
		assertEquals( "must be in the right line", 8, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 9, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'f', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '(', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'I', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'd', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '<', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '1', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '5', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ')', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '{', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '/', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( '/', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'c', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 10, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'p', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '(', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'S', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'a', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '!', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( ')', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 11, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 12, cr.getActualLine() );
		assertEquals( '}', cr.readChar() );
		assertEquals( "must be in the right line", 12, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 13, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'w', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'h', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'l', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '(', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'c', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '_', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( 'x', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '<', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '=', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '2', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '0', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ')', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '{', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		assertEquals( '}', cr.readChar() );
		assertEquals( "must be in the right line", 14, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'm', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '=', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '2', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '0', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '*', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'c', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'o', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'u', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '_', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( 'x', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '+', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( '2', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 15, cr.getActualLine() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		cr.readChar();// end of line
		assertEquals( '	', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'S', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'i', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'n', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'g', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 's', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '3', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '4', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 's', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 't', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '3', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '4', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '=', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'w', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'e', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( 'r', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '_', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '1', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '2', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ' ', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '\'', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( ';', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '?', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( '>', cr.readChar() );
		assertEquals( "must be in the right line", 17, cr.getActualLine() );
		assertEquals( CharReader.EOF_CHAR, cr.readChar() );
	}


	public void testReadLine() throws Exception {

		cr.readLine();
		cr.readLine();
		assertEquals( 'C', cr.readNonBlankChar() );

		cr.readLine();
		cr.readLine();
		assertEquals( '/', cr.readNonBlankChar() );

		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		assertEquals( "Jumped sixty non blank chars and read the rest of the line", 'o', cr.readNonBlankChar() );

		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		cr.readLine();
		assertEquals( "Jumped fifty non blank chars and read the rest of the line", 'S', cr.readNonBlankChar() );

		cr.readLine();
	}


	public void testUnreadChar() throws Exception {

		char next = (char) -1;

		for ( int i = 0; i < 6; i++ ) {
			next = cr.readChar();
			cr.unreadChar( next );
			assertEquals( "I always will read the same char", '<', (next = cr.readChar()) );
			assertEquals( 1, cr.getActualLine() );
			cr.unreadChar( next );
		}
		next = cr.readChar();
		next = cr.readChar();
		assertEquals( '?', next );
		assertEquals( 1, cr.getActualLine() );

		next = cr.readChar();
		assertEquals( ' ', next );
		assertEquals( 1, cr.getActualLine() );

		// THE INTENTION HERE IS TEST THE LINE NUMBER WHEN USE UNREAD TO LINE
		// SEPARATOR CHARATHER
		next = cr.readChar(); // THIS IS THE LINE SEPARATOR
		assertEquals( 1, cr.getActualLine() );
		char next_next = cr.readChar();
		assertEquals( 2, cr.getActualLine() );
		cr.unreadChar( next_next );
		cr.unreadChar( next );
		// RETURNED ONE LINE
		next = cr.readChar();
		assertEquals( 1, cr.getActualLine() );

		next = cr.readNonBlankChar();
		cr.unreadChar( next );
		assertEquals( "Must read the char I've been unread", next, cr.readChar() );
	}

}
