/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.lexical;

import junit.framework.TestCase;

import org.ourgrid.common.spec.TokenDelimiter;
import org.ourgrid.common.spec.token.Token;
import org.ourgrid.test.CommonTest;

public class CommonLexicalAnalyzerTestGrid extends TestCase implements CommonTest {

	private final String SOURCE_FILE_TEST = "test/specs/lexical/GridTest1.gdf";

	private final String SOURCE_FILE_TEST2 = "test/specs/lexical/GridTest2.gdf";

	private CommonLexicalAnalyzer lexical;


	/**
	 * Constructor for CommomLexicalAnalyzerTest1.
	 * 
	 * @param arg0
	 */
	public CommonLexicalAnalyzerTestGrid( String arg0 ) {

		super( arg0 );
	}


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		lexical = null;

		// do nothing here...
	}


	// ///////////////////////////////////////////////////////////////////////////////////
	// TESTS STARTS HERE
	// ///////////////////////////////////////////////////////////////////////////////////

	public void testCommomLexicalAnalyzer() throws Exception {

		try {
			new CommonLexicalAnalyzer( "unexistentFile" );
			fail( "The given file does not exist" );
		} catch ( Exception e ) {
			assertTrue( "Must launch LexicalException", e instanceof LexicalException );
		}
	}


	public void testGetToken() throws Exception {

		lexical = new CommonLexicalAnalyzer( SOURCE_FILE_TEST );
		// the lexical must read all the source file tokens and returns they
		// with the right codes and the right lines
		Token token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "PEER", token.getSymbol() );
		assertEquals( 15, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "name", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "moreia.lsd.ufcg.edu.br", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "gump_port", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( "8888", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "site", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "lsd.ufcg.edu.br", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "storage_size", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( "150", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "storage", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 8, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 8, token.getLine() );
		token = lexical.getToken();
		assertEquals( "/home/grid/", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 8, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 8, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "GUM", token.getSymbol() );
		assertEquals( 16, token.getCode() );
		assertEquals( 11, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 11, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "name", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 12, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 12, token.getLine() );
		token = lexical.getToken();
		assertEquals( "cascudo.lsd.ufcg.edu.br", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 12, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 12, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "uaport", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "7777", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "type", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 14, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 14, token.getLine() );
		token = lexical.getToken();
		assertEquals( "User_Agent", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 14, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 14, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "os", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		assertEquals( "Linux", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "gateway", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( "bla", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( "1,ble", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		assertEquals( "2,bli", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 16, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "GUM", token.getSymbol() );
		assertEquals( 16, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "name", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "robalo.lsd.ufcg.edu.br", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "type", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( "Grid_Script", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "REM_EXEC", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "ssh", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "-x", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%machine", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%command", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "COPY_TO", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "scp", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%localdir/%file", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%machine", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%remotedir/", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "COPY_FROM", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( "scp", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%machine", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%remotedir/%file", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( "%localdir/", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 23, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "os", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 24, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 24, token.getLine() );
		token = lexical.getToken();
		assertEquals( "Linux", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 24, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 24, token.getLine() );
	}


	public void testGetTokenWithDelimiter() throws LexicalException {

		lexical = new CommonLexicalAnalyzer( SOURCE_FILE_TEST2 );
		TokenDelimiter delimiter = new TokenDelimiter();
		delimiter.addDelimiter( '\n' );
		delimiter.addDelimiter( '=' );

		Token token = lexical.getToken( delimiter );
		assertEquals( "PEER :", token.getSymbol() );
		token = lexical.getToken();
		assertEquals( "\\n", token.getSymbol() );

		token = lexical.getToken( delimiter );
		assertEquals( "name", token.getSymbol() );
		token = lexical.getToken( delimiter );
		assertEquals( "moreia.lsd.ufcg.edu.br", token.getSymbol() );
		token = lexical.getToken( delimiter );
		assertEquals( ";", token.getSymbol() );
		token = lexical.getToken();
		assertEquals( "\\n", token.getSymbol() );

		token = lexical.getToken( delimiter );
		assertEquals( "new attribute", token.getSymbol() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		token = lexical.getToken( delimiter );
		assertEquals( "hi", token.getSymbol() );
		token = lexical.getToken();
		assertEquals( "\\n", token.getSymbol() );

		token = lexical.getToken( delimiter );
		assertEquals( "hi", token.getSymbol() );
		token = lexical.getToken( delimiter );
		assertEquals( "new attribute", token.getSymbol() );
		token = lexical.getToken();
		assertEquals( "\\n", token.getSymbol() );
	}

}
