/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.lexical;

import junit.framework.TestCase;

import org.ourgrid.common.spec.TokenDelimiter;
import org.ourgrid.common.spec.token.Token;
import org.ourgrid.test.CommonTest;

public class CommonLexicalAnalyzerTestJob extends TestCase implements CommonTest {

	private final String SOURCE_FILE_TEST = "test/specs/lexical/JobTest1.jdf";

	private final String SOURCE_FILE_TEST2 = "test/specs/lexical/JobTest2.jdf";

	private CommonLexicalAnalyzer lexical;


	/**
	 * Constructor for CommonLexicalAnalyzerTestJob.
	 * 
	 * @param arg0
	 */
	public CommonLexicalAnalyzerTestJob( String arg0 ) {

		super( arg0 );
	}


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		lexical = null;

	}


	// ///////////////////////////////////////////////////////////////////////////////////
	// TESTS STARTS HERE
	// ///////////////////////////////////////////////////////////////////////////////////

	public void testCommomLexicalAnalyzer() throws Exception {

		try {
			new CommonLexicalAnalyzer( "unexistentFile" );
			fail( "The given file does not exist" );
		} catch ( Exception e ) {
			assertTrue( "Must launch LexicalException", e instanceof LexicalException );
		}
	}


	public void testGetToken() throws Exception {

		// the lexical must read all the source file tokens and returns they
		// with the right codes and the right lines
		lexical = new CommonLexicalAnalyzer( SOURCE_FILE_TEST );
		// assertTrue( Character.isLetterOrDigit( '&' ) );
		Token token = lexical.getToken();
		assertEquals( "JOB", token.getSymbol() );
		assertEquals( 1, token.getCode() );
		assertEquals( 1, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 1, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "REQUIREMENTS", token.getSymbol() );
		assertEquals( 3, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "OS", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "==", token.getSymbol() );
		assertEquals( 34, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "Linux", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "&&", token.getSymbol() );
		assertEquals( 31, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PLAYPEN_SIZE", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( ">=", token.getSymbol() );
		assertEquals( 38, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( "200", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "INIT", token.getSymbol() );
		assertEquals( 4, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/init_script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "INPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 12, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/execute", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "IF", token.getSymbol() );
		assertEquals( 9, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "(", token.getSymbol() );
		assertEquals( 72, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PLAYPEN_SIZE", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "<", token.getSymbol() );
		assertEquals( 37, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "300", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( ")", token.getSymbol() );
		assertEquals( 73, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "THEN", token.getSymbol() );
		assertEquals( 10, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "{", token.getSymbol() );
		assertEquals( 70, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 12, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/input1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN/input", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "}", token.getSymbol() );
		assertEquals( 71, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "ELSE", token.getSymbol() );
		assertEquals( 11, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "{", token.getSymbol() );
		assertEquals( 70, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 12, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/input2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN/input", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		assertEquals( "}", token.getSymbol() );
		assertEquals( 71, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "REMOTE", token.getSymbol() );
		assertEquals( 6, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN/execute", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( "input1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "OUTPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "GET", token.getSymbol() );
		assertEquals( 14, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN/output", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/output_$TASK", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "FINAL", token.getSymbol() );
		assertEquals( 8, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/final_script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 7, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "TASK", token.getSymbol() );
		assertEquals( 2, token.getCode() );
		assertEquals( 9, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 9, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "TASK", token.getSymbol() );
		assertEquals( 2, token.getCode() );
		assertEquals( 11, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 11, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "INPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 12, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/input3", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN/input", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 12, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/execute", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$PLAYPEN", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 13, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "TASK", token.getSymbol() );
		assertEquals( 2, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 15, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "TASK", token.getSymbol() );
		assertEquals( 2, token.getCode() );
		assertEquals( 17, token.getLine() );
		token = lexical.getToken();
		assertEquals( ":", token.getSymbol() );
		assertEquals( 62, token.getCode() );
		assertEquals( 17, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "INIT", token.getSymbol() );
		assertEquals( 4, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/init_script2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 18, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "INPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "STORE", token.getSymbol() );
		assertEquals( 13, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/execute2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$STORAGE", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "STORE", token.getSymbol() );
		assertEquals( 13, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/input1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$STORAGE", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 19, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "REMOTE", token.getSymbol() );
		assertEquals( 6, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$STORAGE/execute2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( "input1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 20, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "OUTPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "GET", token.getSymbol() );
		assertEquals( 14, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "$STORAGE/output2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/output_$TASK", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 21, token.getLine() );
		token = lexical.getToken();
		token = lexical.getToken();
		assertEquals( "FINAL", token.getSymbol() );
		assertEquals( 8, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "=", token.getSymbol() );
		assertEquals( 33, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( "sample/final_script2", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 22, token.getLine() );
		token = lexical.getToken();
		assertEquals( ";", token.getSymbol() );
		assertEquals( 60, token.getCode() );
		assertEquals( 22, token.getLine() );
	}


	public void testGetTokenWithDelimiter() throws LexicalException {

		lexical = new CommonLexicalAnalyzer( SOURCE_FILE_TEST2 );
		TokenDelimiter delim = new TokenDelimiter();
		delim.addDelimiter( '\n' );
		delim.addDelimiter( ' ' );
		delim.addDelimiter( '\t' );

		Token token = lexical.getToken( delim );
		assertEquals( "JOB", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 1, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ":", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 1, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

		token = lexical.getToken( delim );
		assertEquals( "INIT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "=", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "sample/init script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ";", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 2, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

		token = lexical.getToken( delim );
		assertEquals( "INPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "=", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "PUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "sample/execute script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "$PLAYPEN", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ";", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 3, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

		token = lexical.getToken( delim );
		assertEquals( "REMOTE", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "=", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "$PLAYPEN/execute script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ";", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 4, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

		token = lexical.getToken( delim );
		assertEquals( "OUTPUT", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "=", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "GET", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "$PLAYPEN/output", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "sample/output $TASK", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ";", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 5, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

		token = lexical.getToken( delim );
		assertEquals( "FINAL", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "=", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( "sample/final script1", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		token = lexical.getToken( delim );
		assertEquals( ";", token.getSymbol() );
		assertEquals( 83, token.getCode() );
		assertEquals( 6, token.getLine() );
		lexical.getToken(); // Ignoring the '\n' character

	}
}
