/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.lexical;

import org.ourgrid.common.spec.TokenDelimiter;
import org.ourgrid.common.spec.token.Token;

public interface LexicalAnalyzer {

	/**
	 * This is the main function of the lexical analyzer. It will read the next
	 * string from source and return the token it represents ( the string, the
	 * code it has and the line it was found ).
	 * 
	 * @return The next valid token object and null if the source finished
	 */
	public Token getToken() throws LexicalException;


	/**
	 * It will read the next string from source and return the token it
	 * represents. This method uses the given set of delimiters to know where
	 * the token stops.
	 * 
	 * @param delimiters the set of delimiters that determines the end of token.
	 * @return A token where the token's symbol is the string read.
	 * @throws LexicalException if a I/O problem ocurres at the reader
	 */
	public Token getToken( TokenDelimiter delimiters ) throws LexicalException;
}
