/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.main;

import java.util.List;

/**
 * This is the compiler entity. It is the responsable for manage the other
 * internal entities to make the work properly. Created on 16/06/2004
 */
public interface Compiler {

	/**
	 * Initialize the compile process for a given source file. It will check the
	 * source syntactical formation and build a answer to the source as defined
	 * at the semantical analyzer.
	 * 
	 * @param sourceFileName The file that contains the source to be compiled.
	 * @param languageType Have to be one of the provided language types at the
	 *        compiler. ( Check types at the implementor of this interface )
	 * @throws CompilerException Thrown when happens any problem. It generally
	 *         packs a more specific exception.
	 */
	public void compile( String sourceFileName, String languageType ) throws CompilerException;


	/**
	 * Its a general form of returning an answer of a compilation.
	 * 
	 * @return A List object that have to contains the answer object(s). A null
	 *         return can be returned to signalize a not necessary answer.
	 */
	public List getResult();

}
