/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.semantic;

import org.ourgrid.common.spec.semantic.exception.SemanticException;
import org.ourgrid.common.spec.token.Token;

/**
 * This implementation of the semantic analyzer uses more then one language to
 * compile. Thats why uses SemanticActions as sets to each different language
 * actions.
 */
public class CommonSemanticAnalyzer implements SemanticAnalyzer {

	SemanticActions actions;


	/**
	 * The constructor.
	 * 
	 * @param actions The set of the actions that will be used for a determined
	 *        language.
	 */
	public CommonSemanticAnalyzer( SemanticActions actions ) {

		this.actions = actions;

	}


	/**
	 * @see org.ourgrid.common.spec.semantic.SemanticAnalyzer#performAction(java.lang.String,
	 *      org.ourgrid.common.spec.token.Token)
	 */
	public void performAction( String action, Token token ) throws SemanticException {

		StringBuffer sb = new StringBuffer( action );
		String actionNumber = sb.substring( 1 ); // the format is #n where n
													// is the number of the
													// action.
		actions.performAction( "action" + actionNumber, token );
	}


	/**
	 * @see org.ourgrid.common.spec.semantic.SemanticAnalyzer#getOperationMode()
	 */
	public int getOperationMode() {

		return actions.getOperationalMode();
	}

}
