/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.semantic.test;

import java.util.LinkedList;

import junit.framework.TestCase;

import org.ourgrid.common.spec.CompilerMessages;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.spec.main.CommonCompiler;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.common.spec.semantic.GDFSemanticActions;
import org.ourgrid.common.spec.semantic.exception.SemanticException;
import org.ourgrid.common.spec.token.Token;
import org.ourgrid.common.url.PeerURLProvider;
import org.ourgrid.test.CommonTest;

/**
 * Created on Jul 8, 2004
 */
public class GDFSemanticActionsTest extends TestCase implements CommonTest {

	CommonCompiler compiler;


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		compiler = new CommonCompiler();
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		compiler = null;
	}


	/**
	 * Constructor for GridSemanticActionsTest.
	 * 
	 * @param arg0
	 */
	public GDFSemanticActionsTest( String arg0 ) {

		super( arg0 );
	}


	public void testPeers() throws Exception {

		compiler.compile( "test/specs/semantic/GridTest2.gdf", CommonCompiler.GDF_TYPE );

		LinkedList gumps = (LinkedList) compiler.getResult();
		assertEquals( 3, gumps.size() );

		PeerSpec gumpSpec = (PeerSpec) gumps.removeFirst();
		assertEquals( "UFRJ", gumpSpec.getLabel() );
		assertEquals( "xpto.ufrj.edu.br", gumpSpec.getName() );
		assertEquals( 7777, gumpSpec.getPort() );

		gumpSpec = (PeerSpec) gumps.removeFirst();
		assertEquals( "LCC", gumpSpec.getLabel() );
		assertEquals( "xpto.lcc.edu.br", gumpSpec.getName() );
		assertEquals( 8888, gumpSpec.getPort() );

		gumpSpec = (PeerSpec) gumps.removeFirst();
		assertEquals( "", gumpSpec.getLabel() );
		assertEquals( "xpto.lsd.edu.br", gumpSpec.getName() );
		assertEquals( 9999, gumpSpec.getPort() );

	}


	public void testBadPeerDefinitions() {

		try {
			compiler.compile( "test/specs/semantic/GridTest8.gdf", CommonCompiler.GDF_TYPE );
			fail( "The GuMP definition on the file compiled is malformed." );
		} catch ( CompilerException cex ) {
			Throwable cause1 = cex.getCause().getCause();
			if ( cause1 instanceof SemanticException ) {
				assertTrue( cause1.getMessage().equals( CompilerMessages.BAD_GUMP_DEFINITION_NAMEORPORT_MISSING ) );
			}
		}

		try {
			compiler.compile( "test/specs/semantic/GridTest9.gdf", CommonCompiler.GDF_TYPE );
			fail( "The GuMP definition on the file compiled is malformed." );
		} catch ( CompilerException cex ) {
			Throwable cause1 = cex.getCause().getCause();
			if ( cause1 instanceof SemanticException ) {
				assertTrue( cause1.getMessage().equals( CompilerMessages.BAD_GUMP_DEFINITION_WRONG_PORT_ATT ) );
			}
		}

	}


	public void testErrorCallsAtPerformAction() {

		GDFSemanticActions actions = new GDFSemanticActions();
		try {
			actions.performAction( "nonexistent_action", new Token() );
			fail( "Sould fail because the action does not exists to be executed." );
		} catch ( SemanticException sex ) {
			assertTrue( sex.getCause() instanceof NoSuchMethodException );
		}

		try {
			actions.performAction( "action2", null );
			fail( "Sould fail because the action uses the token object passed as \"null\"." );
		} catch ( SemanticException sex ) {
			assertTrue( sex.getCause() instanceof NullPointerException );
		}

	}


	public void testPeerSpecification() throws CompilerException {

		compiler.compile( "test/specs/semantic/GridTest10.gdf", CommonCompiler.GDF_TYPE );
		LinkedList result = (LinkedList) compiler.getResult();
		assertEquals( 2, result.size() );
		PeerSpec remoteGump = (PeerSpec) result.removeFirst();
		assertEquals( "UF RJ", remoteGump.getLabel() );
		assertEquals( "xpto.ufrj.edu.br", remoteGump.getName() );
		assertEquals( 7777, remoteGump.getPort() );
		assertEquals( "rmi://xpto.ufrj.edu.br:7777/" + PeerURLProvider.LOCAL_ACCESS, remoteGump.getURL() );
		remoteGump = (PeerSpec) result.removeFirst();
		assertEquals( "LCC", remoteGump.getLabel() );
		assertEquals( "seulunga.lcc.ufcg.edu.br", remoteGump.getName() );
		assertEquals( 8888, remoteGump.getPort() );
		assertEquals( "rmi://seulunga.lcc.ufcg.edu.br:8888/" + PeerURLProvider.LOCAL_ACCESS, remoteGump.getURL() );

	}


	public void testExceptionWhenAttValueIsEmpty() {

		try {
			compiler.compile( "test/specs/semantic/GridTest11.gdf", CommonCompiler.GDF_TYPE );
			fail( "The remote GUMP specified has an attribute with empty value." );
		} catch ( CompilerException cex ) {
			Throwable cause1 = cex.getCause().getCause();
			if ( cause1 instanceof SemanticException ) {
				assertTrue( cause1.getMessage().equals( CompilerMessages.SEMANTIC_EMPTY_ATTRIBUTE_VALUE( "LABEL", 2 ) ) );
			}
		}
	}

}
