/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.semantic.test;

import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.UserAgentConfiguration;
import org.ourgrid.common.spec.CompilerMessages;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.main.CommonCompiler;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.common.spec.semantic.SDFSemanticActions;
import org.ourgrid.common.spec.semantic.exception.SemanticException;
import org.ourgrid.common.spec.syntactical.SyntacticalException;
import org.ourgrid.common.spec.token.Token;
import org.ourgrid.test.CommonTest;

/**
 * Created on Jul 8, 2004
 */
public class SDFSemanticActionsTest extends TestCase implements CommonTest {

	CommonCompiler compiler;


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		compiler = new CommonCompiler();
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		compiler = null;
	}


	/**
	 * Constructor for GridSemanticActionsTest.
	 * 
	 * @param arg0
	 */
	public SDFSemanticActionsTest( String arg0 ) {

		super( arg0 );
	}


	public void testGums() throws Exception {

		compiler.compile( "test/specs/semantic/GridPeerTest1.sdf", CommonCompiler.SDF_TYPE );

		List gums = compiler.getResult();
		Iterator gumsIter = gums.iterator();

		// Tests about the local gump
		GumSpec tested = (GumSpec) gumsIter.next();
		assertEquals( "xpto1.lsd.ufcg.edu.br", tested.getAttribute( Configuration.PROP_NAME ) );
		assertEquals( "9876", tested.getAttribute( Configuration.PROP_PORT ) );
		assertEquals( "linux", tested.getAttribute( UserAgentConfiguration.ATT_OS ) );
		assertEquals( "300", tested.getAttribute( UserAgentConfiguration.ATT_MEM ) );

		tested = (GumSpec) gumsIter.next();
		assertEquals( "xpto2.lsd.ufcg.edu.br", tested.getAttribute( Configuration.PROP_NAME ) );
		assertEquals( "9875", tested.getAttribute( Configuration.PROP_PORT ) );
		assertEquals( "solaris", tested.getAttribute( UserAgentConfiguration.ATT_OS ) );
		assertEquals( "200", tested.getAttribute( UserAgentConfiguration.ATT_MEM ) );

	}


	public void testGumFailed_ATT_TYPE() throws Exception {

		try {
			compiler.compile( "test/specs/semantic/GridPeerTest3.sdf", CommonCompiler.SDF_TYPE );
			fail( "Should fail because the type of the GuM was not defined." );
		} catch ( CompilerException cex ) {
			if ( cex.getCause().getCause() instanceof SemanticException ) {
			} else {
				fail( "Should fail because the type of the GuM was not defined." );
			}
		}
	}


	public void testGumFailed_TYPE_USER_AGENT() throws Exception {

		try {
			compiler.compile( "test/specs/semantic/GridPeerTest4.sdf", CommonCompiler.SDF_TYPE );
			fail( "Should fail because the UA_ATT_PORT of the UserAgentGuM was not defined." );
		} catch ( CompilerException cex ) {
			Throwable cause1 = cex.getCause().getCause();
			if ( cause1 instanceof SemanticException ) {
				assertTrue( cause1.getMessage().equals( CompilerMessages.BAD_GUM_DEFINITION( "xpto1.lsd.ufcg.edu.br" ) ) );
			} else {
				fail( "Should fail because a semantic problem - the machine is bad specified!" );
			}

		}
	}


	public void testGumFailed_TYPE_GRID_SCRIPT() throws Exception {

		try {
			compiler.compile( "test/specs/semantic/GridPeerTest5.sdf", CommonCompiler.SDF_TYPE );
			fail( "Should fail because the ATT_REM_EXEC of the GridScriptGuM was not defined." );
		} catch ( CompilerException cex ) {
			Throwable cause1 = cex.getCause().getCause();
			if ( cause1 instanceof SemanticException ) {
				assertTrue( cause1.getMessage().equals( CompilerMessages.BAD_GUM_DEFINITION( "xpto1.lsd.ufcg.edu.br" ) ) );
			} else {
				fail( "Should fail because a semantic problem - the machine is bad specified!" );
			}

		}
	}


	public void testGoodGum_WrittenInWindows() throws Exception {

		try {
			compiler.compile( "test/specs/semantic/Grid_desc_win.sdf", CommonCompiler.SDF_TYPE );
			List gums = compiler.getResult();
			Iterator gumsIter = gums.iterator();

			// Tests about the My GUMP
			GumSpec tested = (GumSpec) gumsIter.next();
			assertEquals( "xpto1.lsd.ufcg.edu.br", tested.getAttribute( Configuration.PROP_NAME ) );
			assertEquals( "9876", tested.getAttribute( Configuration.PROP_PORT ) );
			assertEquals( "linux", tested.getAttribute( UserAgentConfiguration.ATT_OS ) );
			assertEquals( "UAlinux", tested.getAttribute( UserAgentConfiguration.ATT_TYPE ) );

		} catch ( SyntacticalException sex ) {
			fail( sex.getMessage() );
		}

	}


	public void testErrorCallsAtPerformAction() {

		SDFSemanticActions actions = new SDFSemanticActions();
		try {
			actions.performAction( "nonexistent_action", new Token() );
			fail( "Sould fail because the action does not exists to be executed." );
		} catch ( SemanticException sex ) {
			assertTrue( sex.getCause() instanceof NoSuchMethodException );
		}

		try {
			actions.performAction( "action2", null );
			fail( "Sould fail because the action uses the token object passed as \"null\"." );
		} catch ( SemanticException sex ) {
			assertTrue( sex.getCause() instanceof NullPointerException );
		}

	}

}
