/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.token;

import java.util.Map;

import org.ourgrid.common.spec.CodesTable;
import org.ourgrid.common.spec.io.CharReader;

/**
 * It is a Token object that is recognized as an Parenthetic at the CodesTable.
 */
public class Parenthetic extends Token {

	/**
	 * Checks if the character passed as paramether is a Parenthetic.
	 * 
	 * @param theChar - Is the character that has to be checked.
	 * @param reader - Is the reader that is able to read characters from
	 *        source.
	 * @return A Token object if the character was recognized or "null" if it
	 *         was not.
	 * @see org.ourgrid.common.spec.CodesTable
	 */
	public Token readParantizer( char theChar, CharReader reader ) {

		Map<String,Integer> table = CodesTable.getInstance().getParenthetics();
		String toTest = "" + theChar;

		if ( table.containsKey( toTest ) ) {

			int code = table.get( toTest ).intValue();
			this.setCode( code );
			this.setLine( reader.getActualLine() );
			this.setSymbol( toTest );

			return this;
		}
		return null;
	}

}
