/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.token;

import java.util.Map;

import org.ourgrid.common.spec.CodesTable;
import org.ourgrid.common.spec.io.CharReader;

/**
 * This is a token entity that recognize punctuation symbols.
 * 
 * @see org.ourgrid.common.spec.CodesTable
 */
public class Pointing extends Token {

	/**
	 * The constructor.
	 */
	public Pointing() {

	}


	/**
	 * Tries to recognize a punctuation symbol and return a token using this
	 * symbol.
	 * 
	 * @param theChar The symbol to be analyzed.
	 * @param reader The reader able to read characters from source.
	 * @return this Pointing token object if theChar was recognized as
	 *         punctuation symbol and null otherwise.
	 */
	public Token readPointing( char theChar, CharReader reader ) {

		Map<String,Integer> table = CodesTable.getInstance().getPointing();
		String character = Character.toString( theChar );
		if ( table.containsKey( character ) ) {
			int code = table.get( character ).intValue();
			this.setCode( code );
			this.setLine( reader.getActualLine() );
			if ( character.equals( CharReader.UNIX_LINE_SEPARATOR ) ) {
				this.setSymbol( "\\n" );
			} else {
				this.setSymbol( character );
			}

			return this;
		}
		return null;
	}

}
