/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.token;

import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.common.spec.CodesTable;
import org.ourgrid.common.spec.SpecialCharException;
import org.ourgrid.common.spec.TokenDelimiter;
import org.ourgrid.common.spec.io.CharReader;
import org.ourgrid.test.CommonTest;

/**
 * Created on May 29, 2004
 */
public class StringTokenTest extends TestCase implements CommonTest {

	private final String SOURCE_FILE_TEST = "test/specs/tokens/stringTest";

	private final String SOURCE_FILE_TEST2 = "test/specs/tokens/stringTest2";

	private CharReader reader, reader2;

	private CodesTable codesTable;


	/**
	 * Constructor for StringTokenTest.
	 * 
	 * @param arg0
	 */
	public StringTokenTest( String arg0 ) {

		super( arg0 );
	}


	protected void setUp() throws Exception {

		super.setUp();
		reader = new CharReader( SOURCE_FILE_TEST );
		reader2 = new CharReader( SOURCE_FILE_TEST2 );
		codesTable = CodesTable.getInstance();
	}


	protected void tearDown() throws Exception {

		super.tearDown();
		reader.closeStreams();
		reader2.closeStreams();
		codesTable = null;
	}


	public void test_getStringToken() throws IOException {

		Token toTest = this.getNextString( reader );
		assertEquals( "testando", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		char aux = reader.readChar();
		assertEquals( ';', aux );
		reader.readLine();

		toTest = this.getNextString( reader );
		assertEquals( "bla", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		toTest = this.getNextString( reader );
		assertEquals( "ble", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		toTest = this.getNextString( reader );
		assertEquals( "bli", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		aux = reader.readChar();
		assertEquals( '{', aux );

		toTest = this.getNextString( reader );
		assertEquals( "blo", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		aux = reader.readChar();
		assertEquals( '}', aux );

		toTest = this.getNextString( reader );
		assertEquals( "blu", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );
		reader.readLine();

		toTest = this.getNextString( reader );
		assertEquals( "true", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		aux = reader.readChar();
		assertEquals( '(', aux );

		toTest = this.getNextString( reader );
		assertEquals( "false", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		aux = reader.readChar();
		assertEquals( ')', aux );
		reader.readLine();

		toTest = this.getNextString( reader );
		assertEquals( "xau", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );

		aux = reader.readChar();
		assertEquals( '=', aux );

		toTest = this.getNextString( reader );
		assertEquals( "uax", toTest.getSymbol() );
		assertEquals( codesTable.getCode( CodesTable.STRING ), toTest.getCode() );
		reader.readLine();

		toTest = this.getNextString( reader );
		assertEquals( "IN_PATH", toTest.getSymbol() );
		assertEquals( true, toTest instanceof Operator );

	}


	private Token getNextString( CharReader reader ) throws IOException {

		String aux = "" + reader.readNonBlankChar();
		StringToken stringToken = new StringToken();
		Token toReturn = stringToken.readStringToken( aux, reader );

		return toReturn;
	}


	public void testReadStringWithDelimiter() throws SpecialCharException, IOException {

		StringToken st = new StringToken();

		TokenDelimiter del = new TokenDelimiter();
		del.addDelimiter( '\n' );

		Token toTest = st.readString( this.reader2, del );
		assertEquals( "bla", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 1, reader2.getActualLine() );

		toTest = st.readString( this.reader2, del );
		assertEquals( "ble bli", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 2, reader2.getActualLine() );

		toTest = st.readString( this.reader2, del );
		assertEquals( "blo blu\"", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 3, reader2.getActualLine() );

		toTest = st.readString( this.reader2, del );
		assertEquals( "\"bla\"", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 4, reader2.getActualLine() );

		toTest = st.readString( this.reader2, del );
		assertEquals( "ble \"bli\"", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 5, reader2.getActualLine() );

		del.addDelimiter( ' ' );
		toTest = st.readString( this.reader2, del );
		assertEquals( "eu", toTest.getSymbol() );
		toTest = st.readString( this.reader2, del );
		assertEquals( "sou foda", toTest.getSymbol() );
		toTest = st.readString( this.reader2, del );
		assertEquals( "mesmo", toTest.getSymbol() );
		reader2.readChar();
		assertEquals( 6, reader2.getActualLine() );

		del.addDelimiter( '=' );
		toTest = st.readString( this.reader2, del );
		assertEquals( "", toTest.getSymbol() );
	}
}
