/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.spec.token;

/**
 * A token is a set of informations about the peaces read from the source to be
 * compiled. This is the unic structure that is understandable to the
 * syntactical analyzer.
 */
public class Token {

	private String symbol;

	private int code;

	private int line;


	/**
	 * A Empty Constructor
	 */
	public Token() {

	}


	/**
	 * A Constructor
	 * 
	 * @param symbol The token's string read from source.
	 * @param code The code of the string read. To mode informations see
	 *        CodesTable.
	 * @param line The line where the symbol was found at source.
	 */
	public Token( String symbol, int code, int line ) {

		this.symbol = symbol;
		this.code = code;
		this.line = line;
	}


	/**
	 * @return A string view of the token informations.
	 */
	public String toString() {

		StringBuffer sb = new StringBuffer();
		sb.append( "Lexeme: \"" + getSymbol() + "\" ; " );
		sb.append( "Code: \"" + getCode() + "\" ; " );
		sb.append( "Line: \"" + getLine() + "\"" );

		return sb.toString();
	}


	/**
	 * @return the code of the token read from source.
	 */
	public int getCode() {

		return code;
	}


	/**
	 * @return the line of source where the symbol was read.
	 */
	public int getLine() {

		return line;
	}


	/**
	 * @return the token's string read from source.
	 */
	public String getSymbol() {

		return symbol;
	}


	/**
	 * @param code the code of the token read from source.
	 */
	public void setCode( int code ) {

		this.code = code;
	}


	/**
	 * @param line the line of source where the symbol was read.
	 */
	public void setLine( int line ) {

		this.line = line;
	}


	/**
	 * @param symbol the token's string read from source.
	 */
	public void setSymbol( String symbol ) {

		this.symbol = symbol;
	}

}
