/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.url;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.common.fd.AbstractFailureDetector;

/**
 * This class provides URLs and addresses for the main components that needed to
 * be remotely referenced.
 */
public class MyGridURLProvider extends URLProvider {

	public static final String MYGRID_MANAGER = "MYGRID_UI_MANAGER";


	/**
	 * This method is used like a common point to get the address of the
	 * SCHEDULER.
	 * 
	 * @return the RMI URL that points to the SCHEDULER
	 */
	public static String scheduler() {

		return "rmi://" + getName() + ":" + getPort() + "/SCHEDULER";
	}


	/**
	 * This method is used like a common point to get the address of the
	 * GRID_MANAGER.
	 * 
	 * @return the RMI URL that points to the GUM_MANAGER
	 */
	public static String gridManager() {

		return "rmi://" + getName() + ":" + getPort() + "/GRID_MANAGER";
	}


	/**
	 * This method is used like a common point to get the address of the
	 * GUMP_MANAGER.
	 * 
	 * @return the RMI URL that points to the GUMP_MANAGER
	 */
	public static String gumpManager() {

		return "rmi://" + getName() + ":" + getPort() + "/GUMP_MANAGER";
	}


	/**
	 * This method is used like a common point to get the address of the
	 * GUMP_CLIENT.
	 * 
	 * @return the RMI URL that points to the GUMP_CLIENT
	 */
	public static String gumpClient() {

		return "rmi://" + getName() + ":" + getPort() + "/GUMP_CLIENT";
	}


	public static String externalGumpClient() {

		return "rmi://" + getExternalName() + ":" + getPort() + "/GUMP_CLIENT";
	}


	/**
	 * This method is used like a common point to get the address of the
	 * GUI_LISTENER.
	 * 
	 * @return the RMI URL that points to the MyGridGUIListener
	 */
	public static String guiListener() {

		return "rmi://" + getGuiName() + ":" + getGuiPort() + "/GUI_LISTENER";
	}


	private static String getGuiName() {

		return Configuration.getInstance().getProperty( MyGridConfiguration.PROP_GUI_NAME );
	}


	public static String getGuiPort() {

		return Configuration.getInstance().getProperty( MyGridConfiguration.PROP_GUI_PORT );
	}


	/**
	 * This method is used like a common point to get the address of the
	 * MYGRID_MANAGER.
	 * 
	 * @return the RMI URL that points to the MyGridManager
	 */
	public static String mygridUIManager() {

		return "rmi://" + getName() + ":" + getPort() + "/" + MYGRID_MANAGER;
	}


	/**
	 * This method is used like a common point to get the address of the
	 * MYGRID_FAILURE_DETECTOR.
	 * 
	 * @return the RMI URL that points to the <code>MyGridFailureDetector</code>
	 */
	public static String failureDetector() {

		return "rmi://" + getName() + ":" + getPort() + "/" + AbstractFailureDetector.BOUND_NAME;
	}

}
