/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.url;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.PeerConfiguration;
import org.ourgrid.common.fd.AbstractFailureDetector;

public class PeerURLProvider extends URLProvider {

	public static final String LOCAL_ACCESS = "LOCAL_ACCESS";

	public static final String REMOTE_ACCESS = "REMOTE_ACCESS";

	public static final String PEER_UI_MANAGER = "PEER_UI_MANAGER";


	// Exported Objects
	public static String localAccess() {

		return "rmi://" + getName() + ":" + getPort() + "/" + LOCAL_ACCESS;
	}


	public static String remoteAccess() {

		return "rmi://" + getName() + ":" + getPort() + "/" + REMOTE_ACCESS;
	}


	public static String peerUIManager() {

		return "rmi://" + getName() + ":" + getPort() + "/" + PEER_UI_MANAGER;
	}


	public static String peerFailureDetector() {

		return "rmi://" + getName() + ":" + getPort() + "/" + AbstractFailureDetector.BOUND_NAME;
	}


	public static String publishedObjects() {

		return "rmi://" + getName() + ":" + getPort();
	}


	// Auxiliary Methods
	public static String peerExternalURL() {

		return "rmi://" + getExternalName() + ":" + getPort() + "/" + REMOTE_ACCESS;
	}


	public static String doctor() {

		return "rmi://" + getName() + ":" + getPort() + "/DOCTOR";
	}


	public static String reportManager() {

		return "rmi://" + getName() + ":" + getPort() + "/REPORT_MANAGER";
	}


	public static String guiListener() {

		return "rmi://" + getName() + ":" + getPort() + "/GUI_LISTENER";
	}


	public static String corePeer() {

		return corePeerDefault() + "/CorePeer";
	}


	public static String corePeerFailureDetector() {

		return corePeerDefault() + "/" + AbstractFailureDetector.BOUND_NAME;
	}


	private static String corePeerDefault() {

		return "rmi://" + getProperty( PeerConfiguration.PROP_COREPEER_NAME ) + ":"
				+ getProperty( PeerConfiguration.PROP_COREPEER_PORT );
	}


	private static String getProperty( String propertyName ) {

		return Configuration.getInstance().getProperty( propertyName );
	}


	public static String statusProvider() {

		return "rmi://" + getName() + ":" + getPort() + "/STATUS_PROVIDER";
	}

}
