/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;

public class ArrayUtilTest extends TestCase implements CommonTest {

	/**
	 * Constructor for ArrayUtilTest.
	 * 
	 * @param arg0
	 */
	public ArrayUtilTest( String arg0 ) {

		super( arg0 );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	public void testFind() {

		// String test
		String[ ] numbersStr = { "1", "2", "3", "4", "5" };
		assertTrue( ArrayUtil.find( "1", numbersStr ) == 0 );
		assertTrue( ArrayUtil.find( "5", numbersStr ) == 4 );

		// Integer test
		Integer one = new Integer( 1 );
		Integer two = new Integer( 2 );
		Integer three = new Integer( 3 );
		Integer[ ] numbers = { one, two, three };

		assertTrue( ArrayUtil.find( one, numbers ) == 0 );
		assertTrue( ArrayUtil.find( three, numbers ) == 2 );
		assertTrue( ArrayUtil.find( two, numbers ) == 1 );

	}


	public void testRemoveFirst() {

		// String test
		String[ ] numbersStr = { "1", "2", "3", "4", "5" };

		int lengthBefore = numbersStr.length;

		String[ ] afterNumbersStr = new String[ numbersStr.length - 1 ];
		ArrayUtil.removeFirst( numbersStr, afterNumbersStr );

		int lengthAfter = afterNumbersStr.length;

		assertTrue( lengthBefore == (lengthAfter + 1) );

		for ( int i = 0; i < afterNumbersStr.length; i++ ) {
			if ( !afterNumbersStr[i].equals( numbersStr[i + 1] ) ) {
				fail();
			}
		}

		// Integer test
		Integer[ ] numbers = { new Integer( 1 ), new Integer( 2 ), new Integer( 3 ), new Integer( 4 ), new Integer( 5 ) };

		lengthBefore = numbers.length;

		Integer[ ] afterNumbers = new Integer[ numbers.length - 1 ];
		ArrayUtil.removeFirst( numbers, afterNumbers );

		lengthAfter = afterNumbers.length;

		assertTrue( lengthBefore == (lengthAfter + 1) );

		for ( int i = 0; i < afterNumbers.length; i++ ) {
			if ( !afterNumbers[i].equals( numbers[i + 1] ) ) {
				fail();
			}
		}

	}


	public void testInvert() {

		// testing odd number of elements

		String[ ] numbers = { "1", "2", "3", "4", "5" };
		String[ ] reverseNumbers = { "5", "4", "3", "2", "1" };

		int lengthBefore = numbers.length;

		ArrayUtil.invert( numbers );

		int lengthAfter = numbers.length;

		assertTrue( lengthBefore == lengthAfter );

		for ( int i = 0; i < numbers.length; i++ ) {
			if ( !numbers[i].equals( reverseNumbers[i] ) ) {
				fail( numbers[i] + " != " + reverseNumbers[i] );
			}
		}

		// testing even number of elements

		String[ ] numbersE = { "1", "2", "3", "4", "5", "6" };
		String[ ] reverseNumbersE = { "6", "5", "4", "3", "2", "1" };

		lengthBefore = numbersE.length;

		ArrayUtil.invert( numbersE );

		lengthAfter = numbersE.length;

		assertTrue( lengthBefore == lengthAfter );

		for ( int i = 0; i < numbersE.length; i++ ) {
			if ( !numbersE[i].equals( reverseNumbersE[i] ) ) {
				fail( numbersE[i] + " != " + reverseNumbersE[i] );
			}
		}
	}


	public void testConcat() {

		// String test
		String[ ] firstName = { "J", "O", "H", "N" };
		String[ ] lastName = { "D", "O", "E" };
		String[ ] completeName = { "J", "O", "H", "N", "D", "O", "E" };

		String[ ] obtainedName = new String[ completeName.length ];

		ArrayUtil.concat( firstName, lastName, obtainedName );

		for ( int i = 0; i < completeName.length; i++ ) {
			if ( !completeName[i].equals( obtainedName[i] ) ) {
				fail( completeName[i] + " != " + obtainedName[i] );
			}
		}

		// Integer test
		Integer[ ] initNumbers = { new Integer( 1 ), new Integer( 2 ), new Integer( 3 ), new Integer( 4 ),
									new Integer( 5 ), new Integer( 6 ) };
		Integer[ ] endNumbers = { new Integer( 7 ), new Integer( 8 ), new Integer( 9 ), new Integer( 10 ) };
		Integer[ ] mergedNumbers = { new Integer( 1 ), new Integer( 2 ), new Integer( 3 ), new Integer( 4 ),
									new Integer( 5 ), new Integer( 6 ), new Integer( 7 ), new Integer( 8 ),
									new Integer( 9 ), new Integer( 10 ) };

		Integer[ ] newMergedNumbers = new Integer[ mergedNumbers.length ];

		ArrayUtil.concat( initNumbers, endNumbers, newMergedNumbers );

		for ( int i = 0; i < mergedNumbers.length; i++ ) {
			if ( !mergedNumbers[i].equals( newMergedNumbers[i] ) ) {
				fail( mergedNumbers[i].intValue() + " != " + newMergedNumbers[i].intValue() );
			}
		}

	}

}
