/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.util;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;

public class BidimensionalArrayTest extends TestCase implements CommonTest {

	private final String string1 = "TEST_1";

	private final String string2 = "TEST_2";

	private BidimensionalArray<String> bidimensionalArray;


	protected void setUp() throws Exception {

		super.setUp();
		bidimensionalArray = new BidimensionalArray<String>( 10 );
	}


	protected void tearDown() throws Exception {

		super.tearDown();
	}


	public void testPutAndGet() {

		assertTrue( bidimensionalArray.insert( 0, 0, string1 ) );
		assertEquals( string1, bidimensionalArray.get( 0, 0 ) );
		assertFalse( bidimensionalArray.insert( 0, 0, string1 ) );

		assertTrue( bidimensionalArray.insert( 1, 4, string2 ) );
		assertEquals( string2, bidimensionalArray.get( 1, 4 ) );

		assertEquals( null, bidimensionalArray.get( 4, 1 ) );

		bidimensionalArray.replace( 0, 0, string2 );
		assertEquals( string2, bidimensionalArray.get( 0, 0 ) );

	}


	public void testResize() {

		assertTrue( bidimensionalArray.insert( 10, 10, string1 ) );
		assertEquals( string1, bidimensionalArray.get( 10, 10 ) );
		assertFalse( bidimensionalArray.insert( 10, 10, string1 ) );

		assertTrue( bidimensionalArray.insert( 11, 14, string2 ) );
		assertEquals( string2, bidimensionalArray.get( 11, 14 ) );

		assertEquals( null, bidimensionalArray.get( 14, 11 ) );

		bidimensionalArray.replace( 10, 10, string2 );
		assertEquals( string2, bidimensionalArray.get( 10, 10 ) );

	}


	public void testInsertAtColumn() {

		bidimensionalArray.insertAtColumn( 1, "test1" );
		bidimensionalArray.insertAtColumn( 1, "test2" );
		bidimensionalArray.insertAtColumn( 1, "test3" );
		bidimensionalArray.insertAtColumn( 1, "test4" );

		assertEquals( "test1", bidimensionalArray.get( 0, 1 ) );
		assertEquals( "test2", bidimensionalArray.get( 1, 1 ) );
		assertEquals( "test3", bidimensionalArray.get( 2, 1 ) );
		assertEquals( "test4", bidimensionalArray.get( 3, 1 ) );

	}


	public void testInsertAtRow() {

		bidimensionalArray.insertAtRow( 1, "test1" );
		bidimensionalArray.insertAtRow( 1, "test2" );
		bidimensionalArray.insertAtRow( 1, "test3" );
		bidimensionalArray.insertAtRow( 1, "test4" );

		assertEquals( "test1", bidimensionalArray.get( 1, 0 ) );
		assertEquals( "test2", bidimensionalArray.get( 1, 1 ) );
		assertEquals( "test3", bidimensionalArray.get( 1, 2 ) );
		assertEquals( "test4", bidimensionalArray.get( 1, 3 ) );

	}

}
