/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;
import org.ourgrid.test.TestUtils;

/**
 * This class is used to test the JavaFileUtil class.
 */
public class JavaFileUtilTest extends TestCase implements CommonTest {

	/* Files used to test the digest Representations */
	private File file1, file2, file3;

	private TestUtils testUtils;


	/*
	 * Constructor
	 */
	public JavaFileUtilTest( String name ) {

		super( name );
	}


	/*
	 * Initialization of the test
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		testUtils = new TestUtils();
		testUtils.load();
		file1 = TempFileManager.createTempFileWithBogusData( "mygrid-test-", ".tmp", new File( testUtils
				.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ), 128 );
		file2 = TempFileManager.createTempFileWithBogusData( "mygrid-test-", ".tmp", new File( testUtils
				.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ), 64 );
		file3 = file1;
	}


	/*
	 * Finishes the test
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		file1 = null;
		file2 = null;
		file3 = null;
		// System.gc();
	}


	/* main */
	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( JavaFileUtilTest.class );
	}


	/* Tests the method getDigestRepresentation() */
	public void testGetDigestRepresentation() {

		try {

			assertTrue( "Two File objects referencing the same file must return the same digest representation",
					JavaFileUtil.getDigestRepresentation( file1 )
							.equals( JavaFileUtil.getDigestRepresentation( file3 ) ) );

			System.out.println( JavaFileUtil.getDigestRepresentation( file1 ) );

			assertTrue(
					"Two File objects referencing differents files must return the differents digest representations",
					!JavaFileUtil.getDigestRepresentation( file1 )
							.equals( JavaFileUtil.getDigestRepresentation( file2 ) ) );

			System.out.println( JavaFileUtil.getDigestRepresentation( file2 ) );

		} catch ( Exception exception ) {
			exception.printStackTrace();
			fail();
		}

	}


	public void testDeleteFile() throws IOException {

		File dir1 = new File( System.getProperty( "java.io.tmpdir" ) + File.separator + "deletethisdirecotry"
				+ (int) (Math.random() * Integer.MAX_VALUE) );
		assertFalse( dir1.exists() );
		dir1.mkdirs();
		assertTrue( dir1.exists() );
		assertTrue( dir1.isDirectory() );

		File dir2 = new File( dir1.getAbsolutePath() + File.separator + "bogusdir" );
		assertFalse( dir2.exists() );
		dir2.mkdir();
		assertTrue( dir2.exists() );
		assertTrue( dir2.isDirectory() );

		File[ ] bogusFiles = new File[ 10 ];
		for ( int i = 0; i < bogusFiles.length; i++ ) {
			bogusFiles[i] = TempFileManager.createTempFileWithBogusData( "mygrid-test-", ".tmp", dir2, 128 );
			assertTrue( bogusFiles[i].exists() );
		}

		JavaFileUtil.deleteDir( dir1 );

		for ( File file : bogusFiles ) {
			assertFalse( file.exists() );
		}

		assertFalse( dir2.exists() );
		assertFalse( dir1.exists() );
	}

}
