/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.util;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Similar to <code>StringTokenizer</code>, but not brokes the tokens using
 * the internal deliimiters characters only uses the full delimiter.
 */

public class OGStringTokenizer {

	private Iterator<String> tokenIterator;

	private int countTokens;


	/**
	 * Creates an new OGTokenizer
	 * 
	 * @param string The string to tokenize
	 * @param delim The delimiter
	 */
	public OGStringTokenizer( String string, String delim ) {

		this.countTokens = 0;
		processTokenization( string, delim );
	}


	/**
	 * I always would like to create a method with this name !!!
	 */
	private void processTokenization( String string, String delim ) {

		List<String> tokenList = new LinkedList<String>();
		int index = 0;
		int fromIndex = 0;
		String tokenCandidate;

		index = string.indexOf( delim, fromIndex );
		while ( index != -1 && (fromIndex <= string.length()) ) {
			tokenCandidate = string.substring( fromIndex, index );
			// cases where fromindex == index
			if ( !tokenCandidate.equals( "" ) ) {
				tokenList.add( tokenCandidate );
			}
			fromIndex += tokenCandidate.length() + delim.length();
			index = string.indexOf( delim, fromIndex );
		}

		tokenCandidate = string.substring( fromIndex, string.length() );
		// cases where fromindex == index
		if ( !tokenCandidate.equals( "" ) ) {
			tokenList.add( tokenCandidate );
		}

		this.countTokens = tokenList.size();
		// empty string
		if ( countTokens() == 0 ) {
			tokenList.add( "" );
			this.countTokens = 1;
		}

		this.tokenIterator = tokenList.iterator();
	}


	private Iterator<String> getTokenIterator() {

		return this.tokenIterator;
	}


	/**
	 * Verifies if has more tokens in tokens list.
	 * 
	 * @return true if hasmore tokens in the tokens list, false otherwise
	 */
	public boolean hasMoreTokens() {

		return getTokenIterator().hasNext();
	}


	/**
	 * The number of tokens in the token list.
	 * 
	 * @return The number of tokens in the token list.
	 */
	public int countTokens() {

		return this.countTokens;
	}


	/**
	 * Consumes the first token of token list.
	 * 
	 * @return The first token of token list.
	 */
	public String nextToken() {

		String returned = getTokenIterator().next();
		this.countTokens--;
		return returned;
	}

}
