/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.common.util;

import java.util.StringTokenizer;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;

public class OGStringTokenizerTest extends TestCase implements CommonTest {

	private OGStringTokenizer tokenizer;

	private String string, delim;


	public void testTokenization() {

		string = "NOT np == 5";
		delim = "AND";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( string, tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );

		// Only to show !!!!! See it !!!!
		StringTokenizer badTokenizer = new StringTokenizer( string, delim );
		assertTrue( badTokenizer.hasMoreTokens() );
		assertEquals( 1, badTokenizer.countTokens() );
		assertEquals( "OT np == 5", badTokenizer.nextToken() ); // Look the
		// difference
		assertEquals( 0, badTokenizer.countTokens() );
	}


	public void testTokenization1() {

		string = "NOT np == 5";
		delim = "NOT ";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( "np == 5", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}


	public void testTokenization2() {

		string = "NOT np == 5";
		delim = "NOT";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( " np == 5", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}


	public void testTokenization3() {

		string = "mem == 5 AND np == 5";
		delim = "AND";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 2, tokenizer.countTokens() );
		assertEquals( "mem == 5 ", tokenizer.nextToken() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( " np == 5", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}


	public void testTokenization4() {

		string = "abcd";
		delim = "d";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( "abc", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}


	public void testTokenization5() {

		string = "a < b AND b > c AND d > e";
		delim = " AND ";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 3, tokenizer.countTokens() );
		assertEquals( "a < b", tokenizer.nextToken() );
		assertEquals( 2, tokenizer.countTokens() );
		assertEquals( "b > c", tokenizer.nextToken() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( "d > e", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}


	public void testTokenization6() {

		string = "";
		delim = "d";
		tokenizer = new OGStringTokenizer( string, delim );
		assertTrue( tokenizer.hasMoreTokens() );
		assertEquals( 1, tokenizer.countTokens() );
		assertEquals( "", tokenizer.nextToken() );
		assertEquals( 0, tokenizer.countTokens() );
	}

}
