/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;
import org.ourgrid.test.TestUtils;

/**
 * This TestCase is supposed to test SimpleZip class functionalities.
 */

public class SimpleZipTest extends TestCase implements CommonTest {

	private String tempDir;


	public void setUp() {

		TestUtils testUtils = new TestUtils();
		testUtils.load();
		tempDir = testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR );
	}


	/** tests zip and unzip functionalities */
	public void testZipItUnzipIt() throws IOException {

		/** tests zipping */
		String[ ] files = { tempDir + File.separator + "dummyFile1", tempDir + File.separator + "dummyFile2" };
		createFiles( files );

		String zipFileName = tempDir + File.separator + "zip1.zip";
		SimpleZip.zipIt( files, zipFileName );

		File zipFile = new File( zipFileName );
		assertTrue( zipFile.exists() );

		for ( int k = 0; k < files.length; k++ ) {
			File file = new File( files[k] );
			file.delete();

			assertFalse( file.exists() );
		}

		/** tests unzipping */
		SimpleZip.unzipIt( zipFile, tempDir );

		for ( int k = 0; k < files.length; k++ ) {
			File fileUnzipped = new File( files[k] );
			assertTrue( fileUnzipped.exists() );

			fileUnzipped.delete();
			assertFalse( fileUnzipped.exists() );
		}

		zipFile.delete();
		assertFalse( zipFile.exists() );
	}


	/** creates some files to use in this test */
	private void createFiles( String[ ] filesNames ) throws IOException {

		for ( int k = 0; k < filesNames.length; k++ ) {
			BufferedWriter bw = new BufferedWriter( new FileWriter( filesNames[k] ) );
			bw.write( filesNames[k] );
			bw.write( filesNames[k] );
			bw.write( filesNames[k] );
			bw.write( filesNames[k] );
			bw.close();

			assertTrue( new File( filesNames[k] ).exists() );
		}
	}
}
