/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * This is a auxiliar class that incloses many util functionalities related to
 * strings.
 */
public class StringUtil {

	/**
	 * Writes the given string into the given file.
	 * 
	 * @param file The file where the string have to be written (appended).
	 * @param s the string to be written into the file
	 * @throws IOException If the file could not be used (created) or any other
	 *         I/O problem at the open, writte or close moment.
	 */
	public static void writeStringOnFile( String file, String s ) throws IOException {

		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream( file, true );
			fos.write( s.getBytes() );
		} finally {
			if ( fos != null ) {
				fos.close();
			}
		}
	}


	/**
	 * Replaces at the a String a patter to another.
	 * 
	 * @param str The String where will be made the changes.
	 * @param subOld The pattern that will be removed.
	 * @param subNew The pattern that will be inserted.
	 * @return the new string with the changes made if any was necessary, or the
	 *         same string if any changes was needed.
	 */
	public static String replace( String str, String subOld, String subNew ) {

		int i = str.indexOf( subOld );

		if ( i == -1 ) {
			return str;
		}
		StringBuffer stb = new StringBuffer( str );
		stb.replace( i, i + subOld.length(), subNew );
		return replace( stb.toString(), subOld, subNew );
	}


	/**
	 * Receives an array of String and returns a simple string of all separeted
	 * by a ","
	 */
	public static String passToString( String[ ] orig ) {

		String attrib = new String();

		if ( orig.length != 0 ) {
			attrib = orig[0];
			for ( int i = 1; i < orig.length; i++ ) {
				attrib += ", " + orig[i];
			}
		} else
			attrib = "";

		return attrib;
	}


	/**
	 * Receives a simple String and returns a array containing all words of the
	 * String supressed the "," - it is the delimiter
	 * 
	 * @param strToArray string that must be transformed to an array.
	 * @return an array of strings based on the string parameter.
	 */
	public static String[ ] passToArrayStr( String strToArray ) {

		String[ ] returnArray = {};

		if ( strToArray == null ) {
			return returnArray;
		}

		StringTokenizer noComma = new StringTokenizer( strToArray, "," );
		returnArray = new String[ noComma.countTokens() ];

		for ( int i = 0; noComma.hasMoreTokens(); returnArray[i++] = noComma.nextToken().trim() ) {
			// Trim over all tokens
		}
		return returnArray;
	}


	/**
	 * Inserts a white space at a specific point.
	 * 
	 * @param expr the string where the space have to be inserted.
	 * @param i the specific point where the space have to be inserted
	 * @return the changed string if the position is valid or the same string
	 *         otherwise.
	 */
	public static String insertSpace( String expr, int i ) {

		StringBuffer str = new StringBuffer( expr );
		try {
			str.insert( i, " " );
		} catch ( StringIndexOutOfBoundsException sobex ) {
			// do nothing, returns the string like it was received.
		}
		return str.toString();
	}


	/**
	 * This method fills value with the characters at right.
	 * 
	 * @param value - The string to be filled with white space caracters
	 * @param character - The character to be used in the filling of the string
	 * @param numChars - The number of characters to insert in the string.
	 */
	public static String fillWithChars( String value, char character, int numChars ) {

		StringBuffer formatted = new StringBuffer();

		formatted.append( value );

		for ( int i = 0; i < numChars; i++ ) {
			formatted.append( character );
		}

		return formatted.toString();
	}


	/**
	 * Insert spaces in the sentence expr. This is necessary in order to
	 * StringTokenizer can function properlly when matching sentence of the
	 * task.
	 * 
	 * @param expr The sentence that spaces will be inserted
	 * @return The modified sentence (with spaces)
	 */
	public static String insertSpaces( String expr ) {

		if ( expr != null ) {

			for ( int i = 0; i < expr.length(); i++ ) {
				if ( (expr.charAt( i ) == '(') && (i != (expr.length() - 1)) && (expr.charAt( i + 1 ) != ' ') ) {
					expr = insertSpace( expr, i + 1 );
					i++;
				} else if ( (expr.charAt( i ) == ')') && (i != 0) && (expr.charAt( i - 1 ) != ' ') ) {
					expr = insertSpace( expr, i );
					i++;
				} else if ( (expr.charAt( i ) == '&') || (expr.charAt( i ) == '|') ) { // Symbol
					// can
					// be
					// &&
					// or
					// ||
					int skip = 1;
					boolean changedAhead = false;
					if ( (i != (expr.length() - 2)) && (expr.charAt( i + 2 ) != ' ') ) {
						expr = insertSpace( expr, i + 2 );
						skip++;
						changedAhead = true;
					}
					if ( (i != 0) && (expr.charAt( i - 1 ) != ' ') && changedAhead ) {
						expr = insertSpace( expr, i );
						skip++;
					} else if ( (i != 0) && (expr.charAt( i - 1 ) != ' ') ) {
						expr = insertSpace( expr, i );
						skip++;
					}
					i += skip;
				} else if ( (expr.charAt( i ) == '=') || (expr.charAt( i ) == '<') || (expr.charAt( i ) == '>')
						|| (expr.charAt( i ) == '!') ) {
					int skip = 1;
					if ( expr.charAt( i + 1 ) == '=' ) { // Symbol can be ==
															// or >=
						// or <=
						if ( (i != (expr.length() - 2)) && (expr.charAt( i + 2 ) != ' ') ) {
							expr = insertSpace( expr, i + 2 );
							skip++;
						}
						if ( (i != 0) && (expr.charAt( i - 1 ) != ' ') ) {
							expr = insertSpace( expr, i );
							skip++;
						}
					} else { // Symbol can be only < or >
						if ( i != (expr.length() - 1) && (expr.charAt( i + 1 ) != ' ') ) {
							expr = insertSpace( expr, i + 1 );
						}
						if ( (i != 0) && (expr.charAt( i - 1 ) != ' ') ) {
							expr = insertSpace( expr, i );
							skip++;
						}
					}
					i += skip;
				}
			}
		}
		return expr;
	}


	/**
	 * This method does a job that is expected to be done by the shell: it
	 * converts variables to String literals.
	 * 
	 * @param stringWithVariables A string with 0 or more variables.
	 * @return A String without any variables.
	 */
	public static String replaceVariables( String stringWithVariables, Map<String,String> envVars ) {

		String stringWithoutVariables = stringWithVariables;

		if ( envVars != null ) {

			Iterator<String> it = envVars.keySet().iterator();

			while ( it.hasNext() ) {
				String key = it.next();
				stringWithoutVariables = StringUtil.replace( stringWithoutVariables, "$" + key, envVars.get( key ) );
			}

		}

		return stringWithoutVariables;

	}


	/**
	 * This method parse a permissions file from stringo to int
	 * 
	 * @param permStr The file permission as drwxrwxrwx
	 * @return int
	 */
	public static int parsePermissionString( String permStr ) {

		int perm_o = 0;
		int perm_g = 0;
		int perm_u = 0;

		if ( permStr.charAt( 0 ) == 'r' )
			perm_o = perm_o + 4;
		if ( permStr.charAt( 1 ) == 'w' )
			perm_o = perm_o + 2;
		if ( permStr.charAt( 2 ) == 'x' )
			perm_o = perm_o + 1;

		if ( permStr.charAt( 3 ) == 'r' )
			perm_g = perm_g + 4;
		if ( permStr.charAt( 4 ) == 'w' )
			perm_g = perm_g + 2;
		if ( permStr.charAt( 5 ) == 'x' )
			perm_g = perm_g + 1;

		if ( permStr.charAt( 6 ) == 'r' )
			perm_u = perm_u + 4;
		if ( permStr.charAt( 7 ) == 'w' )
			perm_u = perm_u + 2;
		if ( permStr.charAt( 8 ) == 'x' )
			perm_u = perm_u + 1;

		return (perm_o * 100) + (perm_g * 10) + (perm_u);
	}


	/**
	 * Gets a array of string and return a string that contains the strings
	 * separated by commas
	 * 
	 * @param strings The array of strings
	 * @return A string with strings separated by commas
	 */
	public static String getStringSeparatedByCommas( String[ ] strings ) {

		StringBuffer strAttributes = new StringBuffer();
		if ( strings == null ) {
			return "";
		}
		for ( int indexStrings = 0; indexStrings < strings.length; indexStrings++ ) {
			strAttributes.append( strings[indexStrings] );
			strAttributes.append( "," );
		}
		if ( strAttributes.length() == 0 ) {
			return "";
		}
		return strAttributes.substring( 0, strAttributes.length() - 1 );
	}


	/**
	 * Gets a array of strings after broken with the given delimitator
	 * 
	 * @param string A string that will broken in an array of strings
	 * @param delimitator The delimitator between strings
	 * @return An array with the strings of "string" parameter
	 */
	public static String[ ] getArrayOfStrings( String string, String delimitator ) {

		StringTokenizer strToken = new StringTokenizer( string, delimitator );
		String[ ] strings = new String[ strToken.countTokens() ];
		int indexStrings = 0;
		while ( strToken.hasMoreTokens() ) {
			strings[indexStrings] = (String) strToken.nextElement();
			indexStrings++;
		}
		return strings;
	}

}
