/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import junit.framework.TestCase;

import org.ourgrid.test.CommonTest;
import org.ourgrid.test.TestUtils;

public class StringUtilTest extends TestCase implements CommonTest {

	public static final String TEST_ALL_TEST_TYPE = "UNIT";

	private String FILE = null;

	private TestUtils testUtils;


	public StringUtilTest( String s ) {

		super( s );
	}


	protected void setUp() throws Exception {

		testUtils = new TestUtils();
		testUtils.load();
		File tmpFile = File.createTempFile( "mygrid-test-", ".tmp", new File( testUtils
				.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ) );
		FILE = tmpFile.getAbsolutePath();
		tmpFile.delete();
	}


	protected void tearDown() throws Exception {

		File removedFile = new File( FILE );
		removedFile.delete();
	}


	public void testReplace() {

		String wrong = "MyGrid is the worst program of the world";
		String right = "MyGrid is the almost best program of the world";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "worst", "almost best" ), right );

		wrong = "Anything final something";
		right = "Anything words something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "words" ), right );

		wrong = "Anything final something";
		right = "Anything long end something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "long end" ), right );

		wrong = "Anything long long something";
		right = "Anything short something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "long long", "short" ), right );

		wrong = "Anything final";
		right = "Anything words";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "words" ), right );

		wrong = "Anything final";
		right = "Anything long end";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "long end" ), right );

		wrong = "Anything long long";
		right = "Anything short";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "long long", "short" ), right );

		wrong = "final something";
		right = "words something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "words" ), right );

		wrong = "final something";
		right = "long end something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "final", "long end" ), right );

		wrong = "long long something";
		right = "short something";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "long long", "short" ), right );

		wrong = "scp robalo.lsd.ufpb.br:%remotedir/%file .";
		right = "scp robalo.lsd.ufpb.br:/tmp/%file .";
		assertEquals( "the method must replace the words and produce the expected right form", StringUtil.replace(
				wrong, "%remotedir", "/tmp" ), right );
	}


	public void testParsePermissionString() {

		String permissions = "rwxrwxrwx";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 777 );

		permissions = "rwxrw-r--";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 764 );

		permissions = "--x-wxrwx";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 137 );

		permissions = "r--r--r--";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 444 );

		permissions = "-w--w--w-";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 222 );

		permissions = "--x--x--x";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 111 );

		permissions = "---r-x-wx";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 53 );
		assertTrue( "can't return left side zeros", StringUtil.parsePermissionString( permissions ) != 053 );

		permissions = "---------";
		assertEquals( "the method must recognize the permissions given and return number representing it", StringUtil
				.parsePermissionString( permissions ), 000 );
	}


	public void testWriteStringOnFile() throws Exception {

		File test = null;

		FileReader fReader = null;
		BufferedReader input = null;
		try {
			StringUtil.writeStringOnFile( FILE, "written line" );
			test = new File( FILE );
			assertTrue( "the file must exists", test.exists() );
			fReader = new FileReader( FILE );
			input = new BufferedReader( fReader );
			// InputStream input = new FileInputStream(FILE);
			// DataInputStream dis = new DataInputStream(input);
			assertEquals( "the message must be written correctly", "written line", input.readLine() );
		} catch ( Exception ex ) {
			throw ex;
		} finally {
			test.delete();
			input.close();
			fReader.close();
		}
	}


	public void testPassToString() throws Exception {

		String[ ] list = { "one", "two", "three", "four" };
		String concated = StringUtil.passToString( list );
		assertNotNull( "the method must return something", concated );
		assertEquals( "we must have a list in which the elements are separated by commas", "one, two, three, four",
				concated );
	}


	public void testPassToStringOneElement() throws Exception {

		String[ ] list = { "only this" };
		String concated = StringUtil.passToString( list );
		assertNotNull( "the method must return something", concated );
		assertEquals( "we must have a list in which the elements are separated by commas", "only this", concated );
	}


	public void testPassToStringNoneElement() throws Exception {

		String[ ] list = {};
		String concated = StringUtil.passToString( list );
		assertNotNull( "the method must return something", concated );
		assertEquals( "we must have a list in which the elements are separated by commas", "", concated );
	}


	public void testFillWithChars() throws Exception {

		String initial = "abcde";
		String result = StringUtil.fillWithChars( initial, 'f', 5 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have 5 \"f\" in the initial string's end", "abcdefffff", result );
	}


	public void testFillWithOneChar() throws Exception {

		String initial = "abcde";
		String result = StringUtil.fillWithChars( initial, 'f', 1 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have 1 \"f\" in the initial string's end", "abcdef", result );
	}


	public void testFillWithNoneChar() throws Exception {

		String initial = "abcde";
		String result = StringUtil.fillWithChars( initial, 'f', 0 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "abcde", result );
	}


	public void testFillWithNegativesChars() throws Exception {

		String initial = "abcde";
		String result = StringUtil.fillWithChars( initial, 'f', -5 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "abcde", result );
	}


	public void testFillNothingWithChars() throws Exception {

		String initial = "";
		String result = StringUtil.fillWithChars( initial, 'f', 5 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have 5 \"f\" at the initial string's final", "fffff", result );
	}


	public void testFillNothingWithOneChar() throws Exception {

		String initial = "";
		String result = StringUtil.fillWithChars( initial, 'f', 1 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have 1 \"f\" at the initial string's final", "f", result );
	}


	public void testFillNothingWithNoneChar() throws Exception {

		String initial = "";
		String result = StringUtil.fillWithChars( initial, 'f', 0 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "", result );
	}


	public void testFillNothingWithNegativesChars() throws Exception {

		String initial = "";
		String result = StringUtil.fillWithChars( initial, 'f', -5 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "", result );
	}


	public void testInsertSpace() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 3 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have a blank space after the third char", "som eWord", result );
	}


	public void testInsertSpaceAtAlmostBegin() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 1 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have a blank space after the first char", "s omeWord", result );
	}


	public void testInsertSpaceAtBegin() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 0 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have a blank space before the first char", " someWord", result );
	}


	public void testInsertSpaceBeforeBegin() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, -4 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "someWord", result );
	}


	public void testInsertSpaceAtAlmostEnd() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 7 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have a blank space before the last char", "someWor d", result );
	}


	public void testInsertSpaceAtEnd() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 8 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have a blank space after the last char", "someWord ", result );
	}


	public void testInsertSpaceAfterEnd() throws Exception {

		String initial = "someWord";
		String result = StringUtil.insertSpace( initial, 34 );
		assertNotNull( "the method must return something", result );
		assertEquals( "we must have the same string as the beginning", "someWord", result );
	}


	public void testNotInsertSpaces() throws Exception {

		String expression = "linux";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "we must have the same string as the beginning", "linux", expressionWithSpaces );
	}


	public void testInsertSpacesParenthesis() throws Exception {

		String expression = "(linux)";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis separated by blank spaces", "( linux )",
				expressionWithSpaces );
	}


	public void testInsertSpacesInvertedParenthesis() throws Exception {

		String expression = ")linux(";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "we must have the same string as the beginning", ")linux(", expressionWithSpaces );
	}


	public void testInsertSpacesAnd() throws Exception {

		String expression = "linux&&mandrake";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with logic operators separated by blank spaces", "linux && mandrake",
				expressionWithSpaces );
	}


	public void testInsertSpacesOr() throws Exception {

		String expression = "linux||mandrake";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with logic operators separated by blank spaces", "linux || mandrake",
				expressionWithSpaces );
	}


	public void testInsertSpacesMixOr() throws Exception {

		String expression = "(linux)||(mandrake)";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and logic operators separated by blank spaces",
				"( linux ) || ( mandrake )", expressionWithSpaces );
	}


	public void testInsertSpacesMixAnd() throws Exception {

		String expression = "(linux)&&(mandrake)";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and logic operators separated by blank spaces",
				"( linux ) && ( mandrake )", expressionWithSpaces );
	}


	public void testInsertSpacesMixNot() throws Exception {

		String expression = "(linux)&&!(mandrake)";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and logic operators separated by blank spaces",
				"( linux ) && ! ( mandrake )", expressionWithSpaces );
	}


	public void testInsertSpaces() throws Exception {

		String expression = "(linux||windows&&(office||staroffice))";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and logic operators separated by blank spaces",
				"( linux || windows && ( office || staroffice ) )", expressionWithSpaces );
	}


	public void testInsertSpaces2() throws Exception {

		String expression = "( ( lsd || lcc ) && linux )";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and logic operators separated by blank spaces",
				"( ( lsd || lcc ) && linux )", expressionWithSpaces );
	}


	/**
	 * Including tests for new symbols to be recognized at this method: "==",
	 * ">", "<", ">=", "<="
	 */
	public void testInsertSpaces3() {

		String expression = "(os==linux)";
		String expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and operators separated by blank spaces",
				"( os == linux )", expressionWithSpaces );

		expression = "os==linux&&ram<=400";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with operators separated by blank spaces", "os == linux && ram <= 400",
				expressionWithSpaces );

		expression = "ram<400";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with operators separated by blank spaces", "ram < 400", expressionWithSpaces );

		expression = "(os==windows&&(site==lsd||ram>300))";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and operators separated by blank spaces",
				"( os == windows && ( site == lsd || ram > 300 ) )", expressionWithSpaces );

		expression = "(site==lsd &&ram>= 300)";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and operators separated by blank spaces",
				"( site == lsd && ram >= 300 )", expressionWithSpaces );

		expression = "(site!=lsd)";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and operators separated by blank spaces",
				"( site != lsd )", expressionWithSpaces );

		expression = "!(site==lsd) && (bla<=5000||ble!=4000)";
		expressionWithSpaces = StringUtil.insertSpaces( expression );
		assertNotNull( "the method must return something", expressionWithSpaces );
		assertEquals( "expected expression with parenthesis and operators separated by blank spaces",
				"! ( site == lsd ) && ( bla <= 5000 || ble != 4000 )", expressionWithSpaces );

	}


	public void testPassToArrayString() throws Exception {

		String line = "one, little, indian";
		String[ ] result = StringUtil.passToArrayStr( line );
		assertNotNull( "the method must return something", result );
		assertEquals( "the array must have three elements", 3, result.length );
		assertEquals( "the first element of array must be the same as the initial string", "one", result[0] );
		assertEquals( "the second element of array must be the same as the initial string", "little", result[1] );
		assertEquals( "the third element of array must be the same as the initial string", "indian", result[2] );
	}


	public void testPassToArrayStringConfuse() throws Exception {

		String line = ",one, little, indian,";
		String[ ] result = StringUtil.passToArrayStr( line );
		assertNotNull( "the method must return something", result );
		assertEquals( "the array must have three elements", 3, result.length );
		assertEquals( "the first element of array must be the same as the initial string", "one", result[0] );
		assertEquals( "the second element of array must be the same as the initial string", "little", result[1] );
		assertEquals( "the third element of array must be the same as the initial string", "indian", result[2] );
	}


	public void testPassToArrayStringConfuseAndShort() throws Exception {

		String line = ",one,";
		String[ ] result = StringUtil.passToArrayStr( line );
		assertNotNull( "the method must return something", result );
		assertEquals( "the array must have one elements", 1, result.length );
		assertEquals( "the first element of array must be the same as the initial string", "one", result[0] );
	}


	public void testGetStringSeparatedByCommas() {

		String[ ] test = new String[ ] { "we", "are", "testing" };
		assertTrue( "we,are,testing".equals( StringUtil.getStringSeparatedByCommas( test ) ) );
		test = new String[ ] {};
		assertTrue( "".equals( StringUtil.getStringSeparatedByCommas( test ) ) );
		test = null;
		assertTrue( "".equals( StringUtil.getStringSeparatedByCommas( test ) ) );
	}


	public static void testGetArrayOfStrings() {

		String test = "I,dont,know,if,it,will work!";

		String[ ] arrayTest = new String[ ] { "I", "dont", "know", "if", "it", "will work!" };
		String[ ] arrayReturned = StringUtil.getArrayOfStrings( test, "," );
		for ( int i = 0; i < arrayReturned.length; i++ ) {
			assertTrue( arrayTest[i].equals( arrayReturned[i] ) );
		}
	}


	public void testReplaceUnexistentString() throws Exception {

		String string = "this is a simple string.... Hi mama!!";
		String newString = StringUtil.replace( string, "unexistentString", "newString" );
		assertEquals( "both string must be equals", string, newString );
	}


	public static void main( String args[] ) {

		junit.swingui.TestRunner.run( StringUtilTest.class );
	}

}
