/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.File;
import java.io.FileInputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import junit.framework.TestCase;

import org.ourgrid.common.config.UserAgentConfiguration;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.main.CommonCompiler;
import org.ourgrid.test.CommonTest;
import org.ourgrid.test.TestUtils;

/**
 * Tests TestUtils.
 */
public class TestUtilsTest extends TestCase implements CommonTest {

	private TestUtils testUtils;

	private static final String TEST_TEST_PROPERTIES1 = "test" + File.separatorChar + "resources" + File.separatorChar
			+ "test.test.properties.1";

	private static final String TEST_TEST_PROPERTIES2 = "test" + File.separatorChar + "resources" + File.separatorChar
			+ "test.test.properties.2";


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();
		this.testUtils = new TestUtils();
		testUtils.load( TEST_TEST_PROPERTIES1 );
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		this.testUtils.reset();
		this.testUtils = null;
	}


	public void testLoad() {

		assertEquals( "/tmp", testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );
		assertEquals( "1099", testUtils.getProperty( TestUtils.MG_PORT ) );
		assertEquals( "/playpenroot", testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );
		assertEquals( "{sauron.lsd.ufcg.edu.br}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".1" ) );
		assertEquals( "{frodo.lsd.ufcg.edu.br, 1099}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".2" ) );
		assertNull( testUtils.getProperty( "UNKNOWN key" ) );

		this.testUtils.load( TEST_TEST_PROPERTIES2 );
		assertEquals( "/temporary", testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );
		assertEquals( "10991", testUtils.getProperty( TestUtils.MG_PORT ) );
		assertEquals( "/tmp", testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );
		assertEquals( "{balrog.lsd.ufcg.edu.br}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".1" ) );
		assertEquals( "{gandalf.lsd.ufcg.edu.br, 1099, ssh -x $machine $command}", testUtils
				.getProperty( TestUtils.MYGUMP_MACHINE + "." + TestUtils.TYPE_UALINUX + ".2" ) );
		assertNotNull( testUtils.getProperty( "UNKNOWNkey" ) );

	}


	public void testReset() {

		assertEquals( "/tmp", testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );
		assertEquals( "1099", testUtils.getProperty( TestUtils.MG_PORT ) );
		assertEquals( "/playpenroot", testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );
		assertEquals( "{sauron.lsd.ufcg.edu.br}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".1" ) );
		assertEquals( "{frodo.lsd.ufcg.edu.br, 1099}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".2" ) );
		assertNull( testUtils.getProperty( "UNKNOWN key" ) );

		this.testUtils.reset();

		assertNull( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );
		assertNull( testUtils.getProperty( TestUtils.MG_PORT ) );
		assertNull( testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );
		assertNull( testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "." + TestUtils.TYPE_UALINUX + ".1" ) );
		assertNull( testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "." + TestUtils.TYPE_UALINUX + ".1" ) );
		assertNull( testUtils.getProperty( "UNKNOWN key" ) );

	}


	public void testSetGetProperty() {

		assertEquals( "/tmp", testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );
		this.testUtils.setProperty( TestUtils.SYSTEM_TEMP_DIR, "/temporary_directory" );
		assertEquals( "/temporary_directory", testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );

		assertEquals( "1099", testUtils.getProperty( TestUtils.MG_PORT ) );
		try {
			this.testUtils.setProperty( TestUtils.MG_PORT, null );
			fail( "An exception should be thrown!" );
		} catch ( Exception e ) {
			System.out.println( "EXPECTED EXCEPTION: " + e );
		}
		assertEquals( "1099", testUtils.getProperty( TestUtils.MG_PORT ) );

		assertEquals( "/playpenroot", testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );
		this.testUtils.setProperty( testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ), "/playpenroot" );
		assertEquals( "/playpenroot", testUtils.getProperty( TestUtils.MYGUMP_DEFAULT_PLAYPENROOT ) );

		assertEquals( "{sauron.lsd.ufcg.edu.br}", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "."
				+ TestUtils.TYPE_UALINUX + ".1" ) );
		this.testUtils.setProperty( TestUtils.MYGUMP_MACHINE + "." + TestUtils.TYPE_UALINUX + ".1", "" );
		assertEquals( "", testUtils.getProperty( TestUtils.MYGUMP_MACHINE + "." + TestUtils.TYPE_UALINUX + ".1" ) );

	}


	public void testGetGuMSpecs() throws Exception {

		assertEquals( 1, this.testUtils.getGumSpecs( 1, TestUtils.TYPE_UALINUX ).size() );
		assertEquals( 2, this.testUtils.getGumSpecs( 2, TestUtils.TYPE_UALINUX ).size() );

		List gumSpecs = this.testUtils.getGumSpecs( 2, TestUtils.TYPE_UALINUX );
		Iterator iterator = gumSpecs.iterator();
		while ( iterator.hasNext() ) {
			GumSpec guMSpec = (GumSpec) iterator.next();
			assertEquals( UserAgentConfiguration.TYPE_UA_LINUX, guMSpec.getAttribute( UserAgentConfiguration.ATT_TYPE ) );
		}

		assertEquals( 1, this.testUtils.getGumSpecs( 1, TestUtils.TYPE_UAWINDOWS ).size() );
		// assertEquals(2, this.testUtils.getGuMSpecs(2,
		// GuMSpec.TYPE_UA_WINDOWS).size());

		// gumSpecs = this.testUtils.getGuMSpecs(2, GuMSpec.TYPE_UA_WINDOWS);
		gumSpecs = this.testUtils.getGumSpecs( 1, TestUtils.TYPE_UAWINDOWS );
		iterator = gumSpecs.iterator();
		while ( iterator.hasNext() ) {
			GumSpec guMSpec = (GumSpec) iterator.next();
			assertEquals( UserAgentConfiguration.TYPE_UA_WINDOWS, guMSpec
					.getAttribute( UserAgentConfiguration.ATT_TYPE ) );
		}

		try {
			testUtils.getGumSpecs( Integer.MAX_VALUE, TestUtils.TYPE_UAWINDOWS );
			fail( "An exception should be thrown!" );
		} catch ( Exception e ) {
			System.out.println( "EXPECTED EXCEPTION: " + e );
		}

		assertEquals( 0, this.testUtils.getGumSpecs( 0, "CustomGuM" ).size() );
		assertEquals( 1, this.testUtils.getGumSpecs( 1, "CustomGuM" ).size() );
		assertEquals( 2, this.testUtils.getGumSpecs( 2, "CustomGuM" ).size() );

		gumSpecs = this.testUtils.getGumSpecs( 2, "CustomGuM" );
		iterator = gumSpecs.iterator();
		while ( iterator.hasNext() ) {
			GumSpec guMSpec = (GumSpec) iterator.next();
			assertEquals( "CustomGuM", guMSpec.getAttribute( UserAgentConfiguration.ATT_TYPE ) );
		}
	}


	public void testGetNumberOfGuMSpecsAvailable() {

		assertEquals( 0, testUtils.getNumberOfGuMSpecsAvailable( "Unknown type" ) );
		assertEquals( 2, testUtils.getNumberOfGuMSpecsAvailable( "CustomGuM" ) );
		assertEquals( 1, testUtils.getNumberOfGuMSpecsAvailable( TestUtils.TYPE_UAWINDOWS ) );
	}


	public void testCreateSDF() throws Exception {

		TestUtils testUtils2 = new TestUtils();
		testUtils2.load();

		testUtils.setProperty( TestUtils.SYSTEM_TEMP_DIR, testUtils2.getProperty( TestUtils.SYSTEM_TEMP_DIR ) );

		List<GumSpec> gumSpecs = testUtils.getGumSpecs( 1, TestUtils.TYPE_UALINUX );

		String gdfFileName = testUtils.createSDF( gumSpecs );

		CommonCompiler compiler = new CommonCompiler();
		compiler.compile( gdfFileName, CommonCompiler.SDF_TYPE );

		List<GumSpec> gumsList = compiler.getResult();

		assertEquals( 1, gumsList.size() );
		GumSpec guMSpecFromCompiler = gumsList.get( 0 );

		assertEquals( UserAgentConfiguration.TYPE_UA_LINUX, guMSpecFromCompiler
				.getAttribute( (UserAgentConfiguration.ATT_TYPE  ) ) );

		int expectedUALinux = 1;
		int expectedUAWindows = 1;

		gumSpecs = testUtils.getGumSpecs( expectedUALinux, UserAgentConfiguration.TYPE_UA_LINUX );
		gumSpecs.addAll( testUtils.getGumSpecs( expectedUAWindows, UserAgentConfiguration.TYPE_UA_WINDOWS ) );

		gdfFileName = testUtils.createSDF( gumSpecs );

		compiler = new CommonCompiler();

		compiler.compile( gdfFileName, CommonCompiler.SDF_TYPE );

		gumsList = compiler.getResult();

		assertEquals( 2, gumsList.size() );

		Iterator iterator = gumsList.iterator();

		int uaLinuxCounter = 0;
		int uaWindowsCounter = 0;

		while ( iterator.hasNext() ) {
			GumSpec guMSpec = (GumSpec) iterator.next();
			if ( UserAgentConfiguration.TYPE_UA_LINUX.equals( guMSpec.getAttribute( UserAgentConfiguration.ATT_TYPE ) ) ) {
				uaLinuxCounter++;
			} else if ( UserAgentConfiguration.TYPE_UA_WINDOWS.equals( guMSpec
					.getAttribute( UserAgentConfiguration.ATT_TYPE ) ) ) {
				uaWindowsCounter++;
			}

		}

		assertEquals( expectedUALinux, uaLinuxCounter );
		assertEquals( expectedUAWindows, uaWindowsCounter );

	}


	public void testCreate_MG_properties() throws Exception {

		File originalMGProperties = new File( testUtils.getMGPropertiesFileName() );
		String originalMGPropertiesHash = null;

		if ( originalMGProperties.exists() ) {
			originalMGPropertiesHash = JavaFileUtil.getDigestRepresentation( originalMGProperties );
		}

		testUtils.createMGProperties();

		Properties properties = new Properties();
		FileInputStream fileInputStream = new FileInputStream( testUtils.getMGPropertiesFileName() );
		properties.load( fileInputStream );

		fileInputStream.close();

		assertEquals( testUtils.getProperty( TestUtils.MG_PORT ), properties.getProperty( TestUtils.MG_PORT ) );

		testUtils.reset();

		if ( originalMGPropertiesHash != null ) {
			File newOriginalMGProperties = new File( testUtils.getMGPropertiesFileName() );
			assertEquals( originalMGPropertiesHash, JavaFileUtil.getDigestRepresentation( newOriginalMGProperties ) );
		}
	}


	public void testCreate_OG_properties() throws Exception {

		File originalOGProperties = new File( testUtils.getOGPropertiesFileName() );
		String originalOGPropertiesHash = null;

		if ( originalOGProperties.exists() ) {
			originalOGPropertiesHash = JavaFileUtil.getDigestRepresentation( originalOGProperties );
		}

		testUtils.createOGProperties();

		Properties properties = new Properties();
		FileInputStream fileInputStream = new FileInputStream( testUtils.getOGPropertiesFileName() );
		properties.load( fileInputStream );

		fileInputStream.close();

		assertEquals( testUtils.getProperty( TestUtils.OG_PORT ), properties.getProperty( TestUtils.OG_PORT ) );

		testUtils.reset();

		if ( originalOGPropertiesHash != null ) {
			File newOriginalOGProperties = new File( testUtils.getOGPropertiesFileName() );
			assertEquals( originalOGPropertiesHash, JavaFileUtil.getDigestRepresentation( newOriginalOGProperties ) );
		}
	}

}
