/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.util;

import java.io.Serializable;

/**
 * Description:This class generates data about the execution of a replica
 * 
 * @version 1.0 Created on 23/08/2004
 */
public class TimeDataGenerator implements Serializable {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;

	/**
	 * Logger to store log messages
	 */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( TimeDataGenerator.class );

	private long startTime = -1;

	private long endTime = -1;

	private String preText = "";


	/**
	 * Constructor method
	 * 
	 * @param text the name of the time collection
	 */
	public TimeDataGenerator( String text ) {

		preText = text;

	}


	/**
	 * This method sets the initial part of the caption text of this object
	 * 
	 * @param newText the name of the time collection
	 */
	public void setPreText( String newText ) {

		preText = newText;

	}


	/**
	 * This method returns the initial part of caption text of this object
	 * 
	 * @return A string representing the initial part of caption text of this
	 *         object
	 */
	public String getPreText() {

		return preText;
	}


	/**
	 * This method sets the start time of an execution
	 */
	public void setStartTime() {

		startTime = System.currentTimeMillis();
	}


	/**
	 * This method sets the end time of an execution
	 */
	public void setEndTime() {

		endTime = System.currentTimeMillis();

	}


	/**
	 * This method report the times collect
	 */
	public void report() {

		LOG.info( toString() );

	}


	/**
	 * Returns a string representation ot this object.
	 */
	@Override
	public String toString() {

		if ( startTime == -1 ) {

			return preText + " unstarted!";

		} else if ( endTime == -1 ) {

			return preText + " unfinished!";

		}

		return preText + " " + (endTime - startTime) + " ms";

	}


	public long getStartTime() {

		return startTime;

	}


	public long getEndTime() {

		return endTime;
	}


	/**
	 * This method verifies if the StartTime and EndTime are not a valid time
	 * stamp. By an invalid timestamp we mean that the replica phase were not
	 * started or interrupted during its execution.
	 * 
	 * @return <b>true</b> if StarTime and EndTime timestamps were marked.
	 *         <b>false</b> otherwise.
	 */
	public boolean isValid() {

		return ((this.startTime != -1) && (this.endTime != -1));
	}

}
