/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.common.xml;

import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.common.exception.XMLParserException;
import org.ourgrid.test.CommonTest;
import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.sun.org.apache.xerces.internal.impl.xs.opti.DefaultNode;

public class XMLUtilTest extends TestCase implements CommonTest {

	private String THIS_RELATIVE_PATH = "test" + System.getProperty( "file.separator" ) + "resources";


	public XMLUtilTest( String message ) {

		super( message );
	}


	public void testParseXMLContent() {

		String strFileTestPath = THIS_RELATIVE_PATH + System.getProperty( "file.separator" ) + "JobSchema.xsd";
		String filePath = (ClassLoader.getSystemResource( strFileTestPath )).getFile();

		String test = "<?xml version=\"1.0\" encoding=\"UTF-8\"?> <!--Sample XML file generated by XMLSPY v2004 rel. 2 U (http://www.xmlspy.com)-->"
				+ "<job xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\""
				+ filePath
				+ "\">"
				+ "<task executable=\"exec\" arguments=\"arg\" playpensize=\"3\" cost=\"3\" init=\"init\" final=\"final\">"
				+ "<processorattributes> <attribute value=\"LSD\"/> <attribute value=\"LINUX\"/> </processorattributes>"
				+ "<input><if condition=\"String\"><file location=\"String\" command=\"put\"/></if>"
				+ "</input>"
				+ "<output> <if condition=\"String\"> <file location=\"String\" command=\"get\"/></if>	</output>"
				+ "</task>" + "</job>";
		try {
			XMLUtil.parseXMLContent( test );
		} catch ( Exception e ) {
			assertTrue( false );
		}
	}


	public void testParseXMLFile() {

		try {
			String strXMLFilePath = THIS_RELATIVE_PATH + System.getProperty( "file.separator" ) + "testGrid.xml";
			String filePath = (ClassLoader.getSystemResource( strXMLFilePath )).getFile();
			XMLUtil.parseXMLFile( filePath );
		} catch ( XMLParserException e ) {
			assertTrue( false );
		} catch ( IOException e ) {
			assertTrue( false );
		}
	}


	public void testGetAttributeValue() {

		NamedNodeMap attributes = new NamedNodeMap() {

			public Node getNamedItem( String name ) {

				if ( "yes".equals( name ) || "YES".equals( name ) ) {
					return new TestNode();
				}
				return null;
			}


			public Node setNamedItem( Node arg ) throws DOMException {

				return null;
			}


			public Node removeNamedItem( String name ) throws DOMException {

				return null;
			}


			public Node item( int index ) {

				return null;
			}


			public int getLength() {

				return 0;
			}


			public Node getNamedItemNS( String namespaceURI, String localName ) {

				return null;
			}


			public Node setNamedItemNS( Node arg ) throws DOMException {

				return null;
			}


			public Node removeNamedItemNS( String namespaceURI, String localName ) throws DOMException {

				return null;
			}

		};

		String result = XMLUtil.getAttributeValue( "yes", "YES", attributes );
		if ( "ok".equals( result ) ) {
			assertTrue( true );
		} else {
			assertTrue( false );
		}
	}

	public class TestNode extends DefaultNode {

		public String getNodeValue() throws DOMException {

			return "ok";
		}

	}

}
