/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;

import org.ourgrid.common.id.ObjectID;

/**
 * Description: The core peer is the central peer of the OurGrid Community. When
 * a peer wants to join the community, it contacts the CorePeer to inform its ID
 * and URL and the CorePeer notifies the other community peers of the presence
 * of a new peer.
 * 
 * @version 1.0 Date27/09/2004
 */
public interface CorePeer extends Remote {

	/**
	 * Called when a peer wants to join the OurGrid Community.
	 * 
	 * @param peerId The id of the caller peer.
	 * @return the list of already known peers (including the caller peer).
	 * @throws RemoteException If cannot contact the core peer.
	 */
	public List<ObjectID> hereIAm( ObjectID peerId ) throws RemoteException;


	/**
	 * Returns a list of known peers in the OurGrid Community.
	 * 
	 * @param peerId The identification of the caller "peer".
	 * @return The list of already known peers.
	 * @throws RemoteException If cannot contact the core peer.
	 */
	public List<ObjectID> getKnownPeers( ObjectID peerId ) throws RemoteException;


	/**
	 * Shutdown a core peer. It will stop running threads and also prepare this
	 * object to be unbided.
	 * 
	 * @throws RemoteException If cannot contact the core peer.
	 */
	public void shutdown() throws RemoteException;


	/**
	 * Remove a peer from knowPeers list.
	 * 
	 * @param objectID informations about the peer
	 * @throws RemoteException If cannot contact the core peer.
	 */
	public void removePeer( ObjectID objectID ) throws RemoteException;


	/**
	 * Add a peer to knowPeers list.
	 * 
	 * @param peerID informations about the peer
	 * @throws RemoteException If cannot contact the core peer.
	 */
	public void newPeerArrived( ObjectID peerID ) throws RemoteException;

}
