/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer;

import java.net.InetAddress;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.CorePeerConfiguration;
import org.ourgrid.common.config.PeerConfiguration;
import org.ourgrid.common.fd.AbstractFailureDetector;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.url.CorePeerURLProvider;
import org.ourgrid.peer.community.RemoteAccessImpl;
import org.ourgrid.test.CorepeerPreCommitTest;
import org.ourgrid.test.TestUtils;

public class CoreTest extends TestCase implements CorepeerPreCommitTest {

	private String PORT = null;

	private String hostname;

	private TestUtils testUtils;

	private Registry r;

	private CorePeerImpl corePeer;

	private RemoteAccessImpl peer1;

	private RemoteAccessImpl peer2;

	private RemoteAccessImpl peer3;

	private RemoteAccessImpl peer4;

	private RemoteAccessImpl peer5;

	private RemoteAccessImpl peer6;

	ObjectID peerID1;

	ObjectID peerID2;

	ObjectID peerID3;

	ObjectID peerID4;

	ObjectID peerID5;

	ObjectID peerID6;

	private Map<String,RemoteAccessImpl> peers;


	public void setUp() throws Exception {

		Configuration.reset();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createOGProperties();

		peers = new HashMap<String,RemoteAccessImpl>();

		PORT = testUtils.getProperty( TestUtils.RMI_PORT );

		hostname = InetAddress.getLocalHost().getCanonicalHostName();

		Configuration.getInstance( Configuration.COREPEER );

		int registryPort = Integer.parseInt( testUtils.getProperty( TestUtils.RMI_PORT ) );

		Configuration.getInstance().setProperty( CorePeerConfiguration.PROP_NAME, "localhost" );
		Configuration.getInstance().setProperty( CorePeerConfiguration.PROP_PORT, PORT );
		Configuration.getInstance().setProperty( CorePeerConfiguration.PROP_UPDATE_DELAY, "2000" );

		r = null;

		try {
			r = LocateRegistry.createRegistry( Integer.parseInt( PORT ) );
		} catch ( Exception e ) {
			r = LocateRegistry.getRegistry( Integer.parseInt( PORT ) );
		}

		corePeer = new CorePeerImpl();

		r.rebind( CorePeerURLProvider.COREPEER_OBJECT_NAME, corePeer );

		Configuration.reset();

		Configuration.getInstance( Configuration.PEER ).setProperty( Configuration.PROP_PORT, registryPort + "" );

		Configuration.getInstance().setProperty( PeerConfiguration.PROP_COREPEER_NAME, "localhost" );
		Configuration.getInstance().setProperty( PeerConfiguration.PROP_COREPEER_PORT, PORT );

		FakeFailureDetector fakeFd = new FakeFailureDetector( 0 );
		corePeer.setFailureDetector( fakeFd );

		FakePingerFactory pingerFactory = new FakePingerFactory();
		corePeer.setPingerFactory( pingerFactory );

		peerID1 = new ObjectID( "1", new Long( 1 ), "rmi://" + "peer1" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );
		peerID2 = new ObjectID( "2", new Long( 2 ), "rmi://" + "peer2" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );
		peerID3 = new ObjectID( "3", new Long( 3 ), "rmi://" + "peer3" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );
		peerID4 = new ObjectID( "4", new Long( 4 ), "rmi://" + "peer4" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );
		peerID5 = new ObjectID( "5", new Long( 5 ), "rmi://" + "peer5" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );
		peerID6 = new ObjectID( "6", new Long( 6 ), "rmi://" + "peer6" + ":" + PORT + "/"
				+ AbstractFailureDetector.BOUND_NAME );

		peer1 = new RemoteAccessImpl( peerID1, null, null );
		peer2 = new RemoteAccessImpl( peerID2, null, null );
		peer3 = new RemoteAccessImpl( peerID3, null, null );
		peer4 = new RemoteAccessImpl( peerID4, null, null );
		peer5 = new RemoteAccessImpl( peerID5, null, null );
		peer6 = new RemoteAccessImpl( peerID6, null, null );

	}


	public void tearDown() throws Exception {

		corePeer.shutdown();
		Configuration.reset();
		testUtils.reset();
		r.unbind( CorePeerURLProvider.COREPEER_OBJECT_NAME );

	}


	public void testCommunity() throws Exception {

		peers.put( peerID1.getObjectURL(), peer1 );
		peers.put( peerID2.getObjectURL(), peer2 );
		peers.put( peerID3.getObjectURL(), peer3 );
		peers.put( peerID4.getObjectURL(), peer4 );
		peers.put( peerID5.getObjectURL(), peer5 );
		peers.put( peerID6.getObjectURL(), peer6 );

		ObjectID peerId = new ObjectID( "WebStatus", new Long( Long.MAX_VALUE ), "WebStatus" );
		List listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 0 peers.", 0, listOfKnownPeers.size() );
		assertEquals( "The listOfKnownPeers should have 0 peers.", 0, this.corePeer.getPeerList().size() );

		peer1.joinCommunity();
		peer2.joinCommunity();
		peer3.joinCommunity();
		peer4.joinCommunity();
		peer5.joinCommunity();

		assertNotNull( "The peerlist should not be null!", peer1.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer2.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer3.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer4.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer5.getPeerList() );

		peer1.setPeerList( null );
		peer2.setPeerList( null );
		peer3.setPeerList( null );
		peer4.setPeerList( null );
		peer5.setPeerList( null );

		assertNull( "The peerlist should be null!", peer1.getPeerList() );
		assertNull( "The peerlist should be null!", peer2.getPeerList() );
		assertNull( "The peerlist should be null!", peer3.getPeerList() );
		assertNull( "The peerlist should be null!", peer4.getPeerList() );
		assertNull( "The peerlist should be null!", peer5.getPeerList() );

		peer6.joinCommunity();

		Thread.sleep( 5000 );
		assertNotNull( "The peerlist should not be null!", peer1.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer2.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer3.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer4.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer5.getPeerList() );

		assertEquals( "The peerList should have 6 peers.", 6, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 6 peers.", 6, peer2.getPeerList().size() );
		assertEquals( "The peerList should have 6 peers.", 6, peer3.getPeerList().size() );
		assertEquals( "The peerList should have 6 peers.", 6, peer4.getPeerList().size() );
		assertEquals( "The peerList should have 6 peers.", 6, peer5.getPeerList().size() );

		listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 6 peers.", 6, listOfKnownPeers.size() );
		assertEquals( "The listOfPeers should have 6 peers.", 6, this.corePeer.getPeerList().size() );
	}


	public void testCoreMaps() throws Exception {

		RemoteAccessImpl peer1 = new RemoteAccessImpl( peerID1, null, null );
		peers.put( peerID1.getObjectURL(), peer1 );
		peer1.joinCommunity();

		Thread.sleep( 3000 );

		assertEquals( "The peerList should have 1 peer.", 1, peer1.getPeerList().size() );
		RemoteAccessImpl peer5 = new RemoteAccessImpl( peerID5, null, null );
		peers.put( peerID5.getObjectURL(), peer5 );
		peer5.joinCommunity();

		Thread.sleep( 3000 );

		assertEquals( "The peerList should have 2 peers.", 2, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 2 peers.", 2, peer5.getPeerList().size() );

		peerID6.setIdValue( new Long( 5 ) );
		RemoteAccessImpl peer6 = new RemoteAccessImpl( peerID6, null, null );
		peers.put( peerID6.getObjectURL(), peer6 );
		peer6.joinCommunity();

		Thread.sleep( 3000 );

		assertEquals( "The peerList should have 2 peers.", 2, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 2 peers.", 2, peer5.getPeerList().size() );
		assertEquals( "The peerList should have 2 peers.", 2, peer6.getPeerList().size() );

		ObjectID peerID7 = new ObjectID( "7", new Long( 7 ), "rmi://" + hostname + ":" + PORT + "/Peer6" );
		RemoteAccessImpl peer7 = new RemoteAccessImpl( peerID7, null, null );
		peers.put( peerID7.getObjectURL(), peer7 );
		peer7.joinCommunity();

		Thread.sleep( 3000 );

		assertEquals( "The peerList should have 3 peers.", 3, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 2 peers.", 2, peer5.getPeerList().size() );
		assertEquals( "The peerList should have 3 peers.", 3, peer6.getPeerList().size() );
		assertEquals( "The peerList should have 3 peers.", 3, peer7.getPeerList().size() );

		ObjectID peerID8 = new ObjectID( "8", new Long( 8 ), "rmi://" + hostname + ":" + PORT + "/Peer8" );
		RemoteAccessImpl peer8 = new RemoteAccessImpl( peerID8, null, null );
		peers.put( peerID8.getObjectURL(), peer8 );
		peer8.joinCommunity();

		Thread.sleep( 3000 );

		assertEquals( "The peerList should have 4 peers.", 4, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 2 peers.", 2, peer5.getPeerList().size() );
		assertEquals( "The peerList should have 4 peers.", 4, peer6.getPeerList().size() );
		assertEquals( "The peerList should have 4 peers.", 4, peer7.getPeerList().size() );
		assertEquals( "The peerList should have 4 peers.", 4, peer8.getPeerList().size() );

	}


	public void testRejoinCommunity() throws Exception {

		ObjectID peerId = new ObjectID( "WebStatus", new Long( Long.MAX_VALUE ), "WebStatus" );
		List listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 0 peers.", 0, listOfKnownPeers.size() );

		RemoteAccessImpl peer1 = new RemoteAccessImpl( peerID1, null, null );
		peers.put( peerID1.getObjectURL(), peer1 );

		RemoteAccessImpl peer2 = new RemoteAccessImpl( peerID2, null, null );
		peers.put( peerID2.getObjectURL(), peer2 );

		peer1.joinCommunity();
		peer2.joinCommunity();

		Thread.sleep( 4000 );

		assertEquals( "The peerList should have 2 peers.", 2, peer2.getPeerList().size() );

		listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 2 peers.", 2, listOfKnownPeers.size() );

		corePeer.removePeer( peerID1 );
		Thread.sleep( 4000 );

		assertEquals( "The peerList should have 1 peers.", 1, peer2.getPeerList().size() );

		listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 1 peer.", 1, listOfKnownPeers.size() );

		peer1.joinCommunity();

		Thread.sleep( 4000 );

		assertEquals( "The peerList should have 2 peers.", 2, peer2.getPeerList().size() );

		listOfKnownPeers = this.corePeer.getKnownPeers( peerId );
		assertEquals( "The listOfKnownPeers should have 2 peers.", 2, listOfKnownPeers.size() );

	}

	private class FakePeerPinger implements Pinger {

		private ObjectID peerID;

		private List<ObjectID> peerList;


		public FakePeerPinger( ObjectID peerID, Map<Long,ObjectID> knowPeers, List<ObjectID> peerList,
								Map<String,ObjectID> knowPeersByURL ) {

			this.peerID = peerID;
			this.peerList = peerList;
		}


		/**
		 * Try to contact the peer. If failed, remove it from the know peer's
		 * map.
		 */
		public void run() {

			RemoteAccessImpl peer = peers.get( this.peerID.getObjectURL() );
			peer.setPeerList( peerList );

		}

	}

	private class FakePingerFactory extends PingerFactory {

		public Pinger makePinger( ObjectID peerID, Map<Long,ObjectID> knowPeers, List<ObjectID> peerList,
									Map<String,ObjectID> knowPeersByURL ) {

			return new FakePeerPinger( peerID, knowPeers, peerList, knowPeersByURL );
		}

	}

}
