/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer;

import java.rmi.Naming;
import java.util.List;
import java.util.Map;

import org.ourgrid.common.id.ObjectID;
import org.ourgrid.peer.community.RemoteAccess;

/**
 * Description: Runnable object to test peer status.
 * 
 * @version 1.0 Date27/09/2004
 */
public class PeerPinger implements Pinger {

	/**
	 * The peer to be tested.
	 */
	private ObjectID peerID;

	/**
	 * Logger to store log messages
	 */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( PeerPinger.class );

	/**
	 * The map to be updated if the peer is offline.
	 */
	private Map<Long,ObjectID> knowPeers;

	/**
	 * The list of know peers.
	 */
	private List<ObjectID> peerList;

	private Map<String,ObjectID> knowPeersByURL;


	/**
	 * Creates a pinger.
	 * 
	 * @param peerID The peer to be pinger.
	 * @param knowPeers Map of peers
	 * @param peerList List of knwow peers.
	 */
	public PeerPinger( ObjectID peerID, Map<Long,ObjectID> knowPeers, List<ObjectID> peerList,
						Map<String,ObjectID> knowPeersByURL ) {

		this.peerID = peerID;
		this.knowPeers = knowPeers;
		this.peerList = peerList;
		this.knowPeersByURL = knowPeersByURL;

	}


	/**
	 * Try to contact the peer. If failed, remove it from the know peer's map.
	 */
	public void run() {

		try {

			LOG.info( "Verifying status of peer " + peerID.getName() + " at " + peerID.getObjectURL() + "." );

			RemoteAccess aPeer = (RemoteAccess) Naming.lookup( peerID.getObjectURL() );
			aPeer.hereIsPeerList( peerList );

			LOG.info( "Peer " + peerID.getName() + " at " + peerID.getObjectURL() + " sucessfully contacted." );

		} catch ( Exception e ) {
			LOG.error( "Peer " + peerID.getName() + " at " + peerID.getObjectURL() + " could not be contacted. Cause: "
					+ e.getMessage() );
		}

	}
}
