/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ourgrid.common.id.ObjectID;

/**
 * 
 */
public class RestrictedCorePeerImpl extends CorePeerImpl implements CorePeer, CorePeerStatusProvider {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	public static final long serialVersionUID = 33L;

	/**
	 * Map of peers allowed to join the community
	 */
	private Map<String,String> allowedPeers;

	/**
	 * File with a list of allowed peers.
	 */
	public static final String ALLOWED_PEERS_FILE = "peers.allow";


	/**
	 * Creates a CorePeer that can allow or not peers join the community.
	 * 
	 * @throws RemoteException If cannot create this CorePeer.
	 */
	public RestrictedCorePeerImpl() throws RemoteException {

		super();

		loadAllowedPeers();

	}


	public List<ObjectID> hereIAm( ObjectID peerId ) throws RemoteException {

		if ( isAnAllowedPeer( peerId ) ) {
			return super.hereIAm( peerId );
		}
		throw new RemoteException( "Peer " + peerId.getName() + " is not allowed to join this community!" );

	}


	/**
	 * Loads the allowed peers from the ALLOWED_PEERS_FILE.
	 */
	private void loadAllowedPeers() {

		allowedPeers = new HashMap<String,String>();

		BufferedReader reader = null;

		try {
			reader = new BufferedReader( new FileReader( ALLOWED_PEERS_FILE ) );
			String line;

			while ( (line = reader.readLine()) != null ) {
				allowedPeers.put( line, line );
			}

		} catch ( Exception e ) {

		} finally {
			if ( reader != null ) {
				try {
					reader.close();
				} catch ( IOException e1 ) {
				}
			}
		}

	}


	/**
	 * Returns if the PeerId is allowed or not.
	 * 
	 * @param peerId A PeerId.
	 * @return <code>true</code> case the PeerId is allowed,
	 *         <code>false</code> otherwise.
	 */
	public boolean isAnAllowedPeer( ObjectID peerId ) {

		return allowedPeers.get( peerId.getObjectURL() ) != null;
	}


	public List<ObjectID> getKnownPeers( ObjectID peerId ) throws RemoteException {

		if ( isAnAllowedPeer( peerId ) ) {
			return super.getKnownPeers( peerId );
		}
		throw new RemoteException( "Peer " + peerId.getName() + " is not allowed to join this community!" );
	}

}
