/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.CorePeerConfiguration;
import org.ourgrid.common.config.PeerConfiguration;
import org.ourgrid.common.fd.AbstractFailureDetector;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.url.CorePeerURLProvider;
import org.ourgrid.peer.community.RemoteAccessImpl;
import org.ourgrid.test.CorepeerPreCommitTest;
import org.ourgrid.test.TestUtils;

public class RestrictedCoreTest extends TestCase implements CorepeerPreCommitTest {

	private RestrictedCorePeerImpl corePeer;

	private Registry registry = null;

	private TestUtils testUtils;

	private int port;

	private ObjectID peerID1;

	private ObjectID peerID2;

	private ObjectID peerID3;

	private ObjectID peerID4;

	private ObjectID peerID5;

	private RemoteAccessImpl peer1;

	private RemoteAccessImpl peer2;

	private RemoteAccessImpl peer3;

	private RemoteAccessImpl peer4;

	private RemoteAccessImpl peer5;

	private Map<String,RemoteAccessImpl> peers;


	public void setUp() throws Exception {

		peers = new HashMap<String,RemoteAccessImpl>();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createCorepeerProperties();

		Configuration.getInstance( Configuration.PEER );
		Configuration.getInstance().setProperty( Configuration.PROP_PORT, testUtils.getProperty( TestUtils.RMI_PORT ) );

		Configuration.getInstance().setProperty( PeerConfiguration.PROP_COREPEER_NAME, "localhost" );
		port = Integer.parseInt( Configuration.getInstance().getProperty( Configuration.PROP_PORT ) );
		Configuration.getInstance().setProperty( PeerConfiguration.PROP_COREPEER_PORT, Integer.toString( port ) );

		try {

			registry = LocateRegistry.createRegistry( port );

		} catch ( Exception e ) {
			registry = LocateRegistry.getRegistry( port );
		}

		Configuration.getInstance().setProperty( CorePeerConfiguration.PROP_UPDATE_DELAY, "2000" );

		peerID1 = new ObjectID( "1", new Long( 1 ), "rmi://" + "peer1" + ":" + port
				+ AbstractFailureDetector.BOUND_NAME );
		peerID2 = new ObjectID( "2", new Long( 2 ), "rmi://" + "peer2" + ":" + port
				+ AbstractFailureDetector.BOUND_NAME );
		peerID3 = new ObjectID( "3", new Long( 3 ), "rmi://" + "peer3" + ":" + port
				+ AbstractFailureDetector.BOUND_NAME );
		peerID4 = new ObjectID( "4", new Long( 4 ), "rmi://" + "peer4" + ":" + port
				+ AbstractFailureDetector.BOUND_NAME );
		peerID5 = new ObjectID( "5", new Long( 5 ), "rmi://" + "peer5" + ":" + port
				+ AbstractFailureDetector.BOUND_NAME );

		peer1 = new RemoteAccessImpl( peerID1, null, null );
		peer2 = new RemoteAccessImpl( peerID2, null, null );
		peer3 = new RemoteAccessImpl( peerID3, null, null );
		peer4 = new RemoteAccessImpl( peerID4, null, null );
		peer5 = new RemoteAccessImpl( peerID5, null, null );

		peers.put( peerID1.getObjectURL(), peer1 );
		peers.put( peerID2.getObjectURL(), peer2 );
		peers.put( peerID3.getObjectURL(), peer3 );
		peers.put( peerID4.getObjectURL(), peer4 );
		peers.put( peerID5.getObjectURL(), peer5 );

	}


	public void tearDown() throws Exception {

		corePeer.shutdown();
		Configuration.reset();
		testUtils.reset();
		registry.unbind( CorePeerURLProvider.COREPEER_OBJECT_NAME );
	}


	public void testRestrictedCommunity() throws Exception {

		createAllowedPeersFile();

		corePeer = new RestrictedCorePeerImpl();
		registry.rebind( CorePeerURLProvider.COREPEER_OBJECT_NAME, corePeer );

		FakeFailureDetector fakeFd = new FakeFailureDetector( 0 );
		corePeer.setFailureDetector( fakeFd );

		FakePingerFactory pingerFactory = new FakePingerFactory();
		corePeer.setPingerFactory( pingerFactory );

		ObjectID peerId = new ObjectID( "WebStatus", new Long( Long.MAX_VALUE ), "WebStatus" );
		List listOfKnownPeers = corePeer.getKnownPeers( peerId );
		assertEquals( "The peer list should be empty.", 0, listOfKnownPeers.size() );

		ObjectID notAllowedPeerId = new ObjectID( "NotAllowedWebStatus", new Long( Long.MAX_VALUE ),
				"NotAllowedWebStatus" );
		try {
			listOfKnownPeers = corePeer.getKnownPeers( notAllowedPeerId );
			fail( "This should throw an exception!" );
		} catch ( RemoteException e ) {
			System.out.println( "Expected exception! " + e );
		}

		peer1.joinCommunity();

		try {
			peer2.joinCommunity();
			fail( "This should throw an exception!" );
		} catch ( RemoteException re ) {

		}

		peer3.joinCommunity();

		try {
			peer4.joinCommunity();
			fail( "This should throw an exception!" );
		} catch ( RemoteException re ) {

		}

		peer5.joinCommunity();

		assertNotNull( "The peerlist should not be null!", peer1.getPeerList() );
		assertNull( "The peerlist should be null!", peer2.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer3.getPeerList() );
		assertNull( "The peerlist should be null!", peer4.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer5.getPeerList() );

		peer1.setPeerList( null );
		peer2.setPeerList( null );
		peer3.setPeerList( null );
		peer4.setPeerList( null );
		peer5.setPeerList( null );

		assertNull( "The peerlist should be null!", peer1.getPeerList() );
		assertNull( "The peerlist should be null!", peer2.getPeerList() );
		assertNull( "The peerlist should be null!", peer3.getPeerList() );
		assertNull( "The peerlist should be null!", peer4.getPeerList() );
		assertNull( "The peerlist should be null!", peer5.getPeerList() );

		Thread.sleep( 20000 );

		assertNotNull( "The peerlist should not be null!", peer1.getPeerList() );
		assertNull( "The peerlist should be null!", peer2.getPeerList() );
		assertNotNull( "The peerlist should not be null!", peer3.getPeerList() );
		assertNull( "The peerlist should not be null!", peer4.getPeerList() );
		assertNotNull( "The peerlist should be null!", peer5.getPeerList() );

		assertEquals( "The peerList should have 3 peers.", 3, peer1.getPeerList().size() );
		assertEquals( "The peerList should have 3 peers.", 3, peer3.getPeerList().size() );
		assertEquals( "The peerList should have 3 peers.", 3, peer5.getPeerList().size() );

		listOfKnownPeers = corePeer.getKnownPeers( peerId );
		assertEquals( "The peer list should have 3 peers.", 3, listOfKnownPeers.size() );

	}


	/**
	 *  
	 */
	private void createAllowedPeersFile() throws Exception {

		BufferedWriter writer = null;

		try {
			writer = new BufferedWriter( new FileWriter( RestrictedCorePeerImpl.ALLOWED_PEERS_FILE ) );
			writer.write( "WebStatus" );
			writer.newLine();
			writer.write( "rmi://" + "peer1" + ":" + port + AbstractFailureDetector.BOUND_NAME );
			writer.newLine();
			writer.write( "rmi://" + "peer3" + ":" + port + AbstractFailureDetector.BOUND_NAME );
			writer.newLine();
			writer.write( "rmi://" + "peer5" + ":" + port + AbstractFailureDetector.BOUND_NAME );
			writer.newLine();

		} finally {
			writer.close();
		}

	}

	private class FakePeerPinger implements Pinger {

		private ObjectID peerID;

		private List<ObjectID> peerList;


		public FakePeerPinger( ObjectID peerID, Map<Long,ObjectID> knowPeers, List<ObjectID> peerList,
								Map<String,ObjectID> knowPeersByURL ) {

			this.peerID = peerID;
			this.peerList = peerList;
		}


		/**
		 * Try to contact the peer. If failed, remove it from the know peer's
		 * map.
		 */
		public void run() {

			RemoteAccessImpl peer = peers.get( this.peerID.getObjectURL() );
			peer.setPeerList( peerList );

		}

	}

	private class FakePingerFactory extends PingerFactory {

		public Pinger makePinger( ObjectID peerID, Map<Long,ObjectID> knowPeers, List<ObjectID> peerList,
									Map<String,ObjectID> knowPeersByURL ) {

			return new FakePeerPinger( peerID, knowPeers, peerList, knowPeersByURL );
		}

	}

}
