/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.corepeer.fd;

import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.Notifiable;
import org.ourgrid.common.fd.State;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.url.URLProvider;
import org.ourgrid.corepeer.CorePeer;

public class PeerStateNotifiable implements Notifiable {

	private final String name = "peerStateNotifiable";

	private CorePeer corePeer;


	/**
	 * Default constructor
	 * 
	 * @param corePeer
	 */
	public PeerStateNotifiable( CorePeer corePeer ) {

		this.corePeer = corePeer;
	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#notifyMonitorableStateChange(java.lang.String,
	 *      GenericState)
	 */
	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState ) {

		try {

			String peerUrl = URLProvider.getScheme( monitorableName ) + "://" + URLProvider.getHost( monitorableName )
					+ ":" + URLProvider.getPort( monitorableName );
			ObjectID peerID = new ObjectID( monitorableName, 0l, peerUrl );

			if ( monitorableState.basicState().equals( State.DOWN ) ) {

				corePeer.removePeer( peerID );

			} else {

				corePeer.newPeerArrived( peerID );

			}

		} catch ( Exception e ) {
			e.printStackTrace();
		}

	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#getName()
	 */
	public String getName() {

		return name;
	}

}
