/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.corepeer.ui;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.util.HashMap;
import java.util.Map;

import org.ourgrid.common.config.Configuration;

/**
 * Implements the main of Command Pattern. All CorePeer commands must pass here.
 */
public class CommandManager {

	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( CorePeerUIManager.class );

	private Map<String,Command> corePeerCommands;

	private static final String COREPEER_STARTER = "start";

	private static final String COREPEER_STOPPER = "stop";

	private static final String COREPEER_STATUS = "status";

	private UIManager manager;


	public CommandManager() {

		corePeerCommands = new HashMap<String,Command>();
		initCommandMap();
		initRMIRegistry();
		try {
			manager = CorePeerUIManager.getInstance();
		} catch ( RemoteException e ) {
			LOG.debug( "Could not contact corepeer UI manager" );
		}

	}


	private void initRMIRegistry() {

		int registryPort = Integer.parseInt( Configuration.getInstance( Configuration.COREPEER ).getProperty(
				Configuration.PROP_PORT ) );

		System.setProperty( "java.rmi.server.hostname", Configuration.getInstance().getProperty(
				Configuration.PROP_EXTERNAL_NAME ) );

		try {
			// Starting registry
			LocateRegistry.createRegistry( registryPort );
		} catch ( RemoteException e ) {
			// TODO LOG
		}

	}


	/**
	 * it starts the map with CorePeerCommands.
	 */
	private void initCommandMap() {

		corePeerCommands.put( COREPEER_STOPPER, new CorePeerStopper() );
		corePeerCommands.put( COREPEER_STARTER, new CorePeerStarter() );
		corePeerCommands.put( COREPEER_STATUS, new CorePeerStatus() );
	}


	/**
	 * Method that checks what <code>Command</code> will be able to execute
	 * the command in parameter "param". It will verify in it's map of commands,
	 * and then, call his run method.
	 * 
	 * @param params This parameter must contain the Command string
	 *        representation, followed by it's parameters.
	 */
	public void executeCommand( String[ ] params ) throws Exception {

		String command = params[0];

		String[ ] parameters = discardCommandParameter( params );
		Command gridCommand = corePeerCommands.get( command );

		gridCommand.run( parameters, manager );

	}


	/**
	 * Discard the first parameter from this String[]. This parameter represents
	 * the Command from line command.
	 * 
	 * @param params An String array with all parameter that came from line
	 *        comand, including the especified CorePeer Command.
	 * @return All other parameters except the CorePeer Command
	 */
	private String[ ] discardCommandParameter( String[ ] params ) {

		String[ ] parameters = new String[ params.length - 1 ];

		for ( int i = 1; i < params.length; i++ ) {
			parameters[i - 1] = params[i];
		}

		return parameters;
	}


	public static void main( String[ ] args ) {

		try {
			new CommandManager().executeCommand( args );
		} catch ( Exception e ) {
			System.out.println( "Could not start CorePeer Command " + args[0] + " verfify your corepeer.properties" );
		}

	}

}
