/*
 * Copyright (c) 2002-2004 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.corepeer.ui;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;

import org.ourgrid.common.id.ObjectID;

public interface UIManager extends Remote {

	/**
	 * Starts CorePeer Service
	 * 
	 * @throws RemoteException if a communication problem occurs
	 */
	public void startCorePeerService() throws RemoteException;


	/**
	 * Stops CorePeer Service
	 * 
	 * @throws RemoteException if a communication problem occurs
	 * @throws CorePeerUIException
	 */
	public void stopCorePeerService() throws CorePeerUIException, AccessException, RemoteException, NotBoundException;


	/**
	 * Gets the CorePeer status.
	 * 
	 * @return This CorePeer status.
	 * @throws RemoteException if a communication problem occurs
	 */
	public Collection<ObjectID> statusCorePeerService() throws AccessException, RemoteException, NotBoundException;


	/**
	 * Checks if the corepeer service is running
	 * 
	 * @return true if the corepeer is online
	 * @throws RemoteException if a communication problem occurs
	 */
	public boolean isCorePeerRunning() throws RemoteException;
}
