/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.deployer;

import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.executor.Executor;
import org.ourgrid.common.executor.ExecutorException;
import org.ourgrid.common.executor.ExecutorFactory;
import org.ourgrid.common.executor.ExecutorResult;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.test.PeerPreCommitTest;
import org.ourgrid.test.TestUtils;

/**
 * This TestCase is supposed to test Deployer funcionalities.
 * 
 * @version 1.0 Created on 18/08/2004
 */
public class DeployerTest extends TestCase implements PeerPreCommitTest {

	/** an Executor object */
	private Executor executor;

	/** an ExecutorResult object */
	private ExecutorResult result;

	/** Holds mygrid root directory */
	private String mgroot = System.getProperty( "MGROOT" );

	/** reference to uaadmin script */
	private final String uaadmin = mgroot + "/bin/uaadmin";

	/** directory where UA.properties files are stored */
	private static final String strUAPropertiesDir = "./uasconf";

	/** Grid Description File that will be used */
	private String sdfInvalid;

	private TestUtils testUtils;

	private int uaLinuxMachines;

	private List gumSpecsList;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( DeployerTest.class );
	}


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();

		Configuration.reset();
		Configuration.getInstance( Configuration.USERAGENT );

		executor = ExecutorFactory.getExecutorInstance();
		result = executor.getResult( executor.execute( ".", "/bin/rm -rf " + strUAPropertiesDir ) );

		/* Gets a valid SDF */
		uaLinuxMachines = 2;
		gumSpecsList = testUtils.getGumSpecs( uaLinuxMachines, TestUtils.TYPE_UALINUX );

		sdfInvalid = testUtils.createSDF( gumSpecsList );
	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		result = executor.getResult( executor.execute( ".", "/bin/rm -rf " + strUAPropertiesDir ) );

		result = null;
		gumSpecsList = null;

		Configuration.reset();

		testUtils.reset();

	}


	/**
	 * Constructor for DeployerTest.
	 * 
	 * @param arg0
	 */
	public DeployerTest( String arg0 ) {

		super( arg0 );
	}


	public void testUA() {

		try {
			stop();
			startAndStatus();
			stop();
			restart();
			stop();
		} catch ( ExecutorException e ) {
			fail( "A exception was throw.. cannot execute UaAdmin: " + e.getMessage() );
		}
	}


	/**
	 * Tests uaadmin start option.
	 * 
	 * @throws ExecutorException
	 */
	public void startAndStatus() throws ExecutorException {

		String command = "start";

		/** running uaadmin start */
		System.out.println( uaadmin + " " + command + " " + sdfInvalid );
		result = executor.getResult( executor.execute( ".", uaadmin + " " + command + " " + sdfInvalid ) );
		System.out.println( "Result:" + result );

		// Verify the result of the command execution
		assertEquals( result.getExitValue(), 0 );

		// getting number of auto-installable machines
		Iterator itGuMSpecs = gumSpecsList.iterator();
		int numGums = 0;
		while ( itGuMSpecs.hasNext() ) {
			GumSpec currentGuMSpec = (GumSpec) itGuMSpecs.next();
			if ( Deployer.gumIsUserAgent( currentGuMSpec ) && Deployer.uaIsAutoInstallable( currentGuMSpec ) ) {
				numGums++;
			}
		}

		// verifying whether the the User Agents were successfully started
		command = "status";
		System.out.println( uaadmin + " " + command + " " + sdfInvalid );
		result = executor.getResult( executor.execute( ".", uaadmin + " " + command + " " + sdfInvalid
				+ "| grep -w UP | wc -l " ) );
		System.out.println( "Integer.parseInt(result) == numGums ? ( " + result + " = " + numGums + " ) ?" );
		assertTrue( Integer.parseInt( result.getStdout().toString().trim() ) == numGums );

	}


	/**
	 * Tests uaadmin stop option.
	 * 
	 * @throws ExecutorException
	 */
	public void stop() throws ExecutorException {

		String command = "stop";

		/** running uaadmin stop */
		System.out.println( uaadmin + " " + command + " " + sdfInvalid );
		result = executor.getResult( executor.execute( ".", uaadmin + " " + command + " " + sdfInvalid ) );
		System.out.println( "Result:" + result );

		// Verify the result of the command execution
		assertEquals( result.getExitValue(), 0 );

		// getting number of auto-installable machines
		Iterator itGuMSpecs = gumSpecsList.iterator();
		int numGums = 0;
		while ( itGuMSpecs.hasNext() ) {
			GumSpec currentGuMSpec = (GumSpec) itGuMSpecs.next();
			if ( Deployer.gumIsUserAgent( currentGuMSpec ) && Deployer.uaIsAutoInstallable( currentGuMSpec ) ) {
				numGums++;
			}
		}

		// verifying whether the the User Agents were successfully started
		command = "status";
		System.out.println( uaadmin + " " + command + " " + sdfInvalid );
		result = executor.getResult( executor.execute( ".", uaadmin + " " + command + " " + sdfInvalid
				+ "| grep -w DOWN | wc -l " ) );
		System.out.println( "Integer.parseInt(result) == numGums ? ( " + result.getStdout() + " = " + numGums + " ) ?" );
		assertTrue( Integer.parseInt( result.getStdout().toString().trim() ) == numGums );

	}


	/**
	 * Tests uaadmin stop option.
	 * 
	 * @throws ExecutorException
	 */
	public void restart() throws ExecutorException {

		String command = "restart";

		/** running uaadmin restart */
		System.out.println( uaadmin + " " + command + " " + sdfInvalid );
		result = executor.getResult( executor.execute( ".", uaadmin + " " + command + " " + sdfInvalid ) );
		System.out.println( "Result:" + result );
		assertTrue( result.getExitValue() == 0 );
	}

}
