/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.exception;

import java.text.MessageFormat;

public class FileTransferException extends FailCauseException {

	private static final long serialVersionUID = 33L;

	private String filePath;

	private boolean local;


	public FileTransferException( String filePath, Throwable detail, boolean local ) {

		super( "Filepath: " + filePath, detail );
		this.filePath = filePath;
		this.local = local;

	}


	public FileTransferException( Throwable detail, boolean local ) {

		super( detail );
		this.local = local;
	}


	public FileTransferException( String filePath, boolean local ) {

		super( "Filepath: " + filePath );
		this.filePath = filePath;
		this.local = local;
	}


	/**
	 * Returns the exception message.
	 * 
	 * @return The message of this excpetion
	 */
	public String getMessage() {

		Object params[] = { filePath };
		return MessageFormat.format( super.getMessage(), params );
	}


	@Override
	public String getSimpleMessage() {

		if ( local ) {
			return "Unable to transfer LOCAL file " + filePath + " to Grid Machine";
		}

		return "Unable to transfer REMOTE file " + filePath + " from Grid Machine";
	}

}
