/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.services;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.exception.GumSpecificationException;

/**
 * This class is responsible for validating a given GuMSpec for creating a GuM
 * according to a GuMSpec. <br>
 */
public class GumSpecServices {

	/**
	 * Verifies if the GuMSpec used in the constructor is valid.
	 * 
	 * @return true if the GuMSpec is valid and false,otherwise.
	 * @throws GumSpecificationException
	 */
	public static boolean isSpecValid( GumSpec spec ) { // To Ayla

		try {
			GumServicesBuilder builder = new DefaultGumServicesBuilder( spec );
			return builder.getGumSpecValidator().isValid( spec );
		} catch ( GumSpecificationException e ) {
			return false;
		}
	}


	/**
	 * Creates a GuM according to the GuMSpec passed as the constructor
	 * parameter.
	 * 
	 * @return a GridMachine.
	 * @throws GumSpecificationException if the GuMSpec is not valid.
	 * @throws NotBoundException
	 * @throws RemoteException
	 * @throws MalformedURLException
	 */
	public static Gum createGum( GumSpec gumSpec ) throws GumSpecificationException, MalformedURLException,
		RemoteException, NotBoundException {

		if ( isSpecValid( gumSpec ) ) {
			GumServicesBuilder builder = new DefaultGumServicesBuilder( gumSpec );
			return builder.getGumCreator().createGuM( gumSpec );
		}
		throw new GumSpecificationException( "Unable to create GuM because this specification is not valid" );
	}
}
