/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.ourgrid.common.event.Event;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.mygrid.scheduler.event.SimpleResponseEvent;

/**
 * Description:
 * 
 * @version 1.0 Created on 09/07/2004
 */
public class FileLockManager {

	/**
	 * A logger
	 */

	private Map<String,Vector<EventQueue>> blockedMap;

	public static FileLockManager instance = null;


	public static synchronized FileLockManager getInstance() {

		if ( instance == null ) {
			instance = new FileLockManager();
		}

		return instance;
	}


	protected FileLockManager() {

		blockedMap = new HashMap<String,Vector<EventQueue>>();
	}


	private void grantPermission( EventQueue<Event> responseQueue ) {

		responseQueue.put( new SimpleResponseEvent<Boolean>( true ) );

	}


	public synchronized EventQueue<Event> requestLock( String filePath ) {

		EventQueue<Event> responseQueue = new EventQueue<Event>();

		if ( blockedMap.get( filePath ) == null ) {

			blockedMap.put( filePath, new Vector<EventQueue>() );
			grantPermission( responseQueue );

		} else {

			Vector<EventQueue> v = blockedMap.get( filePath );
			v.add( responseQueue );

		}

		return responseQueue;

	}


	public synchronized void releaseLock( String filePath ) {

		Vector<EventQueue> v = blockedMap.get( filePath );
		if ( v != null ) {
			if ( v.size() == 0 ) {
				blockedMap.remove( filePath );
			} else {
				grantPermission( v.remove( 0 ) );

			}

		}

	}

}
