/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent.test;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.ourgrid.common.executor.ExecutorHandle;
import org.ourgrid.common.executor.ExecutorResult;
import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.gum.GumStatus;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;

/**
 * A FakeUserAgentServer for UserAgentClient's unit tests.
 */
public class FakeUserAgentServer implements Gum {

	private HashMap<String,String> filesPut = new HashMap<String,String>();

	private HashSet<String> filesPutName = new HashSet<String>();

	private int numberOfPutFileCalls = 0;

	private int numberOfGetStorageDirCalls = 0;

	private int numberOfFileExistsCalls = 0;

	private boolean putFileException = false;

	private String storageDirectory = "default";

	private GumSpec spec;

	private GumID gumID;


	/**
	 * @throws RemoteException
	 */
	public FakeUserAgentServer() throws RemoteException {

		super();
	}


	public FakeUserAgentServer( GumSpec spec ) throws RemoteException {

		super();
		this.spec = spec;
	}


	public FakeUserAgentServer( GumID gumID, GumSpec gumSpec ) throws RemoteException {

		super();
		this.spec = gumSpec;
		this.gumID = gumID;
	}


	public ExecutorHandle remoteExecute( String remoteDir, String command, Map envVars ) {

		return null;
	}


	public ExecutorHandle remoteExecute() {

		return null;
	}


	public void putFile( String remoteFile, byte[ ] data, boolean firstSlice ) throws RemoteException, IOException {

		numberOfPutFileCalls++;

		if ( putFileException ) {
			throw new IOException( "Stream must be closed" );
		}

		this.filesPut.put( new File( remoteFile ).getName(), remoteFile );
		this.filesPutName.add( remoteFile );

	}


	public byte[ ] getFile( String filePath, long filePos, int size ) {

		return null;
	}


	public boolean fileExists( File file ) {

		numberOfFileExistsCalls++;
		return this.filesPut.containsKey( file.getName() );
	}


	/**
	 * Returns true if a specified filename was called.
	 * 
	 * @param file To verify file name
	 * @return True if a put was made with specified file name
	 */
	public boolean fileNameExists( String file ) {

		return this.filesPutName.contains( file );
	}


	public void changeFilePermissions( File file, int permissions ) {

	}


	public long getRemoteFileLastModification( File remoteFile, boolean normalized ) {

		return 0;
	}


	public long getRemoteFileLength( File remoteFile ) {

		return 0;
	}


	public String getRemoteFileDigest( File remoteFile ) {

		return null;
	}


	public String createPlaypen( int size ) {

		return null;
	}


	public String createStorageDirectory() {

		return null;
	}


	public String getStorageDirectory() {

		numberOfGetStorageDirCalls++;

		return storageDirectory;
	}


	protected int getPutFileCalls() {

		return numberOfPutFileCalls;
	}


	protected int getFileExistsCalls() {

		return numberOfFileExistsCalls;
	}


	protected int getGetStorageDirCalls() {

		return numberOfGetStorageDirCalls;
	}


	protected void setPutFileException( boolean b ) {

		putFileException = b;
	}


	protected void setStorageDirectory( String s ) {

		storageDirectory = s;
	}


	public ExecutorResult getResult( ExecutorHandle handle ) {

		ExecutorResult result = new ExecutorResult();
		result.setExitValue( 0 );
		result.setStderr( "" );
		result.setStdout( "SUCESSO!" );

		return result;
	}


	public GumStatus getStatus() {

		return GumStatus.UP;
	}


	public GumID getID() {

		return gumID;

	}


	public GumSpec getGuMSpec() {

		return this.spec;
	}


	public void invalidate() {

		return;
	}


	public void shutDown() {

		return;
	}


	public void startReplica() throws RemoteException {

	}


	public void stopReplica() throws RemoteException {

	}
}
