/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.executor.ExecutorException;
import org.ourgrid.common.executor.ExecutorHandle;
import org.ourgrid.common.executor.ExecutorResult;
import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.main.CommonCompiler;
import org.ourgrid.common.url.UserAgentURLProvider;
import org.ourgrid.common.util.JavaFileUtil;
import org.ourgrid.common.util.TempFileManager;
import org.ourgrid.gridmachine.useragent.UserAgentServerImpl;
import org.ourgrid.test.GumPreCommitTest;
import org.ourgrid.test.TestUtils;

public class UserAgentServerTest extends TestCase implements GumPreCommitTest {

	/** User Agent Object */
	Gum ua;

	/** User Agent Object will be created with a port number */
	UserAgentServerImpl uaPort;

	/** User Agent Object will be created with an ObjID and a port number */
	UserAgentServerImpl uaIdAndPort;

	/** The data array used to test the put file method */
	byte[ ] arrayOfData;

	/** The temporary file used to test the putFile() method */
	File file;

	/** Grid Description File that will be used */
	private String gdf;

	private TestUtils testUtils;


	public UserAgentServerTest( String name ) {

		super( name );

	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		Configuration.reset();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createOGProperties();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.USERAGENT );
		Vector<GumSpec> gumList = new Vector<GumSpec>();
		gumList.add( testUtils.getLocalGuMSpec( TestUtils.TYPE_UALINUX ) );

		gdf = testUtils.createSDF( gumList );

		/** get LocalGUMPSpec from GDF */
		CommonCompiler compiler = new CommonCompiler();

		/** tries to compile GDF */
		compiler.compile( gdf, CommonCompiler.SDF_TYPE );

		/** get LocalGUMPSpec from result */
		List gumsList = compiler.getResult();

		File mypids = new File( ".mypids" );
		if ( mypids.exists() ) {
			mypids.delete();
		}

		/** running uaadmin restart */

		Runtime runtime = Runtime.getRuntime();
		System.out.println( "SetUp: doing bin/uaadmin restart " + gdf );
		Process process = runtime.exec( "bin/uaadmin restart " + gdf );
		if ( process.waitFor() != 0 ) {
			fail( "uaadmin failed!" );
		}

		testUtils.testUA( gumList, 10000 );

		GumSpec gumSpec = (GumSpec) gumsList.get( 0 );

		/** Set an array of bytes with null values */
		arrayOfData = new byte[ 13 ];

		/** Init the array of bytes */
		for ( int i = 0; i < 13; i++ ) {
			arrayOfData[i] = 1;
		}

		/** The temporary file used to test the putFile() method */
		file = new File( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + "/UserServerImplTest.testPutFile" );
		file.deleteOnExit();

		/** Abstract representaion for the storage */
		File storage = new File( System.getProperty( "user.home" ) + File.separator + ".mgstorage" );
		storage.delete();

		System.out.println( "ua =>> "
				+ UserAgentURLProvider.userAgent( gumSpec.getAttribute( GumSpec.ATT_NAME ), Integer.parseInt( gumSpec
						.getAttribute( GumSpec.ATT_PORT ) ) ) );
		this.ua = (Gum) Naming.lookup( UserAgentURLProvider.userAgent( gumSpec.getAttribute( GumSpec.ATT_NAME ),
				Integer.parseInt( gumSpec.getAttribute( GumSpec.ATT_PORT ) ) ) );

	}


	protected void tearDown() throws Exception {

		/** running uaadmin shutdown */

		System.out.println( "tearDown Stopping UA..." );
		Runtime runtime = Runtime.getRuntime();
		Process process = runtime.exec( "bin/uaadmin stop " + gdf );
		if ( process.waitFor() != 0 ) {
			fail( "uaadmin failed!" );
		}

		Configuration.reset();
		testUtils.reset();

	}


	/**
	 * This test verifies if the remote file length determination works on
	 * several situations: - File has size Zero. - File is so "big"
	 * 
	 * @throws Exception
	 */
	public void testUA() throws Exception {

		File tempFile = new File( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + "/zeroUATest" );

		ua.startReplica();

		/** Try to create a temporary file */
		try {
			tempFile.createNewFile();

			/** Check the size returned by a getRemoteFileLength() */
			assertEquals( ua.getRemoteFileLength( tempFile ), 0 );

			tempFile.delete();
		} catch ( IOException e ) {
			e.printStackTrace();
			fail();
		}

		/**
		 * This method tests the Put File Method by trying the extreme
		 * situations!
		 */

		File nullFile = new File( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + "/null" );

		try {
			this.ua.putFile( this.ua.getStorageDirectory() + File.separator + nullFile.getName(), null, true );
			fail( "This should throw an IOException!" );
		} catch ( IOException e ) {
			// Expected exception
		}

		/**
		 * The the putFile() method when is intended to create a file on
		 * unauthorized area
		 */

		try {
			this.ua
					.putFile( testUtils.getProperty( TestUtils.PROTECTED_DIR ) + "/" + file.getName(), arrayOfData,
							true );
		} catch ( RemoteException e ) {
			e.printStackTrace();
			fail();
		} catch ( IOException e ) {
			System.out.println( e.getMessage() + ". IT WAS EXPECTED!" );
		}

		/** The temporary file used to test the putFile() method */
		File unauthorizedFile = new File( testUtils.getProperty( TestUtils.PROTECTED_DIR ) + "/xpto" );

		assertTrue( !unauthorizedFile.exists() );

		/**
		 * Test the putFile() method in a common situation. Store a non-null
		 * array of bytes in a file located at an authorized place. The idea is
		 * verify if FIRST-SLICE flag is working well. Store a 13-bytes-length
		 * array twice (firstly FIRST SLICE flag is TRUE, secondly the FIRST
		 * SLICE flag is FALSE) and check the file length. The expected result
		 * is the 26-bytes file length.
		 */

		try {
			this.ua.putFile( file.getAbsolutePath(), arrayOfData, true );
			this.ua.putFile( file.getAbsolutePath(), arrayOfData, false );
		} catch ( RemoteException e ) {
			e.printStackTrace();
			fail();
		} catch ( IOException e ) {
			e.printStackTrace();
			fail();
		}

		/**
		 * Test the get file method using an UserAgent created by default
		 * constructor!
		 */

		/** Received bytes */
		byte[ ] receivedBytes;

		/** Index */
		int sliceIndex;

		/** The received files */
		FileOutputStream outputFile = null;

		/** File created from bytes */
		File receivedFile;

		/** First action try to ping the new UA! */
		this.ua.getStatus();

		try {
			File remoteFile = new File( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + File.separator
					+ file.getName() );

			/** Just creates!!! */
			this.createNewFile( remoteFile );

			receivedFile = new File( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + "/ReceivedFile" );
			outputFile = new FileOutputStream( receivedFile );

			sliceIndex = 0;

			do {

				/** Get the transferred file */
				receivedBytes = this.ua.getFile( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) + File.separator
						+ file.getName(), sliceIndex * 13, 13 );

				sliceIndex++;

				/** Write to the file */
				outputFile.write( receivedBytes );

			} while ( receivedBytes.length == 13 );

			assertEquals( "Received and remote file sizes must be equals", remoteFile.length(), receivedFile.length() );

			/** Close the output stream */
			outputFile.close();

			/** Delete the files */
			receivedFile.delete();

		} catch ( IOException e ) {
			throw e;
		} finally {
			try {
				if ( outputFile != null ) {
					outputFile.close();
				}
			} catch ( IOException e1 ) {
				// Nothing to be done
			}
		}
		/**
		 * Test the get file method using an UserAgent created by default
		 * constructor!
		 */

		/** Received bytes */

		/** Index */
		sliceIndex = 0;

		try {
			/** Get the transferred file */
			receivedBytes = this.ua.getFile( testUtils.getProperty( TestUtils.FAKE_FILE ), sliceIndex * 13, 13 );
		} catch ( IOException e ) {
			System.out.println( "EXPECTED EXCEPTION: " + e.getClass() + ":: File does not exist." );
			assertFalse( "File should not exists.", (new File( testUtils.getProperty( TestUtils.FAKE_FILE ) )).exists() );
		}

		/**
		 * Test the createPlaypen() method by a UA constructed using the default
		 * constructor
		 * 
		 * @throws Exception If some error occurs during the test.
		 */

		/** Just ping to know if the object has been exported */
		this.ua.getStatus();

		/** Create a storage directory */
		File playpen = new File( this.ua.createPlaypen( 0 ) );

		assertTrue( "Playpen should exist!", playpen.exists() );

		/** A temp file used during the test */
		File regularFile = TempFileManager.createTempFile();
		regularFile.deleteOnExit();

		/** Just to test the object */
		this.ua.getStatus();

		/* parse the result */
		String permissions = JavaFileUtil.getFilePermissions( regularFile );

		System.out.println( "[" + getClass().getName() + ".testChangeFilePermissions()] Original file permissions: "
				+ permissions );

		/** Change the file permissions to rwxrwxrwx */
		this.ua.changeFilePermissions( regularFile, 777 );

		permissions = JavaFileUtil.getFilePermissions( regularFile );

		assertTrue( "The permissions should be 777", permissions.equals( "rwxrwxrwx" ) );

		try {
			/** Try to change the file permissions to an invalid number */
			this.ua.changeFilePermissions( regularFile, -1 );
		} catch ( RemoteException e ) {
			System.out.println( "EXPECTED EXCEPTION: Invalid Permissions <-1> :: " + e.getClass() );
			assertTrue( "The permissions should be 777", permissions.equals( "rwxrwxrwx" ) );
		}

		/** Change the file permissions to 000 */
		this.ua.changeFilePermissions( regularFile, 0 );

		permissions = JavaFileUtil.getFilePermissions( regularFile );

		assertTrue( "The permissions should be 000", permissions.equals( "---------" ) );

		try {
			/** Change the file permissions to another invalid number */
			this.ua.changeFilePermissions( regularFile, 999 );
		} catch ( RemoteException e ) {
			System.out.println( "EXPECTED EXCEPTION: Invalid Permissions <999> :: " + e.getClass() );
			assertTrue( "The permissions should be 000", permissions.equals( "---------" ) );
		}

		/**
		 * Try to destroy a playpen in several situations!
		 * 
		 * @throws Exception if some unexpected error occurs.
		 */

		this.ua.getStatus();

		playpen = new File( this.ua.createPlaypen( 0 ) );

		assertTrue( "Playpen [" + playpen.getAbsolutePath() + "] should exists.", playpen.exists() );

		this.ua.stopReplica();

		assertFalse( "Playpen [" + playpen.getAbsolutePath() + "] should not exist anymore.", playpen.exists() );

		/* Tests the existence of a remote file using an UA. */

		this.ua.startReplica();

		this.ua.getStatus();

		playpen = new File( this.ua.createPlaypen( 0 ) );

		assertTrue( "The file should exist.", this.ua.fileExists( playpen ) );

		/* This method tests the checksum generation is correct */

		this.ua.getStatus();

		/** Create a temporary file */
		regularFile = TempFileManager.createTempFile();

		String localDigest = JavaFileUtil.getDigestRepresentation( regularFile );
		String remoteDigest = this.ua.getRemoteFileDigest( regularFile );

		assertTrue( "The checksum should be equal.", localDigest.equals( remoteDigest ) );

		regularFile.delete();

		/** Different files */
		regularFile = TempFileManager.createTempFile();

		File regularRemoteFile = TempFileManager.createTempFileWithBogusData( "mygrid-test-", ".tmp", new File(
				testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ), 64 );

		localDigest = JavaFileUtil.getDigestRepresentation( regularFile );
		remoteDigest = this.ua.getRemoteFileDigest( regularRemoteFile );

		assertFalse( "The checksum should not be equal", localDigest.equals( remoteDigest ) );

		/*
		 * Test the execution of a correct command. The expected result is a
		 * correct execution of command.
		 */

		this.ua.getStatus();

		ExecutorHandle handle = this.ua.remoteExecute( ".", "ls -la /tmp", new HashMap<String,String>() );
		ExecutorResult result = this.ua.getResult( handle );

		assertEquals( result.getExitValue(), 0 );

		/**
		 * This method test the execution of a wrong command. The expected
		 * result is an erroneous execution of command because a permission
		 * denied problem.
		 * 
		 * @throws Exception
		 */

		result = null;
		handle = null;

		this.ua.getStatus();

		try {
			/** Permission denied */
			handle = this.ua.remoteExecute( testUtils.getProperty( TestUtils.REMOTE_PROTECTED_DIR ), "ls -la",
					new HashMap<String,String>() );
			result = this.ua.getResult( handle );
		} catch ( ExecutorException e ) {
			System.out.println( "EXPECTED EXCEPTION: " + e.getClass() + " :: " + e.getMessage() );
		}

		/**
		 * Use a inner thread object to test the kill method.
		 */

		ExecutorResult killResult = null;
		ExecutorHandle killHandle = null;

		killHandle = this.ua.remoteExecute( ".", "ls -Ral /", new HashMap<String,String>() );

		this.ua.stopReplica();

		try {
			killResult = this.ua.getResult( killHandle );
			fail( "An exception should be thrown." );
		} catch ( Exception e ) {
			System.out.println( "EXPECTED EXCEPTION: " + e );
		}

		assertNull( "The kill result must be null, as there is no result because the execution was killed.", killResult );

	}


	/**
	 * An auxiliar method just to create a new file filling with some bytes.
	 * 
	 * @param newFile the filename will be created at /tmp.
	 */

	private void createNewFile( File newFile ) throws Exception {

		FileOutputStream output = null;

		try {

			byte[ ] data = new byte[ 13 ];
			output = new FileOutputStream( newFile );
			newFile.deleteOnExit();

			for ( int i = 0; i < 2; i++ ) {
				output.write( data );
			}
			output.close();

		} catch ( IOException e ) {
			throw e;

		} finally {
			try {
				if ( output != null ) {
					output.close();
				}
			} catch ( IOException e1 ) {
				// Nothing to be done
			}

		}

	}

}
