/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent.ui;

/**
 * A message repository to hold the UserAgent messages to be displayed at any
 * interface.
 */
public enum UIErrors {
	INVALID_ARGUMENT, COULD_NOT_CREATE_REGISTRY, COULD_NOT_CONNECT_GUM, COULD_NOT_BIND, UNBOUNDED_OBJECT, SECURITY_FAIL, USERAGENT_STARTING_FAIL, USERAGENT_ALREADY_RUNNING, USERAGENT_IS_DOWN, SHUTDOWNMANAGER_STARTING_FAIL, MALFORMED_URL, OK, COULD_NOT_START_USERAGENT, INTERRUPTED_EXCEPTION, USERAGENT_ALREADY_DOWN;

	/**
	 * Get a exit code
	 * 
	 * @return
	 */
	public int getExitCode() {

		switch ( this ) {
			case OK:
				return 0;
			case INVALID_ARGUMENT:
				return 1;
			case COULD_NOT_CREATE_REGISTRY:
				return 2;
			case SECURITY_FAIL:
				return 3;
			case USERAGENT_STARTING_FAIL:
				return 4;
			case USERAGENT_ALREADY_RUNNING:
				return 5;
			case SHUTDOWNMANAGER_STARTING_FAIL:
				return 6;
			case MALFORMED_URL:
				return 7;
			case UNBOUNDED_OBJECT:
				return 8;
			case COULD_NOT_CONNECT_GUM:
				return 9;
			case USERAGENT_IS_DOWN:
				return 10;
			case COULD_NOT_BIND:
				return 11;
			case COULD_NOT_START_USERAGENT:
				return 12;
			case INTERRUPTED_EXCEPTION:
				return 13;
			case USERAGENT_ALREADY_DOWN:
				return 14;
			default:
				return -1;
		}
	}


	public String getMessage() {

		switch ( this ) {
			case INVALID_ARGUMENT:
				return "Invalid argument.";
			case USERAGENT_ALREADY_RUNNING:
				return "User Agent already running.";
			case COULD_NOT_CREATE_REGISTRY:
				return "Could not start User Agent." + " Problems when creating the RMI Registry.";
			case SECURITY_FAIL:
				return "Could not start User Agent using SSL communication infrastructure. Problems when creating/exporting UserAgent object.";
			case USERAGENT_STARTING_FAIL:
				return "Could not start User Agent: Change the \"ua.port\" value on \"UA.properties\" file.";
			case SHUTDOWNMANAGER_STARTING_FAIL:
				return "Could not start Shutdown Manager.";
			case MALFORMED_URL:
				return "Malformed URL.";
			case UNBOUNDED_OBJECT:
				return "Object is not bounded at registry.";
			case COULD_NOT_CONNECT_GUM:
				return "Could not connect UserAgent.";
			case USERAGENT_IS_DOWN:
				return "User agent is down.";
			case COULD_NOT_BIND:
				return "There was a RemoteException during the bind of this UserAgent.";
			case COULD_NOT_START_USERAGENT:
				return "Could not start UserAgent. Please, check your peer.properties file!";
			case INTERRUPTED_EXCEPTION:
				return "Interrupted exception.";
			case USERAGENT_ALREADY_DOWN:
				return "User Agent already down.";
			default:
				return "Unknow message.";
		}
	}

}
