/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent.ui;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.gum.GumStatus;

public interface UIManager extends Remote {

	/**
	 * Starts UserAgent Service
	 * 
	 * @throws RemoteException Thrown if occurs any RMI problem.
	 * @throws MalformedURLException Thrown if the bind address is malformed.
	 * @throws InterruptedException Thrown if the hold thread dies.
	 * @throws UserAgentUIException Thrown if the Gum is already started.
	 * @throws ConfigException
	 */
	public void startUserAgentService() throws RemoteException, MalformedURLException, InterruptedException,
		UserAgentUIException, ConfigException;


	/**
	 * Stops UserAgent Service
	 * 
	 * @throws RemoteException Thrown if occurs any RMI problem
	 * @throws UserAgentUIException Thrown if the Gum is already stopped.
	 * @throws NotBoundException Thrown if the Gum is not bounded at a registry.
	 * @throws MalformedURLException Thrown if the Gum address is malformed.
	 */
	public void stopUserAgentService() throws RemoteException, UserAgentUIException, MalformedURLException,
		NotBoundException;


	/**
	 * Gets the UserAgent status.
	 * 
	 * @return This Gum status.
	 * @throws RemoteException Thrown if occurs any RMI problem
	 */
	public GumStatus statusUserAgentService() throws RemoteException;

}
