/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.gridmachine.useragent.ui;

import static org.ourgrid.gridmachine.useragent.ui.UIErrors.OK;

import org.ourgrid.common.config.Configuration;

/**
 * A message repository to hold the UserAgent messages to be displayed at any
 * interface.
 */
public enum UIMessages {

	USERAGENT_STARTING, STARTING_SECURITY, STARTING_BIND, START_SUCCESSFUL, CHECKING_STATUS, STOP_SUCCESSFUL;

	public String getMessage() {

		switch ( this ) {
			case USERAGENT_STARTING:
				return "Starting User Agent...";
			case STARTING_SECURITY:
				return "Starting User Agent with secure socket factories...";
			case STARTING_BIND:
				return "Starting User Agent with default communication options...";
			case START_SUCCESSFUL:
				return "User Agent " + Configuration.VERSION + " started successfully.";
			case CHECKING_STATUS:
				return "Checking user agent status...";
			case STOP_SUCCESSFUL:
				return "User Agent " + Configuration.VERSION + " stopped successfully.";
			default:
				return "Unknow error.";
		}
	}


	/**
	 * Exit the JVM and update log and system output.
	 * 
	 * @param uiError UiError associated wiht this UI.
	 */
	public static void exitError( UIErrors uiError ) {

		System.out.println( uiError.getMessage() );
		System.exit( uiError.getExitCode() );
	}


	/**
	 * Exit the JVM and update log and system output.
	 */
	public static void exitOk( UIMessages uiConstant ) {

		System.out.println( uiConstant.getMessage() );
		System.exit( OK.getExitCode() );
	}

}
