/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.fd;

import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.Notifiable;
import org.ourgrid.common.fd.State;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.url.URLProvider;
import org.ourgrid.mygrid.scheduler.gump.GumpManager;

public class PeerStateNotifiable implements Notifiable {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( PeerStateNotifiable.class );

	private final String name = this.getClass().getName();

	private GumpManager gumpManager;


	public PeerStateNotifiable( GumpManager gumpManager ) {

		this.gumpManager = gumpManager;
	}


	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState ) {

		PeerSpec peer = null;
		try {

			peer = new PeerSpec( URLProvider.getHost( monitorableName ), URLProvider.getPort( monitorableName ) );

			if ( monitorableState.basicState().equals( State.UP ) ) {

				gumpManager.peerAlive( peer.getURL() );

			} else {

				gumpManager.peerIsDead( peer.getURL() );

			}

		} catch ( Exception e ) {
			LOG.debug( "Error receiving peer state change" );
		}

	}


	public String getName() {

		return name;
	}

}
