/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.event;

import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaManager;

/**
 * Description: A request event which specifies that all replicas of the Job
 * with id =<code>jobId</code> should be canceled.
 * 
 * @version 1.0 Created on 08/07/2004
 */
public class CancelReplicasOfJobRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/**
	 * The replica manager
	 */
	private EBReplicaManager ebReplicaManager;

	/**
	 * Id of the job whose replicas will be cancelled.
	 */
	private int jobId;


	/**
	 * Creates a CancelReplicasOfJobRequestEvent with a
	 * <code>EBReplicaManager</code> and a <code>jobId</code>
	 * 
	 * @param ebReplicaManager The current <code>EBReplicaManager</code>
	 * @param jobId The id of the Job whose replicas should be cancelled.
	 */
	public CancelReplicasOfJobRequestEvent( EBReplicaManager ebReplicaManager, int jobId ) {

		this.ebReplicaManager = ebReplicaManager;
		this.jobId = jobId;

	}


	/**
	 * Asks the <code>EBReplicaManager</code> to cancel all replicas of the
	 * job with id =<code>jobId</code>
	 */
	public void process() {

		ebReplicaManager.cancelReplicasOfJob( jobId );

	}

}
