/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.event;

import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Description: A request event which specifies that a <code>Replica</code>
 * should be executed in a given <code>GridMachine</code>
 * 
 * @version 1.0 Created on 08/07/2004
 */
public class ExecuteReplicaRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/**
	 * The replica manager
	 */
	private EBReplicaManager ebReplicaManager;

	/**
	 * The replica to be executed
	 */
	private ReplicaEntry replica;

	/**
	 * The grid machine where the replica will be executed
	 */
	private GumClient gumClient;


	/**
	 * Creates a CancelReplicasOfJobRequestEvent with a
	 * <code>EBReplicaManager</code> a <code>Replica</code> to be executed
	 * and a <code>GridMachine</code> to execute the Replica.
	 * 
	 * @param ebReplicaManager The replica manager
	 * @param replica he replica to be executed
	 * @param gumClient The grid machine where the replica will be executed
	 */
	public ExecuteReplicaRequestEvent( EBReplicaManager ebReplicaManager, ReplicaEntry replica, GumClient gumClient ) {

		this.ebReplicaManager = ebReplicaManager;
		this.replica = replica;
		this.gumClient = gumClient;

	}


	/**
	 * Asks the <code>EBReplicaManager</code> to execute the
	 * <code>Replica</code> int the specified <code>GridExtendedMachine</code>
	 */
	public void process() {

		ebReplicaManager.executeReplica( replica, gumClient );

	}

}
