/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.event;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Description: Description: A request event which specifies that a replica
 * wants to access it's critical region.
 * 
 * @version 1.0 Created on 09/07/2004
 */
public class PermissionRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/**
	 * The replica ebReplicaManager
	 */
	private EBReplicaManager ebReplicaManager;

	/**
	 * The replica who wants to access it's critical region.
	 */
	private ReplicaEntry replica;

	/**
	 * The response queue where the <code>PermissionManager</code> will put
	 * the permission to access the critical region, if it decides to.
	 */
	private EventQueue<ResponseEvent<Boolean>> responseQueue;


	/**
	 * Creates a <code>PermissionRequestEvent</code> with a replica
	 * ebReplicaManager, a replica and a response queue.
	 * 
	 * @param ebReplicaManager The current replica ebReplicaManager
	 * @param replica The replica who wants to access it's critical region.
	 * @param responseQueue The replica's response queue.
	 */
	public PermissionRequestEvent( EBReplicaManager ebReplicaManager, ReplicaEntry replica,
									EventQueue<ResponseEvent<Boolean>> responseQueue ) {

		this.ebReplicaManager = ebReplicaManager;
		this.replica = replica;
		this.responseQueue = responseQueue;
	}


	/**
	 * Sends to the replica ebReplicaManager a replica's permission request.
	 */
	public void process() {

		ebReplicaManager.requestPermission( replica, responseQueue );
	}

}
