/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.event;

import org.ourgrid.common.event.NotifyEvent;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Description: A notify event that informs that a replica has been aborted.
 * 
 * @version 1.0 Created on 09/07/2004
 */
public class ReplicaAbortedNotiftyEvent implements NotifyEvent {

	private static final long serialVersionUID = 33L;

	/**
	 * The replica ebReplicaManager
	 */
	private EBReplicaManager ebReplicaManager;

	/**
	 * The replica who has been aborted.
	 */
	private ReplicaEntry replica;


	/**
	 * Creates a <code>ReplicaAbortedNotifyEvent</code> with a replica
	 * ebReplicaManager and a replica.
	 * 
	 * @param ebReplicaManager The current replica ebReplicaManager
	 * @param replica The replica who has been aborted.
	 */
	public ReplicaAbortedNotiftyEvent( EBReplicaManager ebReplicaManager, ReplicaEntry replica ) {

		this.ebReplicaManager = ebReplicaManager;
		this.replica = replica;
	}


	/**
	 * Notifies the replica ebReplicaManager that a replica has been aborted.
	 */
	public void process() {

		ebReplicaManager.replicaAborted( replica );
	}
}
