/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.exception;

import org.ourgrid.common.executor.ExecutorResult;
import org.ourgrid.gridmachine.exception.FailCauseException;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Description: Thrown whenever a replica fails.
 * 
 * @version 1.0 Created on 14/07/2004
 */
public class ReplicaFailedException extends FailCauseException {

	private static final long serialVersionUID = 33L;

	/**
	 * The Executor result
	 */
	private ExecutorResult result;

	/**
	 * The replica that failed
	 */
	private ReplicaEntry replica;


	/**
	 * Main constructor.
	 * 
	 * @param replica The replica that failed.
	 * @param replicaResult The replica result
	 */
	public ReplicaFailedException( ReplicaEntry replica, ExecutorResult replicaResult ) {

		super();
		this.replica = replica;
		result = replicaResult;

	}


	/**
	 * Constructor.
	 * 
	 * @param replica The replica that failed.
	 * @param replicaResult The replica result
	 * @param newDetail Possible cause for the problem
	 */
	public ReplicaFailedException( ReplicaEntry replica, ExecutorResult replicaResult, Throwable newDetail ) {

		super( newDetail );
		this.replica = replica;
		result = replicaResult;

	}


	/**
	 * Gets the result of the replica that failed
	 * 
	 * @return The replica result
	 */
	public ExecutorResult getReplicaResult() {

		return result;

	}


	/**
	 * Gets the replica
	 * 
	 * @return The replica that failed
	 */
	public ReplicaEntry getReplica() {

		return replica;

	}


	/**
	 * Gets a formatted message for this exception
	 * 
	 * @return A formatted message for this exception
	 */
	public String getMessage() {

		String message = "\n  Replica " + replica.toString() + " has failed during execution!\n";
		message = message + "    Stdout: " + result.getStdout() + "\n";
		message = message + "    Stderr: " + result.getStderr() + "\n";
		message = message + "    Exit value: " + result.getExitValue();
		if ( this.getCause() != null ) {
			message = message + "\n    Nested exception: " + getCause().getClass().getName() + ":"
					+ getCause().getMessage();
		}
		return message;

	}


	@Override
	public String getSimpleMessage() {

		return "Your task did not finish properly on the Grid Machine";
	}

}
