/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.test;

import java.io.File;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.executor.Executor;
import org.ourgrid.common.executor.ExecutorFactory;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.test.LocalPeer;
import org.ourgrid.test.ReplicaExecutorPreCommitTest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

/**
 * This class implements an automatic test for a stress situation when using a
 * grid composed of User Agents. The idea is to test the integration between the
 * User Agent features and the rest of components.
 */
public class EBReplicaExecutorFacadePreCommitUATest extends TestCase implements ReplicaExecutorPreCommitTest,
		SpecialCareTest {

	/* A RMI Registry used to publish UA remote objects */
	Registry registry;

	private TestUtils testUtils;

	private String MGROOT;


	/**
	 * This method init the objects used during this test.
	 */
	public void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();

		testUtils.load();
		testUtils.createMGProperties();
		testUtils.createOGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		testUtils.restartMygrid();

		MGROOT = (new File( Configuration.getInstance( Configuration.MYGRID ).getProperty( Configuration.MGROOT ) ))
				.getCanonicalPath();

	}


	/**
	 * This method release the resources used.
	 */
	public void tearDown() throws Exception {

		super.tearDown();

		/** Stop MyGrid */
		Configuration.reset();
		Configuration.getInstance( Configuration.MYGRID );

		testUtils.stopMygrid();

		Configuration.reset();
		testUtils.reset();
	}


	/** Test the submission and execution of a Job */
	public void testEntireExecutionCycle() throws Exception {

		GumSpec gumSpec = testUtils.getLocalGuMSpec( TestUtils.TYPE_UALINUX );

		List<GumSpec> gumList = new ArrayList<GumSpec>( 1 );
		gumList.add( gumSpec );

		LocalPeer localPeer = new LocalPeer( Integer.parseInt( testUtils.getProperty( TestUtils.OG_PORT ) ) );

		System.out.println( "Starting a local peer ..." );

		localPeer.start();
		System.out.println( "Setting gums to this peer ..." );
		localPeer.setGuMs( new File( testUtils.createSDF( gumList ) ) );

		/* This is a list of GuMPs */
		List<PeerSpec> gumpList = new ArrayList<PeerSpec>( 1 );
		gumpList.add( localPeer.getGuMPSpec() );

		/* Set the list of GuMPs */
		MyGridUIManager.getInstance().setPeers( gumpList );

		Configuration.reset();

		Configuration.getInstance( Configuration.MYGRID );

		Executor exec = ExecutorFactory.getExecutorInstance();

		new File( MGROOT + File.separator + testUtils.getProperty( "job.storage" ) );

		Configuration.reset();

		Configuration.getInstance( Configuration.USERAGENT );

		assertEquals( 0, (exec.getResult( exec.execute( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ), MGROOT
				+ "/bin/mygrid addjob " + MGROOT + "/" + testUtils.getProperty( "job.storage" ) ) ).getExitValue()) );

		assertEquals( 0, (exec.getResult( exec.execute( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ), MGROOT
				+ "/bin/mygrid waitforjob 1" ) ).getExitValue()) );

		File f = new File( Configuration.getInstance().getProperty( Configuration.MGROOT )
				+ "/test/resources/mytask-1.out" );
		assertTrue( f.exists() );

		f.delete();

		localPeer.stop();

	}

}
