/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.test;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.gum.FakeGumClient;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaExecutorFacade;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaManager;
import org.ourgrid.mygrid.replicaexecutor.ReplicaExecutorThreadManager;
import org.ourgrid.mygrid.scheduler.FakeEBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.test.ReplicaExecutorFastTest;
import org.ourgrid.test.TestUtils;
import org.ourgrid.test.threadServices.ThreadPoolServices;
import org.ourgrid.test.threadServices.ThreadServices;

/**
 * <code>EBReplicaManager</code> unit's test.
 * 
 * @version 1.0 Created on 12/07/2004
 */
public class EBReplicaManagerTest extends TestCase implements ReplicaExecutorFastTest {

	private static final int NUMBER_OF_JOBS = 1;

	private static final int TASKS_PER_JOB = 2;

	private static final int REPLICAS_PER_TASK = 3;

	private GumClient gumClient;

	private ReplicaEntry replicas[][][];

	private ReplicaExecutorThreadManager replicaExecutorThreadManager;

	private EBReplicaManager ebReplicaManager;

	private EBReplicaExecutorFacade ebReplicaExecutorFacade;

	private TestUtils testUtils;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( EBReplicaManagerTest.class );
	}


	public void setUp() throws RemoteException {

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		FakeEBSchedulerFacade ebSchedulerFacade = new FakeEBSchedulerFacade();

		ebSchedulerFacade.ignoreCalls();

		ebReplicaExecutorFacade = new EBReplicaExecutorFacade();
		ebReplicaExecutorFacade.config( new FakeReplicaExecutorEventEngine( ebSchedulerFacade,
				new ReplicaExecutorThreadManager( ebReplicaExecutorFacade, ReplicaExecutorThreadManager.FAKE, 200 ) ) );

		ebReplicaManager = ebReplicaExecutorFacade.getEventProcessor().getEBReplicaManager();
		replicaExecutorThreadManager = ((FakeReplicaExecutorEventEngine) ebReplicaExecutorFacade.getEventProcessor())
				.getThreadManager();

		Map<String,String> attrib = new HashMap<String,String>();
		attrib.put( "os", "Linux" );
		attrib.put( "site", "LSD" );

		GumID gumID = new GumID( TestUtils.FAKE_MACHINE_NAME, "0", "peer", "0" );

		GumSpec gumSpec = new GumSpec( attrib );
		gumClient = new FakeGumClient( gumID, gumSpec );

		replicas = new ReplicaEntry[ NUMBER_OF_JOBS ][ TASKS_PER_JOB ][ REPLICAS_PER_TASK ];
		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					replicas[jobId][taskId][replicaId] = new ReplicaEntry( replicaId, taskId, jobId, FakeTaskSpec
							.getFakeTaskSpec() );
				}
			}
		}

	}


	public void tearDown() throws Exception {

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					replicas[jobId][taskId][replicaId] = null;
				}
			}
		}

		replicas = null;

		ThreadPoolServices.waitUntilWorkIsDone();

		ebReplicaManager.shutdown();

		Configuration.reset();
		testUtils.reset();

	}


	public void testExecuteReplica() {

		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaExecutorFacade.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaExecutorFacade.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

	}


	public void testReplicaCanceled() {

		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaExecutorFacade.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();

		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				ebReplicaExecutorFacade.replicaCanceled( replicas[jobId][taskId][0] );
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should have all replicas not cancelled", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * (REPLICAS_PER_TASK - 1) );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaExecutorFacade.replicaCanceled( replicas[jobId][taskId][replicaId] );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

	}


	public void testReplicaFailed() {

		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaExecutorFacade.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				ebReplicaManager.replicaFailed( replicas[jobId][taskId][0] );
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should have all not failed threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * (REPLICAS_PER_TASK - 1) );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.replicaFailed( replicas[jobId][taskId][replicaId] );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

	}


	public void testReplicaFinished() {

		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				ebReplicaManager.replicaFinished( replicas[jobId][taskId][0] );
			}
		}

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.replicaFailed( replicas[jobId][taskId][replicaId] );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

	}


	public void testReplicaAborted() {

		assertEquals( "The threads collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The threads collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				ebReplicaManager.replicaAborted( replicas[jobId][taskId][0] );
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The  threads collection should have all not aborted replicas", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * (REPLICAS_PER_TASK - 1) );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.replicaAborted( replicas[jobId][taskId][replicaId] );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The thread collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );
	}


	public void testCancelReplicasOfJob() {

		assertEquals( "The thread collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			for ( int taskId = 0; taskId < TASKS_PER_JOB; taskId++ ) {
				for ( int replicaId = 0; replicaId < REPLICAS_PER_TASK; replicaId++ ) {
					ebReplicaManager.executeReplica( replicas[jobId][taskId][replicaId], gumClient );
				}
			}
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The thread collection should have all threads ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), NUMBER_OF_JOBS * TASKS_PER_JOB * REPLICAS_PER_TASK );

		ebReplicaExecutorFacade.cancelReplicasOfJob( NUMBER_OF_JOBS - 1 );

		for ( int jobId = 0; jobId < NUMBER_OF_JOBS; jobId++ ) {
			ebReplicaExecutorFacade.cancelReplicasOfJob( jobId );
		}

		ThreadServices.waitUntilWorkIsDone();
		assertEquals( "The thread collection should be empty ", replicaExecutorThreadManager
				.getReplicaExecutorThreads().size(), 0 );

	}

}
