/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.test;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.EBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.ReplicaExecutorResult;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Description: A fake class to be used by unit tests.
 * 
 * @version 1.0 Created on 13/07/2004
 */
public class FakeEBReplicaExecutorFacade extends EBReplicaExecutorFacade {

	private List<ReplicaEntry> replicas;

	private ExecutionStatus replicaStateToBeReturned = ExecutionStatus.RUNNING;

	private EBSchedulerFacade ebSchedulerFacade;

	private ReplicaEntry lastScheduledReplica;


	public FakeEBReplicaExecutorFacade() {

		replicas = new LinkedList<ReplicaEntry>();
	}


	public void executeReplica( ReplicaEntry replica, GumClient gumClient ) {

		replicas.add( replica );
		lastScheduledReplica = replica;
		if ( replicaStateToBeReturned.equals( ExecutionStatus.FINISHED ) ) {
			ebSchedulerFacade.replicaFinished( new ReplicaExecutorResult( replica ) );
		}
		if ( replicaStateToBeReturned.equals( ExecutionStatus.FAILED ) ) {
			ebSchedulerFacade.replicaFailed( new ReplicaExecutorResult( replica ) );
		}
	}


	public void setReplicaStateToBeReturned( ExecutionStatus state ) {

		replicaStateToBeReturned = state;
	}


	public void cancelReplicasOfJob( int jobId ) {

		Iterator replicasIt = replicas.iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry replica = (ReplicaEntry) replicasIt.next();
			ebSchedulerFacade.replicaCanceled( new ReplicaExecutorResult( replica ) );
		}
	}


	public void config( EBSchedulerFacade ebSchedulerFacade ) {

		this.ebSchedulerFacade = ebSchedulerFacade;
	}


	public void replicaCanceled( ReplicaEntry replica ) {

		// Do nothing
	}


	public void replicaFailed( ReplicaEntry replica ) {

		// Do nothing
	}


	public void replicaFinished( ReplicaEntry replica ) {

		// Do nothing
	}


	public void replicaAborted( ReplicaEntry replica ) {

		// Do nothing
	}


	protected EventQueue requestPermission( ReplicaEntry replica ) {

		// Do nothing
		return null;
	}


	public EventQueue shutdown() {

		// Do nothing
		return null;
	}


	public void startEventProcessor() {

		// Do nothing
	}


	public ReplicaEntry getLastScheduledReplica() {

		return lastScheduledReplica;
	}

}
