/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.replicaexecutor.test;

import java.util.Collection;

import junit.framework.TestCase;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.mygrid.replicaexecutor.PermissionManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.test.ReplicaExecutorFastTest;

/**
 * Description: <code>PermissionManager</code>'s unit tests.
 * 
 * @version 1.0 Created on 12/07/2004
 */
public class PermissionManagerTest extends TestCase implements ReplicaExecutorFastTest {

	private PermissionManager permissionManager;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( PermissionManagerTest.class );
	}


	@Override
	public void setUp() throws Exception {

		permissionManager = new PermissionManager();

	}


	@Override
	public void tearDown() throws Exception {

		permissionManager.shutdown();

	}


	public void testReplicaFailed() throws Exception {

		ReplicaEntry replicas[][] = new ReplicaEntry[ 2 ][ 3 ];

		for ( int taskId = 0; taskId < 2; taskId++ ) {
			for ( int replicaId = 0; replicaId < 3; replicaId++ ) {
				replicas[taskId][replicaId] = new ReplicaEntry( replicaId, taskId, 1, FakeTaskSpec.getFakeTaskSpec() );
				permissionManager.requestPermission( replicas[taskId][replicaId],
						new EventQueue<ResponseEvent<Boolean>>() );
			}
		}

		Collection runningReplicas = permissionManager.getRunningReplicas();
		Collection blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 2", 2, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );
		assertTrue( "The runningReplicas collection should contain the replica replicas[1][0]", runningReplicas
				.contains( replicas[1][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 4", 4, blockedReplicas.size() );

		permissionManager.replicaFailed( replicas[0][0] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 2", 2, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas[0][1]", runningReplicas
				.contains( replicas[0][1] ) );

		assertEquals( "The size of the collection blockedReplicas should be 3", 3, blockedReplicas.size() );

		permissionManager.replicaFailed( replicas[1][2] );
		permissionManager.replicaFailed( replicas[1][2] );
		permissionManager.replicaFailed( replicas[1][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection blockedReplicas should be 2", 2, blockedReplicas.size() );
		assertFalse( "The blockingReplicas collection should not contain the replica replicas[1][2]", blockedReplicas
				.contains( replicas[1][2] ) );

		permissionManager.replicaFailed( new ReplicaEntry( 100, 100, 100, FakeTaskSpec.getFakeTaskSpec() ) );

		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection blockedReplicas should be 2", 2, blockedReplicas.size() );

	}


	public void testReplicaAborted() throws Exception {

		ReplicaEntry replicas[][] = new ReplicaEntry[ 2 ][ 3 ];

		for ( int taskId = 0; taskId < 2; taskId++ ) {
			for ( int replicaId = 0; replicaId < 3; replicaId++ ) {
				replicas[taskId][replicaId] = new ReplicaEntry( replicaId, taskId, 1, FakeTaskSpec.getFakeTaskSpec() );
				permissionManager.requestPermission( replicas[taskId][replicaId],
						new EventQueue<ResponseEvent<Boolean>>() );
			}
		}

		Collection runningReplicas = permissionManager.getRunningReplicas();
		Collection blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 2", 2, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );
		assertTrue( "The runningReplicas collection should contain the replica replicas[1][0]", runningReplicas
				.contains( replicas[1][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 4", 4, blockedReplicas.size() );

		permissionManager.replicaAborted( replicas[0][0] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 1", 1, runningReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 2", 2, blockedReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][1]", runningReplicas
				.contains( replicas[0] ) );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][2]", runningReplicas
				.contains( replicas[0] ) );

		permissionManager.replicaAborted( replicas[1][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaAborted( replicas[1][0] );

		permissionManager.replicaAborted( replicas[0][2] );
		permissionManager.replicaAborted( replicas[0][2] );
		permissionManager.replicaAborted( replicas[0][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaAborted( new ReplicaEntry( 100, 100, 100, FakeTaskSpec.getFakeTaskSpec() ) );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

	}


	public void testReplicaFinished() throws Exception {

		ReplicaEntry replicas[][] = new ReplicaEntry[ 2 ][ 3 ];

		for ( int taskId = 0; taskId < 2; taskId++ ) {
			for ( int replicaId = 0; replicaId < 3; replicaId++ ) {
				replicas[taskId][replicaId] = new ReplicaEntry( replicaId, taskId, 1, FakeTaskSpec.getFakeTaskSpec() );
				permissionManager.requestPermission( replicas[taskId][replicaId],
						new EventQueue<ResponseEvent<Boolean>>() );
			}
		}

		Collection runningReplicas = permissionManager.getRunningReplicas();
		Collection blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 2", 2, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );
		assertTrue( "The runningReplicas collection should contain the replica replicas[1][0]", runningReplicas
				.contains( replicas[1][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 4", 4, blockedReplicas.size() );

		permissionManager.replicaFinished( replicas[0][0] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 1", 1, runningReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 2", 2, blockedReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][1]", runningReplicas
				.contains( replicas[0] ) );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][2]", runningReplicas
				.contains( replicas[0] ) );

		permissionManager.replicaFinished( replicas[1][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaFinished( replicas[1][0] );

		permissionManager.replicaFinished( replicas[0][2] );
		permissionManager.replicaFinished( replicas[0][2] );
		permissionManager.replicaFinished( replicas[0][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaFinished( new ReplicaEntry( 100, 100, 100, FakeTaskSpec.getFakeTaskSpec() ) );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

	}


	public void testTwoReplicaFinished() throws Exception {

		ReplicaEntry replica1 = new ReplicaEntry( 1, 1, 1, FakeTaskSpec.getFakeTaskSpec() );
		ReplicaEntry replica2 = new ReplicaEntry( 2, 1, 1, FakeTaskSpec.getFakeTaskSpec() );

		permissionManager.requestPermission( replica1, new EventQueue<ResponseEvent<Boolean>>() );

		Collection runningReplicas = permissionManager.getRunningReplicas();

		assertEquals( "The size of the collection runningReplicas should be 1", 1, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas", runningReplicas
				.contains( replica1 ) );

		permissionManager.replicaFinished( replica1 );

		runningReplicas = permissionManager.getRunningReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas", runningReplicas
				.contains( replica1 ) );

		EventQueue<ResponseEvent<Boolean>> eventQueue = new EventQueue<ResponseEvent<Boolean>>();
		permissionManager.requestPermission( replica2, eventQueue );
		assertTrue( eventQueue.blockingRemove().getResponse() == false );
	}


	public void testReplicaCanceled() throws Exception {

		ReplicaEntry replicas[][] = new ReplicaEntry[ 2 ][ 3 ];

		for ( int taskId = 0; taskId < 2; taskId++ ) {
			for ( int replicaId = 0; replicaId < 3; replicaId++ ) {
				replicas[taskId][replicaId] = new ReplicaEntry( replicaId, taskId, 1, FakeTaskSpec.getFakeTaskSpec() );
				permissionManager.requestPermission( replicas[taskId][replicaId],
						new EventQueue<ResponseEvent<Boolean>>() );
			}
		}

		Collection runningReplicas = permissionManager.getRunningReplicas();
		Collection blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 2", 2, runningReplicas.size() );
		assertTrue( "The runningReplicas collection should contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );
		assertTrue( "The runningReplicas collection should contain the replica replicas[1][0]", runningReplicas
				.contains( replicas[1][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 4", 4, blockedReplicas.size() );

		permissionManager.replicaCanceled( replicas[0][0] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 1", 1, runningReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][0]", runningReplicas
				.contains( replicas[0][0] ) );

		assertEquals( "The size of the collection blockedReplicas should be 2", 2, blockedReplicas.size() );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][1]", runningReplicas
				.contains( replicas[0][1] ) );
		assertFalse( "The runningReplicas collection should not contain the replica replicas[0][2]", runningReplicas
				.contains( replicas[0][2] ) );

		permissionManager.replicaCanceled( replicas[1][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaCanceled( replicas[1][0] );

		permissionManager.replicaCanceled( replicas[0][2] );
		permissionManager.replicaCanceled( replicas[0][2] );
		permissionManager.replicaCanceled( replicas[0][2] );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

		permissionManager.replicaCanceled( new ReplicaEntry( 100, 100, 100, FakeTaskSpec.getFakeTaskSpec() ) );

		runningReplicas = permissionManager.getRunningReplicas();
		blockedReplicas = permissionManager.getBlockedReplicas();

		assertEquals( "The size of the collection runningReplicas should be 0", 0, runningReplicas.size() );

		assertEquals( "The size of the collection blockedReplicas should be 0", 0, blockedReplicas.size() );

	}
}
