/* Created at 26/04/2006 */

package org.ourgrid.mygrid.scheduler;

import java.util.HashSet;
import java.util.Set;

import org.ourgrid.common.id.GumID;

/**
 * Entity used to associate the GumID and a certain number of fails tha can
 * occurs while executing a request.<br>
 * <br>
 */
public class BlackListEntry {

	/**
	 * The GumID represented by this entry
	 */
	private GumID gumID;

	/**
	 * The remaining number of fails to the Gum enter in the black list
	 */
	private int numberOfTasksThatCanFail;

	/**
	 * Failed task
	 */
	private Set<Integer> taskIDs;


	/**
	 * Constructs a new BlackListEntry.
	 * 
	 * @param gumid The GumID represented by this entry
	 * @param numberOfFailures The remaining number of fails to the Gum enter in
	 *        the black list
	 */
	public BlackListEntry( GumID gumid, int numberOfFailures ) {

		gumID = gumid;
		numberOfTasksThatCanFail = numberOfFailures;
		this.taskIDs = new HashSet<Integer>();
	}


	/**
	 * @return The GumID represented by this entry.
	 */
	public GumID getGumID() {

		return gumID;
	}


	@Override
	public boolean equals( Object obj ) {

		if ( obj instanceof BlackListEntry ) {
			BlackListEntry entry = (BlackListEntry) obj;
			return getGumID().equals( entry.getGumID() );
		}
		return false;
	}


	@Override
	public int hashCode() {

		return getGumID().hashCode();
	}


	/**
	 * Add a task to this Gum black list.
	 * 
	 * @param taskID The task to be added
	 * @return True if the task is new in the task list, false otherwise.
	 */
	public boolean taskFail( int taskID ) {

		return this.taskIDs.add( taskID );
	}


	public int getNumberOfRemainingFailedExecutions() {

		return this.numberOfTasksThatCanFail - this.taskIDs.size();
	}


	public Set<Integer> getTaskIDs() {

		return this.taskIDs;
	}
}
