/* Created at 26/04/2006 */

package org.ourgrid.mygrid.scheduler;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.common.id.GumID;
import org.ourgrid.test.SchedulerPreCommitTest;
import org.ourgrid.test.TestUtils;

public class BlackListManagerTest extends TestCase implements SchedulerPreCommitTest {

	private BlackListManager blackListManager;

	private TestUtils testUtils;


	protected void setUp() throws Exception {

		Configuration.reset();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.setProperty( MyGridConfiguration.PREFIX + MyGridConfiguration.PROP_MAX_BL_FAILS,
				MyGridConfiguration.DEFAULT_MAX_BL_FAILS );
		testUtils.createMGProperties();
	}


	protected void tearDown() throws Exception {

		Configuration.reset();
		testUtils.reset();
	}


	public void testConstructor() {

		try {
			this.blackListManager = new BlackListManager();
			fail();
		} catch ( Error e ) {
			// expected exception, it is necessary to create a configuration
			// before create the black
			// list.
		}
	}


	public void testAddEntry() {

		Configuration.getInstance( Configuration.MYGRID );
		this.blackListManager = new BlackListManager();

		final int jobID = 10;
		final GumID gumID = new GumID( "gum", "10", "peeer", "11" );
		final int maxBlFails = this.blackListManager.getMaxBLFails();
		final int jobSize = maxBlFails + 1;
		final int taskID = 1;

		this.blackListManager.initiateBlackListForJob( jobID, jobSize );

		assertEquals( maxBlFails - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		// To guarantee that the second call is not affect the structure
		assertEquals( maxBlFails - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );

		assertEquals( gumID, this.blackListManager.getBlackListedMachines( jobID ).iterator().next().getGumID() );

		assertEquals( maxBlFails - 1, this.blackListManager.getRemainingBLFails( jobID, gumID ) );

		assertFalse( this.blackListManager.canThisGumRunThisTask( jobID, gumID, taskID ) );

		assertTrue( this.blackListManager.canThisGumRunThisTask( jobID, gumID, taskID + 1 ) );
		assertTrue( this.blackListManager.canThisGumRunThisTask( jobID + 1, gumID, taskID + 1 ) );
	}


	public void testAddEntry1() {

		Configuration.getInstance( Configuration.MYGRID );
		this.blackListManager = new BlackListManager();

		final int jobID = 10;
		final GumID gumID = new GumID( "gum", "10", "peeer", "11" );
		final int maxBlFails = this.blackListManager.getMaxBLFails();
		final int jobSize = maxBlFails - 1;
		final int taskID = 1;

		this.blackListManager.initiateBlackListForJob( jobID, jobSize );

		assertEquals( jobSize - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		// To guarantee that the second call is not affect the structure
		assertEquals( jobSize - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );

		assertEquals( gumID, this.blackListManager.getBlackListedMachines( jobID ).iterator().next().getGumID() );

		assertEquals( jobSize - 1, this.blackListManager.getRemainingBLFails( jobID, gumID ) );

		assertFalse( this.blackListManager.canThisGumRunThisTask( jobID, gumID, taskID ) );
		assertTrue( this.blackListManager.canThisGumRunThisTask( jobID, gumID, taskID + 1 ) );
		assertTrue( this.blackListManager.canThisGumRunThisTask( jobID + 1, gumID, taskID + 1 ) );
	}


	public void testIsGumInBlackListForJob() {

		Configuration.getInstance( Configuration.MYGRID );
		this.blackListManager = new BlackListManager();

		final int jobID = 10;
		final GumID gumID = new GumID( "gum", "10", "peeer", "11" );
		final int maxBlFails = this.blackListManager.getMaxBLFails();
		final int jobSize = 1;
		final int taskID = 1;

		this.blackListManager.initiateBlackListForJob( jobID, jobSize );
		assertFalse( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID ) );

		assertEquals( jobSize - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		// To guarantee that the second call is not affect the structure
		assertEquals( jobSize - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		assertEquals( gumID, this.blackListManager.getBlackListedMachines( jobID ).iterator().next().getGumID() );
		assertEquals( jobSize - 1, this.blackListManager.getRemainingBLFails( jobID, gumID ) );

		assertTrue( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID ) );
		final GumID gumID2 = new GumID( "gum2", "102", "peeer2", "112" );
		assertFalse( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID2 ) );
	}


	public void testIsGumInBlackListForJob2() {

		Configuration.getInstance( Configuration.MYGRID );
		this.blackListManager = new BlackListManager( 1 );

		final int jobID = 10;
		final GumID gumID = new GumID( "gum", "10", "peeer", "11" );
		final int maxBlFails = this.blackListManager.getMaxBLFails();
		final int jobSize = 10;
		final int taskID = 1;

		this.blackListManager.initiateBlackListForJob( jobID, jobSize );
		assertFalse( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID ) );

		assertEquals( maxBlFails - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		// To guarantee that the second call is not affect the structure
		assertEquals( maxBlFails - 1, this.blackListManager.executionFailed( jobID, gumID, taskID ) );
		assertEquals( gumID, this.blackListManager.getBlackListedMachines( jobID ).iterator().next().getGumID() );
		assertEquals( maxBlFails - 1, this.blackListManager.getRemainingBLFails( jobID, gumID ) );

		assertTrue( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID ) );
		final GumID gumID2 = new GumID( "gum2", "102", "peeer2", "112" );
		assertFalse( this.blackListManager.isGumBlacklistedForEntireJob( jobID, gumID2 ) );
	}
}
