/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManager;

/**
 * This class is responsible for creating a <code>EBSchedulingHeuristic</code>
 * object according to the <i>heuristic</i> property in the MG.properties file
 */
public class EBSchedulingHeuristicFactory {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( EBSchedulingHeuristicFactory.class );


	/** The constructor. */
	public EBSchedulingHeuristicFactory() {

	}


	/**
	 * Obtains a scheduling heuristic according to the parameters.
	 * 
	 * @param heuristic Heuristic name. Possible values are: Workqueue or
	 *        StorageAffinity
	 * @param jobManager The <code>EBJobManager</code>
	 * @param ebGridManager The <code>EBGridManager</code>
	 * @param ebReplicaExecutorFacade The <code>EBReplicaExecutorFacade</code>
	 * @return The <code>EBSchedulingHeuristic</code>
	 * @throws ConfigException When an error occurred while reading the
	 *         MG.properties file
	 */
	public EBSchedulingHeuristic getHeuristic( String heuristic, JobManager jobManager, GridManager ebGridManager,
												EBReplicaExecutorFacade ebReplicaExecutorFacade )
		throws ConfigException {

		if ( heuristic.equalsIgnoreCase( "StorageAffinity" ) ) {
			LOG.debug( "Loading StorageAffinity scheduler..." );
			return new StorageAffinity( jobManager, ebGridManager, ebReplicaExecutorFacade );
		} else if ( heuristic.equalsIgnoreCase( "Workqueue" ) ) {
			LOG.debug( "Loading Workqueue scheduler..." );
			return new Workqueue( jobManager, ebGridManager, ebReplicaExecutorFacade );
		} else {
			// Returning a default scheduler
			LOG.error( "Invalid Scheduler Name. Loading Workqueue default scheduler..." );
			return new Workqueue( jobManager, ebGridManager, ebReplicaExecutorFacade );
		}
	}

}
