/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler;

/**
 * Enumeration representing the possible states of a Job, Task or Replica. There
 * are six possible states:
 * <ul>
 * <li><b>UNSTARTED</b>: the replica is ready to run. </li>
 * <li><b>RUNNING</b>: replica is running. </li>
 * <li><b>FINISHED</b>: replica has been succesfully executed. </li>
 * <li><b>CANCELLED</b>: replica has been cancelled (the end-user has
 * cancelled the job) </li>
 * <li><b>ABORTED</b>: replica execution has been interrupted (due to the end
 * of another replica of the same task )</li>
 * <li><b>FAILED</b>: replica execution has failed on the remote machine.
 * </li>
 * </ul>
 */
public enum ExecutionStatus {

	/* Available states */
	UNSTARTED, RUNNING, FINISHED, CANCELLED, ABORTED, FAILED;

	/* returns true if the state is RUNNING */
	public boolean isRunning() {

		return (this.equals( RUNNING ));
	}


	/* returns true if the state is UNSTARTED (or READY) */
	public boolean isUnstarted() {

		return (this.equals( UNSTARTED ));
	}


	/* returns true if the state is FINISHED */
	public boolean isFinished() {

		return (this.equals( FINISHED ));
	}


	/* returns true if the state is FAILED */
	public boolean isFailed() {

		return (this.equals( FAILED ));
	}


	/* returns true if the state is CANCELLED */
	public boolean isCancelled() {

		return (this.equals( CANCELLED ));
	}


	/* returns true if the state is ABORTED */
	public boolean isAborted() {

		return (this.equals( ABORTED ));
	}
}
