/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler;

import java.rmi.RemoteException;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.event.Event;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.gump.GumpClient;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.event.WaitForJobResponseEvent;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;
import org.ourgrid.mygrid.scheduler.gump.GumpClientImpl;
import org.ourgrid.mygrid.scheduler.gump.GumpManager;
import org.ourgrid.mygrid.scheduler.gump.GumpManagerImpl;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManager;

/**
 * This class represents a fake <code>EBSchedulerFacade</code> to be used in
 * tests.
 */
public class FakeEBSchedulerFacade extends EBSchedulerFacade {

	private GumpClient gumpClient;

	private GumpManager gumpManager;

	private Scheduler scheduler;

	private boolean ignoreCalls;

	private ExecutionStatus expectedExecutionStatus = null;


	public FakeEBSchedulerFacade() {

		this.ignoreCalls = false;
	}


	public void ignoreCalls() {

		this.ignoreCalls = true;
	}


	@Override
	public void config( EBReplicaExecutorFacade ebReplicaExecutorFacade ) throws ConfigException, RemoteException {

		this.scheduler = new SchedulerImpl( this );
		this.gumpManager = new GumpManagerImpl( this );
		this.gumpClient = new GumpClientImpl( this );

		this.schedulerEventProcessor = new SchedulerEventEngine( scheduler, gumpManager, this.gumpClient,
				ebReplicaExecutorFacade, this );
	}


	@Override
	protected SchedulerEventEngine createSchedulerEventEngine( Scheduler scheduler, GumpManager gumpManager,
																GumpClient gumpClient,
																EBReplicaExecutorFacade ebReplicaExecutorFacade )
		throws ConfigException {

		return new FakeSchedulerEventEngine( scheduler, gumpManager, gumpClient, ebReplicaExecutorFacade, this );
	}


	@Override
	public void hereIsGum( GumClient gumClient, long requestid, ObjectID peerBrokerAccessID ) {

		if ( !ignoreCalls )
			super.hereIsGum( gumClient, requestid, peerBrokerAccessID );
	}


	@Override
	public void replicaFailed( ReplicaExecutorResult result ) {

		if ( !ignoreCalls ) {
			super.replicaFailed( result );
		}
	}


	@Override
	public void replicaFinished( ReplicaExecutorResult result ) {

		if ( !ignoreCalls ) {
			super.replicaFinished( result );
		}
	}


	@Override
	public void replicaCanceled( ReplicaExecutorResult result ) {

		if ( !ignoreCalls ) {
			super.replicaCanceled( result );
		}
	}


	@Override
	public void replicaAborted( ReplicaExecutorResult result ) {

		if ( !ignoreCalls ) {
			super.replicaAborted( result );
		}
	}


	public void setReplicaStateToBeReturned( ExecutionStatus executionStatus ) {

		expectedExecutionStatus = executionStatus;
	}


	@Override
	public EventQueue<Event> waitForJob( int jobId ) {

		if ( this.expectedExecutionStatus == null ) {
			return super.waitForJob( jobId );
		}

		EventQueue<Event> responseQueue = new EventQueue<Event>();
		WaitForJobResponseEvent waitForJobResponseEvent = new WaitForJobResponseEvent();
		responseQueue.put( waitForJobResponseEvent );
		return responseQueue;
	}


	public GridManager getGridManager() {

		return schedulerEventProcessor.getGridManager();
	}


	public JobManager getJobManager() {

		return schedulerEventProcessor.getJobManager();
	}


	public GumpClient getGumpClient() {

		return gumpClient;
	}


	public GumpManager getGumpManager() {

		return gumpManager;
	}


	@Override
	public SchedulerEventEngine getEventProcessor() {

		return schedulerEventProcessor;
	}


	public Scheduler getScheduler() {

		return scheduler;
	}
}
