package org.ourgrid.mygrid.scheduler;

import java.util.Iterator;
import java.util.List;

import org.ourgrid.common.gum.GumClient;
import org.ourgrid.mygrid.replicaexecutor.EBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;

public class FakeSchedulingHeuristic extends EBAbstractSchedulingHeuristic {

	private ReplicaEntry latestScheduledReplica;


	public FakeSchedulingHeuristic( JobManager jobManager, GridManager ebGridManager,
									EBReplicaExecutorFacade ebReplicaExecutorFacade ) {

		super( jobManager, ebGridManager, ebReplicaExecutorFacade );
	}


	@Override
	public boolean schedule() {

		List<TaskEntry> tasks = getTasksToScheduleQueue();

		Iterator<TaskEntry> itTasks = tasks.iterator();
		while ( itTasks.hasNext() ) {
			TaskEntry task = itTasks.next();
			GumClient gum = getGum( task );
			if ( gum != null ) {
				ReplicaEntry replica = jobManager.createNewReplica( task );
				executeReplica( replica, gum );
				latestScheduledReplica = replica;
				itTasks.remove();
				return true;
			}
		}

		return false;
	}


	private GumClient getGum( TaskEntry task ) {

		List<GumClient> gums = gridManager.getAvailableGuMs( task.getJobId(), task.getId() );
		if ( gums.isEmpty() )
			return null;
		return gums.get( 0 );
	}


	public ReplicaEntry getLatestScheduledReplica() {

		return latestScheduledReplica;
	}

}
