package org.ourgrid.mygrid.scheduler;

import java.util.Collection;
import java.util.LinkedList;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.util.StreamCloner;
import org.ourgrid.mygrid.scheduler.event.SimpleResponseEvent;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;

public class GetGumSpecsRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	private final EventQueue<ResponseEvent<Collection<GumSpec>>> eq;

	private final GridManager gridManager;


	public GetGumSpecsRequestEvent( GridManager gridManager, EventQueue<ResponseEvent<Collection<GumSpec>>> eq ) {

		this.gridManager = gridManager;
		this.eq = eq;
	}


	/**
	 * Gets a list of specifications of the gums currently owned by the
	 * EBGridManager. The list is cloned before being put on the response queue
	 */
	public void process() {

		Collection<GumSpec> gumSpecs = gridManager.getGumSpecs();
		Collection<GumSpec> cloneSpecs = null;

		try {
			cloneSpecs = StreamCloner.clone( new LinkedList<GumSpec>( gumSpecs ) );
		} catch ( CloneNotSupportedException e ) {
		}

		eq.put( new SimpleResponseEvent<Collection<GumSpec>>( cloneSpecs ) );
	}
}
