/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.mygrid.scheduler.event.JobNotFoundNotifyEvent;
import org.ourgrid.mygrid.scheduler.event.WaitForJobResponseEvent;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;

/**
 * Object responsible for notifying other objects interested in job completions.
 */
public class JobMonitor implements JobInterested {

	/** A <code>EventQueue</code> that will receive a reponse to a request. */
	private EventQueue responseQueue;

	/** The job identification. */
	private int jobId;


	/**
	 * The constructor.
	 * 
	 * @param responseQueue A response <code>EventQueue</code>.
	 * @param jobId A job identification.
	 */
	public JobMonitor( EventQueue responseQueue, int jobId ) {

		this.responseQueue = responseQueue;
		this.jobId = jobId;
	}


	/**
	 * Notifies who is waiting for the <code>Job</code>. Puts a
	 * WaitForJobResponseEvent in the responseQueue.
	 * 
	 * @param job <code>Job</code> which has finished.
	 */
	public void jobFinished( JobEntry job ) {

		if ( jobId == job.getId() ) {
			WaitForJobResponseEvent waitForJobResponseEvent = new WaitForJobResponseEvent();
			responseQueue.put( waitForJobResponseEvent );
		}
	}


	/**
	 * Returns the <code>Job</code> identification.
	 * 
	 * @return The <code>Job</code> identification.
	 */
	public int getJobId() {

		return jobId;
	}


	/**
	 * @see org.ourgrid.mygrid.scheduler.JobInterested#jobNotFound()
	 */
	public void jobNotFound() {

		responseQueue.put( new JobNotFoundNotifyEvent() );
	}

}
