/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler;

import java.io.Serializable;

import org.ourgrid.common.gump.Gump;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.gump.MyGridGumpSpecServices;

/**
 * This class is intended to store a GuMSpec and its corresponding gump if it
 * could be instantiated. It it could not, every time the <code>getGuMP()</code>
 * method is called,
 */
public class PeerEntry implements Serializable {

	private static final long serialVersionUID = 33L;

	private PeerSpec spec;

	private Gump gump;

	private PeerState peerStatus;


	public PeerEntry( PeerSpec peerSpec ) {

		this.spec = peerSpec;
		this.peerStatus = PeerState.CONTACTING;
	}


	/**
	 * Gets the instance of the GridMachineProvider that corresponds to a given
	 * gumpSpec.
	 * 
	 * @return Returns the gridMachineProvider or null if it could not be
	 *         created.
	 */
	public Gump getPeer() {

		return this.gump;
	}


	/**
	 * Gets the GuMPSpec of this <code>GuMPEntry</code>.
	 * 
	 * @return Returns the gumpSpec.
	 */
	public PeerSpec getPeerSpec() {

		return this.spec;
	}


	public PeerState getStatus() {

		return this.peerStatus;
	}


	public void setPeerStatus( PeerState status ) {

		this.peerStatus = status;

		if ( this.peerStatus == PeerState.UP ) {
			this.peerStatus = status;
			// a new lookup is always performed to avoid using an old instance.
			this.gump = MyGridGumpSpecServices.getGump( spec );
		}

	}


	public boolean equals( PeerSpec peerSpec ) {

		return this.spec.getURL().equals( peerSpec.getURL() );
	}

	public enum PeerState {
		UP, CONTACTING;
	}
}
