/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.mygrid.event.JobRequestEvent;
import org.ourgrid.mygrid.scheduler.EBSchedulingHeuristic;

/**
 * Event used to add jobs to be processed by the Scheduler.
 */
public class AddJobRequestEvent implements JobRequestEvent {

	private static final long serialVersionUID = 33L;

	/** The scheduling ebSchedulingHeuristic that will schedule th job. */
	private EBSchedulingHeuristic ebSchedulingHeuristic;

	/** The job specification. */
	private JobSpec jobSpec;

	/** The job identification. */
	private int jobId;


	/**
	 * The constructor.
	 * 
	 * @param ebSchedulingHeuristic The scheduling heuristic
	 * @param jobId The job identification
	 * @param jobSpec The job specification
	 */
	public AddJobRequestEvent( EBSchedulingHeuristic ebSchedulingHeuristic, int jobId, JobSpec jobSpec ) {

		this.ebSchedulingHeuristic = ebSchedulingHeuristic;
		this.jobId = jobId;
		this.jobSpec = jobSpec;
	}


	/**
	 * Adds the job given in the constructor to the scheduling heuristic.
	 */
	public void process() {

		ebSchedulingHeuristic.addJob( jobId, jobSpec );
	}
}
