/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import java.util.Collection;
import java.util.LinkedList;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.util.StreamCloner;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerListenerRemote;

public class GetGumpSpecsRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	private GridManager gridManager;

	private EventQueue<ResponseEvent<Collection<PeerEntry>>> responseQueue;

	private GridManagerListenerRemote listener;


	public GetGumpSpecsRequestEvent( GridManager gridManager, GridManagerListenerRemote newListener,
										EventQueue<ResponseEvent<Collection<PeerEntry>>> responseQueue ) {

		this.responseQueue = responseQueue;
		this.gridManager = gridManager;
		this.listener = newListener;
	}


	public void process() {

		gridManager.addNewListener( listener );
		Collection<PeerEntry> clonedEntries = null;
		try {
			clonedEntries = StreamCloner.clone( new LinkedList<PeerEntry>( gridManager.getPeerEntries() ) );
		} catch ( CloneNotSupportedException e ) {
		}

		responseQueue.put( new SimpleResponseEvent<Collection<PeerEntry>>( clonedEntries ) );
	}
}
