/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import org.ourgrid.common.id.GumID;
import org.ourgrid.mygrid.event.GridManagerNotifyEvent;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;

/**
 * Event used to notify the <code>EBGridManager</code> that a
 * <code>GridMachine</code> is dead.
 */
public class GumIsDeadNotifyEvent implements GridManagerNotifyEvent {

	private static final long serialVersionUID = 33L;

	/** The <code>EBGridManager</code>. */
	private GridManager ebGridManager;

	/** The request id which the <code>GridMachine</code> is related. * */
	private long requestId;

	/** The <code>GridMachine</code> that died. * */
	private GumID gumID;


	/**
	 * The constructor.
	 * 
	 * @param ebGridManager The EBGridManager
	 * @param requestId The request id which the <code>GridMachine</code> is
	 *        related
	 * @param gumID The <code>GridMachine</code> that died
	 */
	public GumIsDeadNotifyEvent( GridManager ebGridManager, long requestId, GumID gumID ) {

		this.ebGridManager = ebGridManager;
		this.requestId = requestId;
		this.gumID = gumID;
	}


	/**
	 * Notifies the <code>EBGridManager</code> that a <code>GridMachine</code>
	 * died.
	 */
	public void process() {

		ebGridManager.deleteGum( gumID );
	}

}
