/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.mygrid.event.GridManagerNotifyEvent;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;

/**
 * Event used to deliver a <code>GridMachine</code> to a
 * <code>GridMachineConsumer</code>.
 */
public class HereIsGumNotifyEvent implements GridManagerNotifyEvent {

	private static final long serialVersionUID = 33L;

	/** The <code>EBGridManager</code>. */
	private GridManager ebGridManager;

	private GumClient gumClient;

	private long requestID;

	private ObjectID peerBrokerAccessID;


	/**
	 * The constructor.
	 * 
	 * @param ebGridManager The <code>EBGridManager</code>
	 * @param gumClient Client representing received Gum.
	 * @param requestID The request identification
	 * @param peerBrokerAccessID ObjectID representing the Peer we received the
	 *        Gum from.
	 */
	public HereIsGumNotifyEvent( GridManager ebGridManager, GumClient gumClient, long requestID,
									ObjectID peerBrokerAccessID ) {

		this.ebGridManager = ebGridManager;
		this.gumClient = gumClient;
		this.requestID = requestID;
		this.peerBrokerAccessID = peerBrokerAccessID;
	}


	/**
	 * Delivers a <code>GridMachine</code> to the <code>EBGridManager</code>.
	 */
	public void process() {

		ebGridManager.hereIsGum( gumClient, requestID, peerBrokerAccessID );
	}

}
