/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import java.util.Collection;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.mygrid.event.JobRequestEvent;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManager;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManagerListenerRemote;

/**
 * Event used to retrieve a list of jobs from the <code>EBJobManager</code>.
 */
public class JobListRequestEvent implements JobRequestEvent {

	private JobManager jobManager;

	private static final long serialVersionUID = 33L;

	private EventQueue<ResponseEvent<Collection<JobEntry>>> responseQueue;

	private JobManagerListenerRemote listener;


	public JobListRequestEvent( JobManager jobManager, JobManagerListenerRemote newListener,
								EventQueue<ResponseEvent<Collection<JobEntry>>> responseQueue ) {

		this.responseQueue = responseQueue;
		this.jobManager = jobManager;
		this.listener = newListener;
	}


	/**
	 * Retrieves a list of jobs from the <code>EBJobManager</code>. All
	 * objects in this list are clones.
	 */
	public void process() {

		jobManager.addNewListener( listener );
		Collection<JobEntry> jobs = jobManager.clonedJobList();

		responseQueue.put( new SimpleResponseEvent<Collection<JobEntry>>( jobs ) );
	}

}
