/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import org.ourgrid.common.event.Event;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.RequestEvent;

/**
 * SchedulerTestRequestEvent This Event is used only for testing, it only puts a
 * SchedulerTestResponseEvent on its ResponseQueue in order to test if the
 * <code>process</code> method was invoked.
 */
public class SchedulerTestRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/** An EventQueue */
	private EventQueue<Event> responseQueue;


	/**
	 * Constructs the Event.
	 * 
	 * @param responseQueue The EventQueue where the SchedulerTestResponseEvent
	 *        will be put.
	 */
	public SchedulerTestRequestEvent( EventQueue<Event> responseQueue ) {

		this.responseQueue = responseQueue;
	}


	/**
	 * Puts a SchedulerTestResponseEvent on the responseQueue
	 * 
	 * @see org.ourgrid.common.event.ActionEvent#process()
	 */
	public void process() {

		responseQueue.put( new SchedulerTestResponseEvent() );
	}

}
