/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.event;

import java.util.Collection;

import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.EBSchedulingHeuristic;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManager;

/**
 * This event is used to set a new set of GuMPs to be used by the
 * <code>EBGridManager</code>.
 */
public class SetPeersRequestEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/**
	 * The <code>EBGridManager</code>.
	 */
	private GridManager ebGridManager;

	/** A collection with <code>GuMPSpec</code>s. * */
	private Collection<PeerSpec> gumpSpecs;


	/**
	 * The constructor.
	 * 
	 * @param ebSchedulingHeuristic The <code>EBSchedulingHeuristic</code>
	 * @param ebGridManager The EBGridManager
	 * @param gumpSpecs A collection of <code>GridMachineProvider</code>s
	 *        specifications
	 */
	public SetPeersRequestEvent( EBSchedulingHeuristic ebSchedulingHeuristic, GridManager ebGridManager,
									Collection<PeerSpec> gumpSpecs ) {

		this.ebGridManager = ebGridManager;
		this.gumpSpecs = gumpSpecs;
	}


	/**
	 * Sets a new set of GuMPs to be used by the <code>EBGridManager</code>.
	 */
	public void process() {

		ebGridManager.setPeers( gumpSpecs );
		// ebSchedulingHeuristic.newPeersHaveBeenSet();
	}
}
