/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.exception;

import org.ourgrid.common.exception.OurgridException;

/**
 * Exception to be thrown when a job is not found.
 */
public class JobNotFoundException extends OurgridException {

	static final long serialVersionUID = 33L;

	/** The job identification */
	private int job;


	/**
	 * The constructor.
	 * 
	 * @param job The job identification
	 * @param detail The cause of this exception
	 */
	public JobNotFoundException( int job, Throwable detail ) {

		super( detail );
		this.job = job;
	}


	/**
	 * The constructor.
	 * 
	 * @param job The job identification
	 */
	public JobNotFoundException( int job ) {

		super();
		this.job = job;
	}


	/**
	 * Returns the job identification.
	 * 
	 * @return The job identification
	 */
	public int getJob() {

		return job;
	}


	/**
	 * Returns the message of this exception
	 * 
	 * @return The message
	 */
	public String getMessage() {

		return "Job " + String.valueOf( getJob() ) + " does not exist";
	}
}
