/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.exception;

import java.text.MessageFormat;

import org.ourgrid.common.exception.OurgridException;

/**
 * Exception to be thrown when a problem occurs while trying to kill a replica
 * execution.
 */
public class KillReplicaException extends OurgridException {

	private static final long serialVersionUID = 33L;

	/** The command to be killed or a string that is present in the command line. */
	private String commandToKill;


	/**
	 * The constructor.
	 * 
	 * @param commandToKill
	 */
	public KillReplicaException( String commandToKill ) {

		super();
		this.commandToKill = commandToKill;
	}


	/**
	 * The constructor.
	 * 
	 * @param commandToKill The command or the string present in the command
	 *        line
	 * @param detail The throwable that is the cause of this exception.
	 */
	public KillReplicaException( String commandToKill, Throwable detail ) {

		super( detail );
		this.commandToKill = commandToKill;
	}


	/**
	 * Returns the command to be killed or a string that is present in the
	 * command line.
	 * 
	 * @return The command or the string present in the command line
	 */
	public String getCommandToKill() {

		return this.commandToKill;
	}


	/**
	 * Returns the exception message.
	 * 
	 * @return The message
	 */
	public String getMessage() {

		Object params[] = { this.commandToKill };
		return MessageFormat.format( super.getMessage(), params );
	}

}
