package org.ourgrid.mygrid.scheduler.gridmanager;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;

import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

public abstract class AbstractGridManager implements GridManager {

	private Collection<GridManagerListenerRemote> listeners;

	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( AbstractGridManager.class );


	public AbstractGridManager() {

		listeners = new LinkedHashSet<GridManagerListenerRemote>();
	}


	public void hereIsGum( GumClient gumClient, long requestID, ObjectID peerBrokerAccessID ) {

		int jobId = realHereIsGum( gumClient, requestID, peerBrokerAccessID );

		if ( jobId != 0 ) {
			Iterator<GridManagerListenerRemote> it = listeners.iterator();
			while ( it.hasNext() ) {
				GridManagerListenerRemote listener = it.next();
				try {
					listener.hereIsGum( gumClient.getGumSpec(), jobId, peerBrokerAccessID );
				} catch ( RemoteException e ) {
					it.remove();
					LOG.error( "Could not contact GridManagerListener " + listener );
				}
			}
		}
	}


	protected abstract int realHereIsGum( GumClient gumClient, long requestID, ObjectID peerBrokerAccessID );


	public void setPeers( Collection<PeerSpec> peerSpecs ) {

		realSetPeers( peerSpecs );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.setPeers( peerSpecs );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realSetPeers( Collection<PeerSpec> peerSpecs );


	public void deleteGum( GumID gumID ) {

		realDeleteGum( gumID );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.deleteGum( gumID );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realDeleteGum( GumID gumID );


	public void allocate( GumID gumID, ReplicaEntry replica ) {

		realAllocate( gumID, replica );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.allocate( gumID, replica );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realAllocate( GumID gumID, ReplicaEntry replica );


	public void ready( GumID gumID ) {

		realGumIsReady( gumID );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.gumIsReady( gumID );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realGumIsReady( GumID gumID );


	public void peerAlive( PeerSpec peerSpec ) {

		realPeerAlive( peerSpec );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.peerAlive( peerSpec );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realPeerAlive( PeerSpec peerSpec );


	public void peerIsDead( PeerSpec peerSpec ) {

		realPeerIsDead( peerSpec );

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.peerIsDead( peerSpec );
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Could not contact GridManagerListener " + listener );
			}
		}
	}


	protected abstract void realPeerIsDead( PeerSpec peerSpec );


	public void needGums( int jobId, String requirements, int numRequiredGums ) {

		realNeedGums( jobId, requirements, numRequiredGums );
	}


	protected abstract void realNeedGums( int jobId, String requirements, int numRequiredGums );


	public boolean updateBlackList( int jobID, int taskID, GumID gumID ) {

		return realUpdateBlackList( jobID, taskID, gumID );
	}


	protected abstract boolean realUpdateBlackList( int jobID, int taskID, GumID gumID );


	public boolean unwantedGum( GumID gumID, int jobID ) {

		return realUnwantedGum( gumID, jobID );
	}


	protected abstract boolean realUnwantedGum( GumID gumID, int jobID );


	public void pauseRequest( int jobId ) {

		realPauseRequest( jobId );
	}


	protected abstract void realPauseRequest( int jobId );


	public void resumeRequest( int jobId ) {

		realResumeRequest( jobId );
	}


	protected abstract void realResumeRequest( int jobId );


	public void addNewListener( GridManagerListenerRemote newListener ) {

		if ( newListener != null ) {
			listeners.add( newListener );
		}
	}


	public void removeListener( GridManagerListenerRemote listener ) {

		listeners.remove( listener );
	}


	public void shutdown() {

		realShutdown();

		Iterator<GridManagerListenerRemote> it = listeners.iterator();
		while ( it.hasNext() ) {
			GridManagerListenerRemote listener = it.next();
			try {
				listener.myGridDown();
			} catch ( RemoteException e ) {
				it.remove();
				LOG.error( "Unable to contact listener " + listener, e );
			}
		}
	}


	protected abstract void realShutdown();
}
