/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.gridmanager;

import java.io.Serializable;

import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.gum.GumStatus;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

/**
 * Objects of this class are responsible for maintaining the state of Gums on
 * behalf of the Scheduler. It also associates a Gum with its Gump.
 */
public class GridManagerEntry implements Serializable {

	private static final long serialVersionUID = 33L;

	/** The Gum, wrapped by a GumClient */
	private GumClient gumClient;

	/** The request identification. */
	private long requestID;

	/** Enumeration that represents the state of the Gum */
	private GumStatus gumState;

	/**
	 * ObjectID used to identify the peer we are talking to.
	 */
	private ObjectID peerBrokerAccessID;

	private final int jobID;

	private GumSpec gumSpec;

	private ReplicaEntry replica;


	public GridManagerEntry( long requestId, int jobID, GumClient gumClient, ObjectID peerBrokerAccessID ) {

		this.requestID = requestId;
		this.jobID = jobID;
		this.gumClient = gumClient;
		this.gumSpec = gumClient.getGumSpec();
		this.peerBrokerAccessID = peerBrokerAccessID;
		this.gumState = GumStatus.READY;
	}


	public GridManagerEntry( GumSpec spec, int jobID, ObjectID peerBrokerAccessId ) {

		this.gumSpec = spec;
		this.jobID = jobID;
		this.peerBrokerAccessID = peerBrokerAccessId;
	}


	public GumClient getGum() {

		return this.gumClient;
	}


	public GumStatus getState() {

		return this.gumState;
	}


	public GumID getGumID() {

		return this.getGumSpec().getGumID();
	}


	public long getRequestId() {

		return this.requestID;
	}


	public void allocate( ReplicaEntry replica ) {

		this.replica = replica;
		this.gumState = GumStatus.RUNNING;
	}


	/**
	 * Verifies if the gum is ready.
	 * 
	 * @return True if the gum is free, false otherwise
	 */
	public boolean isReady() {

		return this.gumState.equals( GumStatus.READY );
	}


	/**
	 * Sets the gum state to READY
	 */
	public void ready() {

		this.replica = null;
		this.gumState = GumStatus.READY;
	}


	public ObjectID getPeerBrokerAccessID() {

		return peerBrokerAccessID;
	}


	public GumSpec getGumSpec() {

		return gumSpec;
	}


	public int getJobID() {

		return jobID;
	}


	public ReplicaEntry getReplica() {

		return replica;
	}
}
