/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.gridmanager;

import java.io.Serializable;

import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;

/**
 * This class encapsulates the information regarding a <code>Gum</code>
 * delivery. This information will travel from the grid machine provider (<code>Gump</code>)
 * to the grid machine consumer (<code>GumpClient</code>). The required
 * information to deliver a <code>Gum</code> is basically: (1) The
 * identification of the request to which the machine will be delivered. (2) The
 * machine specification (<code>GumSpec</code>). (3) A reference to the
 * entity which represents the machine, which will be an instance of
 * <code>Gum</code> (4) The ID of the machine provider.
 */
public class GumResponseEntry implements Serializable {

	private static final long serialVersionUID = 33L;

	/**
	 * The identification of the request to which the machine will be delivered.
	 */
	private long requestID;

	/** The machine specification. */
	private GumSpec gumSpec;

	/**
	 * A reference to the entity which represents the machine, which will be an
	 * instance of <code>Gum</code>.
	 */
	private Gum gum;

	/**
	 * The ID of the machine provider.
	 */
	private ObjectID requesterObjectID;


	/**
	 * Default constructor.
	 */
	public GumResponseEntry( Gum gum, GumSpec spec, long requestid, ObjectID providerObjectID ) {

		this.gum = gum;
		this.gumSpec = spec;
		this.requestID = requestid;
		this.requesterObjectID = providerObjectID;
	}


	/**
	 * This constructor is used only by tests.
	 */
	public GumResponseEntry( long requestid ) {

		this.requestID = requestid;
		this.gumSpec = null;
	}


	public GumSpec getGumSpec() {

		return gumSpec;
	}


	public long getRequestID() {

		return requestID;
	}


	public Gum getGum() {

		return gum;
	}


	public ObjectID getObjectID() {

		return requesterObjectID;
	}


	@Override
	public boolean equals( Object obj ) {

		if ( obj != null && obj instanceof GumResponseEntry ) {
			GumResponseEntry otherGre = (GumResponseEntry) obj;
			return otherGre.requestID == requestID && otherGre.gumSpec.equals( gumSpec )
					&& otherGre.requesterObjectID.equals( requesterObjectID );

		}

		return false;
	}
}
