/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.gump;

import java.rmi.RemoteException;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.gump.GumpClient;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.security.MGSecureClientSocketFactory;
import org.ourgrid.common.security.MGSecureServerSocketFactory;
import org.ourgrid.common.url.MyGridURLProvider;
import org.ourgrid.gridmachine.useragent.UserAgentClient;
import org.ourgrid.mygrid.scheduler.EBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.gridmanager.GumResponseEntry;

/**
 * The <code>GridMachineConsumer</code> remote object responsible for
 * receiving RMI calls and translating them to calls on the
 * <code>EBSchedulerFacade</code>.
 */
public class GumpClientImpl extends OurgridUnicastRemoteObject implements GumpClient {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;

	/** The <code>EBSchedulerFacade</code>. */
	private EBSchedulerFacade ebSchedulerFacade;

	private ObjectID objectID;


	/**
	 * The constructor.
	 * 
	 * @param ebSchedulerFacade The <code>EBSchedulerFacade</code>
	 * @throws RemoteException When this remote object could not be created and
	 *         exported
	 */
	public GumpClientImpl( EBSchedulerFacade ebSchedulerFacade ) throws RemoteException {

		super();
		this.ebSchedulerFacade = ebSchedulerFacade;
		objectID = new ObjectID( "GumpClientImpl", chooseId(), MyGridURLProvider.externalGumpClient() );
	}


	/**
	 * The constructor with SecureSocketFactories.
	 * 
	 * @param ebSchedulerFacade The EBSchedulerFacade.
	 * @param secureClientSocketFactory The SecureClientSocketFactory.
	 * @param secureServerSocketFactory The SecureServerSocketFactory.
	 * @param securePort The port where the secure services will be bound.
	 */
	public GumpClientImpl( EBSchedulerFacade ebSchedulerFacade, MGSecureClientSocketFactory secureClientSocketFactory,
							MGSecureServerSocketFactory secureServerSocketFactory, int securePort )
		throws RemoteException {

		super( securePort, secureClientSocketFactory, secureServerSocketFactory );
		this.ebSchedulerFacade = ebSchedulerFacade;
		objectID = new ObjectID( "GumpClientImpl", chooseId(), "rmi://"
				+ Configuration.getInstance().getProperty( Configuration.PROP_NAME ) + ":" + securePort
				+ "/GUMP_CLIENT" );
	}


	public void hereIsGum( GumResponseEntry gumResponseEntry ) {

		GumClient gumClient = new UserAgentClient( gumResponseEntry.getGum(), gumResponseEntry.getGumSpec() );
		ebSchedulerFacade.hereIsGum( gumClient, gumResponseEntry.getRequestID(), gumResponseEntry.getObjectID() );
	}


	/**
	 * @return A randomly chosen 64 bit number.
	 */
	private Long chooseId() {

		return new Long( ((long) (Math.random() * Long.MAX_VALUE)) );
	}


	/**
	 * @see org.ourgrid.common.gump.GumpClient#getObjectID()
	 */
	public ObjectID getObjectID() {

		return objectID;
	}
}
