/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.gump;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;

import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerListenerRemote;

/**
 * A <code>GuMPManager</code> is a <code>Remote</code> object that manages
 * GuMPs.
 */
public interface GumpManager extends Remote {

	/**
	 * Sets the <code>GuMPSpec</code>s.
	 * 
	 * @param gumpSpecs the new <code>GuMPSpec</code>s.
	 * @throws RemoteException when a communication error occurs.
	 */
	public void setPeers( Collection<PeerSpec> gumpSpecs ) throws RemoteException;


	/**
	 * Gets the <code>GuMPSpec</code>s.
	 * 
	 * @param newListener TODO
	 * @return a collection of <code>GuMPSpec</code>s.
	 * @throws RemoteException when a communication error occurs.
	 */
	public Collection<PeerEntry> getPeerEntries( GridManagerListenerRemote newListener ) throws RemoteException;


	/**
	 * Notifies that the peer <code>PeerSpec</code> is alive.
	 * 
	 * @param url Url of the Peer that is now alive.
	 */
	public void peerAlive( String url ) throws RemoteException;


	/**
	 * Notifies that the peer <code>PeerSpec</code> is dead.
	 * 
	 * @param url Url of the Peer that is now dead.
	 */
	public void peerIsDead( String url ) throws RemoteException;

}
