/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.gump;

import java.rmi.Naming;

import org.ourgrid.common.gump.Gump;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.test.FakeGump;

/**
 * This class is responsible for validating a given GumSpec or creating a GuM
 * according to a GumSpec.
 */
public class MyGridGumpSpecServices {

	/**
	 * A constant used to identify a <code>FakeGridMachineProvider</code>.
	 */
	public final static String FAKE_GUMP = "fake_gump";


	/**
	 * Obtains a <code>GridMachineProvider</code> from a URL where it should
	 * be published. If it could not obtain a <code>GridMachineProvider</code>,
	 * returns null.
	 * 
	 * @param gumpSpec The <code>PeerSpec</code> used to create a
	 *        GridMachineProvider.
	 * @return the obtained GuMP or null if it could not be created.
	 */
	public static Gump getGump( PeerSpec gumpSpec ) {

		if ( gumpSpec.getLabel().equals( FAKE_GUMP ) ) {
			return new FakeGump();
		}

		try {
			return (Gump) Naming.lookup( gumpSpec.getURL() );

		} catch ( Exception e ) {
			return null;
		}
	}
}
