/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.ExportException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.common.event.Event;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.NotifyEvent;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.event.ShutdownResponseEvent;
import org.ourgrid.common.gum.GumClient;
import org.ourgrid.common.gum.GumStatus;
import org.ourgrid.common.gump.BindableMockGumpWrapper;
import org.ourgrid.common.gump.Gump;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.spec.TaskSpec;
import org.ourgrid.common.spec.exception.JobSpecificationException;
import org.ourgrid.common.spec.exception.TaskSpecificationException;
import org.ourgrid.common.url.MyGridURLProvider;
import org.ourgrid.mygrid.replicaexecutor.test.FakeEBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.EBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.JobRequestEntry;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.ReplicaExecutorResult;
import org.ourgrid.mygrid.scheduler.event.JobCancelledNotifyEvent;
import org.ourgrid.mygrid.scheduler.event.JobCouldNotBeCancelledNotifyEvent;
import org.ourgrid.mygrid.scheduler.event.JobNotFoundNotifyEvent;
import org.ourgrid.mygrid.scheduler.gridmanager.EBGridManager;
import org.ourgrid.mygrid.scheduler.gump.MyGridGumpSpecServices;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManager;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;
import org.ourgrid.test.SchedulerFastTest;
import org.ourgrid.test.TestUtils;
import org.ourgrid.test.threadServices.ThreadServices;

public class EBSchedulerFacadeTest extends TestCase implements SchedulerFastTest {

	private static final String MOCK_GUM_NAME2 = "mockGum2";

	private static final String MOCK_GUM_NAME1 = "mockGum1";

	private static final String MOCK_GUM_NAME = "mockGum";

	private FakeEBReplicaExecutorFacade fakeEBReplicaExecutorFacade;

	private Gump mockGump;

	private GumClient mockGum;

	private GumClient mockGum1;

	private GumClient mockGum2;

	private boolean verifyMocks;

	private TestUtils testUtils;

	private GumSpec gumSpec;

	private GumSpec gumSpec1;

	private GumSpec gumSpec2;

	private boolean verify2Mocks;

	private int registryPort;

	private ObjectID mockGumpID;

	private BindableMockGumpWrapper boundableMockGumpWrapper;

	private EBSchedulerFacade ebSchedulerFacade;

	private EBGridManager gridManager;


	/**
	 * @param name Test name
	 */
	public EBSchedulerFacadeTest( String name ) {

		super( name );
	}


	@Override
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration config = Configuration.getInstance( Configuration.MYGRID );

		registryPort = Integer.parseInt( config.getProperty( Configuration.PROP_PORT ) );

		try {
			LocateRegistry.createRegistry( registryPort );
		} catch ( ExportException e ) {
			LocateRegistry.getRegistry( registryPort );
		}

		verifyMocks = false;
		verify2Mocks = false;
		createEBSchedulerFacade();

		// Setup mock Gum
		gumSpec = new GumSpec();
		gumSpec.putAttribute( GumSpec.ATT_NAME, MOCK_GUM_NAME );
		gumSpec.putAttribute( "os", "linux" );

		mockGum = EasyMock.createMock( GumClient.class );
		mockGum.getGumSpec();
		EasyMock.expectLastCall().andReturn( gumSpec ).anyTimes();

		mockGump = EasyMock.createMock( Gump.class );
		mockGumpID = new ObjectID( "MockGump", (long) 42, "rmi://localhost:" + registryPort + "/MockGump" );

		mockGump.getObjectID();
		EasyMock.expectLastCall().andReturn( mockGumpID ).anyTimes();

		boundableMockGumpWrapper = new BindableMockGumpWrapper( mockGump );

		Naming.rebind( mockGumpID.getObjectURL(), boundableMockGumpWrapper );
	}


	private void createEBSchedulerFacade() throws ConfigException, RemoteException {

		ebSchedulerFacade = new EBSchedulerFacade();
		fakeEBReplicaExecutorFacade = new FakeEBReplicaExecutorFacade();

		ebSchedulerFacade.config( fakeEBReplicaExecutorFacade );
		fakeEBReplicaExecutorFacade.config( ebSchedulerFacade );
		ebSchedulerFacade.startEventProcessor();
		gridManager = (EBGridManager) ebSchedulerFacade.getEventProcessor().getGridManager();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
		EventQueue shutdownResponse = ebSchedulerFacade.shutdown();
		shutdownResponse.blockingRemove();
		ebSchedulerFacade = null;
		if ( verifyMocks ) {
			EasyMock.verify( mockGump );
			EasyMock.verify( mockGum );
		}
		if ( verify2Mocks ) {
			EasyMock.verify( mockGum1 );
			EasyMock.verify( mockGum2 );
		}

		try {
			Naming.unbind( mockGumpID.getObjectURL() );
			OurgridUnicastRemoteObject.unexportObject( boundableMockGumpWrapper, true );
		} catch ( Exception e ) {
		}

		Configuration.reset();
		testUtils.reset();

	}


	public void testPersistJobId() throws Exception {

		Configuration config = Configuration.getInstance( Configuration.MYGRID );

		config.setProperty( MyGridConfiguration.PROP_PERSISTJOBID, "yes" );

		assertTrue( config.isEnabled( MyGridConfiguration.PROP_PERSISTJOBID ) );

		String expression = "location==lsd && os==linux";
		int numTasks = 5;
		int numJobs = 3;
		int numId = 0;
		JobSpec jobSpec = createJobSpec( expression, numTasks );

		for ( int jobCount = 1; jobCount <= numJobs; jobCount++ ) {
			int jobId = ebSchedulerFacade.addJob( jobSpec );
			assertEquals( ++numId, jobId );
		}

		ebSchedulerFacade.shutdown();
		createEBSchedulerFacade();

		numId = 3;
		for ( int jobCount = 1; jobCount <= numJobs; jobCount++ ) {
			int jobId = ebSchedulerFacade.addJob( jobSpec );
			assertEquals( ++numId, jobId );
		}
	}


	public void testJobIdWithoutPersistence() throws Exception {

		Configuration config = Configuration.getInstance( Configuration.MYGRID );

		config.setProperty( MyGridConfiguration.PROP_PERSISTJOBID, "no" );

		assertFalse( config.isEnabled( MyGridConfiguration.PROP_PERSISTJOBID ) );

		String expression = "location==lsd && os==linux";
		int numTasks = 5;
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		int jobId = ebSchedulerFacade.addJob( jobSpec );
		assertEquals( "The first job id should be '1'", 1, jobId );

		ebSchedulerFacade.shutdown();
		createEBSchedulerFacade();

		jobId = ebSchedulerFacade.addJob( jobSpec );
		assertEquals( "The first job id should be '1'", 1, jobId );

	}


	/* org.ourgrid.scheduler.Scheduler */

	/**
	 * This tests requests the list of jobs from the scheduler. It must be empty
	 * because no job was added.
	 */
	public void testJobList() {

		Collection<JobEntry> jobs = ebSchedulerFacade.jobList( null ).blockingRemove().getResponse();

		assertTrue( jobs.isEmpty() );
	}


	/**
	 * This test adds one job and requests for wait on this job. After that, the
	 * job is canceled and a call is made to block until the job is finished or
	 * canceled (<code>responseQueue.blockingRemove()</code>). This call
	 * should not block because the job was canceled.
	 * 
	 * @throws RemoteException
	 * @throws TaskSpecificationException
	 * @throws JobSpecificationException
	 */
	public void testWaitForJobWithCancelJob() throws RemoteException, TaskSpecificationException,
		JobSpecificationException {

		mockGum.getStatus();
		EasyMock.expectLastCall().andReturn( GumStatus.READY ).anyTimes();

		EasyMock.replay( mockGum );

		JobRequestEntry jobRequestEntry = new JobRequestEntry( 1, 1 );
		jobRequestEntry.addPeer( mockGump );

		this.gridManager.jobRequestsEntryMap.put( 1, jobRequestEntry );

		mockGump.finishRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		mockGump.pauseRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		// Sending gum and gumex
		ebSchedulerFacade.hereIsGum( mockGum, 1, mockGumpID );

		// Cancelling a job that does not exists
		EventQueue<NotifyEvent> responseQueue = ebSchedulerFacade.cancelJob( 1 );
		assertTrue( "The job should not exist", responseQueue.blockingRemove() instanceof JobNotFoundNotifyEvent );

		String expression = "os == linux";
		int numTasks = 1;
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		int jobId = ebSchedulerFacade.addJob( jobSpec );
		assertEquals( "The first job id should be '1'", 1, jobId );

		EventQueue<Event> responseQueueWFJ = ebSchedulerFacade.waitForJob( 1 );

		// Cancelling the job that exists
		responseQueue = ebSchedulerFacade.cancelJob( 1 );
		assertTrue( "The job should have been canceled",
				responseQueue.blockingRemove() instanceof JobCancelledNotifyEvent );

		// This method should not block. The job was canceled.
		ResponseEvent re = (ResponseEvent) responseQueueWFJ.blockingRemove();

		Collection<JobEntry> jobs = ebSchedulerFacade.jobList( null ).blockingRemove().getResponse();
		Iterator<JobEntry> jobsIt = jobs.iterator();
		while ( jobsIt.hasNext() ) {
			JobEntry job = jobsIt.next();
			if ( job.getId() == 1 ) {
				assertEquals( "The job should be in 'Canceled' state.", ExecutionStatus.CANCELLED, job.getStatus() );
			}
		}

		// Cancelling the job that has already been canceled
		responseQueue = ebSchedulerFacade.cancelJob( 1 );
		assertTrue( "The job should have already been canceled",
				responseQueue.blockingRemove() instanceof JobCancelledNotifyEvent );

		verifyMocks = true;
	}


	public void testWaitForJobWithRunningTasks() throws RemoteException, TaskSpecificationException,
		JobSpecificationException {

		mockGum.getStatus();
		EasyMock.expectLastCall().andReturn( GumStatus.UP ).anyTimes();

		// Creating the mockGump
		mockGump.disposeGum( EasyMock.anyLong(), (GumID) EasyMock.anyObject() );
		EasyMock.expectLastCall().anyTimes();

		mockGump.finishRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		mockGump.pauseRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGum );
		EasyMock.replay( mockGump );

		String expression = "os==linux";
		int numTasks = 5;
		fakeEBReplicaExecutorFacade.setReplicaStateToBeReturned( ExecutionStatus.FINISHED );
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		int jobId1 = ebSchedulerFacade.addJob( jobSpec );
		int jobId2 = ebSchedulerFacade.addJob( jobSpec );
		int jobId3 = ebSchedulerFacade.addJob( jobSpec );

		JobRequestEntry jobRequestEntry;

		ThreadServices.waitUntilWorkIsDone();

		jobRequestEntry = new JobRequestEntry( jobId1, 1 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( jobId1, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( jobId2, 2 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( jobId2, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( jobId3, 3 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( jobId3, jobRequestEntry );

		// Sending gum and gumex
		ebSchedulerFacade.hereIsGum( mockGum, 1, mockGumpID );
		EventQueue<Event> responseQueue = ebSchedulerFacade.waitForJob( jobId1 );
		responseQueue.blockingRemove();

		ebSchedulerFacade.hereIsGum( mockGum, 2, mockGumpID );
		responseQueue = ebSchedulerFacade.waitForJob( jobId2 );
		responseQueue.blockingRemove();

		ebSchedulerFacade.hereIsGum( mockGum, 3, mockGumpID );
		responseQueue = ebSchedulerFacade.waitForJob( jobId3 );
		responseQueue.blockingRemove();

		assertEquals( "The third job id should be '3'", 3, jobId3 );

		EventQueue<NotifyEvent> responseQueue2 = ebSchedulerFacade.cancelJob( jobId1 );
		assertTrue( "The job " + jobId1 + " should have already been canceled",
				responseQueue2.blockingRemove() instanceof JobCouldNotBeCancelledNotifyEvent );
		responseQueue2 = ebSchedulerFacade.cancelJob( jobId2 );
		assertTrue( "The job " + jobId2 + " should have already been canceled",
				responseQueue2.blockingRemove() instanceof JobCouldNotBeCancelledNotifyEvent );
		responseQueue2 = ebSchedulerFacade.cancelJob( jobId3 );
		assertTrue( "The job " + jobId3 + " should have already been canceled",
				responseQueue2.blockingRemove() instanceof JobCouldNotBeCancelledNotifyEvent );

		verifyMocks = true;
	}


	public void testAddJob() throws TaskSpecificationException, JobSpecificationException {

		String expression = "location==lsd && os==linux";
		int numTasks = 5;
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		int jobId = ebSchedulerFacade.addJob( jobSpec );
		assertEquals( "The first job id should be '1'", 1, jobId );

		// Verifying if the added job is in the list of jobs
		EventQueue<ResponseEvent<Collection<JobEntry>>> responseQueue = ebSchedulerFacade.jobList( null );
		ResponseEvent<Collection<JobEntry>> re = responseQueue.blockingRemove();
		Collection<JobEntry> jobs = re.getResponse();
		assertTrue( "The job must be in the job list", containsJob( jobs, 1 ) );
	}


	public void testAdd3Jobs() throws TaskSpecificationException, JobSpecificationException {

		String expression = "location==lsd && os==linux";
		int numTasks = 5;
		int numJobs = 3;
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		for ( int jobCount = 1; jobCount <= numJobs; jobCount++ ) {
			int jobId = ebSchedulerFacade.addJob( jobSpec );
			assertEquals( "The job id should be '" + jobCount + "'", jobCount, jobId );
		}

		// Verifying if the added job is in the list of jobs
		EventQueue<ResponseEvent<Collection<JobEntry>>> responseQueue = ebSchedulerFacade.jobList( null );
		ResponseEvent<Collection<JobEntry>> responseEvent = responseQueue.blockingRemove();
		Collection<JobEntry> jobs = responseEvent.getResponse();
		for ( int jobCount = 1; jobCount <= numJobs; jobCount++ ) {
			assertTrue( "The job must be in the job list", containsJob( jobs, jobCount ) );
		}
	}


	/**
	 * This test is intended to be a catcher of BID #121.
	 */
	public void testAddJobWithoutRemote() {

		/** Job requirements */
		String expression = "location==lsd && os==linux";

		/** List of tasks specifications to put into the Job */
		List<TaskSpec> taskSpecList = new LinkedList<TaskSpec>();

		TaskSpec t = null;

		try {

			t = new TaskSpec( null, null, null );
			fail( getClass().getName() + ".testAddJobWithoutRemote(): The task should be invaldated! REMOTE = null! " );

		} catch ( TaskSpecificationException e ) {
			System.out.println( "EXPECTED EXCEPTION: Cannot create a Task Spec without REMOTE." );
		}

		taskSpecList.add( t );

		try {
			new JobSpec( "TestJob", expression, taskSpecList );
			fail( "What the hell is going on?! The Job should not be created because the List of Tasks Specs is empty!" );
		} catch ( JobSpecificationException e1 ) {
			System.out.println( "EXPECTED EXCEPTION: Job was not correctly specified." );
		}
	}


	/* org.ourgrid.scheduler.GridMachineConsumer */

	public void testHereIsGuMGetAllGuMs() throws RemoteException {

		createTwoMockGumsAndOneMockGump();

		mockGump.disposeGum( 2, gumSpec2.getGumID() );
		EasyMock.expectLastCall().once();

		mockGump.disposeGum( 1, gumSpec1.getGumID() );
		EasyMock.expectLastCall().once();

		JobRequestEntry jobRequestEntry = new JobRequestEntry( 1, 1 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 1, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( 1, 2 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 2, jobRequestEntry );

		mockGump.finishRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		mockGump.finishRequestGums( 2 );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		// Getting an empty list of GuMeXEntries
		EventQueue<ResponseEvent<Collection<GumSpec>>> responseQueue = ebSchedulerFacade.getGumSpecs();
		ResponseEvent<Collection<GumSpec>> re = responseQueue.blockingRemove();
		Collection<GumSpec> gums = re.getResponse();
		assertTrue( gums.isEmpty() );

		ebSchedulerFacade.hereIsGum( mockGum1, 1, mockGumpID );
		ebSchedulerFacade.hereIsGum( mockGum2, 2, mockGumpID );

		// Getting the gums from the EBSchedulerFacade
		responseQueue = ebSchedulerFacade.getGumSpecs();
		re = responseQueue.blockingRemove();
		gums = re.getResponse();

		// verificar se mockGum1 est na lista
		assertTrue( verifyIfGumIsInList( mockGum1.getGumSpec(), gums ) );

		// verificar se mockGum2 est na lista
		assertTrue( verifyIfGumIsInList( mockGum2.getGumSpec(), gums ) );

		verify2Mocks = true;
	}


	/* org.ourgrid.scheduler.SchedulerHeuristic */

	public void testReplicaFinished() throws RemoteException, TaskSpecificationException, JobSpecificationException {

		ReplicaEntry replica = commonReplicaStart();

		ReplicaExecutorResult result = new ReplicaExecutorResult( replica );
		ebSchedulerFacade.replicaFinished( result );

		Collection<JobEntry> jobs = ebSchedulerFacade.jobList( null ).blockingRemove().getResponse();

		ReplicaEntry schedulerReplica = getReplicaById( jobs, replica );
		assertEquals( ExecutionStatus.FINISHED, schedulerReplica.getStatus() );
	}


	public void testReplicaFailed() throws Exception {

		ReplicaEntry replica = commonReplicaStartFail();

		ReplicaExecutorResult result = new ReplicaExecutorResult( replica );
		ebSchedulerFacade.replicaFailed( result );

		Collection<JobEntry> jobs = ebSchedulerFacade.jobList( null ).blockingRemove().getResponse();

		ReplicaEntry schedulerReplica = getReplicaById( jobs, replica );
		assertEquals( ExecutionStatus.FAILED, schedulerReplica.getStatus() );

	}


	public void testReplicaCanceled() throws RemoteException, TaskSpecificationException, JobSpecificationException {

		ReplicaEntry replica = commonReplicaStart();

		Map<String,String> expressions = new HashMap<String,String>();
		expressions.put( "os", "Linux" );

		ReplicaExecutorResult result = new ReplicaExecutorResult( replica );
		ebSchedulerFacade.replicaCanceled( result );

		EventQueue<ResponseEvent<Collection<JobEntry>>> responseQueue = ebSchedulerFacade.jobList( null );
		ResponseEvent<Collection<JobEntry>> re = responseQueue.blockingRemove();
		Collection<JobEntry> jobs = re.getResponse();

		ReplicaEntry schedulerReplica = getReplicaById( jobs, replica );
		assertEquals( ExecutionStatus.CANCELLED, schedulerReplica.getStatus() );

	}


	private ReplicaEntry commonReplicaStart() throws RemoteException, TaskSpecificationException,
		JobSpecificationException {

		createTwoMockGumsAndOneMockGump();

		mockGump.disposeGum( EasyMock.eq( 1 ), EasyMock.eq( mockGum1.getGumSpec().getGumID() ) );
		EasyMock.expectLastCall().once();

		mockGump.pauseRequestGums( 1 );
		EasyMock.expectLastCall().once();

		mockGump.finishRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		ReplicaEntry replica = addAJobAndGetARunningReplica();
		return replica;
	}


	private ReplicaEntry commonReplicaStartFail() throws Exception {

		createTwoMockGumsAndOneMockGump();

		mockGump.disposeGum( EasyMock.eq( 1 ), EasyMock.eq( mockGum1.getGumSpec().getGumID() ) );
		EasyMock.expectLastCall().once();

		mockGump.pauseRequestGums( 1 );
		EasyMock.expectLastCall().once();

		mockGump.resumeRequestGums( 1 );
		EasyMock.expectLastCall().once();

		mockGump.finishRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		ReplicaEntry replica = addAJobAndGetARunningReplica();
		return replica;
	}


	/* org.ourgrid.gridmachine.GuMManager */

	public void testGetGuMs() throws RemoteException {

		EventQueue<ResponseEvent<Collection<GumSpec>>> responseQueue = ebSchedulerFacade.getGumSpecs();
		ResponseEvent<Collection<GumSpec>> re = responseQueue.blockingRemove();
		Collection<GumSpec> gums = re.getResponse();

		assertTrue( gums.isEmpty() );

		createTwoMockGumsAndOneMockGump();

		mockGump.disposeGum( 1, mockGum1.getGumSpec().getGumID() );
		EasyMock.expectLastCall().once();

		mockGump.disposeGum( 2, mockGum2.getGumSpec().getGumID() );
		EasyMock.expectLastCall().once();

		mockGump.pauseRequestGums( 1 );
		EasyMock.expectLastCall().once();

		mockGump.pauseRequestGums( 2 );
		EasyMock.expectLastCall().once();

		mockGump.finishRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		JobRequestEntry jobRequestEntry = new JobRequestEntry( 1, 1 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 1, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( 2, 2 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 2, jobRequestEntry );

		// Sending mockGum1 and mockGum2
		ebSchedulerFacade.hereIsGum( mockGum1, 1, mockGumpID );
		ebSchedulerFacade.hereIsGum( mockGum2, 2, mockGumpID );

		// Getting gums from the EBSchedulerFacade
		responseQueue = ebSchedulerFacade.getGumSpecs();
		re = responseQueue.blockingRemove();
		gums = re.getResponse();

		// Verifying if the GuM was returned
		assertEquals( 2, gums.size() );
		assertTrue( gums.contains( mockGum1.getGumSpec() ) );
		assertTrue( gums.contains( mockGum2.getGumSpec() ) );

		verify2Mocks = true;
	}


	/* org.ourgrid.gump.GuMPManager */

	public void testSetAndGetGuMPs() {

		EventQueue<ResponseEvent<Collection<PeerEntry>>> responseQueue = ebSchedulerFacade.getPeerEntries( null );
		ResponseEvent<Collection<PeerEntry>> re = responseQueue.blockingRemove();
		Collection<PeerEntry> gumps = re.getResponse();

		assertTrue( gumps.isEmpty() );

		PeerSpec specLSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "1", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specLSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );
		PeerSpec specUCSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "2", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specUCSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );

		LinkedList<PeerSpec> gumpSpecs = new LinkedList<PeerSpec>();
		gumpSpecs.add( specLSD );
		gumpSpecs.add( specUCSD );

		ebSchedulerFacade.setPeers( gumpSpecs );

		responseQueue = ebSchedulerFacade.getPeerEntries( null );
		re = responseQueue.blockingRemove();
		gumps = re.getResponse();

		assertEquals( "Two gumps must be returned", 2, gumps.size() );
	}


	public void testCancelJob() throws RemoteException, TaskSpecificationException, JobSpecificationException {

		EventQueue<ResponseEvent<Collection<GumSpec>>> responseQueue = ebSchedulerFacade.getGumSpecs();
		ResponseEvent<Collection<GumSpec>> re = responseQueue.blockingRemove();
		Collection<GumSpec> gums = re.getResponse();

		assertTrue( gums.isEmpty() );

		createTwoMockGumsAndOneMockGump();

		mockGump.finishRequestGums( 1 );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		JobSpec jobSpec = createJobSpec( "", 1 );
		ebSchedulerFacade.addJob( jobSpec );
		ebSchedulerFacade.cancelJob( 1 );

		ThreadServices.waitUntilWorkIsDone();

		verify2Mocks = true;
	}


	/* org.ourgrid.mygrid.common.EBSyncShutdownable */

	public void testShutdown() throws Exception {

		EventQueue<ShutdownResponseEvent> shutdownResponse = ebSchedulerFacade.shutdown();
		shutdownResponse.blockingRemove();
		try {
			Naming.lookup( MyGridURLProvider.scheduler() );
			fail( "This should throw an exception!" );
		} catch ( Exception e ) {

		}
		try {
			Naming.lookup( MyGridURLProvider.gumpManager() );
			fail( "This should throw an exception!" );
		} catch ( Exception e ) {

		}
		try {
			Naming.lookup( MyGridURLProvider.gridManager() );
			fail( "This should throw an exception!" );
		} catch ( Exception e ) {

		}
		try {
			Naming.lookup( MyGridURLProvider.gumpClient() );
			fail( "This should throw an exception!" );
		} catch ( Exception e ) {

		}

	}


	/* Others */

	public void testGumIsDead() throws RemoteException {

		EventQueue<ResponseEvent<Collection<GumSpec>>> responseQueue = ebSchedulerFacade.getGumSpecs();
		ResponseEvent<Collection<GumSpec>> re = responseQueue.blockingRemove();
		Collection<GumSpec> gums = re.getResponse();
		assertTrue( gums.isEmpty() );

		createTwoMockGumsAndOneMockGump();

		mockGump.disposeGum( 1, mockGum1.getGumSpec().getGumID() );
		EasyMock.expectLastCall().anyTimes();

		mockGump.pauseRequestGums( 1 );
		EasyMock.expectLastCall().once();

		mockGump.finishRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGump );

		JobRequestEntry jobRequestEntry = new JobRequestEntry( 1, 1L );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 1, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( 2, 2L );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 2, jobRequestEntry );

		// Sending gum and gumex
		ebSchedulerFacade.hereIsGum( mockGum1, 1, mockGumpID );
		ebSchedulerFacade.hereIsGum( mockGum2, 2, mockGumpID );

		// Getting the gums from the EBSchedulerFacade
		responseQueue = ebSchedulerFacade.getGumSpecs();
		re = responseQueue.blockingRemove();
		gums = re.getResponse();

		assertTrue( verifyIfGumIsInList( mockGum1.getGumSpec(), gums ) );

		assertTrue( verifyIfGumIsInList( mockGum2.getGumSpec(), gums ) );

		// Warning the EBGridManager that the gum is dead
		ebSchedulerFacade.gumIsDead( 2, mockGum2.getGumSpec().getGumID() );

		responseQueue = ebSchedulerFacade.getGumSpecs();
		re = responseQueue.blockingRemove();
		gums = re.getResponse();
		assertFalse( verifyIfGumIsInList( mockGum2.getGumSpec(), gums ) );

		verify2Mocks = true;
	}


	/* Utils */

	private ReplicaEntry addAJobAndGetARunningReplica() throws TaskSpecificationException, JobSpecificationException {

		String expression = "os==linux";
		int numTasks = 1;
		JobSpec jobSpec = createJobSpec( expression, numTasks );
		int jobId = ebSchedulerFacade.addJob( jobSpec );
		assertEquals( "The first job id should be '1'", 1, jobId );

		JobManager jobManager = this.ebSchedulerFacade.getEventProcessor().getJobManager();

		ThreadServices.waitUntilWorkIsDone();

		Collection<JobEntry> jobs = jobManager.jobList();

		assertTrue( "The job added must be in the job list", containsJob( jobs, 1 ) );

		JobRequestEntry jobRequestEntry = new JobRequestEntry( jobId, 1 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( jobId, jobRequestEntry );

		ebSchedulerFacade.hereIsGum( mockGum1, 1, mockGumpID );

		ThreadServices.waitUntilWorkIsDone();

		ReplicaEntry replica = null;
		ReplicaEntry returnedReplica = null;
		Iterator<JobEntry> jobsIt = jobs.iterator();
		while ( jobsIt.hasNext() ) {
			JobEntry job = jobsIt.next();
			Iterator<TaskEntry> tasksIt = job.getTasks().iterator();
			while ( tasksIt.hasNext() ) {
				TaskEntry task = tasksIt.next();

				Collection<ReplicaEntry> replicas = task.getReplicas();
				System.out.println( "replicas " + replicas );
				Iterator<ReplicaEntry> replicasIt = replicas.iterator();
				while ( replicasIt.hasNext() ) {
					replica = replicasIt.next();
					if ( replica.getStatus().isRunning() ) {
						returnedReplica = replica;
						break;
					}
				}
				if ( returnedReplica != null ) {
					break;
				}
			}
			break;
		}

		assertEquals( ExecutionStatus.RUNNING, returnedReplica.getStatus() );
		return replica;
	}


	private ReplicaEntry getReplicaById( Collection<JobEntry> jobs, ReplicaEntry replica ) {

		Iterator<JobEntry> jobsIt = jobs.iterator();
		while ( jobsIt.hasNext() ) {
			JobEntry job = jobsIt.next();
			if ( job.getId() == replica.getJobId() ) {
				List<TaskEntry> tasks = job.getTasks();
				Iterator<TaskEntry> tasksIt = tasks.iterator();
				while ( tasksIt.hasNext() ) {
					TaskEntry task = tasksIt.next();
					if ( task.getId() == replica.getTaskId() ) {
						Collection<ReplicaEntry> replicas = task.getReplicas();
						Iterator<ReplicaEntry> replicasIt = replicas.iterator();
						while ( replicasIt.hasNext() ) {
							ReplicaEntry schedulerReplica = replicasIt.next();
							if ( schedulerReplica.getId() == replica.getId() ) {
								return schedulerReplica;
							}
						}
					}
				}
			}
		}
		return null;
	}


	/**
	 * Creates a job with <code>numTasks</code> identical tasks. Each task
	 * simply performs "echo 0".
	 * 
	 * @param expression The job attribute expression
	 * @param numTasks The number of tasks in the job
	 * @return The created job
	 * @throws TaskSpecificationException When the <code>TaskSpec</code> is
	 *         not valid
	 * @throws JobSpecificationException
	 */
	private JobSpec createJobSpec( String expression, int numTasks ) throws TaskSpecificationException,
		JobSpecificationException {

		List<TaskSpec> taskSpecList = new LinkedList<TaskSpec>();

		for ( int i = 0; i < numTasks; i++ ) {
			TaskSpec t = new TaskSpec( null, "echo 0", null );
			taskSpecList.add( t );
		}

		return new JobSpec( "TestJob", expression, taskSpecList );
	}


	/**
	 * Verifies if a job with 'jobId' is an element of a given collection of
	 * jobs.
	 * 
	 * @param jobs Job collection
	 * @param jobId The job id
	 * @return True if the job is present in the collection of jobs, false
	 *         otherwise
	 */
	private boolean containsJob( Collection<JobEntry> jobs, int jobId ) {

		Iterator<JobEntry> jobsIt = jobs.iterator();
		boolean containsJob = false;
		while ( jobsIt.hasNext() ) {
			JobEntry job = jobsIt.next();
			if ( job.getId() == jobId ) {
				containsJob = true;
			}
		}
		return containsJob;
	}


	private void createTwoMockGumsAndOneMockGump() {

		// Setting the GridMachine

		gumSpec1 = new GumSpec();
		gumSpec1.putAttribute( GumSpec.ATT_NAME, MOCK_GUM_NAME1 );
		gumSpec1.putAttribute( "os", "linux" );

		gumSpec2 = new GumSpec();
		gumSpec2.putAttribute( GumSpec.ATT_NAME, MOCK_GUM_NAME2 );
		gumSpec2.putAttribute( "os", "linux" );

		// Setting the GridMachine 1
		mockGum1 = EasyMock.createMock( GumClient.class );
		mockGum1.getGumSpec();
		EasyMock.expectLastCall().andReturn( gumSpec1 ).anyTimes();
		mockGum1.getStatus();
		EasyMock.expectLastCall().andReturn( GumStatus.UP ).anyTimes();

		// Setting the GridMachine 2
		mockGum2 = EasyMock.createMock( GumClient.class );
		mockGum2.getGumSpec();
		EasyMock.expectLastCall().andReturn( gumSpec2 ).anyTimes();
		mockGum2.getStatus();
		EasyMock.expectLastCall().andReturn( GumStatus.UP ).anyTimes();

		EasyMock.replay( mockGum1 );
		EasyMock.replay( mockGum2 );
	}


	private boolean verifyIfGumIsInList( GumSpec spec, Collection<GumSpec> gums ) {

		for ( GumSpec gum : gums ) {
			if ( spec.getGumID().equals( gum.getGumID() ) ) {
				return true;
			}
		}

		return false;
	}
}
