/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.rmi.RemoteException;

import org.ourgrid.common.gump.Gump;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.RequestSpec;

/**
 * Fake object.
 */
public class FakeGump implements Gump {

	protected int expectedWannaGuMs = 0;

	protected int expectedNoMoreGuMs = 0;

	protected int expectedDisposeOf = 0;

	protected int expectedLostGuM = 0;

	protected int wannaGuMsCount = 0;

	protected int noMoreGuMsCount = 0;

	protected int disposeOfCount = 0;

	protected int lostGuMCount = 0;


	public void setExpectedWannaGuMs( int expectedWannaGuMs ) {

		this.expectedWannaGuMs = expectedWannaGuMs;
	}


	public void setExpectedNMoreGuMs( int expectedNoMoreGuMs ) {

		this.expectedNoMoreGuMs = expectedNoMoreGuMs;
	}


	public void setExpectedDisposeOf( int expectedDisposeOf ) {

		this.expectedDisposeOf = expectedDisposeOf;
	}


	public void setExpectedLostGuM( int expectedLostGuM ) {

		this.expectedLostGuM = expectedLostGuM;
	}


	public void requestGums( RequestSpec requestSpec ) throws RemoteException {

		wannaGuMsCount++;
	}


	public void finishRequestGums( long requestID ) throws RemoteException {

		noMoreGuMsCount++;
	}


	public void disposeGum( long requestID, GumID gumID ) throws RemoteException {

		disposeOfCount++;
	}


	public void lostGum( long requestId, GumID gumID ) throws RemoteException {

		lostGuMCount++;
	}


	public void verify() {

		if ( expectedWannaGuMs != wannaGuMsCount ) {
			throw new RuntimeException( "wannaGuMs() was expected to be called " + expectedWannaGuMs
					+ " times but was " + wannaGuMsCount );
		}

		if ( expectedNoMoreGuMs != noMoreGuMsCount ) {
			throw new RuntimeException( "noMoreGuMs() was expected to be called " + expectedNoMoreGuMs
					+ " times but was " + noMoreGuMsCount );
		}

		if ( expectedDisposeOf != disposeOfCount ) {
			throw new RuntimeException( "disposeOf() was expected to be called " + expectedDisposeOf
					+ " times but was " + disposeOfCount );
		}

		if ( expectedLostGuM != lostGuMCount ) {
			throw new RuntimeException( "lostGuM() was expected to be called " + expectedLostGuM + " times but was "
					+ lostGuMCount );
		}
	}


	public void pauseRequestGums( long requestID ) throws RemoteException {

	}


	public void resumeRequestGums( long requestID ) throws RemoteException {

	}


	public ObjectID getObjectID() throws RemoteException {

		return null;
	}


	public void unwantedGum( long requestID, GumID gumID ) throws RemoteException {

		// TODO Auto-generated method stub

	}
}
