/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.rmi.RemoteException;
import java.util.Collection;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.gump.Gump;
import org.ourgrid.common.gump.GumpClient;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.mygrid.replicaexecutor.test.FakeEBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.FakeEBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.JobRequestEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.EBGridManager;
import org.ourgrid.mygrid.scheduler.gridmanager.GumResponseEntry;
import org.ourgrid.test.SchedulerFastTest;
import org.ourgrid.test.TestUtils;
import org.ourgrid.test.threadServices.ThreadServices;

public class GumpClientImplTest extends TestCase implements SchedulerFastTest {

	private static final ObjectID OBJECT_ID_2 = new ObjectID( "ArthurDent", (long) 4242,
			"rmi://arthurdent:4242/HeartOfGold" );

	private static final ObjectID OBJECT_ID_1 = new ObjectID( "ZaphodBeeblerox", (long) 4242,
			"rmi://zaphodbeeblerox:4242/HeartOfGold" );

	private Gump mockGump2;

	private Gump mockGump;

	private Gum mockGum2;

	private Gum mockGum1;

	private EBGridManager gridManager;

	private GumpClient gumClient;

	private FakeEBSchedulerFacade fakeEBSchedulerFacade;

	private TestUtils testUtils;

	private GumSpec gumSpec2;

	private GumSpec gumSpec1;


	/**
	 * Constructor for GuMManagerImplTest.
	 * 
	 * @param name Test name
	 */
	public GumpClientImplTest( String name ) {

		super( name );
	}


	@Override
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
		fakeEBSchedulerFacade = new FakeEBSchedulerFacade();
		fakeEBSchedulerFacade.config( new FakeEBReplicaExecutorFacade() );
		fakeEBSchedulerFacade.startEventProcessor();
		gridManager = (EBGridManager) fakeEBSchedulerFacade.getGridManager();
		gumClient = fakeEBSchedulerFacade.getGumpClient();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
		EventQueue responseQueue = fakeEBSchedulerFacade.shutdown();
		responseQueue.blockingRemove();

		/*
		 * This usage of aspects is needed because the Threads invokes the
		 * Configuration before finishing. Therefore, the Configuration just can
		 * be reseted after thread goes down.
		 */
		ThreadServices.waitUntilWorkIsDone();

		EasyMock.verify( mockGum1 );
		EasyMock.verify( mockGum2 );
		EasyMock.verify( mockGump );
		EasyMock.verify( mockGump2 );

		Configuration.reset();
		testUtils.reset();

	}


	public void testGetGuMs() throws RemoteException {

		Collection<GumSpec> gumClients = gridManager.getGumSpecs();
		assertTrue( gumClients.isEmpty() );

		createTwoMockGuMPs();

		mockGump.finishRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		mockGump2.finishRequestGums( EasyMock.anyLong() );
		EasyMock.expectLastCall().anyTimes();

		EasyMock.replay( mockGum1 );
		EasyMock.replay( mockGum2 );
		EasyMock.replay( mockGump );
		EasyMock.replay( mockGump2 );

		JobRequestEntry jobRequestEntry = new JobRequestEntry( 1, 1 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 1, jobRequestEntry );

		jobRequestEntry = new JobRequestEntry( 2, 2 );
		jobRequestEntry.addPeer( mockGump );

		gridManager.jobRequestsEntryMap.put( 2, jobRequestEntry );

		// Sending gum and gumex

		gumClient.hereIsGum( new GumResponseEntry( mockGum1, gumSpec1, 1, OBJECT_ID_1 ) );
		gumClient.hereIsGum( new GumResponseEntry( mockGum2, gumSpec2, 2, OBJECT_ID_2 ) );

		ThreadServices.waitUntilWorkIsDone();
		// Getting gums from the FakeEBSchedulerFacade
		gumClients = gridManager.getGumSpecs();

		// Verifying if the mockGuM was returned
		assertEquals( 2, gumClients.size() );
	}


	private void createTwoMockGuMPs() throws RemoteException {

		gumSpec1 = new GumSpec();
		gumSpec1.putAttribute( GumSpec.ATT_NAME, "MockGum1" );

		gumSpec2 = new GumSpec();
		gumSpec2.putAttribute( GumSpec.ATT_NAME, "MockGum2" );

		mockGum1 = EasyMock.createMock( Gum.class );
		mockGum2 = EasyMock.createMock( Gum.class );

		// Creating the mockGump
		mockGump = EasyMock.createMock( Gump.class );
		mockGump.disposeGum( 1, gumSpec1.getGumID() );
		EasyMock.expectLastCall().anyTimes();

		// Creating the mockGump2
		mockGump2 = EasyMock.createMock( Gump.class );
		mockGump2.disposeGum( 2, gumSpec2.getGumID() );
		EasyMock.expectLastCall().anyTimes();

	}
}
