/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ShutdownResponseEvent;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.replicaexecutor.test.FakeEBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.EBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.FakeEBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.gump.GumpManager;
import org.ourgrid.mygrid.scheduler.gump.GumpManagerImpl;
import org.ourgrid.mygrid.scheduler.gump.MyGridGumpSpecServices;
import org.ourgrid.test.SchedulerFastTest;
import org.ourgrid.test.TestUtils;

public class GumpManagerImplTest extends TestCase implements SchedulerFastTest {

	private GumpManager gumpManager;

	private FakeEBSchedulerFacade fakeEBSchedulerFacade;

	private TestUtils testUtils;


	/**
	 * Constructor for GuMPManagerImplTest.
	 * 
	 * @param name Test name
	 */
	public GumpManagerImplTest( String name ) {

		super( name );
	}


	@Override
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
		fakeEBSchedulerFacade = new FakeEBSchedulerFacade();
		fakeEBSchedulerFacade.config( new FakeEBReplicaExecutorFacade() );
		fakeEBSchedulerFacade.startEventProcessor();
		gumpManager = fakeEBSchedulerFacade.getGumpManager();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
		EventQueue<ShutdownResponseEvent> responseQueue = fakeEBSchedulerFacade.shutdown();
		responseQueue.blockingRemove();
		Configuration.reset();
		testUtils.reset();
	}


	public void testSetAndGetGuMPs() throws RemoteException {

		List<PeerEntry> receivedGumpSpecs = (List<PeerEntry>) gumpManager.getPeerEntries( null );
		assertTrue( receivedGumpSpecs.isEmpty() );

		PeerSpec specLSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "1", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specLSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );
		PeerSpec specUCSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "2", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specUCSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );

		LinkedList<PeerSpec> gumpSpecs = new LinkedList<PeerSpec>();
		gumpSpecs.add( specLSD );
		gumpSpecs.add( specUCSD );

		gumpManager.setPeers( gumpSpecs );

		receivedGumpSpecs = (List<PeerEntry>) gumpManager.getPeerEntries( null );
		assertEquals( "Two gumps must be returned", 2, receivedGumpSpecs.size() );
	}


	public void testDeadAndAlive() throws RemoteException {

		EBSchedulerFacade schedulerFacade = EasyMock.createMock( EBSchedulerFacade.class );
		GumpManager gumpManager = new GumpManagerImpl( schedulerFacade );

		PeerSpec specLSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "1", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specLSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );
		PeerSpec specUCSD = new PeerSpec( TestUtils.FAKE_MACHINE_NAME + "2", Integer.parseInt( testUtils
				.getProperty( TestUtils.FAKE_PORT ) ) );
		specUCSD.setLabel( MyGridGumpSpecServices.FAKE_GUMP );

		LinkedList<PeerSpec> gumpSpecs = new LinkedList<PeerSpec>();
		gumpSpecs.add( specLSD );
		gumpSpecs.add( specUCSD );

		schedulerFacade.setPeers( gumpSpecs );
		EasyMock.expectLastCall().anyTimes();

		schedulerFacade.peerAlive( specLSD );
		EasyMock.expectLastCall().once();

		schedulerFacade.peerIsDead( specLSD );
		EasyMock.expectLastCall().once();

		schedulerFacade.peerAlive( specUCSD );
		EasyMock.expectLastCall().once();

		schedulerFacade.peerIsDead( specUCSD );
		EasyMock.expectLastCall().once();

		EasyMock.replay( schedulerFacade );

		gumpManager.peerAlive( specLSD.getURL() );
		gumpManager.peerIsDead( specLSD.getURL() );

		gumpManager.peerAlive( specUCSD.getURL() );
		gumpManager.peerIsDead( specUCSD.getURL() );

		gumpManager.setPeers( gumpSpecs );

		gumpManager.peerAlive( "aha" );
		gumpManager.peerAlive( "eha" );
		gumpManager.peerAlive( "iha" );
		gumpManager.peerAlive( "uha" );
		gumpManager.peerIsDead( "aha" );
		gumpManager.peerIsDead( "eha" );
		gumpManager.peerIsDead( "iha" );
		gumpManager.peerIsDead( "uha" );

		gumpManager.peerAlive( specLSD.getURL() );
		gumpManager.peerIsDead( specLSD.getURL() );

		gumpManager.peerAlive( specUCSD.getURL() );
		gumpManager.peerIsDead( specUCSD.getURL() );

		EasyMock.verify( schedulerFacade );
	}
}
