/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.rmi.RemoteException;

import junit.framework.TestCase;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.event.ShutdownRequestEvent;
import org.ourgrid.common.event.ShutdownResponseEvent;
import org.ourgrid.mygrid.replicaexecutor.test.FakeEBReplicaExecutorFacade;
import org.ourgrid.mygrid.scheduler.FakeEBSchedulerFacade;
import org.ourgrid.mygrid.scheduler.SchedulerEventEngine;
import org.ourgrid.mygrid.scheduler.event.SchedulerTestRequestEvent;
import org.ourgrid.mygrid.scheduler.event.SchedulerTestResponseEvent;
import org.ourgrid.test.SchedulerFastTest;
import org.ourgrid.test.TestUtils;
import org.ourgrid.test.threadServices.ThreadServices;

public class SchedulerEventEngineTest extends TestCase implements SchedulerFastTest {

	/**
	 * Object that is tested.
	 */
	private SchedulerEventEngine schedulerEventProcessor;

	private FakeEBSchedulerFacade fakeEBSchedulerFacade;

	private TestUtils testUtils;


	/**
	 * The constructor for SchedulerEventEngineTest.
	 * 
	 * @param name Test name
	 */
	public SchedulerEventEngineTest( String name ) {

		super( name );
	}


	@Override
	protected void setUp() throws ConfigException, RemoteException {

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
		fakeEBSchedulerFacade = new FakeEBSchedulerFacade();
		fakeEBSchedulerFacade.config( new FakeEBReplicaExecutorFacade() );
		schedulerEventProcessor = fakeEBSchedulerFacade.getEventProcessor();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
		EventQueue responseQueue = fakeEBSchedulerFacade.shutdown();
		responseQueue.blockingRemove();
		Configuration.reset();
		testUtils.reset();
	}


	/**
	 * Tests all methods from SchedulerEventEngine.
	 */
	public void testAll() {

		EventQueue eventQueue = new EventQueue();
		RequestEvent requestEvent = new SchedulerTestRequestEvent( eventQueue );

		// Put Event, StartProcessing and isAlive
		schedulerEventProcessor.putEvent( requestEvent );
		ThreadServices.waitUntilWorkIsDone();

		Object response = eventQueue.unblockingRemove();
		assertNull( response );
		assertFalse( schedulerEventProcessor.isAlive() );

		schedulerEventProcessor.startProcessing();
		ThreadServices.waitUntilWorkIsDone();
		assertTrue( schedulerEventProcessor.isAlive() );
		response = eventQueue.blockingRemove();
		assertTrue( response instanceof SchedulerTestResponseEvent );

		int numberOfEventQueues = 10000;
		EventQueue[ ] eventQueues = new EventQueue[ numberOfEventQueues ];
		for ( int k = 0; k < eventQueues.length; k++ ) {
			eventQueues[k] = new EventQueue();
			RequestEvent re = new SchedulerTestRequestEvent( eventQueues[k] );
			schedulerEventProcessor.putEvent( re );
		}

		for ( int k = 0; k < eventQueues.length; k++ ) {
			Object responseEvent = eventQueues[k].blockingRemove();
			assertNotNull( responseEvent );
			assertTrue( responseEvent instanceof SchedulerTestResponseEvent );
		}

		// Shutdown, isAlive
		assertTrue( schedulerEventProcessor.isAlive() );
		schedulerEventProcessor.putEvent( new ShutdownRequestEvent( eventQueue, schedulerEventProcessor ) );
		ThreadServices.waitUntilWorkIsDone();
		assertFalse( schedulerEventProcessor.isAlive() );
		response = eventQueue.unblockingRemove();
		assertTrue( response instanceof ShutdownResponseEvent );
	}

}
