/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.io.File;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.test.LocalPeer;
import org.ourgrid.test.SchedulerPreCommitTest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class SpecialTest extends TestCase implements SchedulerPreCommitTest, SpecialCareTest {

	private TestUtils testUtils;


	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		MyGridUIManager.reset();

		testUtils.restartMygrid();
	}


	/**
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();

		testUtils.stopMygrid();

		MyGridUIManager.reset();
		Configuration.reset();
		testUtils.reset();
	}


	/**
	 * A fail in this test is identified by the blocking of the waitForJob
	 * method
	 * 
	 * @throws Exception when something wrong occurs
	 */
	public void testAddJobAndSetGuMPs() throws Exception {

		int jobId = MyGridUIManager.getInstance().addJob( testUtils.getSimpleJobSpec() );
		// GridSetter mgSetter = new GridSetter();

		LocalPeer localPeer = new LocalPeer( Integer.parseInt( testUtils.getProperty( TestUtils.OG_PORT ) ) );

		System.out.println( this.getClass().getCanonicalName() + " Starting a local peer " );
		localPeer.start();

		System.out.println( this.getClass().getCanonicalName() + " Setting gums to this local peer " );
		localPeer.setGuMs( new File( testUtils.createSDF( testUtils.getGumSpecs( testUtils
				.getNumberOfGuMSpecsAvailable( TestUtils.TYPE_UALINUX ), TestUtils.TYPE_UALINUX ) ) ) );

		System.out.println( this.getClass().getCanonicalName() + " Setting a grid " );
		TestUtils.executeCommand( TestUtils.MYGRID_COMMAND,
				new String[ ] { "setgrid",
								localPeer.generateTempGDF( testUtils.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ) } );

		System.out.println( this.getClass().getCanonicalName() + " Waiting for a job " );
		MyGridUIManager.getInstance().waitForJob( jobId );

		System.out.println( this.getClass().getCanonicalName() + " Stopping local peer " );
		localPeer.stop();
	}

}
