/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.scheduler.test;

import java.util.Iterator;
import java.util.LinkedList;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.exception.IllegalStateException;
import org.ourgrid.common.spec.TaskSpec;
import org.ourgrid.common.spec.exception.TaskSpecificationException;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.ReplicaExecutorResult;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;
import org.ourgrid.test.SchedulerFastTest;
import org.ourgrid.test.TestUtils;

public class TaskTest extends TestCase implements SchedulerFastTest {

	private TestUtils testUtils;


	/**
	 * Constructor for TaskTest.
	 * 
	 * @param name Test name
	 */
	public TaskTest( String name ) {

		super( name );
	}


	public void testReplicaFinished() throws TaskSpecificationException {

		TaskSpec taskSpec = new TaskSpec( null, "echo 0", null );
		TaskEntry task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		ReplicaEntry realReplica = task.getNewReplica();
		assertTrue( "Initially the replica must be unstarted", realReplica.getStatus().equals(
				ExecutionStatus.UNSTARTED ) );

		ReplicaEntry clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		ReplicaExecutorResult result = new ReplicaExecutorResult( clonedReplica );
		task.replicaFinished( result );

		Iterator replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			assertEquals( "The replica was not running. It should not become finished.", returnedReplica.getStatus(),
					ExecutionStatus.UNSTARTED );
		}

		taskSpec = new TaskSpec( null, "echo 0", null );
		task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		realReplica = task.getNewReplica();
		assertEquals( "Initially the replica must be unstarted", realReplica.getStatus(), ExecutionStatus.UNSTARTED );
		realReplica.setStatus( ExecutionStatus.RUNNING );
		assertEquals( "The replica should have its state changed to running", realReplica.getStatus(),
				ExecutionStatus.RUNNING );

		clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		result = new ReplicaExecutorResult( clonedReplica );
		task.replicaFinished( result );

		replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			System.out.println( "replica atual: " + returnedReplica );
			assertTrue( "The replica was running. It should become finished.", returnedReplica.getStatus().equals(
					ExecutionStatus.FINISHED ) );
		}

	}


	public void testReplicaFailed() throws TaskSpecificationException {

		TaskSpec taskSpec = new TaskSpec( null, "echo 0", null );
		TaskEntry task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		ReplicaEntry realReplica = task.getNewReplica();
		assertTrue( "Initially the replica must be unstarted", realReplica.getStatus().equals(
				ExecutionStatus.UNSTARTED ) );

		ReplicaEntry clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		ReplicaExecutorResult result = new ReplicaExecutorResult( clonedReplica );
		task.replicaFailed( result );

		Iterator replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			assertTrue( "The replica was not running. It should not become failed.", returnedReplica.getStatus()
					.equals( ExecutionStatus.UNSTARTED ) );
		}

		taskSpec = new TaskSpec( null, "echo 0", null );
		task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		realReplica = task.getNewReplica();
		assertEquals( "Initially the replica must be unstarted", realReplica.getStatus(), ExecutionStatus.UNSTARTED );
		realReplica.setStatus( ExecutionStatus.RUNNING );
		assertEquals( "The replica should have its state changed to running", realReplica.getStatus(),
				ExecutionStatus.RUNNING );

		clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		result = new ReplicaExecutorResult( clonedReplica );
		task.replicaFailed( result );

		replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			assertTrue( "The replica was running. It should become failed.", returnedReplica.getStatus().equals(
					ExecutionStatus.FAILED ) );
		}

	}


	public void testReplicaCanceled() throws TaskSpecificationException {

		TaskSpec taskSpec = new TaskSpec( null, "echo 0", null );
		TaskEntry task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		ReplicaEntry realReplica = task.getNewReplica();
		assertEquals( "Initially the replica must be unstarted", realReplica.getStatus(), ExecutionStatus.UNSTARTED );

		ReplicaEntry clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		ReplicaExecutorResult result = new ReplicaExecutorResult( clonedReplica );
		task.replicaCanceled( result );

		Iterator replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			assertTrue( "The replica should become canceled.", returnedReplica.getStatus().equals(
					ExecutionStatus.CANCELLED ) );
		}
	}


	public void testReplicaAborted() throws TaskSpecificationException {

		TaskSpec taskSpec = new TaskSpec( null, "echo 0", null );
		TaskEntry task = new TaskEntry( taskSpec, 1, 1, 1, 1 );
		ReplicaEntry realReplica = task.getNewReplica();
		assertEquals( "Initially the replica must be unstarted", realReplica.getStatus(), ExecutionStatus.UNSTARTED );
		realReplica.setStatus( ExecutionStatus.RUNNING );
		assertEquals( "The replica should have its state changed to running", realReplica.getStatus(),
				ExecutionStatus.RUNNING );

		ReplicaEntry clonedReplica = new ReplicaEntry( 1, 1, 1, taskSpec );
		ReplicaExecutorResult result = new ReplicaExecutorResult( clonedReplica );
		task.replicaAborted( result );

		Iterator replicasIt = task.getReplicas().iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry returnedReplica = (ReplicaEntry) replicasIt.next();
			assertTrue( "The replica  should become aborted.", returnedReplica.getStatus().equals(
					ExecutionStatus.ABORTED ) );
		}
	}


	/**
	 * Tests if, according to the Replicas states, a task is in the correct
	 * state. It tests a several combination of Tasks states.
	 */
	public void testGetState() throws TaskSpecificationException {

		int maxReplicas = 6;
		int maxFails = 2;
		TaskEntry task = createTask( maxFails, maxReplicas );
		assertEquals( "The state should be 'Ready'", ExecutionStatus.UNSTARTED, task.getState() );
		createReplicas( task );
		assertEquals( "The state should be 'Ready'", ExecutionStatus.UNSTARTED, task.getState() );

		// There is at least one running replica
		LinkedList<String> states = new LinkedList<String>();
		String[ ] statesArrayRunning = { "1", "2", "3", "4", "5", "6" };
		ExecutionStatus[ ] replicasStatesRunning = { ExecutionStatus.UNSTARTED, ExecutionStatus.RUNNING,
													ExecutionStatus.FAILED, ExecutionStatus.CANCELLED,
													ExecutionStatus.ABORTED, ExecutionStatus.FINISHED };
		assertAll( ExecutionStatus.RUNNING, ExecutionStatus.RUNNING, states, statesArrayRunning, replicasStatesRunning,
				maxReplicas, maxFails );

		// There is at least one unstarted replica (no running replicas)
		maxReplicas = 5;
		states = new LinkedList<String>();
		String[ ] statesArrayUnstarted = { "1", "2", "3", "4", "5" };
		ExecutionStatus[ ] replicasStatesUnstarted = { ExecutionStatus.UNSTARTED, ExecutionStatus.FAILED,
														ExecutionStatus.CANCELLED, ExecutionStatus.ABORTED,
														ExecutionStatus.FINISHED };
		assertAll( ExecutionStatus.RUNNING, ExecutionStatus.UNSTARTED, states, statesArrayUnstarted,
				replicasStatesUnstarted, maxReplicas, maxFails );

		// There is at least one canceled replica (no running and unstarted
		// replicas)
		maxReplicas = 4;
		states = new LinkedList<String>();
		String[ ] statesArrayCanceled = { "1", "2", "3", "4" };
		ExecutionStatus[ ] replicasStatesCanceled = { ExecutionStatus.FAILED, ExecutionStatus.CANCELLED,
														ExecutionStatus.ABORTED, ExecutionStatus.FINISHED };
		assertAll( ExecutionStatus.CANCELLED, ExecutionStatus.CANCELLED, states, statesArrayCanceled,
				replicasStatesCanceled, maxReplicas, maxFails );

		// There is at least one finished replica (no running, unstarted and
		// canceled replicas)
		maxReplicas = 3;
		states = new LinkedList<String>();
		String[ ] statesArrayFinished = { "1", "2", "3" };
		ExecutionStatus[ ] replicasStatesFinished = { ExecutionStatus.FAILED, ExecutionStatus.ABORTED,
														ExecutionStatus.FINISHED };
		assertAll( ExecutionStatus.FINISHED, ExecutionStatus.FINISHED, states, statesArrayFinished,
				replicasStatesFinished, maxReplicas, maxFails );

		// There is at least one failed replica (no running, unstarted, canceled
		// and finished replicas)
		maxReplicas = 2;
		states = new LinkedList<String>();
		String[ ] statesArrayFailed = { "1", "2" };
		ExecutionStatus[ ] replicasStatesFailed = { ExecutionStatus.FAILED, ExecutionStatus.ABORTED };
		assertAll( ExecutionStatus.FAILED, ExecutionStatus.FAILED, states, statesArrayFailed, replicasStatesFailed,
				maxReplicas, maxFails );

		// There is only aborted replicas
		maxReplicas = 2;
		states = new LinkedList<String>();
		String[ ] statesArrayAborted = { "1", "2" };
		ExecutionStatus[ ] replicasStatesAborted = { ExecutionStatus.ABORTED, ExecutionStatus.ABORTED };
		assertAll( null, ExecutionStatus.ABORTED, states, statesArrayAborted, replicasStatesAborted, maxReplicas,
				maxFails );
	}


	public void testCompareTo() {

		int task1Id = 1;
		int task1JobId = 1;
		int task1NumberOfRunningReplicas = 0;
		TaskEntry task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		int task2Id = 1;
		int task2JobId = 1;
		int task2NumberOfRunningReplicas = 0;
		TaskEntry task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The tasks should have the same priority", 0, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 2;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 3;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task1 should have priority", -1, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 3;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 2;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 3;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task1 should have priority", -1, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 3;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 2;
		task2NumberOfRunningReplicas = 3;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task1 should have priority", -1, task1.compareTo( task2 ) );

		task1Id = 10;
		task1JobId = 5;
		task1NumberOfRunningReplicas = 1;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 10;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task1 should have priority", -1, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 3;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 2;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task2 should have priority", 1, task1.compareTo( task2 ) );

		task1Id = 2;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 3;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 3;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task2 should have priority", 1, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 2;
		task1NumberOfRunningReplicas = 3;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 1;
		task2JobId = 1;
		task2NumberOfRunningReplicas = 3;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task2 should have priority", 1, task1.compareTo( task2 ) );

		task1Id = 1;
		task1JobId = 1;
		task1NumberOfRunningReplicas = 10;
		task1 = new TaskEntry( null, task1Id, task1JobId, 1, 3 );
		createRunningReplicas( task1, task1NumberOfRunningReplicas );

		task2Id = 10;
		task2JobId = 5;
		task2NumberOfRunningReplicas = 1;
		task2 = new TaskEntry( null, task2Id, task2JobId, 1, 3 );
		createRunningReplicas( task2, task2NumberOfRunningReplicas );

		assertEquals( "The task2 should have priority", 1, task1.compareTo( task2 ) );

	}


	/**
	 * Asserts if the a task is in the expected state in several combinations of
	 * Replicas states.
	 * 
	 * @param status The expected state the Task will be in all combinations.
	 * @param fixedReplicaState A Replica state that will be fixed to fix one
	 *        Task state - to be possible to know which state Job will be in
	 *        advance.
	 * @param states Where the Replica states will be held.
	 * @param statesArrayRunning The possible Replica state array - to foresee
	 *        which state the Task will be.
	 * @param replicasStates The possible replica state array - to foresee which
	 *        state the Task will be.
	 * @param maxReplicas The maximum number of replicas that a task can have
	 * @param maxFails The maximum number of replica fails that a task can have
	 */
	private void assertAll( ExecutionStatus expectedTaskState, ExecutionStatus fixedReplicaState,
							LinkedList<String> states, String[ ] statesArray, ExecutionStatus[ ] replicasStates,
							int maxReplicas, int maxFails ) throws TaskSpecificationException {

		int depth = maxReplicas - 1;
		for ( int x = 0; x < statesArray.length; x++ ) {
			states.add( statesArray[x] );
		}

		for ( int a = 1; a < depth; a++ ) {
			states = copyStates( statesArray, states );
		}

		for ( int x = 0; x < states.size(); x++ ) {
			TaskEntry task = createTask( maxFails, maxReplicas );
			LinkedList replicas = createReplicas( task );
			ReplicaEntry replica = (ReplicaEntry) replicas.get( 0 );
			if ( fixedReplicaState.equals( ExecutionStatus.FAILED ) ) {
				replica.setStatus( ExecutionStatus.RUNNING );
				ReplicaExecutorResult rer = new ReplicaExecutorResult( replica );
				task.replicaFailed( rer );
			} else {
				replica.setStatus( fixedReplicaState );
			}

			String statesString = states.get( x );
			for ( int y = 0; y < statesString.length(); y++ ) {
				replica = (ReplicaEntry) replicas.get( y + 1 );
				int intState = Integer.parseInt( statesString.charAt( y ) + "" ) - 1;
				if ( replicasStates[intState].equals( ExecutionStatus.FAILED ) ) {
					replica.setStatus( ExecutionStatus.RUNNING );
					ReplicaExecutorResult rer = new ReplicaExecutorResult( replica );
					task.replicaFailed( rer );
				} else {
					replica.setStatus( replicasStates[intState] );
				}
			}

			// Exceptional cases
			if ( hasAbortedReplicaWithoutAFinishedReplica( replicas ) ) {
				try {
					task.getState();
					fail( "An IllegalStateException should be thrown" );
				} catch ( IllegalStateException ise ) {
					// The exception was thrown as expected
				}
			} else if ( areAllReplicasUnstarted( replicas ) ) {
				assertEquals( ExecutionStatus.UNSTARTED, task.getState() );
			} else if ( (task.getActualFails() >= task.getMaxFails())
					&& !task.getState().equals( ExecutionStatus.RUNNING )
					&& !task.getState().equals( ExecutionStatus.FINISHED )
					&& !task.getState().equals( ExecutionStatus.CANCELLED ) ) {
				assertEquals( ExecutionStatus.FAILED, task.getState() );
			} else {
				assertEquals( expectedTaskState, task.getState() );
			}
		}
	}


	/**
	 * Verifies if there exists an aborted replica without a finished one.
	 * 
	 * @return True if there exists an aborted replica without a finished one,
	 *         false otherwise
	 */
	private boolean hasAbortedReplicaWithoutAFinishedReplica( LinkedList replicas ) {

		boolean hasFinished = false;
		boolean hasAborted = false;
		Iterator replicasIt = replicas.iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry replica = (ReplicaEntry) replicasIt.next();
			if ( replica.getStatus().equals( ExecutionStatus.FINISHED ) ) {
				hasFinished = true;
			}
			if ( replica.getStatus().equals( ExecutionStatus.ABORTED ) ) {
				hasAborted = true;
			}
		}

		return !hasFinished && hasAborted;
	}


	/**
	 * Checks if all Replicas are unstarted.
	 * 
	 * @param replicas The a list of Replicas.
	 * @return <code>true</code> if all Replicas are unstarted,
	 *         <code>false</code> otherwise.
	 */
	private boolean areAllReplicasUnstarted( LinkedList replicas ) {

		Iterator replicasIt = replicas.iterator();
		while ( replicasIt.hasNext() ) {
			ReplicaEntry replica = (ReplicaEntry) replicasIt.next();
			if ( !replica.getStatus().equals( ExecutionStatus.UNSTARTED ) ) {
				return false;
			}
		}

		return true;
	}


	private LinkedList<String> copyStates( String[ ] statesArray, LinkedList states ) {

		LinkedList<String> newStates = new LinkedList<String>();
		for ( int x = 0; x < statesArray.length; x++ ) {
			for ( int y = 0; y < states.size(); y++ ) {
				newStates.add( ((String) states.get( y )) + statesArray[x] );
			}
		}
		return newStates;
	}


	private TaskEntry createTask( int maxFails, int maxReplicas ) throws TaskSpecificationException {

		TaskSpec taskSpec = new TaskSpec( null, "echo 0", null );
		TaskEntry task = new TaskEntry( taskSpec, 1, 1, maxFails, maxReplicas );

		return task;
	}


	private LinkedList createReplicas( TaskEntry task ) {

		LinkedList<ReplicaEntry> replicasList = new LinkedList<ReplicaEntry>();
		while ( task.canReplicate() ) {
			replicasList.add( task.getNewReplica() );
		}
		return replicasList;
	}


	private void createRunningReplicas( TaskEntry task, int numberOfRunningReplicas ) {

		for ( int x = 0; x < numberOfRunningReplicas; x++ ) {
			if ( task.canReplicate() ) {
				ReplicaEntry replica = task.getNewReplica();
				replica.setStatus( ExecutionStatus.RUNNING );
			}
		}
	}


	public static void main( String[ ] args ) throws TaskSpecificationException {

		TaskTest tt = new TaskTest( "test" );
		tt.testGetState();
	}


	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
	}


	protected void tearDown() throws Exception {

		super.tearDown();
		Configuration.reset();
		testUtils.reset();
	}
}
