/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.mygrid.ui.UIManager;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.PreCommitTest;

public class CommandManagerTest extends TestCase implements Command, GUITest, PreCommitTest {

	private CommandManager commandManager;

	private boolean runCalled;

	private final String[ ] TEST = { "test1", "test2", "test3" };

	private String[ ] paramsResult;


	protected void setUp() throws Exception {

		super.setUp();
		runCalled = false;
		UIManager mockMygridUIManager = EasyMock.createNiceMock( UIManager.class );
		commandManager = new CommandManager( mockMygridUIManager );
	}


	protected void tearDown() throws Exception {

		super.tearDown();
		runCalled = false;
		paramsResult = null;
	}


	/*
	 * Test method for
	 * 'org.ourgrid.mygrid.ui.MyGridCommandManager.executeCommand(String[])'
	 */
	public void testExecuteCommand() throws Exception {

		// Insert first two commands
		commandManager.addNewCommand( TEST[0], this );
		commandManager.addNewCommand( TEST[1], this );

		commandManager.executeCommand( new String[ ] { TEST[0] } );
		assertTrue( "It should be true", runCalled );
		runCalled = false;

		commandManager.executeCommand( new String[ ] { TEST[1] } );
		assertTrue( "It should be true", runCalled );
		runCalled = false;

		// Insert one command more
		commandManager.addNewCommand( TEST[2], this );

		commandManager.executeCommand( new String[ ] { TEST[2] } );
		assertTrue( "It should be true", runCalled );
		runCalled = false;

	}


	/*
	 * Test method for
	 * 'org.ourgrid.mygrid.ui.MyGridCommandManager.executeCommand(String[])'
	 */
	public void testExecuteCommandWithManyParameters() {

		String[ ] params = TEST;
		commandManager.addNewCommand( TEST[0], this );
		try {
			commandManager.executeCommand( params );
			assertTrue( "It should be true", runCalled );
			assertTrue( paramsResult[0] == TEST[1] );
			assertTrue( paramsResult[1] == TEST[2] );
		} catch ( Exception e ) {
			fail( "Problems executing run() method. Maybe the command does not exists." );
		}
	}


	public void testInvalidCommand() {

		commandManager.addNewCommand( TEST[0], this );
		commandManager.addNewCommand( TEST[1], this );
		commandManager.addNewCommand( TEST[2], this );

		try {
			commandManager.executeCommand( new String[ ] { "UnknownCommand" } );
			fail( "Command does not exist. Exception should be thrown" );
		} catch ( Exception e ) {
		}

		assertFalse( "It should be false", runCalled );
	}


	public void testAddJob() {

		commandManager.initCommandMap();

	}


	public void testConfig() throws Exception {

		commandManager.initCommandMap();

		commandManager.executeCommand( new String[ ] { CommandManager.MYGRID_CONFIG } );
	}


	public int run( String[ ] params ) {

		runCalled = true;
		paramsResult = params;

		return ErrorCode.OK;
	}


	public boolean configureEnvironment() {

		return true;
	}

}
