/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.io.File;

import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.mygrid.ui.Wizard;

/**
 * This class checks and configures the configuration files directory, i.e. it
 * assures that ".mygrid" is created at /home/user/ and that MG.defaults and
 * MG.properties are available there.
 */
public class EnvironmentConfigurator implements Command {

	/** File separator symbol */
	private static final String fileSeparator = System.getProperty( "file.separator" );

	/** Holds Mygrid configuration directory ($OGROOT) */
	private static final String mgConfDir = MyGridConfiguration.ROOT_DIR + fileSeparator;


	/**
	 * Builds correctly the MyGrid configuation directory, cheking if it already
	 * exists and if the necessary contents are into this.
	 */
	public int run( String[ ] params ) {

		boolean newMGProperties = false;
		for ( int k = 0; k < params.length; k++ ) {
			if ( params[k].equals( "--mgproperties" ) ) {
				newMGProperties = true;
				break;
			}
		}

		/* assures that the configuration directory (.mygrid) exists */
		File mygridConfDir = new File( mgConfDir );
		if ( !mygridConfDir.exists() ) {
			mygridConfDir.mkdir();
		}

		/*
		 * tests whether mg.properties is available at ".mygrid". If it is not,
		 * the Wizard is called to solve this problem either by importing or
		 * creating a new one
		 */
		File mgProperties = new File( MyGridConfiguration.PROPERTIES_FILENAME );

		if ( newMGProperties ) {
			mgProperties.delete();
		}

		if ( !mgProperties.exists() ) {
			new Wizard().start();
		}

		return ErrorCode.OK;
	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return false;
	}

}
