/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.common.url.MyGridURLProvider;
import org.ourgrid.mygrid.ui.UIManager;
import org.ourgrid.mygrid.ui.gui.EBGuiFacade;
import org.ourgrid.mygrid.ui.gui.GUIListenerImpl;

/**
 * Description: This class is responsible to print mygrid current job and gums
 * state. This can be made by getting an instance from <i>Status </i> and using
 * its API or through <i>mygrid status </i> script.
 */
public class GUIStarter implements Command {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( GUIStarter.class );

	private Registry registry;


	public GUIStarter() {

	}


	/**
	 * Main method. This method should be called with no command line arguments
	 */
	public int run( String[ ] params ) {

		EBGuiFacade facade;
		GUIListenerImpl listener;
		try {
			UIManager manager = (UIManager) Naming.lookup( MyGridURLProvider.mygridUIManager() );

			if ( !manager.isMyGridRunning() ) {
				System.out.println( "Cannot start GUI: Mygrid is not running yet" );
				return ErrorCode.COULD_NOT_START_GUI;
			}
			facade = new EBGuiFacade();
			listener = createListener();

			listener.setFacade( facade );
			facade.config( manager, listener );

			facade.startEventProcessor();

			facade.getGUI().show();
		} catch ( RemoteException e ) {
			System.out.println( "Cannot start GUI: Mygrid is not running yet" );
			return ErrorCode.COULD_NOT_START_GUI;
		} catch ( MalformedURLException e ) {
			System.out.println( "Cannot start GUI: Mygrid is not running yet" );
			return ErrorCode.COULD_NOT_START_GUI;
		} catch ( ConfigException e ) {
			System.out.println( "Error starting Mygrid GUI! Cannot contact Mygrid daemon: " + e.getMessage() );
			return ErrorCode.COULD_NOT_START_GUI;
		} catch ( NotBoundException e ) {
			System.out.println( "Cannot start GUI: Mygrid is not running yet" );
			return ErrorCode.COULD_NOT_START_GUI;
		}

		return ErrorCode.START_CODE;
	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return true;
	}


	private GUIListenerImpl createListener() throws RemoteException, MalformedURLException {

		GUIListenerImpl guiListener = null;

		initRMIRegistry();
		guiListener = new GUIListenerImpl();
		registry.rebind( MyGridURLProvider.guiListener(), guiListener );

		return guiListener;
	}


	private void initRMIRegistry() throws RemoteException {

		int registryPort = Integer.parseInt( Configuration.getInstance( Configuration.MYGRID ).getProperty(
				MyGridConfiguration.PROP_GUI_PORT ) );

		// Starting registry
		registry = LocateRegistry.createRegistry( registryPort );
	}
}
