/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.io.File;
import java.rmi.RemoteException;
import java.util.List;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.common.spec.main.DescriptionFileCompile;
import org.ourgrid.mygrid.ui.UIManager;

public class GridSetter implements Command {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( GridSetter.class );

	private UIManager manager;


	public GridSetter( UIManager manager ) {

		this.manager = manager;
	}


	/**
	 * Main method. This method should be called with a GridDescriptionFile as
	 * command line argument1
	 */
	public int run( String[ ] params ) {

		Configuration.getInstance( Configuration.MYGRID );

		if ( params.length != 1 ) {

			printUsage();
			return ErrorCode.FAIL;
		}

		try {
			System.out.println( UIMessages.LOADING_GRID_DESCRIPTION_FILE );
			List<PeerSpec> peerSpecs = DescriptionFileCompile.compileGDF( new File( params[0] ).getAbsolutePath() );
			manager.setPeers( peerSpecs );
			System.out.println( UIMessages.GRID_SUCCESSFULLY_LOADED );
			return ErrorCode.OK;
		} catch ( RemoteException e ) {
			System.err.println( UIMessages.GRID_COULD_NOT_BE_LOADED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
		} catch ( CompilerException ce ) {
			System.err.println( UIMessages.GRID_COULD_NOT_BE_LOADED );
			System.err.println( UIMessages.ERROR_CAUSE + ce.getMessage() );
		}
		return ErrorCode.GRID_COULD_NOT_BE_LOADED;
	}


	/**
	 * Print's this class command line usage.
	 */
	public static void printUsage() {

		System.err.println( UIMessages.MYGRIDSETTER_USAGE );

	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return true;
	}

}
