/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.io.File;
import java.rmi.RemoteException;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.common.spec.main.DescriptionFileCompile;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.mygrid.ui.UIManager;
import org.ourgrid.mygrid.ui.exception.CouldNotAddJobOutsideHomeMachineException;

/**
 * Description: This class is responsible to add new jobs to the scheduler. This
 * can be made by getting an instance from <i>JobAdder </i> and using its API or
 * through <i>mygrid addjob </i> script. At both of them a job is readed from
 * the given file and passed to MyGrid, that'll schedule the job's tasks. To
 * describe a job the user must create a description file where the job
 * attributes are explicited and each task is described. To obtaind a better
 * description about how to write this file, read: //TODO:refer to the
 * manual!!!!
 * 
 * @version 1.0 Created on 02/08/2004
 */
public class JobAdder implements Command {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( JobAdder.class );

	private UIManager manager;


	public JobAdder( UIManager manager ) {

		this.manager = manager;
	}


	/**
	 * Main method. This method should be called with a JobDescriptionFile as
	 * command line argument1
	 */
	public int run( String[ ] params ) {

		Configuration.getInstance( Configuration.MYGRID );

		if ( params.length != 1 ) {

			printUsage();
			return ErrorCode.FAIL;
		}

		try {
			manager = MyGridUIManager.getInstance();
			String descriptionFilePath = new File( params[0] ).getAbsolutePath();
			LOG.info( UIMessages.ADDING_JOB_FILE + " " + descriptionFilePath );

			JobSpec theJob = DescriptionFileCompile.compileJDF( descriptionFilePath );
			int jobId = manager.addJob( theJob );
			LOG.info( UIMessages.JOB_SUCCESSFULLY_ADDED + " JobId[" + jobId + "]" );

			System.out.println( UIMessages.JOB_SUCCESSFULLY_ADDED + " JobId: [" + jobId + "]" );

			return ErrorCode.OK;
		} catch ( RemoteException e ) {
			System.err.println( UIMessages.THE_JOB_COULD_NOT_BE_ADDED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
		} catch ( CompilerException e ) {
			System.err.println( UIMessages.THE_JOB_COULD_NOT_BE_ADDED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
		} catch ( CouldNotAddJobOutsideHomeMachineException e ) {
			System.err.println( UIMessages.THE_JOB_COULD_NOT_BE_ADDED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
		}

		return ErrorCode.THE_JOB_COULD_NOT_BE_ADDED;
	}


	/**
	 * Print's this class command line usage.
	 */
	public static void printUsage() {

		System.err.println( UIMessages.JOBADDER_USAGE );

	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return true;
	}

}
