/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.rmi.RemoteException;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.mygrid.scheduler.exception.JobCouldNotBeCancelledException;
import org.ourgrid.mygrid.scheduler.exception.JobNotFoundException;
import org.ourgrid.mygrid.ui.UIManager;

/**
 * Description: This class is responsible to send a cancel job request to the
 * scheduler. This can be made by getting an instance from <i>JobCanceller </i>
 * and using its API or through <i>mygrid cancejob </i> script. At both of them
 * a job is cancelled based on it's id.
 * 
 * @version 1.0 Created on 03/08/2004
 */
public class JobCanceller implements Command {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( JobCanceller.class );

	private UIManager manager;


	public JobCanceller( UIManager manager ) {

		this.manager = manager;
	}


	/**
	 * Main method. This method should be called with a Job Id as command line
	 * argument.
	 * 
	 * @param params Job Id
	 */
	public int run( String[ ] params ) {

		Configuration.getInstance( Configuration.MYGRID );

		int jobId = -1;
		if ( params.length != 1 ) {

			printUsage();
			return ErrorCode.FAIL;

		}

		try {
			jobId = Integer.parseInt( params[0] );

			System.out.println( UIMessages.CANCELLING_JOB + jobId );
			this.cancelJob( jobId );
			System.out.println( UIMessages.JOB_SUCCESSFULLY_CANCELLED );

			return ErrorCode.OK;
		} catch ( Exception e ) {
			LOG.error( UIMessages.THE_JOB_COULD_NOT_BE_CANCELLED );
			LOG.error( UIMessages.ERROR_CAUSE + e.getMessage() );

			System.err.println( UIMessages.THE_JOB_COULD_NOT_BE_CANCELLED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
		}

		return ErrorCode.THE_JOB_COULD_NOT_BE_CANCELLED;
	}


	/**
	 * Print's this class command line usage.
	 */
	public static void printUsage() {

		System.err.println( UIMessages.JOBCANCELLER_USAGE );

	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return true;
	}


	public void cancelJob( int jobId ) throws JobNotFoundException, RemoteException, JobCouldNotBeCancelledException {

		manager.cancelJob( jobId );
	}

}
