/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import org.ourgrid.mygrid.ui.UIManager;

/**
 * The MyGrid main class. It can initialize all the environment by command line
 * starting the necessary entities that manager the MyGrid operations.
 */
public class Starter implements Command {

	private UIManager manager;


	public Starter( UIManager manager ) {

		this.manager = manager;
	}


	/**
	 * Will verify if the environment is already alive and if not will start
	 * every necessary part of the MyGrid environment as: scheduler, grid
	 * machine provider, tasks executor, etc.
	 */
	public int run( String[ ] params ) {

		try {

			System.out.println( UIMessages.STARTING_MYGRID );
			manager.startMyGridService();
			System.out.println( UIMessages.MYGRID_IS_UP_AND_RUNNING );

			return ErrorCode.START_CODE;

		} catch ( Exception e ) {
			System.out.println( "Unable to start Mygrid. [" + e.getMessage() + "]" );
		}

		return ErrorCode.MYGRID_COULD_NOT_BE_STARTED;
	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return false;
	}

}
