/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.command;

import java.rmi.UnmarshalException;

import org.ourgrid.mygrid.ui.UIManager;

/**
 * Description: This class is responsible to stop mygrid. This can be made by
 * getting an instance from <i>Stopper </i> and using its API or through
 * <i>mygrid stop </i> script.
 * 
 * @version 1.0 Created on 03/08/2004
 */
public class Stopper implements Command {

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( Stopper.class );

	private UIManager manager;


	public Stopper( UIManager manager ) {

		this.manager = manager;
	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#run(java.lang.String[])
	 */
	public int run( String[ ] params ) {

		try {

			if ( manager.isMyGridRunning() ) {

				System.out.println( UIMessages.STOPPING_MYGRID );
				LOG.info( UIMessages.STOPPING_MYGRID );

				if ( params != null && params[0] != null && params[0].equals( "-f" ) ) {
					manager.stopMyGridService( false, true );
				} else {
					manager.stopMyGridService( true, true );
				}

				System.out.println( UIMessages.MYGRID_SUCCESSFULLY_STOPPED );
				LOG.info( UIMessages.MYGRID_SUCCESSFULLY_STOPPED );
				return ErrorCode.OK;
			}
			System.out.println( UIMessages.MYGRID_NOT_STOPPED + " [ " + UIMessages.MYGRID_IS_ALREADY_STOPPED + " ]" );
			return ErrorCode.OK;
		} catch ( UnmarshalException e ) {
			System.out.println( UIMessages.MYGRID_SUCCESSFULLY_STOPPED );
			LOG.info( UIMessages.MYGRID_SUCCESSFULLY_STOPPED );
			return ErrorCode.OK;
		} catch ( Exception e ) {
			System.err.println( UIMessages.MYGRID_COULD_NOT_BE_STOPPED );
			System.err.println( UIMessages.ERROR_CAUSE + e.getMessage() );
			LOG.error( UIMessages.MYGRID_COULD_NOT_BE_STOPPED );
			LOG.error( UIMessages.ERROR_CAUSE + e.getMessage(), e );
		}

		return ErrorCode.MYGRID_COULD_NOT_BE_STOPPED;
	}


	/**
	 * @see org.ourgrid.mygrid.ui.command.Command#configureEnvironment()
	 */
	public boolean configureEnvironment() {

		return true;
	}

}
