/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.rmi.RemoteException;
import java.util.Collection;

import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.url.MyGridURLProvider;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerListenerRemote;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManagerListenerRemote;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

public class GUIListenerImpl extends OurgridUnicastRemoteObject implements JobManagerListenerRemote,
		GridManagerListenerRemote {

	private static final long serialVersionUID = 33L;

	private EBGuiFacade guiFacade;


	public GUIListenerImpl() throws RemoteException {

		super( Integer.parseInt( MyGridURLProvider.getGuiPort() ) );
	}


	public void setFacade( EBGuiFacade guiFacade ) {

		this.guiFacade = guiFacade;
	}


	public void jobCreated( JobEntry jobEntry ) throws RemoteException {

		guiFacade.jobCreated( jobEntry );
	}


	public void replicaStateChanged( ReplicaEntry replicaEntry, ExecutionStatus jobState, ExecutionStatus taskState,
										int numberOfTaskFails ) throws RemoteException {

		guiFacade.replicaStateChanged( replicaEntry, jobState, taskState, numberOfTaskFails );
	}


	public void removeJob( int jobId ) throws RemoteException {

		guiFacade.removeJob( jobId );
	}


	public void hereIsGum( GumSpec spec, int jobID, ObjectID peerBrokerAccessID ) throws RemoteException {

		guiFacade.hereIsGum( spec, jobID, peerBrokerAccessID );
	}


	public void setPeers( Collection<PeerSpec> peerSpecs ) throws RemoteException {

		guiFacade.setPeers( peerSpecs );
	}


	public void deleteGum( GumID gumID ) throws RemoteException {

		guiFacade.deleteGum( gumID );
	}


	public void allocate( GumID gumID, ReplicaEntry replica ) throws RemoteException {

		guiFacade.allocate( gumID, replica );

	}


	public void gumIsReady( GumID gumID ) throws RemoteException {

		guiFacade.gumIsReady( gumID );
	}


	public void peerAlive( PeerSpec peerSpec ) throws RemoteException {

		guiFacade.peerAlive( peerSpec );
	}


	public void peerIsDead( PeerSpec peerSpec ) throws RemoteException {

		guiFacade.peerIsDead( peerSpec );
	}


	public void myGridDown() {

		guiFacade.myGridDown();
	}


	public void jobCanceled( int jobId ) throws RemoteException {

		guiFacade.jobCanceled( jobId );
	}
}
