/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.PeerEntry.PeerState;

/**
 * This class generate the Grid Tree, it has a instance of MyGridUIServices to
 * obtain informations to create tree.<br>
 * It has too, maps to store all GuMPs and MyGuMs that alredy have been inserted
 * in tree to prevent exhausting algorithms to verify if already had been
 * inserted in the tree.
 */
public class GridTreeModel {

	/** Default Tree Model */
	private DefaultTreeModel tree;

	/** Default Mutable Tree Node */
	private DefaultMutableTreeNode root;

	/** List of UP and DOWN Peers */
	private Map<PeerSpec,PeerNode> peer2node;


	/**
	 * Create a GridTreeModel.
	 * 
	 * @throws RemoteException
	 */
	public GridTreeModel() {

		root = new PeerNode( null, null, "Grid" );
		tree = new DefaultTreeModel( root );
		resetTree();
		peer2node = new HashMap<PeerSpec,PeerNode>();
	}


	/** Returns the tree to adds in panel */
	public DefaultTreeModel getTreeModel() {

		return tree;
	}


	private void resetTree() {

		root.removeAllChildren();
		tree.reload( root );
	}


	/**
	 * Refresh the tree
	 */
	public void refreshTree( Map<PeerSpec,PeerState> peers ) {

		updateTree( peers );
	}


	private void updateTree( Map<PeerSpec,PeerState> peers ) {

		Set<PeerSpec> knownPeers = new HashSet<PeerSpec>( peer2node.keySet() );

		synchronized ( tree ) {
			for ( PeerSpec entry : knownPeers ) {
				if ( !peers.containsKey( entry ) ) {
					DefaultMutableTreeNode jobNode = peer2node.remove( entry );
					tree.removeNodeFromParent( jobNode );
				}
			}
		}

		for ( Entry<PeerSpec,PeerState> entry : peers.entrySet() ) {
			PeerNode node = peer2node.get( entry.getKey() );
			String peerLabel = entry.getKey().getLabel();

			if ( peerLabel.length() == 0 ) {
				peerLabel = entry.getKey().getName();
			}

			String peerInfo = "Peer: " + peerLabel + " [ " + entry.getValue().toString() + " ]";

			if ( node == null ) {
				PeerNode peerNode = new PeerNode( entry.getKey(), entry.getValue(), peerInfo );
				tree.insertNodeInto( peerNode, root, root.getChildCount() );
				peer2node.put( entry.getKey(), peerNode );
			} else {
				node.setState( entry.getValue() );
				node.setUserObject( peerInfo );
			}
		}
	}


	/** This method is friendly because test needs a reference to Node Root */
	public DefaultMutableTreeNode getPeerRoot() {

		return root;
	}


	public TreeModel getTree() {

		return tree;
	}
}
