/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

/**
 * Tests for GridTreeModel
 */
public class GridTreeModelTest extends TestCase implements GUITest, SpecialCareTest {

	private GridTreeModel gridTreeModel;

	private TestUtils testUtils;

	private SimpleSynchronizedGridManager gridManager;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( GridTreeModelTest.class );
	}


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
		gridTreeModel = new GridTreeModel();
		gridManager = new SimpleSynchronizedGridManager();
	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		gridTreeModel = null;
		Configuration.reset();
		testUtils.reset();
	}


	/** Test when getGuMPSpecs() and getLocalGuMSpecs() returns null */
	public void testNull() throws RemoteException {

		DefaultMutableTreeNode root = gridTreeModel.getPeerRoot();
		assertEquals( root.getChildCount(), 0 );

		gridTreeModel.refreshTree( gridManager.getPeers() );

		assertEquals( root.getChildCount(), 0 );
	}


	/** Test if tree is empty */
	public void testTreeEmpty() {

		DefaultMutableTreeNode root = gridTreeModel.getPeerRoot();
		assertEquals( root.getChildCount(), 0 );
	}


	/** Test Number Of GuMPs in Tree */
	public void testNumberOfPeers() throws RemoteException {

		DefaultMutableTreeNode root = gridTreeModel.getPeerRoot();

		List<PeerSpec> peerSpecs = new LinkedList<PeerSpec>();

		peerSpecs.add( new PeerSpec( "1", 1 ) );
		peerSpecs.add( new PeerSpec( "2", 2 ) );
		peerSpecs.add( new PeerSpec( "3", 3 ) );

		gridManager.setPeers( peerSpecs );

		gridTreeModel.refreshTree( gridManager.getPeers() );

		assertEquals( root.getChildCount(), gridManager.getPeers().size() );

	}

}
