/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerEntry;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class GumTableTest extends TestCase implements GUITest, SpecialCareTest {

	private GumTable gumTable;

	private TestUtils testUtils;

	private SimpleSynchronizedGridManager simpleGridManager;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( GumTableTest.class );
	}


	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );
		simpleGridManager = new SimpleSynchronizedGridManager();
		gumTable = new GumTable( simpleGridManager );

	}


	protected void tearDown() throws Exception {

		super.tearDown();
		Configuration.reset();
		testUtils.reset();
	}


	public void testNull() {

		assertTrue( gumTable.getRowCount() == 0 );

		gumTable.refreshGums();
		assertTrue( gumTable.getRowCount() == 0 );

		gumTable.refreshGums();
		assertTrue( gumTable.getRowCount() == 0 );
	}


	public void testNumberOfRows() {

		assertTrue( gumTable.getRowCount() == 0 );

		GumSpec spec1 = new GumSpec();
		spec1.putAttribute( GumSpec.ATT_NAME, "gum1" );
		spec1.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec2 = new GumSpec();
		spec1.putAttribute( GumSpec.ATT_NAME, "gum2" );
		spec1.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec3 = new GumSpec();
		spec1.putAttribute( GumSpec.ATT_NAME, "gum3" );
		spec1.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		ObjectID peerID = new ObjectID( "peer1", 1L, "url://www" );

		simpleGridManager.hereIsGum( spec1, 1, peerID );
		gumTable.refreshGums();
		assertTrue( gumTable.getRowCount() == simpleGridManager.getGums().size() );

		simpleGridManager.hereIsGum( spec2, 1, peerID );
		gumTable.refreshGums();
		assertTrue( gumTable.getRowCount() == simpleGridManager.getGums().size() );

		simpleGridManager.hereIsGum( spec3, 1, peerID );
		gumTable.refreshGums();
		assertTrue( gumTable.getRowCount() == simpleGridManager.getGums().size() );
	}


	public void testIfTableContainsAllGuMs() {

		GumSpec spec1 = new GumSpec();
		spec1.putAttribute( GumSpec.ATT_NAME, "gum1" );
		spec1.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec2 = new GumSpec();
		spec2.putAttribute( GumSpec.ATT_NAME, "gum2" );
		spec2.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec3 = new GumSpec();
		spec3.putAttribute( GumSpec.ATT_NAME, "gum3" );
		spec3.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec4 = new GumSpec();
		spec4.putAttribute( GumSpec.ATT_NAME, "gum4" );
		spec4.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		GumSpec spec5 = new GumSpec();
		spec5.putAttribute( GumSpec.ATT_NAME, "gum5" );
		spec5.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );

		ObjectID peerID = new ObjectID( "peer1", 1L, "url://www" );

		simpleGridManager.hereIsGum( spec1, 1, peerID );

		simpleGridManager.hereIsGum( spec2, 1, peerID );

		simpleGridManager.hereIsGum( spec3, 1, peerID );

		gumTable.refreshGums();

		assertGumsAreOnTable();

		simpleGridManager.hereIsGum( spec4, 1, peerID );

		simpleGridManager.hereIsGum( spec5, 1, peerID );

		gumTable.refreshGums();

		assertGumsAreOnTable();
	}


	private void assertGumsAreOnTable() {

		assertTrue( gumTable.getRowCount() == simpleGridManager.getGums().size() );

		for ( GridManagerEntry gum : simpleGridManager.getGums().values() ) {
			String gumNameScheduler = gum.getGumSpec().getGumID().getGumName();
			int rowCount = gumTable.getRowCount();
			boolean foundGum = false;
			for ( int i = 0; i < rowCount; i++ ) {
				String gumNameTable = (String) gumTable.getValueAt( i, 1 );
				if ( gumNameTable.equals( gumNameScheduler ) )
					foundGum = true;
			}
			assertTrue( foundGum );
		}
	}
}
