/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 * Description:
 * 
 * @version 1.0 Created on 26/08/2004
 */
public class JobsTreeCellRenderer extends DefaultTreeCellRenderer {

	private static final long serialVersionUID = 33L;

	private static ImageIcon runningIcon = createIcon( "running.gif" );

	private static ImageIcon finishedIcon = createIcon( "finished.gif" );

	private static ImageIcon abortedIcon = createIcon( "aborted.gif" );

	private static ImageIcon cancelledIcon = createIcon( "cancelled.gif" );

	private static ImageIcon failedIcon = createIcon( "error.gif" );

	private static ImageIcon readyIcon = createIcon( "ready.gif" );

	public static final int RUNNING = 0;

	public static final int FINISHED = 1;

	public static final int ABORTED = 2;

	public static final int CANCELLED = 3;

	public static final int FAILED = 4;

	public static final int READY = 5;


	public JobsTreeCellRenderer() {

	}


	private static ImageIcon createIcon( String fileName ) {

		String basedir = System.getProperty( "MGROOT" ) + "/resources/images/";
		ImageIcon icon = new ImageIcon( basedir + fileName );
		return icon;

	}


	public Component getTreeCellRendererComponent( JTree tree, Object value, boolean sel, boolean expanded,
													boolean leaf, int row, boolean hasFocus ) {

		super.getTreeCellRendererComponent( tree, value, sel, expanded, leaf, row, hasFocus );

		switch ( getType( value ) ) {

			case RUNNING: {

				setIcon( runningIcon );
				setToolTipText( "Running" );
				break;
			}

			case FINISHED: {
				setIcon( finishedIcon );
				setToolTipText( "Finished" );
				break;
			}

			case ABORTED: {
				setIcon( abortedIcon );
				setToolTipText( "Aborted" );
				break;
			}

			case CANCELLED: {
				setIcon( cancelledIcon );
				setToolTipText( "Cancelled" );
				break;
			}

			case FAILED: {
				setIcon( failedIcon );
				setToolTipText( "Failed" );
				break;
			}

			case READY: {
				setIcon( readyIcon );
				setToolTipText( "Unstarted" );
				break;
			}

		}

		setDoubleBuffered( true );
		return this;
	}


	protected int getType( Object value ) {

		DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;

		String title = ((String) node.getUserObject()).toUpperCase();

		if ( title.indexOf( "RUNNING" ) >= 0 ) {
			return RUNNING;
		}

		if ( title.indexOf( "FINISHED" ) >= 0 ) {
			return FINISHED;
		}

		if ( title.indexOf( "ABORTED" ) >= 0 ) {
			return ABORTED;
		}

		if ( title.indexOf( "CANCEL" ) >= 0 ) {
			return CANCELLED;
		}

		if ( title.indexOf( "FAILED" ) >= 0 ) {
			return FAILED;
		}

		if ( title.indexOf( "UNSTARTED" ) >= 0 ) {
			return READY;
		}

		return -1;

	}
}
