/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Iterator;

import javax.swing.tree.DefaultMutableTreeNode;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.IOBlock;
import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.common.spec.TaskSpec;
import org.ourgrid.common.spec.exception.JobSpecificationException;
import org.ourgrid.common.spec.exception.TaskSpecificationException;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class JobsTreeModelTest extends TestCase implements GUITest, SpecialCareTest {

	private JobsTreeModel stm;

	private TestUtils testUtils;

	private SimpleSynchronizedJobManager jobManager;


	public static void main( String[ ] args ) {

		junit.swingui.TestRunner.run( JobsTreeModelTest.class );
	}


	@Override
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		jobManager = new SimpleSynchronizedJobManager();

		Configuration.getInstance( Configuration.MYGRID );
		stm = new JobsTreeModel();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
		Configuration.reset();
		stm = null;
		testUtils.reset();
	}


	public void testNull() {

		DefaultMutableTreeNode root = stm.getGuMRoot();
		assertTrue( root.getChildCount() == 0 );

		stm.refreshTree( jobManager.getJobs() );
		assertTrue( root.getChildCount() == 0 );
	}


	public void testTreeEmpty() {

		DefaultMutableTreeNode root = stm.getGuMRoot();
		assertTrue( root.getChildCount() == 0 );
	}


	public void testNumberOfJobs() throws JobSpecificationException, TaskSpecificationException {

		DefaultMutableTreeNode root = stm.getGuMRoot();
		stm.refreshTree( jobManager.getJobs() );

		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		JobSpec mockJobSpec1 = new JobSpec( "Mock 1", "", Arrays.asList( new TaskSpec( new IOBlock(), "exit 0",
				new IOBlock() ) ) );
		JobEntry jobEntry1 = new JobEntry( 1, mockJobSpec1, 3, 1 );
		jobManager.jobCreated( jobEntry1 );
		stm.refreshTree( jobManager.getJobs() );
		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		JobSpec mockJobSpec2 = new JobSpec( "Mock 2", "", Arrays.asList( new TaskSpec( new IOBlock(), "exit 0",
				new IOBlock() ) ) );
		JobEntry jobEntry2 = new JobEntry( 2, mockJobSpec2, 3, 1 );
		jobManager.jobCreated( jobEntry2 );
		stm.refreshTree( jobManager.getJobs() );
		assertTrue( root.getChildCount() == jobManager.getJobs().size() );
	}


	public void testNumberOfTasks() throws JobSpecificationException, TaskSpecificationException {

		DefaultMutableTreeNode root = stm.getGuMRoot();
		stm.refreshTree( jobManager.getJobs() );

		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		JobSpec mockJobSpec1 = new JobSpec( "Mock 1", "", Arrays.asList( new TaskSpec( new IOBlock(), "exit 0",
				new IOBlock() ) ) );
		JobEntry jobEntry1 = new JobEntry( 1, mockJobSpec1, 3, 1 );
		jobManager.jobCreated( jobEntry1 );
		stm.refreshTree( jobManager.getJobs() );
		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		DefaultMutableTreeNode jobNode = (DefaultMutableTreeNode) root.getFirstChild();
		Iterator<SimpleJobEntry> jobIt = jobManager.getJobs().values().iterator();
		SimpleJobEntry job = jobIt.next();
		assertTrue( jobNode.getChildCount() == job.getTasks().size() );
	}


	public void testNumberOfReplicas() throws TaskSpecificationException, JobSpecificationException {

		DefaultMutableTreeNode root = stm.getGuMRoot();
		stm.refreshTree( jobManager.getJobs() );

		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		JobSpec mockJobSpec1 = new JobSpec( "Mock 1", "", Arrays.asList( new TaskSpec( new IOBlock(), "exit 0",
				new IOBlock() ) ) );
		JobEntry jobEntry1 = new JobEntry( 1, mockJobSpec1, 3, 1 );
		jobManager.jobCreated( jobEntry1 );
		stm.refreshTree( jobManager.getJobs() );

		assertTrue( root.getChildCount() == jobManager.getJobs().size() );

		DefaultMutableTreeNode jobNode = (DefaultMutableTreeNode) root.getFirstChild();
		Iterator<SimpleJobEntry> jobIt = jobManager.getJobs().values().iterator();
		SimpleJobEntry job = jobIt.next();
		assertTrue( jobNode.getChildCount() == job.getTasks().size() );

		int numberOfReplicasJob = 0;
		int numberOfReplicasNode = 0;
		Iterator<SimpleTaskEntry> taskIt = job.getTasks().values().iterator();
		Enumeration jobNodeEn = jobNode.children();

		while ( taskIt.hasNext() ) {
			SimpleTaskEntry t = taskIt.next();
			numberOfReplicasJob += t.getReplicas().size();
		}

		while ( jobNodeEn.hasMoreElements() ) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) jobNodeEn.nextElement();
			numberOfReplicasNode += node.getChildCount();
		}

		assertTrue( numberOfReplicasJob == numberOfReplicasNode );
	}
}
