/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;

/**
 * This class contains the presentation screen of MyGrid GUI and also some
 * buttons to set Grids, add and cancel Jobs and start stop MyGrid.
 */
public class SettingsPanel {

	/** MyGrid's background file path */
	private final String BACKGROUND_MYGRID_PATH = Configuration.getInstance().getProperty( Configuration.MGROOT )
			+ File.separator + "resources/images" + File.separator + "background.png";

	/** MyGrid Panel */
	private JPanel mygridPanel;

	/** Value Labels */
	private JLabel nameValue;

	private JLabel logfileValue;

	private JLabel portValue;

	private JLabel maxfailsValue;

	private JLabel maxblfailsValue;

	private JLabel heuristicValue;

	private JLabel maxreplicasValue;

	private JLabel experimentValue;


	/** Private constructor, Singleton Pattern */
	public SettingsPanel() {

		initComponents();

		readMGProperties();
	}


	/** Return the mygrid panel */
	public JPanel getPanel() {

		return mygridPanel;
	}


	/** Init components */
	private void initComponents() {

		mygridPanel = new JPanel();
		mygridPanel.setLayout( new GridLayout( 1, 2 ) );

		JPanel westPanel = new JPanel();
		westPanel.setLayout( new BorderLayout() );
		westPanel.setBackground( Color.WHITE );
		JLabel background = new JLabel( new ImageIcon( BACKGROUND_MYGRID_PATH ) );
		westPanel.add( background, BorderLayout.LINE_START );
		mygridPanel.add( westPanel );

		JPanel propertiesPanel = new JPanel();
		propertiesPanel.setLayout( new GridLayout( 9, 2 ) );
		propertiesPanel.setBackground( Color.WHITE );
		propertiesPanel.setBorder( new TitledBorder( "MyGrid Configuration" ) );

		mygridPanel.add( propertiesPanel );

		JLabel homemachine = new JLabel( " mg.homemachine" );
		JLabel debug = new JLabel( " mg.debug" );
		JLabel logfile = new JLabel( " mg.logfile" );
		JLabel port = new JLabel( " mg.port" );
		JLabel maxfails = new JLabel( " mg.maxfails" );
		JLabel maxblfails = new JLabel( " max.blacklist.fails" );
		JLabel heuristic = new JLabel( " mg.heuristic" );
		JLabel maxreplicas = new JLabel( " mg.maxreplicas" );

		Font font = new Font( "Arial", Font.BOLD, 12 );
		homemachine.setFont( font );
		debug.setFont( font );
		logfile.setFont( font );
		port.setFont( font );
		maxfails.setFont( font );
		maxblfails.setFont( font );
		heuristic.setFont( font );
		maxreplicas.setFont( font );

		nameValue = new JLabel( "" );
		logfileValue = new JLabel( "" );
		portValue = new JLabel( "" );
		maxfailsValue = new JLabel( "" );
		maxblfailsValue = new JLabel( "" );
		heuristicValue = new JLabel( "" );
		maxreplicasValue = new JLabel( "" );
		experimentValue = new JLabel( "" );

		Font fontValue = new Font( "Arial", Font.PLAIN, 12 );
		nameValue.setFont( fontValue );
		logfileValue.setFont( fontValue );
		portValue.setFont( fontValue );
		maxfailsValue.setFont( fontValue );
		maxblfailsValue.setFont( fontValue );
		heuristicValue.setFont( fontValue );
		maxreplicasValue.setFont( fontValue );
		experimentValue.setFont( fontValue );

		propertiesPanel.add( homemachine );
		propertiesPanel.add( nameValue );

		propertiesPanel.add( port );
		propertiesPanel.add( portValue );

		propertiesPanel.add( logfile );
		propertiesPanel.add( logfileValue );

		propertiesPanel.add( maxfails );
		propertiesPanel.add( maxfailsValue );

		propertiesPanel.add( maxblfails );
		propertiesPanel.add( maxblfailsValue );

		propertiesPanel.add( maxreplicas );
		propertiesPanel.add( maxreplicasValue );

		propertiesPanel.add( heuristic );
		propertiesPanel.add( heuristicValue );

	}


	/**
	 * Read MG.properties file and shows values in MyGrid Panel.
	 */
	public void readMGProperties() {

		String notDefined = "Not defined";
		String space = "  ";

		Configuration config = Configuration.getInstance();

		String value = config.getProperty( MyGridConfiguration.PROP_NAME );

		if ( value == null || value.trim().equals( "" ) ) {
			nameValue.setText( notDefined );
		} else {
			nameValue.setText( value.trim() + space );
		}

		value = config.getProperty( MyGridConfiguration.PROP_LOGFILE );

		if ( value == null || value.trim().equals( "" ) ) {
			logfileValue.setText( notDefined );
		} else {
			logfileValue.setText( value.trim() );
		}

		value = config.getProperty( MyGridConfiguration.PROP_PORT );

		if ( value == null || value.trim().equals( "" ) ) {
			portValue.setText( notDefined );
		} else {
			portValue.setText( value.trim() );
		}

		value = config.getProperty( MyGridConfiguration.PROP_MAX_FAILS );

		if ( value == null || value.trim().equals( "" ) ) {
			maxfailsValue.setText( notDefined );
		} else {
			maxfailsValue.setText( value.trim() );
		}

		value = config.getProperty( MyGridConfiguration.PROP_MAX_BL_FAILS );

		if ( value == null || value.trim().equals( "" ) ) {
			maxblfailsValue.setText( notDefined );
		} else {
			maxblfailsValue.setText( value.trim() );
		}

		value = config.getProperty( MyGridConfiguration.PROP_HEURISTIC );

		if ( value == null || value.trim().equals( "" ) ) {
			heuristicValue.setText( notDefined );
		} else {
			heuristicValue.setText( value.trim() );
		}

		value = config.getProperty( MyGridConfiguration.PROP_MAX_REPLICAS );

		if ( value == null || value.trim().equals( "" ) ) {
			maxreplicasValue.setText( notDefined );
		} else {
			maxreplicasValue.setText( value.trim() );
		}

	}

}
