/*
 * Copyright (c) 2002-2005 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.util.LinkedHashMap;
import java.util.Map;

import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;

public class SimpleJobEntry {

	private int jobID;

	private JobSpec jobSpec;

	private ExecutionStatus state;

	private Map<Integer,SimpleTaskEntry> tasks;


	public SimpleJobEntry( JobEntry jobEntry ) {

		this.jobID = jobEntry.getId();
		this.jobSpec = jobEntry.getJobSpec();
		this.state = jobEntry.getStatus();
		this.tasks = new LinkedHashMap<Integer,SimpleTaskEntry>();

		createTasks( jobEntry );
	}


	private void createTasks( JobEntry jobEntry ) {

		for ( TaskEntry taskEntry : jobEntry.getTasks() ) {
			this.tasks.put( taskEntry.getId(), new SimpleTaskEntry( taskEntry ) );
		}
	}


	public synchronized void replicaStateChanged( ReplicaEntry replicaEntry, ExecutionStatus jobState,
													ExecutionStatus taskState, int numberOfTaskFails ) {

		SimpleTaskEntry taskEntry = this.tasks.get( replicaEntry.getTaskId() );

		if ( taskEntry != null ) {
			taskEntry.replicaStateChanged( replicaEntry, taskState, numberOfTaskFails );
		}

		this.state = jobState;
	}


	public synchronized int getJobID() {

		return jobID;
	}


	public synchronized JobSpec getJobSpec() {

		return jobSpec;
	}


	public synchronized ExecutionStatus getState() {

		return state;
	}


	public synchronized SimpleTaskEntry getTaskByID( int id ) {

		return tasks.get( id );
	}


	public synchronized Map<Integer,SimpleTaskEntry> getTasks() {

		return new LinkedHashMap<Integer,SimpleTaskEntry>( tasks );
	}


	public synchronized void jobCanceled() {

		this.state = ExecutionStatus.CANCELLED;

		for ( SimpleTaskEntry taskEntry : tasks.values() ) {
			taskEntry.jobCancelled();
		}
	}
}
