package org.ourgrid.mygrid.ui.gui;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.PeerEntry.PeerState;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

public class SimpleSynchronizedGridManager implements GridManagerListener {

	private Map<GumID,GridManagerEntry> gums;

	private Map<PeerSpec,PeerEntry.PeerState> peers;


	public SimpleSynchronizedGridManager() {

		this.gums = new HashMap<GumID,GridManagerEntry>();
		this.peers = new HashMap<PeerSpec,PeerState>();
	}


	public synchronized void createEntries( Collection<GridManagerEntry> gums, Collection<PeerEntry> peers ) {

		for ( GridManagerEntry entry : gums ) {
			this.gums.put( entry.getGumID(), entry );
		}

		for ( PeerEntry entry : peers ) {
			this.peers.put( entry.getPeerSpec(), entry.getStatus() );
		}
	}


	public synchronized void hereIsGum( GumSpec spec, int jobID, ObjectID peerBrokerAccessID ) {

		gums.put( spec.getGumID(), new GridManagerEntry( spec, jobID, peerBrokerAccessID ) );
	}


	public synchronized void setPeers( Collection<PeerSpec> peerSpecs ) {

		peers = new HashMap<PeerSpec,PeerState>();

		for ( PeerSpec peerSpec : peerSpecs ) {
			peers.put( peerSpec, PeerState.CONTACTING );
		}
	}


	public synchronized void deleteGum( GumID gumID ) {

		gums.remove( gumID );
	}


	public synchronized void allocate( GumID gumID, ReplicaEntry replica ) {

		GridManagerEntry entry = gums.get( gumID );

		if ( entry != null ) {
			entry.allocate( replica );
		}
	}


	public synchronized void gumIsReady( GumID gumID ) {

		GridManagerEntry entry = gums.get( gumID );

		if ( entry != null ) {
			entry.ready();
		}
	}


	public synchronized void peerAlive( PeerSpec peerSpec ) {

		peers.put( peerSpec, PeerState.UP );
	}


	public synchronized void peerIsDead( PeerSpec peerSpec ) {

		peers.put( peerSpec, PeerState.CONTACTING );
	}


	public synchronized Map<PeerSpec,PeerState> getPeers() {

		return new HashMap<PeerSpec,PeerState>( this.peers );
	}


	public synchronized Map<GumID,GridManagerEntry> getGums() {

		return new HashMap<GumID,GridManagerEntry>( this.gums );
	}
}
