package org.ourgrid.mygrid.ui.gui;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

import junit.framework.TestCase;

import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerEntry;
import org.ourgrid.test.GUITest;

public class SimpleSynchronizedGridManagerTest extends TestCase implements GUITest {

	private SimpleSynchronizedGridManager manager;


	@Override
	protected void setUp() throws Exception {

		super.setUp();
		manager = new SimpleSynchronizedGridManager();
	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	/*
	 * Test method for
	 * 'org.ourgrid.mygrid.ui.gui.SimpleSynchronizedGridManager.SimpleSynchronizedGridManager(Collection<GridManagerEntry>,
	 * Collection<PeerEntry>)'
	 */
	public final void testCreation() {

		SimpleSynchronizedGridManager gridManager = new SimpleSynchronizedGridManager();

		assertTrue( gridManager.getGums().isEmpty() );
		assertTrue( gridManager.getPeers().isEmpty() );

		Collection<PeerEntry> peers = new LinkedList<PeerEntry>();

		final PeerEntry peerEntry1 = new PeerEntry( new PeerSpec( "peer1", 1010 ) );
		peers.add( peerEntry1 );
		final PeerEntry peerEntry2 = new PeerEntry( new PeerSpec( "peer2", 1011 ) );
		peers.add( peerEntry2 );

		Collection<GridManagerEntry> gums = new LinkedList<GridManagerEntry>();

		final GridManagerEntry gridManagerEntry1 = new GridManagerEntry( new GumSpec(), 1, null );
		gums.add( gridManagerEntry1 );
		final GridManagerEntry gridManagerEntry2 = new GridManagerEntry( new GumSpec(), 2, null );
		gums.add( gridManagerEntry2 );

		gridManager = new SimpleSynchronizedGridManager();
		gridManager.createEntries( gums, peers );

		assertTrue( gridManager.getPeers().containsKey( peerEntry1.getPeerSpec() ) );
		assertTrue( gridManager.getPeers().containsKey( peerEntry2.getPeerSpec() ) );

		assertTrue( gridManager.getGums().containsKey( gridManagerEntry1.getGumID() ) );
		assertTrue( gridManager.getGums().containsKey( gridManagerEntry2.getGumID() ) );
	}


	public final void testHereIsGum() {

		ObjectID peerID = new ObjectID( "name", 1L, "url" );

		GumSpec spec = new GumSpec();
		spec.putAttribute( "name1", "value1" );
		spec.putAttribute( "name2", "value2" );

		GumSpec spec2 = new GumSpec();
		spec2.putAttribute( "name3", "value3" );
		spec2.putAttribute( "name4", "value4" );

		manager.hereIsGum( spec, 1, peerID );
		manager.hereIsGum( spec2, 1, peerID );

		Map<GumID,GridManagerEntry> gums = manager.getGums();

		assertTrue( gums.containsKey( spec.getGumID() ) );
		assertTrue( gums.containsKey( spec2.getGumID() ) );
	}
}
