/*
 * Copyright (c) 2002-2005 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;

public class SimpleSynchronizedJobManager implements JobManagerListener {

	private Map<Integer,SimpleJobEntry> jobs;


	public SimpleSynchronizedJobManager() {

		jobs = new LinkedHashMap<Integer,SimpleJobEntry>();
	}


	public synchronized void createEntries( Collection<JobEntry> jobEntries ) {

		for ( JobEntry jobEntry : jobEntries ) {
			jobCreated( jobEntry );
		}
	}


	public synchronized void jobCanceled( int jobId ) {

		SimpleJobEntry jobEntry = jobs.get( jobId );

		if ( jobEntry != null ) {
			jobEntry.jobCanceled();
		}
	}


	public synchronized void replicaStateChanged( ReplicaEntry replicaEntry, ExecutionStatus jobState,
													ExecutionStatus taskState, int numberOfTaskFails ) {

		SimpleJobEntry jobEntry = jobs.get( replicaEntry.getJobId() );

		if ( jobEntry != null ) {
			jobEntry.replicaStateChanged( replicaEntry, jobState, taskState, numberOfTaskFails );
		}
	}


	public synchronized void removeJob( int jobId ) {

		jobs.remove( jobId );
	}


	public synchronized void jobCreated( JobEntry jobEntry ) {

		SimpleJobEntry simpleSinchronizedJobEntry = new SimpleJobEntry( jobEntry );
		jobs.put( jobEntry.getId(), simpleSinchronizedJobEntry );
	}


	public synchronized Map<Integer,SimpleJobEntry> getJobs() {

		return new LinkedHashMap<Integer,SimpleJobEntry>( jobs );
	}


	public synchronized SimpleJobEntry getJobByID( int id ) {

		return jobs.get( id );
	}
}
