/*
 * Copyright (c) 2002-2005 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.gui;

import java.util.LinkedHashMap;
import java.util.Map;

import org.ourgrid.common.spec.TaskSpec;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;

public class SimpleTaskEntry {

	private int jobID;

	private int taskID;

	private TaskSpec taskSpec;

	private ExecutionStatus taskState;

	private Map<Integer,SimpleReplicaEntry> replicas;

	private int actualFails;


	public SimpleTaskEntry( TaskEntry taskEntry ) {

		this.jobID = taskEntry.getJobId();
		this.taskID = taskEntry.getId();
		this.taskSpec = taskEntry.getSpec();
		this.taskState = taskEntry.getState();

		this.replicas = new LinkedHashMap<Integer,SimpleReplicaEntry>();

		createReplicas( taskEntry );
	}


	private void createReplicas( TaskEntry taskEntry ) {

		for ( ReplicaEntry replicaEntry : taskEntry.getReplicas() ) {
			this.replicas.put( replicaEntry.getId(), new SimpleReplicaEntry( replicaEntry ) );
		}
	}


	public synchronized void replicaStateChanged( ReplicaEntry replicaEntry, ExecutionStatus taskState,
													int numberOfTaskFails ) {

		this.taskState = taskState;
		this.actualFails = numberOfTaskFails;
		this.replicas.put( replicaEntry.getId(), new SimpleReplicaEntry( replicaEntry ) );
	}


	public synchronized int getJobID() {

		return jobID;
	}


	public synchronized int getTaskID() {

		return taskID;
	}


	public synchronized TaskSpec getTaskSpec() {

		return taskSpec;
	}


	public synchronized ExecutionStatus getTaskState() {

		return taskState;
	}


	public synchronized SimpleReplicaEntry getReplicaByID( int id ) {

		return replicas.get( id );
	}


	public synchronized Map<Integer,SimpleReplicaEntry> getReplicas() {

		return new LinkedHashMap<Integer,SimpleReplicaEntry>( replicas );
	}


	public synchronized int getNumberOfRunningReplicas() {

		int returnValue = 0;

		for ( SimpleReplicaEntry replicaEntry : replicas.values() ) {
			if ( replicaEntry.getReplicaState().equals( ExecutionStatus.RUNNING ) ) {
				returnValue++;
			}
		}

		return returnValue;
	}


	public synchronized int getNumberOfUnstartedReplicas() {

		int returnValue = 0;

		for ( SimpleReplicaEntry replicaEntry : replicas.values() ) {
			if ( replicaEntry.getReplicaState().equals( ExecutionStatus.UNSTARTED ) ) {
				returnValue++;
			}
		}

		return returnValue;
	}


	public synchronized int getActualFails() {

		return actualFails;
	}


	public synchronized void jobCancelled() {

		this.taskState = ExecutionStatus.CANCELLED;
	}
}
