/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;

import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.mygrid.scheduler.PeerEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerListenerRemote;
import org.ourgrid.mygrid.scheduler.gump.GumpManager;

public class FakeGumpManager implements GumpManager {

	private Collection<PeerSpec> gumpSpecs;


	public FakeGumpManager() {

		gumpSpecs = new ArrayList<PeerSpec>();
		createRandomGuMPs();
	}


	public void setPeers( Collection<PeerSpec> gumpSpecs ) {

		this.gumpSpecs = gumpSpecs;
	}


	public Collection<PeerEntry> getPeerEntries( GridManagerListenerRemote newListener ) {

		Collection<PeerEntry> gumpEntries = new LinkedList<PeerEntry>();

		for ( PeerSpec peerSpec : this.gumpSpecs ) {
			gumpEntries.add( new PeerEntry( peerSpec ) );
		}
		return gumpEntries;
	}


	public void addMoreGuMPs() {

		createRandomGuMPs();
	}


	public int getNumberOfGuMPs() {

		return gumpSpecs.size();
	}


	private void createRandomGuMPs() {

		int n = (int) (Math.random() * 5 + 1);
		for ( int i = 1; i <= n; i++ ) {
			FakeGuMPSpec spec = new FakeGuMPSpec();
			gumpSpecs.add( spec );
		}
	}


	public void peerAlive( String url ) throws RemoteException {

	}


	public void peerIsDead( String url ) throws RemoteException {

	}

}


class FakeGuMPSpec extends PeerSpec {

	private static final long serialVersionUID = 33L;

	int number;


	FakeGuMPSpec() {

		super( "fakename", 0 );
		number = (int) (Math.random() * Integer.MAX_VALUE - 1);
	}


	/**
	 * @see org.ourgrid.common.spec.GuMPSpec#getLabel()
	 */
	public String getLabel() {

		return String.valueOf( number );
	}


	/**
	 * @see org.ourgrid.common.spec.GuMPSpec#getURL()
	 */
	public String getURL() {

		return String.valueOf( number );
	}


	/**
	 * @see org.ourgrid.common.spec.GuMPSpec#getURL()
	 */
	public String getName() {

		return String.valueOf( number );
	}
}
