/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.common.spec.TaskSpec;
import org.ourgrid.common.spec.exception.TaskSpecificationException;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.Scheduler;
import org.ourgrid.mygrid.scheduler.exception.JobNotFoundException;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerEntry;
import org.ourgrid.mygrid.scheduler.gridmanager.GridManagerListenerRemote;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.JobManagerListenerRemote;
import org.ourgrid.mygrid.scheduler.jobmanager.ReplicaEntry;
import org.ourgrid.mygrid.scheduler.jobmanager.TaskEntry;

public class FakeScheduler implements Scheduler {

	private Map<Integer,JobEntry> jobs;

	private int nextJobId;


	public FakeScheduler() {

		jobs = new HashMap<Integer,JobEntry>();
		nextJobId = 1;
	}


	public Collection<JobEntry> jobList( JobManagerListenerRemote newListener ) throws RemoteException {

		if ( jobs != null )
			return jobs.values();
		return null;
	}


	public void waitForJob( int jobId ) throws JobNotFoundException, RemoteException {

	}


	public void cancelJob( int jobId ) throws JobNotFoundException, RemoteException {

		JobEntry je = jobs.remove( new Integer( jobId ) );
		if ( je == null )
			throw new JobNotFoundException( jobId );
	}


	public int addJob( JobSpec newJob ) throws RemoteException {

		try {
			jobs.put( new Integer( nextJobId ), new FakeJob( nextJobId ) );
		} catch ( TaskSpecificationException e ) {
			e.printStackTrace();
		}
		return nextJobId++;
	}


	public void setJobList( Map<Integer,JobEntry> c ) {

		jobs = c;
	}

	/*
	 * ==== MockJob ====
	 */
	public class FakeJob extends JobEntry {

		private static final long serialVersionUID = 33L;


		public FakeJob( int id ) throws TaskSpecificationException {

			super( id, new JobSpec( "FakeJob " + id ), 1, 1 );
			initTasks();
		}


		public FakeJob( int id, JobSpec jobSpec ) throws TaskSpecificationException {

			super( id, jobSpec, 1, 1 );
			initTasks();
		}


		private void initTasks() throws TaskSpecificationException {

			tasks = new LinkedList<TaskEntry>();
			addRandomTasks();
		}


		public void addRandomTasks() throws TaskSpecificationException {

			int size = tasks.size();
			for ( int i = size + 1; i <= (int) (Math.random() * 10) + 1 + size; i++ )
				tasks.add( new FakeTask( i, this.getId() ) );
		}


		@Override
		public ExecutionStatus getStatus() {

			return ExecutionStatus.UNSTARTED;
		}
	}

	/*
	 * ==== MockTask ====
	 */
	public class FakeTask extends TaskEntry {

		private static final long serialVersionUID = 33L;


		public FakeTask( int id, int jobId ) throws TaskSpecificationException {

			super( new TaskSpec( null, "FakeTask", null ), id, jobId, 1, 1 );
			replicas = new LinkedList<ReplicaEntry>();
			addRandomReplicas();
		}


		public Collection getAllReplicas() {

			return replicas;
		}


		public void addRandomReplicas() throws TaskSpecificationException {

			int size = replicas.size();

			for ( int i = size + 1; i <= (int) (Math.random() * 20) + 1 + size; i++ ) {
				ReplicaEntry fakeReplica = new FakeReplica( i, this.getId(), this.getJobId() );
				fakeReplica.setGumID( new GumID( "a", "b", "c", "d" ) );
				replicas.add( fakeReplica );
			}
		}
	}

	public class FakeReplica extends ReplicaEntry {

		private static final long serialVersionUID = 33L;


		public FakeReplica( int id, int taskId, int jobId ) throws TaskSpecificationException {

			super( id, taskId, jobId, new TaskSpec( null, "FakeTask", null ) );
		}

	}


	public void cleanFinishedJobs() throws RemoteException {

	}


	public void removeJob( int jobId ) throws RemoteException {

	}


	public Collection<GridManagerEntry> getGumEntries( GridManagerListenerRemote newListener ) throws RemoteException {

		// TODO Auto-generated method stub
		return null;
	}


	public Collection<GumSpec> getGumSpecs() throws RemoteException {

		// TODO Auto-generated method stub
		return null;
	}


	public void removeJobManagerListener( JobManagerListenerRemote listener ) {

		// TODO Auto-generated method stub

	}


	public void removeGridManagerListener( GridManagerListenerRemote listener ) {

		// TODO Auto-generated method stub

	}
}
