/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.io.File;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.PeerSpec;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.exception.JobNotFoundException;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.mygrid.ui.command.JobExpecter;
import org.ourgrid.mygrid.ui.command.Status;
import org.ourgrid.mygrid.ui.exception.CouldNotAddJobOutsideHomeMachineException;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.LocalPeer;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

/**
 * Description:
 * 
 * @version 1.0 Created on 03/08/2004
 */
public class JobExpecterTest extends TestCase implements GUITest, SpecialCareTest {

	private JobExpecter expecter;

	private TestUtils testUtils;

	private LocalPeer localPeer;


	/*
	 * @see TestCase#setUp()
	 */
	public void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		MyGridUIManager.reset();

		testUtils.restartMygrid();

		List gumsList = testUtils.getGumSpecs( testUtils.getNumberOfGuMSpecsAvailable( TestUtils.TYPE_UALINUX ),
				TestUtils.TYPE_UALINUX );

		localPeer = new LocalPeer( Integer.parseInt( testUtils.getProperty( TestUtils.OG_PORT ) ) );

		localPeer.start();
		localPeer.setGuMs( new File( testUtils.createSDF( gumsList ) ) );

		ArrayList<PeerSpec> myGumpSpecsList = new ArrayList<PeerSpec>();
		myGumpSpecsList.add( localPeer.getGuMPSpec() );

		if ( MyGridUIManager.getInstance().isMyGridRunning() ) {
			MyGridUIManager.getInstance().setPeers( myGumpSpecsList );
		}

		expecter = new JobExpecter( MyGridUIManager.getInstance() );
	}


	/*
	 * @see TestCase#tearDown()
	 */
	public void tearDown() throws Exception {

		super.tearDown();
		expecter = null;

		localPeer.stop();

		testUtils.stopMygrid();

		MyGridUIManager.reset();
		Configuration.reset();
		testUtils.reset();

	}


	public void testCorrectWaitForJob() throws JobNotFoundException, InterruptedException, RemoteException,
		CompilerException, CouldNotAddJobOutsideHomeMachineException {

		int jobId = MyGridUIManager.getInstance().addJob( testUtils.getSimpleJobSpec() );

		Status status = new Status( MyGridUIManager.getInstance() );

		while ( !isJobFinished( jobId ) ) {

			status.printGridStatus( new String[ 0 ] );
			Thread.sleep( 1000 );
		}

		expecter.waitForJob( jobId );

		status.printGridStatus( new String[ 0 ] );

		List jobs = (List<JobEntry>) MyGridUIManager.getInstance().getJobs( null );
		Iterator it = jobs.iterator();

		boolean found = false;

		JobEntry job = null;

		while ( it.hasNext() && !found ) {
			job = (JobEntry) it.next();
			if ( job.getId() == jobId ) {
				found = true;
			}
		}

		assertTrue( "The job should be in the scheduler job list!", found );
		assertEquals( "The job should be finished.", job.getStatus(), ExecutionStatus.FINISHED );

	}


	private boolean isJobFinished( int jobId ) throws RemoteException {

		Collection jobs = MyGridUIManager.getInstance().getJobs( null );
		Iterator it = jobs.iterator();

		boolean found = false;

		JobEntry job = null;

		while ( it.hasNext() && !found ) {
			job = (JobEntry) it.next();
			if ( job.getId() == jobId ) {
				found = true;
			}
		}

		return job.getStatus().equals( ExecutionStatus.FINISHED );
	}


	public void testIncorrectWaitForJob() throws RemoteException {

		try {
			expecter.waitForJob( -1 );
			fail( "This should throw a JobNotFoundException!" );
		} catch ( JobNotFoundException jnfe ) {

		}

		try {
			expecter.waitForJob( 283293 );
			fail( "This should throw a JobNotFoundException!" );
		} catch ( JobNotFoundException jnfe ) {

		}
	}
}
