/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.JobSpec;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.mygrid.ui.command.JobAdder;
import org.ourgrid.mygrid.ui.exception.CouldNotAddJobOutsideHomeMachineException;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class MyGridJobAdderTest extends TestCase implements GUITest, SpecialCareTest {

	private JobAdder adder;

	private TestUtils testUtils;


	/*
	 * @see TestCase#setUp()
	 */
	public void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		MyGridUIManager.reset();

		testUtils.restartMygrid();

		adder = new JobAdder( MyGridUIManager.getInstance() );
	}


	/*
	 * @see TestCase#tearDown()
	 */
	public void tearDown() throws Exception {

		super.tearDown();
		adder = null;

		testUtils.stopMygrid();

		Configuration.reset();

		testUtils.reset();

	}


	public void testJobAdd() throws RemoteException, CompilerException, CouldNotAddJobOutsideHomeMachineException {

		// Should not throw any exceptions since it's a valid job description
		// file.

		JobSpec jobSpec = testUtils.getSimpleJobSpec();

		int jobId = MyGridUIManager.getInstance().addJob( jobSpec );

		Collection jobs = MyGridUIManager.getInstance().getJobs( null );
		Iterator it = jobs.iterator();
		boolean found = false;

		while ( it.hasNext() && !found ) {
			JobEntry next = (JobEntry) it.next();
			if ( next.getId() == jobId ) {
				found = true;
			}
		}

		assertTrue( "The job should be in the scheduler jobs!", found );
	}
}
