/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.mygrid.scheduler.exception.JobCouldNotBeCancelledException;
import org.ourgrid.mygrid.scheduler.exception.JobNotFoundException;
import org.ourgrid.mygrid.scheduler.jobmanager.JobEntry;
import org.ourgrid.mygrid.ui.MyGridUIManager;
import org.ourgrid.mygrid.ui.command.JobCanceller;
import org.ourgrid.mygrid.ui.exception.CouldNotAddJobOutsideHomeMachineException;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class MyGridJobCancellerTest extends TestCase implements GUITest, SpecialCareTest {

	private JobCanceller canceller;

	private TestUtils testUtils;


	/*
	 * @see TestCase#setUp()
	 */
	public void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		MyGridUIManager.reset();

		testUtils.restartMygrid();

		canceller = new JobCanceller( MyGridUIManager.getInstance() );

	}


	/*
	 * @see TestCase#tearDown()
	 */
	public void tearDown() throws Exception {

		super.tearDown();
		canceller = null;

		testUtils.stopMygrid();

		MyGridUIManager.reset();

		Configuration.reset();

		testUtils.reset();
	}


	public void testCorrectJobCancel() throws JobNotFoundException, JobCouldNotBeCancelledException, RemoteException,
		CompilerException, CouldNotAddJobOutsideHomeMachineException {

		int jobId = MyGridUIManager.getInstance().addJob( testUtils.getSimpleJobSpec() );

		Collection jobs = MyGridUIManager.getInstance().getJobs( null );
		Iterator it = jobs.iterator();

		boolean found = false;

		JobEntry job = null;

		while ( it.hasNext() && !found ) {
			job = (JobEntry) it.next();
			if ( job.getId() == jobId ) {
				found = true;
			}
		}

		assertTrue( "The job should be in the scheduler jobs!", found );
		assertFalse( "The job should not be cancelled.", job.getStatus().equals( ExecutionStatus.CANCELLED ) );

		canceller.cancelJob( jobId );
		jobs = MyGridUIManager.getInstance().getJobs( null );
		it = jobs.iterator();

		found = false;

		job = null;

		while ( it.hasNext() && !found ) {
			job = (JobEntry) it.next();
			if ( job.getId() == jobId ) {
				found = true;
			}
		}

		assertTrue( "The job should be in the scheduler job list!", found );
		assertTrue( "The job should be cancelled.", job.getStatus().equals( ExecutionStatus.CANCELLED ) );

	}


	public void testIncorectJobCancel() throws RemoteException, JobCouldNotBeCancelledException {

		try {
			canceller.cancelJob( -1 );
			fail( "This should throw a JobNotFoundException!" );
		} catch ( JobNotFoundException jnfe ) {

		}

		try {
			canceller.cancelJob( 283293 );
			fail( "This should throw a JobNotFoundException!" );
		} catch ( JobNotFoundException jnfe ) {

		}

	}

}
