/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.mygrid.ui.test;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.test.GUITest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class MyGridSetterTest extends TestCase implements GUITest, SpecialCareTest {

	private TestUtils testUtils;


	public void setUp() throws Exception {

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();

		Configuration.getInstance( Configuration.MYGRID );

		testUtils.restartMygrid();

	}


	public void tearDown() throws Exception {

		testUtils.stopMygrid();

		Configuration.reset();

		testUtils.reset();
	}


	public void testSetGrid() throws Exception {

		TestUtils.executeCommand( TestUtils.MYGRID_COMMAND, new String[ ] { "setgrid", generateTempGDF() } );

	}


	/**
	 * Creates a temporary GDF file to be used in tests
	 * 
	 * @return Path of temporary GDF File
	 * @throws IOException If cannot write to file
	 */
	private String generateTempGDF() throws IOException {

		File tempGDF = File.createTempFile( "mygrid-test", ".gdf", new File( testUtils
				.getProperty( TestUtils.SYSTEM_TEMP_DIR ) ) );

		BufferedWriter writer = new BufferedWriter( new FileWriter( tempGDF ) );

		writer.write( "Peer : " );
		writer.newLine();
		writer.write( " label : testpeer" );
		writer.newLine();
		writer.write( " name : testpeer" );
		writer.newLine();
		writer.write( "port : 1099" );
		writer.newLine();

		writer.close();

		tempGDF.deleteOnExit();

		return tempGDF.toString();
	}

}
