/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.community.nof;

import java.io.IOException;
import java.util.TimerTask;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.PeerConfiguration;

/**
 * <p>
 * TimerTask that stores the Ranking of the peer when called.
 * </p>
 */
class AccountingMaintainer extends TimerTask {

	private AccountingManager accountingManager;

	/** Logger to store log info */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( AccountingMaintainer.class );


	/**
	 * Creates a new AccountingMaintainer
	 * 
	 * @param accountingManager The accounting manager that will provide the
	 *        ranking to be saved.
	 */
	public AccountingMaintainer( AccountingManager accountingManager ) {

		this.accountingManager = accountingManager;

	}


	public void run() {

		try {

			accountingManager.saveRankingInFile( Configuration.getInstance().getProperty(
					PeerConfiguration.PROP_RANKINGFILE ) );
		} catch ( IOException e ) {
			LOG.error( "The ranking couldn't be saved." + e.getLocalizedMessage() );
		}
	}

}
