/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ebgum;

import java.io.File;
import java.util.Map;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.event.ShutdownResponseEvent;
import org.ourgrid.common.executor.ExecutorHandle;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;

/**
 * An event-based gum, located in the peer. It process the calls received by the
 * <code>GumProxy</code>.
 */

public interface EBGum {

	/**
	 * @see org.ourgrid.common.gum.Gum#remoteExecute(java.lang.String,
	 *      java.lang.String, java.util.Map)
	 */
	public void remoteExecute( EventQueue<ResponseEvent> responseQueue, String remoteDir, String command,
								Map<String,String> environmentVars );


	/**
	 * @see org.ourgrid.common.gum.Gum#putFile(String, byte[], boolean)
	 */
	public void putFile( EventQueue<ResponseEvent> responseQueue, String remoteFile, byte[ ] data, boolean firstSlice );


	/**
	 * @see org.ourgrid.common.gum.Gum#getFile(String, long, int)
	 */
	public void getFile( EventQueue<ResponseEvent> responseQueue, String filePath, long filePos, int size );


	/**
	 * @see org.ourgrid.common.gum.Gum#getResult
	 *      (org.ourgrid.common.executor.ExecutorHandle)
	 */
	public void getResult( EventQueue<ResponseEvent> responseQueue, ExecutorHandle handle );


	/**
	 * @see org.ourgrid.common.gum.Gum#fileExists(java.io.File)
	 */
	public void fileExists( EventQueue<ResponseEvent> responseQueue, File file );


	/**
	 * @see org.ourgrid.common.gum.Gum#changeFilePermissions(java.io.File, int)
	 */
	public void changeFilePermissions( EventQueue<ResponseEvent> responseQueue, File file, int permissions );


	/**
	 * @see org.ourgrid.common.gum.Gum#getRemoteFileLength(java.io.File)
	 */
	public void getRemoteFileLength( EventQueue<ResponseEvent> responseQueue, File file );


	/**
	 * @see org.ourgrid.common.gum.Gum#getRemoteFileLastModification
	 *      (java.io.File, boolean)
	 */
	public void getRemoteFileLastModification( EventQueue<ResponseEvent> responseQueue, File remoteFile,
												boolean normalized );


	/**
	 * @see org.ourgrid.common.gum.Gum#createPlaypen(int)
	 */
	public void createPlaypen( EventQueue<ResponseEvent> responseQueue, int size );


	/**
	 * @see org.ourgrid.common.gum.Gum#createStorageDirectory()
	 */
	public void createStorageDirectory( EventQueue<ResponseEvent> responseQueue );


	/**
	 * @see org.ourgrid.common.gum.Gum#getStorageDirectory()
	 */
	public void getStorageDirectory( EventQueue<ResponseEvent> responseQueue );


	/**
	 * @see org.ourgrid.common.gum.Gum#getStatus()
	 */
	public void getStatus( EventQueue<ResponseEvent> responseQueue );


	/**
	 * @see org.ourgrid.common.gum.Gum#getRemoteFileDigest(File)
	 */
	public void getRemoteFileDigest( EventQueue<ResponseEvent> responseQueue, File remoteFile );


	/**
	 * @see org.ourgrid.common.gum.Gum#startReplica()
	 */
	public void startReplica( EventQueue<ResponseEvent> responseQueue );


	/**
	 * @see org.ourgrid.common.gum.Gum#stopReplica()
	 */
	public void stopReplica( EventQueue<ResponseEvent> responseQueue );


	/**
	 * Retrieves the gum identification cached in the peer
	 * 
	 * @return the gum identification cached in the peer
	 */
	public GumID getID();


	/**
	 * Retrieves the <code>GumSpec</code> cached in the peer
	 * 
	 * @return the <code>GumSpec</code> cached in the peer
	 */
	public GumSpec getGumSpec();


	/**
	 * Must be implemented by the facade to stop event processing
	 */
	public EventQueue<ShutdownResponseEvent> stop();


	/**
	 * Start the event processor
	 */
	public void start();
}
