/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ebgum;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.InvalidationResponseEvent;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.peer.manager.FakeEventEngine;
import org.ourgrid.peer.manager.gummanager.events.GumChangeFilePermissionsEvent;
import org.ourgrid.peer.manager.gummanager.events.GumCreatePlaypenEvent;
import org.ourgrid.peer.manager.gummanager.events.GumCreateStorageDirectoryEvent;
import org.ourgrid.peer.manager.gummanager.events.GumFileExistsEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetFileEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetRemoteFileDigestEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetRemoteFileLastModificationEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetRemoteFileLengthEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetResultEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetStatusEvent;
import org.ourgrid.peer.manager.gummanager.events.GumGetStorageDirectoryEvent;
import org.ourgrid.peer.manager.gummanager.events.GumPutFileEvent;
import org.ourgrid.peer.manager.gummanager.events.GumRemoteExecuteEvent;
import org.ourgrid.peer.manager.gummanager.events.GumStartReplicaEvent;
import org.ourgrid.peer.manager.gummanager.events.GumStopReplicaEvent;
import org.ourgrid.test.PeerPreCommitTest;

public class EBGumFacadeTest extends TestCase implements PeerPreCommitTest {

	private FakeEventEngine fakeEventEngine;

	private EBGumFacade ebGridMachineFacade;

	private GumSpec spec;

	private EBGum gum;

	private GumID gumID;


	public void setUp() {

		fakeEventEngine = new FakeEventEngine();
		gum = EasyMock.createMock( EBGum.class );
		ebGridMachineFacade = new EBGumFacade( gum, fakeEventEngine );
		ebGridMachineFacade.start();
		HashMap<String,String> attributeMaps = new HashMap<String,String>();
		attributeMaps.put( GumSpec.ATT_NAME, "gumName1" );
		attributeMaps.put( GumSpec.ATT_TYPE, GumSpec.TYPE_UA_LINUX );
		this.spec = new GumSpec( attributeMaps );
		this.gumID = new GumID( this.spec );
	}


	public void testRemoteExecuteStringStringMap() {

		Map<String,String> myMap = new HashMap<String,String>();
		ebGridMachineFacade.remoteExecute( null, "remoteDir", "command", myMap );
		assertTrue( fakeEventEngine.getEvent() instanceof GumRemoteExecuteEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();

		ebGridMachineFacade.remoteExecute( eq, "remoteDir", "command", myMap );

		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}


	public void testPutFile() {

		ebGridMachineFacade.putFile( null, "remoteFile", null, true );
		assertTrue( fakeEventEngine.getEvent() instanceof GumPutFileEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();

		ebGridMachineFacade.putFile( eq, "remoteFile", null, true );

		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testGetFile() {

		ebGridMachineFacade.getFile( null, "filePath", 0l, 0 );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetFileEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();

		ebGridMachineFacade.getFile( eq, "filePath", 0l, 0 );

		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testGetAttributes() {

		gum.getGumSpec();
		EasyMock.expectLastCall().andReturn( this.spec ).once();
		EasyMock.replay( gum );

		assertEquals( ebGridMachineFacade.getGumSpec().getAttributes(), this.spec.getAttributes() );

		EasyMock.verify( gum );

	}


	public void testGetName() {

		gum.getID();
		EasyMock.expectLastCall().andReturn( gumID ).once();

		EasyMock.replay( gum );
		assertEquals( gumID, ebGridMachineFacade.getID() );
		EasyMock.verify( gum );
	}


	public void testGetResult() {

		ebGridMachineFacade.getResult( null, null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetResultEvent );

		ebGridMachineFacade.stop();

		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();

		ebGridMachineFacade.getResult( eq, null );

		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testFileExists() {

		ebGridMachineFacade.fileExists( null, null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumFileExistsEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.fileExists( eq, null );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testChangeFilePermissions() {

		ebGridMachineFacade.changeFilePermissions( null, null, 777 );
		assertTrue( fakeEventEngine.getEvent() instanceof GumChangeFilePermissionsEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.changeFilePermissions( eq, null, 777 );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}


	public void testGetRemoteFileLength() {

		ebGridMachineFacade.getRemoteFileLength( null, null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetRemoteFileLengthEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.getRemoteFileLength( eq, null );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testGetRemoteFileLastModification() {

		ebGridMachineFacade.getRemoteFileLastModification( null, null, true );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetRemoteFileLastModificationEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.getRemoteFileLastModification( eq, null, true );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testCreatePlaypen() {

		ebGridMachineFacade.createPlaypen( null, 0 );
		assertTrue( fakeEventEngine.getEvent() instanceof GumCreatePlaypenEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.createPlaypen( eq, 0 );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testCreateStorageDirectory() {

		ebGridMachineFacade.createStorageDirectory( null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumCreateStorageDirectoryEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.createStorageDirectory( eq );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testGetStorageDirectory() {

		ebGridMachineFacade.getStorageDirectory( null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetStorageDirectoryEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.getStorageDirectory( eq );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}


	public void testGetStatus() {

		ebGridMachineFacade.getStatus( null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetStatusEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.getStatus( eq );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );

	}


	public void testGetGuMSpec() {

		EasyMock.expect( gum.getGumSpec() ).andReturn( this.spec ).once();

		EasyMock.replay( gum );

		assertEquals( ebGridMachineFacade.getGumSpec().getAttributes(), this.spec.getAttributes() );

		EasyMock.verify( gum );
	}


	public void testGetRemoteFileDigest() {

		ebGridMachineFacade.getRemoteFileDigest( null, null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumGetRemoteFileDigestEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.getRemoteFileDigest( eq, null );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}


	public void testStartReplica() {

		ebGridMachineFacade.startReplica( null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumStartReplicaEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.startReplica( eq );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}


	public void testStopReplica() {

		ebGridMachineFacade.stopReplica( null );
		assertTrue( fakeEventEngine.getEvent() instanceof GumStopReplicaEvent );

		ebGridMachineFacade.stop();
		EventQueue<ResponseEvent> eq = new EventQueue<ResponseEvent>();
		ebGridMachineFacade.stopReplica( eq );
		assertTrue( eq.blockingRemove() instanceof InvalidationResponseEvent );
	}
}
