/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ebgum;

import java.io.File;
import java.util.Map;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.event.ShutdownResponseEvent;
import org.ourgrid.common.executor.ExecutorHandle;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;

/**
 * A fake EBGridMachine used only for tests
 */
public class FakeEBGum implements EBGum {

	private final GumSpec gumSpec;


	public FakeEBGum( GumSpec gumSpec ) {

		this.gumSpec = gumSpec;

	}


	/**
	 * @see org.ourgrid.peer.ebgum.EBGum#getID()
	 */
	public GumID getID() {

		return gumSpec.getGumID();
	}


	/**
	 * @see org.ourgrid.peer.ebgum.EBGum#getGumSpec()
	 */
	public GumSpec getGumSpec() {

		return gumSpec;
	}


	@Override
	public boolean equals( Object o ) {

		if ( !(o instanceof FakeEBGum) )
			return false;
		return this.gumSpec.equals( ((FakeEBGum) o).getGumSpec() );
	}


	public void changeFilePermissions( EventQueue<ResponseEvent> responseQueue, File file, int permissions ) {

	}


	public void createPlaypen( EventQueue<ResponseEvent> responseQueue, int size ) {

	}


	public void createStorageDirectory( EventQueue<ResponseEvent> responseQueue ) {

	}


	public void fileExists( EventQueue<ResponseEvent> responseQueue, File file ) {

	}


	public void getFile( EventQueue<ResponseEvent> responseQueue, String filePath, long filePos, int size ) {

	}


	public void getRemoteFileDigest( EventQueue<ResponseEvent> responseQueue, File remoteFile ) {

	}


	public void getRemoteFileLastModification( EventQueue<ResponseEvent> responseQueue, File remoteFile,
												boolean normalized ) {

	}


	public void getRemoteFileLength( EventQueue<ResponseEvent> responseQueue, File file ) {

	}


	public void getResult( EventQueue<ResponseEvent> responseQueue, ExecutorHandle handle ) {

	}


	public void getStatus( EventQueue<ResponseEvent> responseQueue ) {

	}


	public void getStorageDirectory( EventQueue<ResponseEvent> responseQueue ) {

	}


	public void putFile( EventQueue<ResponseEvent> responseQueue, String remoteFile, byte[ ] data, boolean firstSlice ) {

	}


	public void remoteExecute( EventQueue<ResponseEvent> responseQueue, String remoteDir, String command,
								Map<String,String> environmentVars ) {

	}


	public void startReplica( EventQueue<ResponseEvent> responseQueue ) {

	}


	public void stopReplica( EventQueue<ResponseEvent> responseQueue ) {

	}


	public EventQueue<ShutdownResponseEvent> stop() {

		return null;
	}


	public void start() {

	}
}
