/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ebgum;

import java.rmi.ConnectException;
import java.rmi.RemoteException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.gum.Gum;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.peer.manager.EBPeerManagerFacade;
import org.ourgrid.test.PeerPreCommitTest;

/**
 * The <code>GumProxy</code> test. It will try to execute all steps from the
 * proxy creation, to the <code>EBGum<code> and the GridMachine.
 */
public class GumProxyTest2 extends TestCase implements PeerPreCommitTest {

	private static final long REQID_1 = 1;

	private GumSpec gumSpec;


	@Override
	protected void setUp() throws Exception {

		gumSpec = new GumSpec();
		gumSpec.putAttribute( GumSpec.ATT_NAME, "mockGum1" );

		Configuration.getInstance( Configuration.PEER );

	}


	@Override
	protected void tearDown() throws Exception {

		super.tearDown();
	}


	public void testGumFailure() throws RemoteException {

		Gum mockGum = EasyMock.createMock( Gum.class );
		EBPeerManagerFacade mockPeerManagerFacade = EasyMock.createMock( EBPeerManagerFacade.class );

		final EBGumFacade ebGum = EBGumCreator.createGuM( mockGum, gumSpec );
		ebGum.start();
		GumProxy proxyWithMockGum = new GumProxy( ebGum, mockPeerManagerFacade, REQID_1 );

		mockGum.startReplica();
		EasyMock.expectLastCall().andThrow( new ConnectException( "" ) ).once();

		mockPeerManagerFacade.lostGum( EasyMock.eq( gumSpec.getGumID() ), EasyMock.eq( REQID_1 ) );
		EasyMock.expectLastCall().once();

		EasyMock.replay( mockGum );
		EasyMock.replay( mockPeerManagerFacade );

		try {
			proxyWithMockGum.startReplica();
			fail();
		} catch ( RemoteException e ) {
		}

		EasyMock.verify( mockGum );
		EasyMock.verify( mockPeerManagerFacade );

	}

}
