/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.event;

import org.ourgrid.common.event.RequestEvent;
import org.ourgrid.common.exception.RequestAlreadyExistsException;
import org.ourgrid.common.gump.EBGump;
import org.ourgrid.common.spec.RequestSpec;

public class RequestGumsEvent implements RequestEvent {

	private static final long serialVersionUID = 33L;

	/** Logger (log4j) object to store log events */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( RequestGumsEvent.class );

	/**
	 * Object that really treats the event.
	 */
	private EBGump gump;

	private RequestSpec requestSpec;


	public RequestGumsEvent( EBGump gump, RequestSpec requestSpec ) {

		this.gump = gump;
		this.requestSpec = requestSpec;
	}


	public void process() {

		try {
			this.gump.requestGums( requestSpec );
		} catch ( RequestAlreadyExistsException e ) {
			LOG.error( "A request with ID " + requestSpec.getID() + " already exists." );
		}
	}

}
