/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.fd;

import java.net.URISyntaxException;
import java.rmi.RemoteException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.Notifiable;
import org.ourgrid.common.fd.State;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.url.URLProvider;
import org.ourgrid.peer.manager.EBPeerManagerFacade;
import org.ourgrid.peer.ui.PeerUIManager;
import org.ourgrid.peer.ui.PeerUIManager.Notifiables;

public class ConsumerStateNotifiable implements Notifiable {

	private final String name = this.getClass().getName();

	private EBPeerManagerFacade managerFacade;

	private Map<String,ObjectID> url2objectID;

	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( ConsumerStateNotifiable.class );


	/**
	 * Default constructor
	 */
	public ConsumerStateNotifiable() {

		this.url2objectID = Collections.synchronizedMap( new HashMap<String,ObjectID>() );
	}


	public void newMonitorable( ObjectID objectID ) {

		try {
			url2objectID.put( genericURL( objectID.getObjectURL() ), objectID );
		} catch ( URISyntaxException e ) {
			LOG.error( "Unable to define url", e );
		}
	}


	public void removeMonitorable( ObjectID objectID ) {

		try {
			url2objectID.remove( genericURL( objectID.getObjectURL() ) );
		} catch ( URISyntaxException e ) {
			LOG.error( "Unable to define url", e );
		}
	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#notifyMonitorableStateChange(java.lang.String,
	 *      GenericState)
	 */
	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState ) {

		if ( monitorableState.basicState().equals( State.DOWN ) ) {
			try {
				String consumerURL = genericURL( monitorableName );
				ObjectID consumerID = url2objectID.remove( consumerURL );

				if ( consumerID != null && managerFacade != null ) {
					PeerUIManager.getInstance().unregisterNewMonitorable( Notifiables.CONSUMER, monitorableName );
					managerFacade.consumerIsDead( consumerID );
				}
			} catch ( URISyntaxException e ) {
				LOG.error( "Invalid URI Syntax of a peer. ", e );
			} catch ( RemoteException e ) {
				LOG.error( "Could not contact gump. ", e );
			}

		}

	}


	private String genericURL( String url ) throws URISyntaxException {

		return "rmi://" + URLProvider.getHost( url ) + ":" + URLProvider.getPort( url );
	}


	public void setFacade( EBPeerManagerFacade managerFacade ) {

		this.managerFacade = managerFacade;
	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#getName()
	 */
	public String getName() {

		return name;
	}

}
