/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.fd;

import java.rmi.Naming;

import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.Notifiable;
import org.ourgrid.common.fd.State;
import org.ourgrid.common.url.PeerURLProvider;
import org.ourgrid.peer.community.RemoteAccess;

public class CorePeerStateNotifiable implements Notifiable {

	private final String name = this.getClass().getName();

	/** Logger to store log info */
	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( CorePeerStateNotifiable.class );


	/**
	 * @see org.ourgrid.common.fd.Notifiable#notifyMonitorableStateChange(java.lang.String,
	 *      GenericState)
	 */
	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState ) {

		if ( monitorableState.basicState().equals( State.UP ) ) {

			RemoteAccess peer;
			try {

				peer = (RemoteAccess) Naming.lookup( PeerURLProvider.remoteAccess() );

				peer.joinCommunity();

			} catch ( Exception e ) {
				LOG.error( "Could not join the community", e );
			}

		}

	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#getName()
	 */
	public String getName() {

		return name;
	}

}
