/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.fd;

import java.net.URISyntaxException;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.fd.Notifiable;
import org.ourgrid.common.gum.GumStateFD;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.url.URLProvider;
import org.ourgrid.peer.manager.EBPeerManagerFacade;

public class GumStateNotifiable implements Notifiable {

	private final String name = this.getClass().getName();

	private EBPeerManagerFacade managerFacade;

	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger
			.getLogger( GumStateNotifiable.class );


	public GumStateNotifiable() {

	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#notifyMonitorableStateChange(java.lang.String,
	 *      GenericState)
	 */
	public void notifyMonitorableStateChange( String monitorableName, GenericState monitorableState ) {

		Configuration.getInstance( Configuration.PEER );

		String peerName = Configuration.getInstance().getProperty( Configuration.PROP_EXTERNAL_NAME );
		String peerPort = Configuration.getInstance().getProperty( Configuration.PROP_PORT );

		GumStateFD gumStateFD;
		if ( monitorableState instanceof GumStateFD ) {
			gumStateFD = (GumStateFD) monitorableState;
		} else {
			gumStateFD = new GumStateFD( monitorableState.basicState() );
		}

		try {

			GumID gumID = new GumID( URLProvider.getHost( monitorableName ), URLProvider.getPort( monitorableName )
					+ "", peerName, peerPort );
			managerFacade.notifyGumStateChange( gumID, gumStateFD.gumState() );

		} catch ( URISyntaxException e ) {
			LOG.error( "Invalid URI Syntax of a gum. ", e );
		}

	}


	public void setFacade( EBPeerManagerFacade managerFacade ) {

		this.managerFacade = managerFacade;
	}


	/**
	 * @see org.ourgrid.common.fd.Notifiable#getName()
	 */
	public String getName() {

		return name;
	}

}
