/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager;

import java.util.Collection;
import java.util.Map;

import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.event.ShutdownEventEngine;
import org.ourgrid.common.gum.GumStatus;
import org.ourgrid.common.gump.GumpClientThreadManager;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.RequestSpec.RequestSource;
import org.ourgrid.peer.community.obtainer.EBCommunityObtainerFacade;
import org.ourgrid.peer.event.UnwantedGumGumNotifyEvent;
import org.ourgrid.peer.manager.allocation.AllocationStatus;
import org.ourgrid.peer.manager.allocation.AllocationEntry.GumSource;
import org.ourgrid.peer.manager.event.ConsumerIsDeadEvent;
import org.ourgrid.peer.manager.event.GetGumsEvent;
import org.ourgrid.peer.manager.event.GetGumsStateEvent;
import org.ourgrid.peer.manager.event.GetPeerBalanceEvent;
import org.ourgrid.peer.manager.event.LostGumNotifyEvent;
import org.ourgrid.peer.manager.event.NotifyGumStateChangeEvent;
import org.ourgrid.peer.manager.event.SetGumsNotifyEvent;
import org.ourgrid.peer.manager.gummanager.EBGumManager;
import org.ourgrid.peer.manager.gummanager.EBGumManagerImpl;
import org.ourgrid.peer.manager.status.ConsumerStatus;
import org.ourgrid.peer.manager.status.EBStatusProvider;
import org.ourgrid.peer.manager.status.EBStatusProviderImpl;
import org.ourgrid.peer.manager.status.StatusEntry;
import org.ourgrid.peer.ui.UIManager;

/**
 * The facade which has to used by the modules that wish to interact with the
 * <code>EBPeerManager</code>. This makes the latest a concurrency free
 * class.
 */
public class EBPeerManagerFacade extends EBPeerFacade implements EBGumManager {

	private EBGumManagerImpl gumManager;

	private EBStatusProvider statusProvider;

	private EBPeerManager ebPeerManager;

	private GumpClientThreadManager consumerThreadManager;


	/**
	 * Default constructor.
	 */
	public EBPeerManagerFacade() {

		super();
	}


	/**
	 * Constructor used only by tests.
	 * 
	 * @param eventProcessor an event processor created by the test
	 */
	public EBPeerManagerFacade( ShutdownEventEngine eventProcessor ) {

		this.eventProcessor = eventProcessor;
	}


	public void config( EBCommunityObtainerFacade communityObtainerFacade, ObjectID remoteAccessID,
						ObjectID brokerAccessID, UIManager peerUIManager ) {

		consumerThreadManager = new GumpClientThreadManager( this );
		ebPeerManager = new EBPeerManager( consumerThreadManager, remoteAccessID, brokerAccessID, peerUIManager );
		EBGumManagerImpl ebGumManager = new EBGumManagerImpl( ebPeerManager, this );

		ebPeerManager.config( communityObtainerFacade, ebGumManager );
		statusProvider = new EBStatusProviderImpl( ebGumManager );

		this.ebGump = ebPeerManager;
		this.ebGumpClient = ebPeerManager;
		this.gumManager = ebGumManager;
	}


	public void setGums( Collection<GumSpec> gumSpecCollection ) {

		eventProcessor.putEvent( new SetGumsNotifyEvent( this.gumManager, gumSpecCollection ) );
	}


	public void getGums( EventQueue<ResponseEvent<Collection<StatusEntry>>> responseQueue,
							AllocationStatus allocationStatus, RequestSource source ) {

		eventProcessor.putEvent( new GetGumsEvent( responseQueue, this.statusProvider, allocationStatus, source ) );

	}


	public void getPeerBalance( EventQueue<ResponseEvent<Map<ObjectID,Double>>> responseQueue ) {

		eventProcessor.putEvent( new GetPeerBalanceEvent( responseQueue, this.statusProvider ) );

	}


	public void getAllGums( EventQueue<ResponseEvent<Collection<StatusEntry>>> responseQueue, GumSource source ) {

		eventProcessor.putEvent( new GetGumsStateEvent( responseQueue, this.statusProvider, source ) );

	}


	public void consumerIsDead( ObjectID objectID ) {

		eventProcessor.putEvent( new ConsumerIsDeadEvent( this.ebPeerManager, objectID ) );
	}


	public void notifyGumStateChange( GumID gumID, GumStatus gumState ) {

		eventProcessor.putEvent( new NotifyGumStateChangeEvent( gumManager, gumID, gumState ) );
	}


	public void unwantedGum( long requestID, GumID gumID ) {

		eventProcessor.putEvent( new UnwantedGumGumNotifyEvent( ebPeerManager, requestID, gumID ) );
	}


	public void lostGum( GumID gumID, long requestID ) {

		eventProcessor.putEvent( new LostGumNotifyEvent( ebPeerManager, gumID, requestID ) );
	}


	public void getConsumers( EventQueue<ResponseEvent<Collection<ConsumerStatus>>> responseQueue, RequestSource source ) {

		eventProcessor.putEvent( new GetConsumersEvent( responseQueue, this.statusProvider, source ) );
	}
}
