/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager;

import org.ourgrid.common.event.ActionEvent;
import org.ourgrid.common.event.Event;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ShutdownEventEngine;
import org.ourgrid.common.event.ShutdownRequestEvent;
import org.ourgrid.common.event.ShutdownResponseEvent;

/**
 * The FakeEventEngine.
 */
public class FakeEventEngine implements ShutdownEventEngine {

	private EventQueue<ActionEvent> eventQueue;

	private boolean isAlive;


	/**
	 * Creates a fake <code>JICEventHandler</code> which will process the
	 * events.
	 */
	public FakeEventEngine() {

		this.eventQueue = new EventQueue<ActionEvent>();
		this.isAlive = false;
	}


	public void putEvent( ActionEvent event ) {

		if ( event instanceof ShutdownRequestEvent ) {
			shutdown( null );
		}
		eventQueue.put( event );
	}


	/**
	 * Returns the next <code>Event</code> to be processed.
	 * 
	 * @return The next event.
	 */
	public Event getEvent() {

		return eventQueue.unblockingRemove();
	}


	public void startProcessing() {

		this.isAlive = true;
	}


	public boolean isAlive() {

		return isAlive;
	}


	public void run() {

		// do nothing
	}


	public void shutdown( EventQueue<ShutdownResponseEvent> responseQueue ) {

		this.isAlive = false;
	}
}
