/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager;

import java.io.File;
import java.util.Vector;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.test.PeerPreCommitTest;
import org.ourgrid.test.SpecialCareTest;
import org.ourgrid.test.TestUtils;

public class PeerTest extends TestCase implements SpecialCareTest, PeerPreCommitTest {

	private String mgroot;

	private String gdfOneGridMachine;

	private TestUtils testUtils;

	private String OUTPUT_1_FILENAME;

	private final String EXPECTED_OUTPUT_1 = "47";

	private String gdfLocalPeer;

	private final int numberOfGuMs = 1;


	/**
	 * Sets up the environment.
	 */
	protected void setUp() throws Exception {

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();
		testUtils.createOGProperties();

		Configuration configuration = Configuration.getInstance( Configuration.MYGRID );
		mgroot = (new File( configuration.getProperty( Configuration.MGROOT ) )).getCanonicalPath();
		OUTPUT_1_FILENAME = mgroot + "/test/resources/output-1.1";
		File output1 = new File( OUTPUT_1_FILENAME );
		output1.delete();
	}


	protected void tearDown() {

		testUtils.reset();
	}


	/**
	 * Tests the peer in a simple execution that it would simply stop responding
	 * requests.
	 */
	public void testPeerSimpleExecution() throws Exception {

		Runtime runtime = Runtime.getRuntime();

		Vector<GumSpec> GuMSpecList = new Vector<GumSpec>( testUtils.getGumSpecs( numberOfGuMs, TestUtils.TYPE_UALINUX ) );
		gdfOneGridMachine = testUtils.createSDF( GuMSpecList );
		gdfLocalPeer = testUtils.createLocalPeerGDF();

		System.out.println( mgroot + File.separator + "bin" + File.separator + "mygrid stop" );
		Process process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid stop" );
		process.waitFor();

		System.out.println( mgroot + File.separator + "bin" + File.separator + "mygrid start" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid start" );
		assertEquals( 0, process.waitFor() );

		Thread.sleep( 5000 );

		Configuration.getInstance( Configuration.PEER );

		System.out.println( "peer stop" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer stop" );
		process.waitFor();

		System.out.println( "peer start" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer start" );
		assertEquals( 0, process.waitFor() );

		Thread.sleep( 5000 );

		Configuration.getInstance( Configuration.MYGRID );

		System.out.println( "uaadmin start " + gdfOneGridMachine );
		process = runtime
				.exec( mgroot + File.separator + "bin" + File.separator + "uaadmin start " + gdfOneGridMachine );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid status" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid status" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer status" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer status" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer setgums " + gdfOneGridMachine );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer setgums " + gdfOneGridMachine );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid setgrid " + gdfLocalPeer );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid setgrid " + gdfLocalPeer );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid addjob " + testUtils.getProperty( "job.simpletask" ) );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid addjob "
				+ testUtils.getProperty( "job.simpletask" ) );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid waitforjob 1" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid waitforjob 1" );
		assertEquals( 0, process.waitFor() );

		assertTrue( testUtils.checkFileContent( EXPECTED_OUTPUT_1, new File( OUTPUT_1_FILENAME ) ) );

		System.out.println( "mygrid stop" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid stop" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer stop" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer stop" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid status" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "mygrid status" );
		assertFalse( 0 == process.waitFor() );

		System.out.println( "peer status" );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "peer status" );
		assertFalse( 0 == process.waitFor() );

		System.out.println( "uaadmin shutdown " + gdfOneGridMachine );
		process = runtime.exec( mgroot + File.separator + "bin" + File.separator + "uaadmin shutdown "
				+ gdfOneGridMachine );
		assertEquals( 0, process.waitFor() );

	}

}
