/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager.allocation;

import java.util.HashMap;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.RequestSpec;
import org.ourgrid.peer.ebgum.EBGum;
import org.ourgrid.peer.ebgum.EBGumFacade;
import org.ourgrid.peer.ebgum.FakeEBGum;
import org.ourgrid.peer.ebgum.FakeGumFacade;
import org.ourgrid.peer.manager.FakeEventEngine;
import org.ourgrid.peer.manager.FakeGumpClient;
import org.ourgrid.peer.manager.request.RequestManager;
import org.ourgrid.test.PeerFastTest;
import org.ourgrid.test.TestUtils;

public class AllocationEntryTest extends TestCase implements PeerFastTest {

	private static final int REQID1 = 1254;

	protected AllocationEntry allocation;

	private EBGumFacade gumFacade;

	private HashMap<String,String> attributes;

	private TestUtils testUtils;

	private EBGum gridMachine;


	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

		super.setUp();

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createOGProperties();

		Configuration.getInstance( Configuration.PEER );

		attributes = new HashMap<String,String>();
		attributes.put( "os", "linux" );
		attributes.put( "ram", "1024" );
		attributes.put( GumSpec.ATT_PEERNAME, Configuration.getInstance()
				.getProperty( Configuration.PROP_EXTERNAL_NAME ) );
		attributes.put( GumSpec.ATT_PEERPORT, Configuration.getInstance().getProperty( Configuration.PROP_PORT ) );

		GumSpec gumSpec = new GumSpec( attributes );

		gridMachine = new FakeEBGum( gumSpec );

		gumFacade = new FakeGumFacade( gridMachine, new FakeEventEngine() );

		allocation = new AllocationEntry( gumFacade, gumSpec );

		RequestSpec request = new RequestSpec( REQID1, "", 1, new FakeGumpClient( "" ) );
		request.setAsFromLocalBroker();
		RequestManager.getInstance().createRequestEntry( request );

	}


	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		Configuration.reset();
		RequestManager.reset();
		testUtils.reset();
	}


	/**
	 * Tests the constructor.
	 */
	public void testCreateAllocation() {

		assertEquals( gumFacade.getID(), getAllocationEntry().getEBGum().getID() );
		assertEquals( gumFacade.getGumSpec().getAttributes(), getAllocationEntry().getEBGum().getGumSpec()
				.getAttributes() );
		assertFalse( getAllocationEntry().isAllocated() );
		assertTrue( getAllocationEntry().isALocalGum() );
		assertTrue( getAllocationEntry().getRequestID() < 0 );
	}


	/**
	 * Tests for getOurAllocationRegistry() and degetOurAllocationRegistry().
	 */
	public void testAllocateDeallocate() {

		assertFalse( getAllocationEntry().isAllocated() );
		getAllocationEntry().allocate( REQID1, true );
		assertTrue( getAllocationEntry().isAllocated() );
		assertTrue( getAllocationEntry().isAllocatedToRequest( REQID1 ) );
		getAllocationEntry().deallocate();
		assertFalse( getAllocationEntry().isAllocated() );
	}


	protected AllocationEntry getAllocationEntry() {

		return this.allocation;
	}

}
