/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager.allocation;

/**
 * This enumeration represents the status that an allocation can assume. These
 * status can be classified in the following way:
 * <ul>
 * <li><b>ONLINE</b>: The gum is up, and can be either:</li>
 * <ul>
 * <li><b>IN_USE</b>: When it's a gum allocated</li>
 * <li><b>DONATED</b>: When the gum is local, but is donated for the community</li>
 * <li><b>IDLE</b>: The gum is available for usage</li>
 * <li><b>OWNER</b>: The gum was in use by owner machine</li>
 * </ul>
 * </ul>
 */
public enum AllocationStatus {

	/* Allocation status */
	IN_USE, IDLE, DONATED, CONTACTING, OWNER;

	public boolean isAllocated() {

		return (this.equals( IN_USE ) || this.equals( DONATED ));
	}


	public boolean isDonated() {

		return this.equals( DONATED );
	}


	public boolean isIdle() {

		return this.equals( IDLE );
	}


	public boolean isOwner() {

		return this.equals( OWNER );
	}

}
