/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager.gummanager;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.gum.GumStatus;
import org.ourgrid.common.id.GumID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.peer.ebgum.EBGum;
import org.ourgrid.peer.ebgum.FakeGumFacade;
import org.ourgrid.peer.manager.EBPeerManager;
import org.ourgrid.test.PeerFastTest;

/**
 * Test for the class <code>EBGumManagerImpl</code>.
 */
public class GumManagerImplTest extends TestCase implements PeerFastTest {

	private EBGumManagerImpl gridManager;

	private EBPeerManager peerManagerMock;

	private Collection<GumSpec> gumSpecs;

	private EBGum fakeGum1;

	private EBGum fakeGum2;

	private EBGum fakeGum3;

	private HashMap<GumID,EBGum> gumMap;

	private GumSpec fakeGumSpec1;

	private GumSpec fakeGumSpec2;

	private GumSpec fakeGumSpec3;


	@Override
	public void setUp() {

		peerManagerMock = EasyMock.createMock( EBPeerManager.class );
		gridManager = new EBGumManagerImpl( peerManagerMock, null );
		Configuration.getInstance( Configuration.PEER );
		try {
			prepareGuMs();
		} catch ( Exception e ) {
			e.printStackTrace();
			fail( "An exception has been throw." );
		}

	}


	@Override
	public void tearDown() {

		Configuration.reset();
	}


	/**
	 * Prepare the GuMs to be used at any test.
	 * 
	 * @throws RemoteException
	 */
	private void prepareGuMs() {

		gumSpecs = new ArrayList<GumSpec>();

		fakeGumSpec1 = new GumSpec();
		fakeGumSpec2 = new GumSpec();
		fakeGumSpec3 = new GumSpec();

		fakeGumSpec1.putAttribute( GumSpec.ATT_NAME, "GuM1" );
		fakeGumSpec1.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_FAKE_GUM );
		fakeGumSpec2.putAttribute( GumSpec.ATT_NAME, "GuM2" );
		fakeGumSpec2.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_FAKE_GUM );
		fakeGumSpec3.putAttribute( GumSpec.ATT_NAME, "GuM3" );
		fakeGumSpec3.putAttribute( GumSpec.ATT_TYPE, GumSpec.TYPE_FAKE_GUM );

		fakeGum1 = new FakeGumFacade( fakeGumSpec1 );
		fakeGum2 = new FakeGumFacade( fakeGumSpec2 );
		fakeGum3 = new FakeGumFacade( fakeGumSpec3 );

		gumMap = new HashMap<GumID,EBGum>();

		gumMap.put( fakeGum1.getID(), fakeGum1 );
		gumMap.put( fakeGum2.getID(), fakeGum2 );
		gumMap.put( fakeGum3.getID(), fakeGum3 );

		gumSpecs.add( fakeGumSpec1 );
		gumSpecs.add( fakeGumSpec2 );
		gumSpecs.add( fakeGumSpec3 );
	}


	/**
	 * Test method for 'EBGumManagerImpl.notifyGuMStateChange(String, String)'
	 */
	public void testNotifyGuMStateChange() {

		gridManager.setGums( gumSpecs );

		peerManagerMock.hereIsLocalGum( EasyMock.eq( fakeGum1 ), EasyMock.eq( fakeGumSpec1 ) );
		peerManagerMock.gumWasLost( fakeGum1.getID() );
		peerManagerMock.hereIsLocalGum( EasyMock.eq( fakeGum1 ), EasyMock.eq( fakeGumSpec1 ) );
		peerManagerMock.hereIsLocalGum( EasyMock.eq( fakeGum2 ), EasyMock.eq( fakeGumSpec2 ) );
		peerManagerMock.gumWasLost( fakeGum2.getID() );

		EasyMock.replay( peerManagerMock );

		gridManager.notifyGumStateChange( fakeGum1.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum1.getID(), GumStatus.CONTACTING );
		gridManager.notifyGumStateChange( fakeGum1.getID(), GumStatus.UP );

		gridManager.notifyGumStateChange( fakeGum2.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum2.getID(), GumStatus.CONTACTING );

		GumID gumID = new GumID( "XPTO Gum", Integer.toString( 0 ), "XPTO Peer", Integer.toString( 0 ) );
		gridManager.notifyGumStateChange( gumID, GumStatus.UP );
		gridManager.notifyGumStateChange( gumID, GumStatus.CONTACTING );

		EasyMock.verify( peerManagerMock );
	}


	/**
	 * Test method for 'EBGumManagerImpl.setGuMs(Collection)'
	 */
	public void testSetGuMsAndGetGuMs() {

		peerManagerMock.hereIsLocalGum( fakeGum1, fakeGumSpec1 );
		peerManagerMock.hereIsLocalGum( fakeGum2, fakeGumSpec2 );
		peerManagerMock.hereIsLocalGum( fakeGum3, fakeGumSpec3 );

		EasyMock.replay( peerManagerMock );

		gridManager.setGums( gumSpecs );

		// Simulating reaction to failure detector commands.
		gridManager.notifyGumStateChange( fakeGum1.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum2.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum3.getID(), GumStatus.UP );

		assertTrue( gridManager.getGumSpecs().equals( gumSpecs ) );

		EasyMock.verify( peerManagerMock );

		EasyMock.reset( peerManagerMock );

		peerManagerMock.gumWasLost( fakeGum1.getID() );
		peerManagerMock.gumWasLost( fakeGum2.getID() );
		peerManagerMock.gumWasLost( fakeGum3.getID() );

		peerManagerMock.hereIsLocalGum( fakeGum1, fakeGumSpec1 );
		peerManagerMock.hereIsLocalGum( fakeGum2, fakeGumSpec2 );
		peerManagerMock.hereIsLocalGum( fakeGum3, fakeGumSpec3 );

		EasyMock.replay( peerManagerMock );

		gridManager.setGums( gumSpecs );

		// Simulating GuMStateOracle reaction.
		gridManager.notifyGumStateChange( fakeGum1.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum2.getID(), GumStatus.UP );
		gridManager.notifyGumStateChange( fakeGum3.getID(), GumStatus.UP );

		EasyMock.verify( peerManagerMock );
	}
}
