package org.ourgrid.peer.manager.status;

import java.io.Serializable;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.peer.manager.allocation.AllocationStatus;

public class StatusEntry implements Serializable {

	private long requestID = -1;

	private GumSpec gumSpec;

	private AllocationStatus status;

	private GumSource gumSource;


	public StatusEntry( GumSpec gumSpec, AllocationStatus status ) {

		this.gumSpec = gumSpec;
		this.status = status;
		this.gumSource = getGumSource();
	}


	public GumSource getGumSource() {

		String thisPeerName = Configuration.getInstance().getProperty( Configuration.PROP_EXTERNAL_NAME );
		String thisPeerPort = Configuration.getInstance().getProperty( Configuration.PROP_PORT );

		String gumPeerName = gumSpec.getAttribute( GumSpec.ATT_PEERNAME );
		String gumPeerPort = gumSpec.getAttribute( GumSpec.ATT_PEERPORT );

		if ( gumPeerName.equalsIgnoreCase( thisPeerName ) && gumPeerPort.equals( thisPeerPort ) ) {
			return GumSource.LOCAL;
		}
		return GumSource.REMOTE;
	}


	public GumSpec getGumSpec() {

		return this.gumSpec;
	}


	public long getRequestID() {

		return this.requestID;
	}


	public AllocationStatus getStatus() {

		return status;
	}


	@Override
	public String toString() {

		return this.gumSpec.getAttribute( GumSpec.ATT_NAME );
	}

	public enum GumSource {
		// indicates the gum source (from local peer or from remote ones)
		LOCAL, REMOTE;

		public boolean isLocal() {

			return (this.equals( LOCAL ));
		}


		public boolean isRemote() {

			return (this.equals( REMOTE ));
		}
	}

}
