/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.manager.status;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.PeerConfiguration;
import org.ourgrid.common.event.EventQueue;
import org.ourgrid.common.event.ResponseEvent;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.rmi.OurgridUnicastRemoteObject;
import org.ourgrid.common.spec.RequestSpec.RequestSource;
import org.ourgrid.peer.manager.EBPeerManagerFacade;
import org.ourgrid.peer.manager.allocation.AllocationStatus;
import org.ourgrid.peer.manager.allocation.AllocationEntry.GumSource;

public class StatusProviderImpl extends OurgridUnicastRemoteObject implements StatusProvider {

	/**
	 * Serial Version
	 */
	private static final long serialVersionUID = 33L;

	private transient Configuration configuration;

	private transient EBPeerManagerFacade ebPeerManagerFacade;


	public StatusProviderImpl( EBPeerManagerFacade ebPeerManagerFacade ) throws RemoteException {

		this.ebPeerManagerFacade = ebPeerManagerFacade;
		configuration = Configuration.getInstance( Configuration.PEER );
	}


	public Collection<StatusEntry> getGums( AllocationStatus allocationStatus, RequestSource source ) {

		EventQueue<ResponseEvent<Collection<StatusEntry>>> responseQueue = new EventQueue<ResponseEvent<Collection<StatusEntry>>>();
		ebPeerManagerFacade.getGums( responseQueue, allocationStatus, source );
		return responseQueue.blockingRemove().getResponse();

	}


	public Collection<StatusEntry> getAllGums( GumSource gumSource ) {

		EventQueue<ResponseEvent<Collection<StatusEntry>>> responseQueue = new EventQueue<ResponseEvent<Collection<StatusEntry>>>();
		ebPeerManagerFacade.getAllGums( responseQueue, gumSource );
		return responseQueue.blockingRemove().getResponse();

	}


	public Collection<ConsumerStatus> getConsumersStatus( RequestSource source ) {

		EventQueue<ResponseEvent<Collection<ConsumerStatus>>> responseQueue = new EventQueue<ResponseEvent<Collection<ConsumerStatus>>>();

		ebPeerManagerFacade.getConsumers( responseQueue, source );

		return responseQueue.blockingRemove().getResponse();
	}


	public Map<ObjectID,Double> getNetworkOfFavors() {

		EventQueue<ResponseEvent<Map<ObjectID,Double>>> responseQueue = new EventQueue<ResponseEvent<Map<ObjectID,Double>>>();

		ebPeerManagerFacade.getPeerBalance( responseQueue );

		return responseQueue.blockingRemove().getResponse();
	}


	public String getPeerName() {

		return configuration.getProperty( Configuration.PROP_NAME );
	}


	public String getPeerExternalName() {

		return configuration.getProperty( Configuration.PROP_EXTERNAL_NAME );
	}


	public String getPeerPort() {

		return configuration.getProperty( Configuration.PROP_PORT );
	}


	public String getVersion() {

		return Configuration.VERSION;
	}


	public String getEmail() {

		return configuration.getProperty( PeerConfiguration.PROP_EMAIL );
	}


	public String getDescription() {

		return configuration.getProperty( PeerConfiguration.PROP_DESCRIPTION );
	}


	public Float getLatitude() {

		return Float.parseFloat( configuration.getProperty( PeerConfiguration.PROP_LATITUDE ) );
	}


	public Float getLongitude() {

		return Float.parseFloat( configuration.getProperty( PeerConfiguration.PROP_LONGITUDE ) );
	}


	public String getDefaultSDF() {

		return configuration.getProperty( PeerConfiguration.PROP_DEFAULT_SDF );
	}


	public String getCorepeerName() {

		return configuration.getProperty( PeerConfiguration.PROP_COREPEER_NAME );
	}


	public String getCorepeerPort() {

		return configuration.getProperty( PeerConfiguration.PROP_COREPEER_PORT );
	}


	public boolean isCommunityMember() {

		return configuration.getProperty( PeerConfiguration.PROP_JOIN_COMMUNITY ).equalsIgnoreCase( "yes" );
	}


	public String getUptime() {

		return configuration.getProperty( PeerConfiguration.PROP_STARTED_TIME );
	}
}
