/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui;

import java.net.MalformedURLException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;

import org.ourgrid.common.config.ConfigException;
import org.ourgrid.common.fd.GenericState;
import org.ourgrid.common.id.ObjectID;
import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.common.spec.RequestSpec.RequestSource;
import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.peer.manager.allocation.AllocationStatus;
import org.ourgrid.peer.manager.status.StatusEntry;
import org.ourgrid.peer.ui.PeerUIManager.Notifiables;
import org.ourgrid.peer.ui.exception.PeerUIException;

public interface UIManager extends Remote {

	/**
	 * Starts Peer Service
	 */
	public void startPeerService() throws RemoteException, MalformedURLException, NumberFormatException,
		ConfigException;


	/**
	 * Stops Peer Service
	 */
	public void stopPeerService() throws RemoteException, PeerUIException;


	public void startGUIService() throws RemoteException;


	public void noMoreListeners() throws RemoteException;


	public void setLocalGums( Collection<GumSpec> gums ) throws RemoteException, PeerUIException;


	public void setLocalGums( String filename ) throws PeerUIException, CompilerException, RemoteException;


	public boolean isPeerRunning() throws RemoteException;


	public ObserverDispatcher getObserverDispatcher() throws RemoteException;


	public Collection<StatusEntry> getGums( AllocationStatus status, RequestSource source ) throws RemoteException;


	public Map<ObjectID,Double> getPeerBalances() throws RemoteException;


	public void registerNewMonitorable( Notifiables notifiableType, String monitorableAddress ) throws RemoteException;


	public void registerNewMonitorable( Notifiables notifiableType, ObjectID objectID ) throws RemoteException;


	public void unregisterNewMonitorable( Notifiables notifiableType, String monitorableAddress )
		throws RemoteException;


	public void failureDetectorAdvice( String monitorableAddress, GenericState newState ) throws RemoteException;


	public void unregisterNewMonitorable( Notifiables consumer, ObjectID consumerID ) throws RemoteException;

}
