/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui.command;

import java.io.File;
import java.rmi.RemoteException;

import org.ourgrid.common.spec.main.CompilerException;
import org.ourgrid.peer.ui.PeerUIManager;
import org.ourgrid.peer.ui.UIManager;
import org.ourgrid.peer.ui.exception.PeerUIException;

/**
 * Description: Class that sets the gums configurated on Peer.
 */
public class PeerSetter implements Command {

	private static transient final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger( PeerSetter.class );


	/**
	 * Main method. This method should be called with a GridDescriptionFile as
	 * command line argument1
	 */
	public void run( String[ ] params, UIManager manager ) {

		try {

			PeerUIManager.getInstance();
			System.out.println( UIMessages.SETTING_GRID );
			// TODO Find better sollution. Since the UIManager is running in
			// another JVM there may be path problems, so
			// we send the absolute path relative to where setgrid was invoked.
			manager.setLocalGums( new File( params[0] ).getAbsolutePath() );
			System.out.println( UIMessages.GRID_SET );
			System.exit( ErrorCode.OK );

		} catch ( PeerUIException pe ) {

			LOG.debug( " The Peer is not running." );
			System.err.println( UIMessages.GRID_COULD_NOT_BE_LOADED );
			System.err.println( UIMessages.ERROR_CAUSE + " " + pe.getMessage() );
			System.exit( ErrorCode.PEER_IS_NOT_RUNNING );

		} catch ( RemoteException re ) {

			System.err.println( UIMessages.GRID_COULD_NOT_BE_LOADED );
			System.err.println( UIMessages.ERROR_CAUSE + " " + re.getMessage() );
			System.exit( ErrorCode.PEER_IS_NOT_RUNNING );

		} catch ( CompilerException ce ) {
			LOG.debug( ce.getMessage() );
			System.err.println( UIMessages.GRID_COULD_NOT_BE_LOADED );
			System.err.println( UIMessages.ERROR_CAUSE + " " + ce.getMessage() );
			System.exit( ErrorCode.FAIL );
		}

	}

}
