/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui.gui;

import java.awt.Component;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.ourgrid.peer.manager.allocation.AllocationStatus;

public class GridTreeCellRenderer extends DefaultTreeCellRenderer {

	private static final long serialVersionUID = 33L;

	private static Icon allocatedIcon = createIcon( "peer_allocated.gif" );

	private static Icon donatedIcon = createIcon( "peer_donated.gif" );

	private static Icon idleIcon = createIcon( "peer_idle.gif" );

	private static Icon offlineIcon = createIcon( "peer_offline.gif" );


	private static Icon createIcon( String fileName ) {

		String basedir = System.getProperty( "MGROOT" ) + "/resources/images/";
		ImageIcon icon = new ImageIcon( basedir + fileName );

		return icon;

	}


	public Component getTreeCellRendererComponent( JTree tree, Object value, boolean sel, boolean expanded,
													boolean leaf, int row, boolean hasFocus ) {

		super.getTreeCellRendererComponent( tree, value, sel, expanded, leaf, row, hasFocus );

		switch ( (AllocationStatus) getType( (DefaultMutableTreeNode) value ) ) {

			case IN_USE: {
				// TODO change this icon
				setIcon( allocatedIcon );
				setToolTipText( AllocationStatus.IN_USE.toString() );
				break;
			}

			case DONATED: {
				setIcon( donatedIcon );
				setToolTipText( AllocationStatus.DONATED.toString() );
				break;
			}

			case IDLE: {
				setIcon( idleIcon );
				setToolTipText( AllocationStatus.IDLE.toString() );
				break;
			}

			case CONTACTING: {
				setIcon( offlineIcon );
				setToolTipText( AllocationStatus.CONTACTING.toString() );
				break;

			}

			default: {
				if ( leaf ) {
					setIcon( (new DefaultTreeCellRenderer()).getDefaultClosedIcon() );
				}
			}

		}

		return this;
	}


	protected Object getType( DefaultMutableTreeNode value ) {

		DefaultMutableTreeNode node = value;

		Object type = ((GridTreeNode) node.getUserObject()).getType();

		return type;

	}

}
