/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui.gui;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.peer.manager.allocation.AllocationStatus;

/**
 * This class generate the Grid Tree, it has a instance of StatusProvider to
 * obtain informations to create tree.<br>
 * It has too, maps to store all gums that have already been inserted in tree to
 * prevent exhausting algorithms to verify if already had been inserted in the
 * tree.
 */
public class GridTreeModel {

	/** Default Tree Model */
	private DefaultTreeModel tree;

	/** Default Mutable Tree Node */
	private DefaultMutableTreeNode root;

	private DefaultMutableTreeNode localGuMPRoot;

	private DefaultMutableTreeNode communityGuMPRoot;

	private Collection remoteInUseGums;

	private Collection offlineGums;

	private Collection idleGums;

	private Collection donatedGums;

	private Collection localInUseGums;

	private Collection configuredGums;


	/**
	 * To obtain unique instance of this class
	 */
	public static GridTreeModel createTree() {

		DefaultMutableTreeNode root = new DefaultMutableTreeNode( new GridTreeNode( "Grid", "Grids" ) );
		return new GridTreeModel( root );
	}


	/**
	 * Create a GridTreeModel.
	 */
	private GridTreeModel( DefaultMutableTreeNode root ) {

		tree = new DefaultTreeModel( root );

		// statusProvider = StatusProviderImpl.getInstance();

		this.root = root;
		// TODO Finish this implementation
		// localGuMPRoot = new DefaultMutableTreeNode( new GridTreeNode( "Local
		// Gums" , "Peers" ) );
		// tree.insertNodeInto(localGuMPRoot,root,root.getChildCount());
		//		
		// communityGuMPRoot = new DefaultMutableTreeNode( new GridTreeNode(
		// "Community Gums", "Peers" ) );
		// tree.insertNodeInto(communityGuMPRoot, root, root.getChildCount());

		remoteInUseGums = new HashSet();
		offlineGums = new HashSet();
		idleGums = new HashSet();
		donatedGums = new HashSet();
		localInUseGums = new HashSet();
		configuredGums = new HashSet();
	}


	/** Returns the tree to adds in panel */
	public DefaultTreeModel getTreeModel() {

		return tree;
	}


	/** Refresh the tree */
	public void refreshTree() {

		// TODO Finish this implementation
		// configuredGums = statusProvider.getGums(AllocationStatus.CONFIGURED);
		// remoteInUseGums =
		// statusProvider.getGums(AllocationStatus.REMOTE_IN_USE);
		// offlineGums = statusProvider.getGums(AllocationStatus.OFFLINE);
		// idleGums = statusProvider.getGums(AllocationStatus.IDLE);
		// donatedGums = statusProvider.getGums(AllocationStatus.DONATED);
		// localInUseGums =
		// statusProvider.getGums(AllocationStatus.LOCAL_IN_USE);

		updateTree();
	}


	/** Update the tree data structure */
	private void updateTree() {

		Iterator gumsIt = configuredGums.iterator();
		while ( gumsIt.hasNext() ) {

			GumSpec gum = (GumSpec) gumsIt.next();

			DefaultMutableTreeNode gumNode = null;

			if ( donatedGums.contains( gum ) ) {
				gumNode = new DefaultMutableTreeNode( new GridTreeNode( AllocationStatus.DONATED ) );
			}

			if ( remoteInUseGums.contains( gum ) ) {
				gumNode = new DefaultMutableTreeNode( new GridTreeNode( AllocationStatus.IN_USE ) );
			}

			if ( idleGums.contains( gum ) ) {
				gumNode = new DefaultMutableTreeNode( new GridTreeNode( AllocationStatus.IDLE ) );
			}

			if ( offlineGums.contains( gum ) ) {
				gumNode = new DefaultMutableTreeNode( new GridTreeNode( AllocationStatus.CONTACTING ) );
			}

			tree.insertNodeInto( gumNode, localGuMPRoot, localGuMPRoot.getChildCount() );

		}

		Iterator localInUseGumsIt = localInUseGums.iterator();

		while ( localInUseGumsIt.hasNext() ) {

			localInUseGumsIt.next();

			DefaultMutableTreeNode gumNode = new DefaultMutableTreeNode( new GridTreeNode( AllocationStatus.IN_USE ) );
			tree.insertNodeInto( gumNode, communityGuMPRoot, communityGuMPRoot.getChildCount() );
		}
	}


	/** This method is friendly because test needs a reference to Node Root */
	DefaultMutableTreeNode getMyGuMPRoot() {

		return localGuMPRoot;
	}


	/** This method is friendly because test needs a reference to Node Root */
	DefaultMutableTreeNode getGuMPRoot() {

		return root;
	}

}
