/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui.gui;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.border.LineBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import org.ourgrid.common.spec.GumSpec;

/**
 * This class generate the GuMP Manager, it has a instance of PeerUIServices to
 * obtain informations to create table.<br>
 * Implements the abstract methods of AbstractTableModel to instantiate JTable.
 */
public class GumPanel implements TreeSelectionListener, ActionListener {

	private final static String LINE_SEPARATOR = System.getProperty( "line.separator" );

	/** Static instance, for use Singleton Pattern */
	private static GumPanel instance = null;

	/** JPanel */
	private JPanel masterPanel;

	/** Properties Area */
	private JTextArea propertiesArea;

	/** JTree */
	private JTree tree;

	/** Tree's Scroll */
	private JScrollPane scroll;

	/** PropertiesArea's Scroll */
	private JScrollPane propertiesScroll;

	/** DefaultTreeModel */
	private GridTreeModel gtm;

	// AQUI
	private JPopupMenu popupMenu;


	/** Private constructor for use Singleton Pattern */
	private GumPanel() {

		// TODO statusProvider = StatusProviderImpl.getInstance();

		masterPanel = new JPanel();
		masterPanel.setLayout( new GridLayout( 1, 2 ) );

		// Tree
		gtm = GridTreeModel.createTree();
		tree = new JTree( gtm.getTreeModel() );
		tree.setLargeModel( true );
		tree.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
		tree.addTreeSelectionListener( this );
		tree.setCellRenderer( new GridTreeCellRenderer() );
		tree.setDoubleBuffered( true );
		tree.setBorder( new LineBorder( Color.BLACK ) );

		// PopupMenu
		popupMenu = new JPopupMenu();
		JMenuItem doctorTest = new JMenuItem( "Doctor Test" );
		doctorTest.addActionListener( this );
		popupMenu.add( doctorTest );
		MouseListener popupListener = new PopupListener( popupMenu );
		tree.addMouseListener( popupListener );

		// Scroll
		scroll = new JScrollPane( tree );
		scroll.setViewportView( tree );
		masterPanel.add( scroll );

		// Propertioes Area
		propertiesArea = new JTextArea();
		propertiesArea.setBorder( new LineBorder( Color.BLACK ) );
		propertiesArea.setEditable( false );
		propertiesScroll = new JScrollPane( propertiesArea );
		masterPanel.add( propertiesScroll );
		masterPanel.setDoubleBuffered( true );

	}


	/** Static method for get instance of this class */
	public synchronized static GumPanel getInstance() {

		if ( instance == null )
			instance = new GumPanel();
		return instance;
	}


	/** Return JPanel with JScrollPane */
	public JPanel getPanel() {

		return masterPanel;
	}


	/** Get List of GuMs and put in table */
	public void refreshTree() {

		this.clean();
		gtm.refreshTree();
	}


	/** Clean the table */
	public void clean() {

		masterPanel.remove( scroll );
		gtm = GridTreeModel.createTree();
		tree = new JTree( gtm.getTreeModel() );
		tree.setDoubleBuffered( true );
		tree.addTreeSelectionListener( this );
		tree.setLargeModel( true );
		tree.setCellRenderer( new GridTreeCellRenderer() );
		tree.setBorder( new LineBorder( Color.BLACK ) );
		scroll = new JScrollPane( tree );
		masterPanel.add( scroll );

		masterPanel.remove( propertiesScroll );
		propertiesArea = new JTextArea();
		propertiesArea.setBorder( new LineBorder( Color.BLACK ) );
		propertiesArea.setEditable( false );
		propertiesScroll = new JScrollPane( propertiesArea );
		masterPanel.add( propertiesScroll );

		// PopupMenu
		popupMenu = new JPopupMenu();
		JMenuItem doctorTest = new JMenuItem( "Doctor Test" );
		doctorTest.addActionListener( this );
		popupMenu.add( doctorTest );
		MouseListener popupListener = new PopupListener( popupMenu );
		tree.addMouseListener( popupListener );

		propertiesArea.repaint();
		tree.repaint();
		scroll.repaint();
	}


	/**
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged( TreeSelectionEvent e ) {

		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();

		if ( node == null )
			return;

		String nodeInfo = "Grid";

		try {

			nodeInfo = ((GridTreeNode) node.getUserObject()).toString();

		} catch ( Exception ee ) {
			ee.printStackTrace();
		}

		if ( nodeInfo.startsWith( "Local Machine" ) || nodeInfo.startsWith( "Grid" )
				|| nodeInfo.startsWith( "Community" ) ) {
			propertiesArea.setText( "" );
			return;
		}

		showPropertiesOfGuMs( nodeInfo );

	}


	/**
	 * Search the GuM properties and put in properties Area.
	 * 
	 * @param gumLabel GuM label in tree.
	 */
	private void showPropertiesOfGuMs( String gumLabel ) {

		// TODO statusProvider = StatusProviderImpl.getInstance();

		Collection allGuMs = new HashSet();

		// TODO allGuMs.addAll(statusProvider.getGums(
		// AllocationStatus.CONFIGURED ));
		// TODO allGuMs.addAll(statusProvider.getGums(
		// AllocationStatus.REMOTE_IN_USE ));

		String gumName = gumLabel;
		Iterator gumIt;

		gumIt = allGuMs.iterator();

		while ( gumIt.hasNext() ) {
			GumSpec gumSpec = (GumSpec) gumIt.next();
			if ( gumName.equals( gumSpec.getGumID() ) ) {

				Map<String,String> p = new HashMap<String,String>();

				for ( String propName : gumSpec.getAttributes().keySet() ) {
					String uperPropName = propName.toUpperCase();
					p.put( uperPropName, gumSpec.getAttributes().get( propName ) );
				}

				writeGuMInTextArea( p );

				break;

			}

		}

	}


	/**
	 * Put all properties in properties area.
	 * 
	 * @param p Properties.
	 */
	private void writeGuMInTextArea( Map p ) {

		propertiesArea.setText( "" );

		StringBuffer sb = new StringBuffer();
		sb.append( " GUM PROPERTIES:" );

		Iterator it = p.keySet().iterator();
		while ( it.hasNext() ) {
			String key = (String) it.next();
			sb.append( LINE_SEPARATOR + "    " + key.toUpperCase() + ": " + p.get( key ) );
		}

		propertiesArea.setText( sb.toString() );
	}


	public void actionPerformed( ActionEvent e ) {

	}

}
