/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.peer.ui.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.PeerConfiguration;

/**
 * This class contains the presentation screen of MyGrid GUI and also some
 * buttons to set Grids, add and cancel Jobs and start stop MyGrid.
 */
public class SettingsPanel {

	/** MyGrid's background file path */
	private final String BACKGROUND_PEER_PATH = Configuration.getInstance( Configuration.MYGRID ).getProperty(
			Configuration.MGROOT )
			+ File.separator + "resources/images" + File.separator + "peer_background.png";

	/** Static reference to unique instance of this class, Singleton Pattern */
	private static SettingsPanel instance = null;

	/** MyGrid Panel */
	private JPanel settingsPanel;

	/** Value Labels */
	private JLabel nameValue;

	private JLabel externalnameValue;

	private JLabel labelValue;

	private JLabel portValue;

	private JLabel joincommunityValue;

	private JLabel savingintervalValue;

	private JLabel redistributeintervalValue;

	private JLabel pingsintervalValue;

	private JLabel emailValue;

	private JLabel descriptionValue;


	/** Private constructor, Singleton Pattern */
	private SettingsPanel() {

		initComponents();
		readPeerProperties();
	}


	/** Static method to obtain the reference of this class, Singleton Pattern */
	public static SettingsPanel getInstance() {

		if ( instance == null )
			instance = new SettingsPanel();
		return instance;
	}


	/** Return the mygrid panel */
	public JPanel getPanel() {

		return settingsPanel;
	}


	/** Init components */
	private void initComponents() {

		settingsPanel = new JPanel();
		settingsPanel.setLayout( new GridLayout( 1, 2 ) );

		JPanel westPanel = new JPanel();
		westPanel.setLayout( new BorderLayout() );
		westPanel.setAlignmentX( JPanel.CENTER_ALIGNMENT );
		westPanel.setBackground( Color.WHITE );
		JLabel background = new JLabel( new ImageIcon( BACKGROUND_PEER_PATH ) );
		westPanel.add( background, BorderLayout.LINE_START );
		settingsPanel.add( westPanel );

		JPanel propertiesPanel = new JPanel();
		propertiesPanel.setLayout( new GridLayout( 11, 2 ) );
		propertiesPanel.setBackground( Color.WHITE );
		propertiesPanel.setBorder( new TitledBorder( "Peer Settings" ) );

		settingsPanel.add( propertiesPanel );

		JLabel name = new JLabel( " peer.name" );
		JLabel externalname = new JLabel( " peer.externalname" );
		JLabel label = new JLabel( " peer.label" );
		JLabel port = new JLabel( " peer.port" );
		JLabel joincommunity = new JLabel( " peer.joincommunity" );
		JLabel savinginterval = new JLabel( " peer.savinginterval" );
		JLabel redistributeinterval = new JLabel( " peer.redistributeinterval" );
		JLabel pingsinterval = new JLabel( " peer.pingsinterval" );
		JLabel email = new JLabel( " peer.email" );
		JLabel description = new JLabel( " peer.description" );

		Font font = new Font( "Arial", Font.BOLD, 12 );
		name.setFont( font );
		externalname.setFont( font );
		label.setFont( font );
		port.setFont( font );
		joincommunity.setFont( font );
		savinginterval.setFont( font );
		redistributeinterval.setFont( font );
		pingsinterval.setFont( font );
		email.setFont( font );
		description.setFont( font );

		nameValue = new JLabel( "" );
		externalnameValue = new JLabel( "" );
		labelValue = new JLabel( "" );
		portValue = new JLabel( "" );
		joincommunityValue = new JLabel( "" );
		savingintervalValue = new JLabel( "" );
		redistributeintervalValue = new JLabel( "" );
		pingsintervalValue = new JLabel( "" );
		emailValue = new JLabel( "" );
		descriptionValue = new JLabel( "" );

		Font fontValue = new Font( "Arial", Font.PLAIN, 12 );
		name.setFont( fontValue );
		externalname.setFont( fontValue );
		label.setFont( fontValue );
		port.setFont( fontValue );
		joincommunity.setFont( fontValue );
		savinginterval.setFont( fontValue );
		redistributeinterval.setFont( fontValue );
		pingsinterval.setFont( fontValue );
		email.setFont( fontValue );
		description.setFont( fontValue );

		propertiesPanel.add( name );
		propertiesPanel.add( nameValue );

		propertiesPanel.add( externalname );
		propertiesPanel.add( externalnameValue );

		propertiesPanel.add( label );
		propertiesPanel.add( labelValue );

		propertiesPanel.add( port );
		propertiesPanel.add( portValue );

		propertiesPanel.add( joincommunity );
		propertiesPanel.add( joincommunityValue );

		propertiesPanel.add( savinginterval );
		propertiesPanel.add( savingintervalValue );

		propertiesPanel.add( redistributeinterval );
		propertiesPanel.add( redistributeintervalValue );

		propertiesPanel.add( pingsinterval );
		propertiesPanel.add( pingsintervalValue );

		propertiesPanel.add( email );
		propertiesPanel.add( emailValue );

		propertiesPanel.add( description );
		propertiesPanel.add( descriptionValue );

	}


	/**
	 * Read MG.properties file and shows values in MyGrid Panel.
	 */
	public void readPeerProperties() {

		String notDefined = "Not defined";
		String space = "  ";

		Configuration config = Configuration.getInstance( Configuration.PEER );

		String value = config.getProperty( PeerConfiguration.PROP_NAME );

		if ( value == null || value.trim().equals( "" ) ) {
			nameValue.setText( notDefined );
		} else {
			nameValue.setText( value.trim() + space );
		}

		value = config.getProperty( PeerConfiguration.PROP_EXTERNAL_NAME );

		if ( value == null || value.trim().equals( "" ) ) {
			externalnameValue.setText( notDefined );
		} else {
			externalnameValue.setText( value.trim() );
		}
		value = config.getProperty( PeerConfiguration.PROP_LABEL );

		if ( value == null || value.trim().equals( "" ) ) {
			labelValue.setText( notDefined );
		} else {
			labelValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_PORT );

		if ( value == null || value.trim().equals( "" ) ) {
			portValue.setText( notDefined );
		} else {
			portValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_JOIN_COMMUNITY );

		if ( value == null || value.trim().equals( "" ) ) {
			joincommunityValue.setText( notDefined );
		} else {
			joincommunityValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_SAVING_INTERVAL );

		if ( value == null || value.trim().equals( "" ) ) {
			savingintervalValue.setText( notDefined );
		} else {
			savingintervalValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_REDISTRIBUTING_INTERVAL );

		if ( value == null || value.trim().equals( "" ) ) {
			redistributeintervalValue.setText( notDefined );
		} else {
			redistributeintervalValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_PING_INTERVAL );

		if ( value == null || value.trim().equals( "" ) ) {
			pingsintervalValue.setText( notDefined );
		} else {
			pingsintervalValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_EMAIL );

		if ( value == null || value.trim().equals( "" ) ) {
			emailValue.setText( notDefined );
		} else {
			emailValue.setText( value.trim() );
		}

		value = config.getProperty( PeerConfiguration.PROP_DESCRIPTION );

		if ( value == null || value.trim().equals( "" ) ) {
			descriptionValue.setText( notDefined );
		} else {
			descriptionValue.setText( value.trim() );
		}

	}

}
