/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test;

import java.io.File;
import java.io.IOException;
import java.util.List;

import junit.framework.TestCase;

import org.ourgrid.common.config.UserAgentConfiguration;
import org.ourgrid.common.exception.NotEnoughGumSpecsException;

/**
 * Tests for <code>LocalPeer</code> class
 */
public class LocalPeerTest extends TestCase implements CommonTest, SpecialCareTest {

	private int port;

	private TestUtils testUtils;

	private List gumSpecList;


	public void setUp() throws NotEnoughGumSpecsException {

		testUtils = new TestUtils();
		testUtils.load();
		port = Integer.parseInt( testUtils.getProperty( TestUtils.OG_PORT_II ) );
		gumSpecList = testUtils.getGumSpecs( 2, UserAgentConfiguration.TYPE_UA_LINUX );
	}


	public void tearDown() {

		testUtils.reset();
	}


	public void testStartAndStop() {

		LocalPeer myLocalPeer = new LocalPeer( port );

		try {
			myLocalPeer.stop(); // Fails, peer is not running
			fail();
		} catch ( Error e ) {
			assertTrue( "LocalPeer.stop throws an error", true );
		}

		System.out.println( "Starting the peer" );
		myLocalPeer.start();
		assertTrue( myLocalPeer.isRunning() );
		System.out.println( "Stopping the peer" );
		myLocalPeer.stop();
		assertFalse( myLocalPeer.isRunning() );
	}


	public void testSetGuMs() throws IOException {

		// Prepares a LocalPeer
		LocalPeer myLocalPeer = new LocalPeer( port );
		File myGDF = new File( testUtils.createSDF( gumSpecList ) );

		try {
			myLocalPeer.stop();
		} catch ( Error e ) {
			// Forcing that the peer will be stopped.
		}

		assertFalse( myLocalPeer.setGuMs( myGDF ) ); // Fails, peer is not
														// running

		myLocalPeer.start();
		assertTrue( myLocalPeer.setGuMs( myGDF ) );
		myLocalPeer.stop();

	}

}
