/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;

import junit.framework.TestCase;

import org.ourgrid.common.config.Configuration;
import org.ourgrid.common.config.MyGridConfiguration;
import org.ourgrid.common.spec.GumSpec;

/**
 * Tests the scripts
 */
public class PathWithEmptySpacesTest extends TestCase implements GUITest, SpecialCareTest {

	private File mgrootWithEmptySpace;

	private String OUTPUT_1_FILENAME;

	private final String EXPECTED_OUTPUT_1 = "47";

	private String OUTPUT_2_FILENAME;

	private final String EXPECTED_OUTPUT_2 = "6819792792357414911";

	private String OUTPUT_3_FILENAME;

	private final String EXPECTED_OUTPUT_3 = "6819792792357414911";

	private TestUtils testUtils;

	private String gdfAnotherSimpleGrid;


	/**
	 * Sets up the environment.
	 */
	protected void setUp() throws Exception {

		testUtils = new TestUtils();
		testUtils.load();
		testUtils.createMGProperties();
		testUtils.createOGProperties();

		gdfAnotherSimpleGrid = testUtils.createSDF( testUtils.getGumSpecs( testUtils
				.getNumberOfGuMSpecsAvailable( TestUtils.TYPE_UALINUX ), TestUtils.TYPE_UALINUX ) );

		Configuration configuration = Configuration.getInstance( Configuration.MYGRID );
		mgrootWithEmptySpace = new File( configuration.getProperty( MyGridConfiguration.MGROOT ) );

		OUTPUT_1_FILENAME = mgrootWithEmptySpace.getCanonicalPath() + "/test/resources/output-1.1";
		OUTPUT_2_FILENAME = mgrootWithEmptySpace.getCanonicalPath() + "/test/resources/output-1.2";
		OUTPUT_3_FILENAME = mgrootWithEmptySpace.getCanonicalPath() + "/test/resources/output-1.3";

		File output1 = new File( OUTPUT_1_FILENAME );
		output1.delete();
		File output2 = new File( OUTPUT_2_FILENAME );
		output2.delete();
		File output3 = new File( OUTPUT_3_FILENAME );
		output3.delete();
	}


	protected void tearDown() {

		testUtils.reset();
	}


	private boolean checkFileContent( String expectedContent, File file ) throws Exception {

		BufferedReader bufferedReader = new BufferedReader( new FileReader( file ) );
		String line = bufferedReader.readLine();
		String contentInFile = "";
		while ( line != null ) {
			contentInFile += line;
			line = bufferedReader.readLine();
		}
		System.out.println( "expected: " + expectedContent );
		System.out.println( "obtained: " + contentInFile );
		bufferedReader.close();
		return expectedContent.equals( contentInFile );
	}


	/**
	 * Tests the scripts.
	 */
	public void testScripts() throws Exception {

		System.out.println( mgrootWithEmptySpace.getAbsolutePath() );

		Vector<GumSpec> GuMSpecList = new Vector<GumSpec>( testUtils.getGumSpecs( 3, TestUtils.TYPE_UALINUX ) );

		String simpleGridUAGDF = testUtils.createSDF( GuMSpecList );

		Process process;

		Runtime runtime = Runtime.getRuntime();

		System.out.println( "uaadmin start " + simpleGridUAGDF );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "uaadmin init "
				+ gdfAnotherSimpleGrid );
		assertEquals( 0, process.waitFor() );

		testUtils.testUA( GuMSpecList, 10000 );

		System.out.println( "uaadmin status " + simpleGridUAGDF );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "uaadmin test "
				+ gdfAnotherSimpleGrid );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer start" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "peer start" );
		assertEquals( 0, process.waitFor() );

		Thread.sleep( 5000 );

		System.out.println( "peer status" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "peer status" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer setgums " + simpleGridUAGDF );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "peer setgums "
				+ gdfAnotherSimpleGrid );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid start" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "mygrid start" );
		assertEquals( 0, process.waitFor() );

		Thread.sleep( 5000 );

		String localGDF = testUtils.createLocalPeerGDF();

		System.out.println( "mygrid setgrid " + localGDF );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "mygrid setgrid "
				+ localGDF );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid addjob " + testUtils.getProperty( "job.stress" ) );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "mygrid addjob "
				+ testUtils.getProperty( "job.stress" ) );
		assertEquals( 0, process.waitFor() );

		System.out.println( "mygrid waitforjob 1" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "mygrid waitforjob 1" );
		assertEquals( 0, process.waitFor() );

		assertTrue( checkFileContent( EXPECTED_OUTPUT_1, new File( OUTPUT_1_FILENAME ) ) );
		assertTrue( checkFileContent( EXPECTED_OUTPUT_2, new File( OUTPUT_2_FILENAME ) ) );
		assertTrue( checkFileContent( EXPECTED_OUTPUT_3, new File( OUTPUT_3_FILENAME ) ) );

		System.out.println( "mygrid stop" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "mygrid stop" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer stop" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "peer stop" );
		assertEquals( 0, process.waitFor() );

		System.out.println( "peer status" );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "peer status" );
		assertFalse( 0 == process.waitFor() );

		testUtils.testUA( GuMSpecList, 5000 );

		System.out.println( "uaadmin stop " + simpleGridUAGDF );
		process = runtime.exec( mgrootWithEmptySpace + File.separator + "bin" + File.separator + "uaadmin shutdown "
				+ gdfAnotherSimpleGrid );
		assertEquals( 0, process.waitFor() );

		testUtils.testUA( GuMSpecList, 5000 );

	}

}
