/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.functionaltests;

import java.io.File;

import junit.framework.TestCase;

import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.test.functionaltests.busywait.JobWaiterRunnable;
import org.ourgrid.test.functionalunits.CorePeerUnit;
import org.ourgrid.test.functionalunits.FunctionalUnitsControl;
import org.ourgrid.test.functionalunits.MyGridUnit;
import org.ourgrid.test.functionalunits.PeerUnit;
import org.ourgrid.test.functionalunits.UserAgentUnit;
import org.ourgrid.test.threadServices.ThreadServices;

/**
 * Stressfully test job executions on OurGrid communities under different
 * conditions. Test details are documented on the test methods.
 * 
 * @author Flavio Vinicius Diniz de Figueiredo - flaviov@lsd.ufcg.edu.br
 */
public class ExecutionLoadFunctionalTests extends TestCase /*
															 * implements
															 * FunctionalStressTest
															 */{

	/**
	 * Reset the <code>FunctionTestsControl</code> to clear any properties
	 * configuration that was used before.
	 * 
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {

		FunctionalUnitsControl.getInstance().destroyAllRemainingUnits();
		FunctionalUnitsControl.getInstance().reset();
	}


	/**
	 * Test scenario:<br> - Peer A, MyGrid A at Peer A, Gums at Peer A
	 * Description:<br> - Tests execution of a wide number of jobs in a Single
	 * Peer from one MyGrid. Functional tests units:<br> - Peer A,<br> -
	 * MyGrid A.<br>
	 * Test scenario workflow:<br> - Start Peer A,<br> - Set Gums at Peer A,<br> -
	 * Start MyGrid A,<br> - Set MyGrid A with Peer A,<br> - Add job at MyGrid
	 * A,<br> - Wait until MyGrid A job is running,<br> - Wait until MyGrid A
	 * job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testOneMyGridALotOfEchoJobsAndOnePeer() throws Exception {

		runStressTestWithEnviroment( 1, 1, 5, FunctionalUnitsControl.JOB_ECHO, 200 );
	}


	/**
	 * Test scenario:<br> - Peer A, MyGrid [1 to 5] at Peer A, Gums at Peer A
	 * Description:<br> - Tests execution of a wide number of jobs in a Single
	 * Peer from various MyGrid. Functional tests units:<br> - Peer A,<br> -
	 * MyGrid [1 to 5].<br>
	 * Test scenario workflow:<br> - Start Peer A,<br> - Set Gums at Peer A,<br> -
	 * Start MyGrid [1 to 5],<br> - Set MyGrid [1 to 5] with Peer A,<br> - Add
	 * job at MyGrid [1 to 5],<br> - Wait until MyGrid [1 to 5] job is running,<br> -
	 * Wait until MyGrid [1 to 5] job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testVariousMyGridALotOfEchoJobsAndOnePeer() throws Exception {

		runStressTestWithEnviroment( 5, 1, 5, FunctionalUnitsControl.JOB_ECHO, 200 );
	}


	/**
	 * Test scenario:<br> - Peer [1 to 5], MyGrid A at Peer 1, Gums at Peers [1
	 * to 5], Peer [1 to 5] at Core Peer A Description:<br> - Tests execution
	 * of a wide number of jobs in various Peer from one MyGrid. Functional
	 * tests units:<br> - Peer [1 to 5],<br> - MyGrid A.<br>
	 * Test scenario workflow:<br> - Start Core Peer A,<br> - Start Peer [1 to
	 * 5],<br> - Set Gums at Peer [1 to 5],<br> - Set Peer [1 to 5] at Core
	 * Peer A,<br> - Start MyGrid A,<br> - Set MyGrid A with Peer 1,<br> -
	 * Add jobs at MyGrid A,<br> - Wait until MyGrid A jobs are running,<br> -
	 * Wait until MyGrid A job are finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testOneMyGridALotOfEchoJobsAndVariousPeer() throws Exception {

		runStressTestWithEnviroment( 1, 5, 2, FunctionalUnitsControl.JOB_ECHO, 200 );
	}


	/**
	 * Test scenario:<br> - Peer [1 to 5], MyGrid [1 to 5] randomly at Peers [1
	 * to 5], Gums at Peers [1 to 5], Peer [1 to 5] at Core Peer 1 Description:<br> -
	 * Tests execution of a wide number of jobs in various Peer from various
	 * MyGrid. Functional tests units:<br> - Peer [1 to 5],<br> - MyGrid [1 to
	 * 5].<br> - Core Peer 1 Test scenario workflow:<br> - Start Core Peer 1,<br> -
	 * Start Peer [1 to 5],<br> - Set Gums at Peer [1 to 5],<br> - Set Peer [1
	 * to 5] at Core Peer 1,<br> - Start MyGrid [1 to 5],<br> - Set MyGrid [1
	 * to y] with Peers [1 to 5] randomly,<br> - Add job at MyGrid [1 to 5],<br> -
	 * Wait until MyGrid [1 to 5] job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testVariousMyGridALotOfEchoJobsAndVariousPeer() throws Exception {

		runStressTestWithEnviroment( 5, 5, 2, FunctionalUnitsControl.JOB_ECHO, 200 );
	}


	/**
	 * Test scenario:<br> - Peer A, MyGrid A at Peer A, Gums at Peer A
	 * Description:<br> - Tests execution of a wide number of jobs in a Single
	 * Peer from one MyGrid. Functional tests units:<br> - Peer A,<br> -
	 * MyGrid A.<br>
	 * Test scenario workflow:<br> - Start Peer A,<br> - Set Gums at Peer A,<br> -
	 * Start MyGrid A,<br> - Set MyGrid A with Peer A,<br> - Add job at MyGrid
	 * A,<br> - Wait until MyGrid A job is running,<br> - Wait until MyGrid A
	 * job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testOneMyGridAFewOfEchoTaskBasedAndOnePeer() throws Exception {

		runStressTestWithEnviroment( 1, 1, 5, FunctionalUnitsControl.JOB_ECHO_MANY_TASKS, 200 );
	}


	/**
	 * Test scenario:<br> - Peer A, MyGrid [1 to 5] at Peer A, Gums at Peer A
	 * Description:<br> - Tests execution of a wide number of jobs in a Single
	 * Peer from various MyGrid. Functional tests units:<br> - Peer A,<br> -
	 * MyGrid [1 to 5].<br>
	 * Test scenario workflow:<br> - Start Peer A,<br> - Set Gums at Peer A,<br> -
	 * Start MyGrid [1 to 5],<br> - Set MyGrid [1 to 5] with Peer A,<br> - Add
	 * job at MyGrid [1 to 5],<br> - Wait until MyGrid [1 to 5] job is running,<br> -
	 * Wait until MyGrid [1 to 5] job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testVariousMyGridAFewOfEchoTaskBasedAndOnePeer() throws Exception {

		runStressTestWithEnviroment( 5, 1, 5, FunctionalUnitsControl.JOB_ECHO_MANY_TASKS, 200 );
	}


	/**
	 * Test scenario:<br> - Peer [1 to 5], MyGrid A at Peer 1, Gums at Peers [1
	 * to 5], Peer [1 to 5] at Core Peer A Description:<br> - Tests execution
	 * of a wide number of jobs in various Peer from one MyGrid. Functional
	 * tests units:<br> - Peer [1 to 5],<br> - MyGrid A.<br>
	 * Test scenario workflow:<br> - Start Core Peer A,<br> - Start Peer [1 to
	 * 5],<br> - Set Gums at Peer [1 to 5],<br> - Set Peer [1 to 5] at Core
	 * Peer A,<br> - Start MyGrid A,<br> - Set MyGrid A with Peer 1,<br> -
	 * Add jobs at MyGrid A,<br> - Wait until MyGrid A jobs are running,<br> -
	 * Wait until MyGrid A job are finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testOneMyGridAFewOfEchoTaskBasedJobsAndVariousPeer() throws Exception {

		runStressTestWithEnviroment( 1, 5, 2, FunctionalUnitsControl.JOB_ECHO_MANY_TASKS, 200 );
	}


	/**
	 * Test scenario:<br> - Peer [1 to 5], MyGrid [1 to 5] randomly at Peers [1
	 * to 5], Gums at Peers [1 to 5], Peer [1 to 5] at Core Peer 1 Description:<br> -
	 * Tests execution of a wide number of jobs in various Peer from various
	 * MyGrid. Functional tests units:<br> - Peer [1 to 5],<br> - MyGrid [1 to
	 * 5].<br> - Core Peer 1 Test scenario workflow:<br> - Start Core Peer 1,<br> -
	 * Start Peer [1 to 5],<br> - Set Gums at Peer [1 to 5],<br> - Set Peer [1
	 * to 5] at Core Peer 1,<br> - Start MyGrid [1 to 5],<br> - Set MyGrid [1
	 * to y] with Peers [1 to 5] randomly,<br> - Add job at MyGrid [1 to 5],<br> -
	 * Wait until MyGrid [1 to 5] job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void testVariousMyGridAFewOfEchoTaskBasedJobsAndVariousPeer() throws Exception {

		runStressTestWithEnviroment( 5, 5, 2, FunctionalUnitsControl.JOB_ECHO_MANY_TASKS, 200 );
	}


	/**
	 * Test scenario:<br> - Peer [1 to x], MyGrid [1 to y] randomly at Peers [1
	 * to x], Gums at Peers [1 to x], Peer [1 to x] at Core Peer 1 Description:<br> -
	 * Tests execution of a wide number of jobs in various Peer from various
	 * MyGrid. Functional tests units:<br> - Peer [1 to x],<br> - MyGrid [1 to
	 * y].<br> - Core Peer 1 Test scenario workflow:<br> - Start Core Peer 1,<br> -
	 * Start Peer [1 to x],<br> - Set Gums at Peer [1 to x],<br> - Set Peer [1
	 * to x] at Core Peer 1,<br> - Start MyGrid [1 to y],<br> - Set MyGrid [1
	 * to y] with Peers [1 to x] randomly,<br> - Add job at MyGrid [1 to y],<br> -
	 * Wait until MyGrid [1 to y] job is finished,<br> - Clear scenario.
	 * 
	 * @throws Exception If any exception is thrown the test fails.
	 */
	public void runStressTestWithEnviroment( int numMyGrids, int numPeers, int numGumsPerPeer, File jobFile, int numJobs )
		throws Exception {

		CorePeerUnit testCorePeer = FunctionalUnitsControl.getInstance().createANewCorePeerUnit( "TestCorePeer1" );
		testCorePeer.start();

		MyGridUnit[ ] testMygrids = new MyGridUnit[ numMyGrids ];
		for ( int i = 0; i < testMygrids.length; i++ ) {
			testMygrids[i] = FunctionalUnitsControl.getInstance().createANewMyGridUnit( "TestMyGrid" + i );
			testMygrids[i].start();
		}

		PeerUnit[ ] testPeers = new PeerUnit[ numPeers ];
		for ( int i = 0; i < testPeers.length; i++ ) {
			testPeers[i] = FunctionalUnitsControl.getInstance().createANewPeerUnit( "TestPeer" + i );
			testPeers[i].setCorePeer( testCorePeer );
			testPeers[i].start();

			UserAgentUnit[ ] userAgentUnits = FunctionalUnitsControl.getInstance().createANumberOfUserAgentUnits(
					numGumsPerPeer );
			for ( UserAgentUnit userAgentUnit : userAgentUnits ) {
				userAgentUnit.start();
			}

			testPeers[i].setGums( userAgentUnits );
		}

		int[ ] jobids = new int[ numJobs ];
		for ( int i = 0; i < testMygrids.length; i++ ) {
			for ( int j = 0; j < jobids.length; j++ ) {
				jobids[j] = testMygrids[i].addJob( FunctionalUnitsControl.JOB_ECHO );
			}
		}

		for ( int i = 0; i < testMygrids.length; i++ ) {
			testMygrids[i].setPeer( testPeers[(int) (Math.random() * numPeers)] );
		}

		for ( int i = 0; i < testMygrids.length; i++ ) {
			for ( int j = 0; j < jobids.length; j++ ) {
				new Thread( new JobWaiterRunnable( jobids[j], testMygrids[i] ) ).start();
			}
		}

		ThreadServices.waitUntilWorkIsDone();

		for ( int i = 0; i < testMygrids.length; i++ ) {
			for ( int j = 0; j < jobids.length; j++ ) {
				verifyStatus( testMygrids[i], jobids[j], ExecutionStatus.FINISHED );
			}
		}
	}


	/*
	 * TODO: Same idea as above for jobs with jobs that use PUT/STORE (large zip
	 * files might be good), jobs with a lot of processing (maybe using
	 * PUT/STORE also). Repeat tests with a large number of Gums.
	 */

	/**
	 * A busy wait call to test the status at a desired mygrid.
	 * 
	 * @param mygrid MyGrid to be test.
	 * @param runninGums Desired number of running gums.
	 * @throws Exception
	 */
	private void verifyStatus( MyGridUnit mygrid, int jobId, ExecutionStatus executionStatus ) throws Exception {

		if ( mygrid.getJobStatus( jobId ) != executionStatus ) {
			fail( "Job " + jobId + " should be in state " + executionStatus );
		}
	}
}
