/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.functionalunits;

import java.rmi.Naming;

import junit.framework.TestCase;

import org.ourgrid.test.FunctionalStressTest;

/**
 * Test for <code>CorePeerUnit</code>.
 */
public class CorePeerUnitTest extends TestCase implements FunctionalStressTest {

	protected void tearDown() throws Exception {

		FunctionalUnitsControl.getInstance().destroyAllRemainingUnits();
		FunctionalUnitsControl.getInstance().reset();
	}


	/**
	 * Tests a stop execution with a not started peer.
	 * 
	 * @throws Exception If any exeception is trhown.
	 */
	public void testStopAStoppedCorePeer() throws Exception {

		CorePeerUnit corePeer = FunctionalUnitsControl.getInstance().createANewCorePeerUnit( "Test core peer" );
		assertFalse( corePeer.isRunning() );

		try {
			corePeer.stop();
			fail();
		} catch ( Exception e ) {
		}

		corePeer.destroy();
	}


	/**
	 * Tests a normal corepeer execution:<br>
	 * 1. Verify that the corepeer is down.<br>
	 * 2. Starts a corepeer.<br>
	 * 3. Verify that the corepeer is running.<br>
	 * 4. Stops a corepeer.<br>
	 * 5. Verify that the corepeer is down.<br>
	 * 
	 * @throws Exception If any exeception is trhown.
	 */
	public void testTestCorePeerExecution() throws Exception {

		CorePeerUnit corePeer = FunctionalUnitsControl.getInstance().createANewCorePeerUnit( "Test core peer" );
		assertFalse( corePeer.isRunning() );
		corePeer.start();
		try {
			assertNotNull( Naming.lookup( corePeer.getAddress() ) );
		} catch ( Exception e ) {
			fail( "The core peer is not initialized. Exception: " + e.getMessage() );
		}

		assertTrue( corePeer.isRunning() );
		corePeer.stop();
		assertFalse( corePeer.isRunning() );
		corePeer.destroy();
	}

}
