package org.ourgrid.test.functionalunits;

/**
 * A <code>FunctionalTestUnit</code> will represents a test unit that will be
 * used to compose a test scenario. FunctioTestUnit interacts with the
 * FunctionalTestControls to mantain its state and properties file.
 */
public interface FunctionalTestUnit {

	/**
	 * Start this <code>FunctionalTestUnit</code>.
	 */
	public void start() throws Exception;


	/**
	 * Stop this <code>FunctionalTestUnit</code>.
	 */
	public void stop() throws Exception;


	/**
	 * This method is used to delete enviroment files that the
	 * <code>FunctionalTestUnit</code> uses.
	 */
	public void destroy();


	/**
	 * Verify if this <code>FunctionalTestUnit</code> is running.
	 * 
	 * @return Returns true if this <code>FunctionalTestUnit</code> is
	 *         running.
	 */
	public boolean isRunning();


	/**
	 * Returns the name of this <code>FunctionalTestUnit</code>.
	 * 
	 * @return The name of this <code>FunctionalTestUnit</code>.
	 */
	public String getName();


	/**
	 * Get's the hostname of this functional unit.
	 * 
	 * @return Hostname.
	 */
	public String getHostname();


	/**
	 * Get's the port of this functional unit.
	 * 
	 * @return Port
	 */
	public int getPort();


	/**
	 * Returns the default properties file that will be used by this
	 * <code>FunctionalTestUnit</code>.
	 */
	public String getDefaultPropertiesFile();


	/**
	 * This method will block until this <code>FunctionalTestUnit</code> has
	 * stopped all work.
	 * 
	 * @throws Exception
	 */
	public void waitUntilWorkIsDone() throws Exception;


	/**
	 * This method verifies if the functional test unit was not waken up since
	 * the last <code>waitUntilWorkIsDone()</code> call.
	 * 
	 * @return True if it hasnt.
	 * @throws Exception
	 */
	public boolean stillIdle() throws Exception;


	/**
	 * Kills this unit by calling <code>System.exit()</code>
	 * 
	 * @throws Exception
	 */
	public void kill() throws Exception;
}
