package org.ourgrid.test.functionalunits;

import java.rmi.Naming;

import junit.framework.TestCase;

import org.ourgrid.mygrid.scheduler.ExecutionStatus;
import org.ourgrid.test.FunctionalStressTest;

/**
 * Test class for MyGridUnit.
 */
public class MyGridUnitTest extends TestCase implements FunctionalStressTest {

	protected void tearDown() throws Exception {

		FunctionalUnitsControl.getInstance().destroyAllRemainingUnits();
		FunctionalUnitsControl.getInstance().reset();
	}


	/**
	 * Try to stops an already stopped mygrid.
	 * 
	 * @throws Exception
	 */
	public void testStopAStoppedMygrid() throws Exception {

		MyGridUnit mygrid = FunctionalUnitsControl.getInstance().createANewMyGridUnit( "testmygrid1" );
		assertFalse( mygrid.isRunning() );

		try {
			mygrid.stop();
			fail();
		} catch ( Exception e ) {
		}

		mygrid.destroy();
	}


	public void testTestMyGridExecution() throws Exception {

		MyGridUnit mygrid = FunctionalUnitsControl.getInstance().createANewMyGridUnit( "testmygrid1" );
		assertFalse( mygrid.isRunning() );
		mygrid.start();
		try {
			assertNotNull( Naming.lookup( mygrid.getAddress() ) );
		} catch ( Exception e ) {
			fail( "The core peer is not initialized. Exception: " + e.getMessage() );
		}
		assertTrue( mygrid.isRunning() );
		mygrid.stop();
		assertFalse( mygrid.isRunning() );
		mygrid.destroy();
	}


	public void testAlgo() throws Exception {

		MyGridUnit mygrid = FunctionalUnitsControl.getInstance().createANewMyGridUnit( "testmygrid1" );
		mygrid.start();
		int jobid = mygrid.addJob( FunctionalUnitsControl.JOB_ECHO );
		assertEquals( ExecutionStatus.UNSTARTED, mygrid.getJobStatus( jobid ) );
		mygrid.stop();
		assertFalse( mygrid.isRunning() );
		mygrid.destroy();
	}

}
