/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.functionalunits;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * This interface defines functionalities that OurGrid services must have while
 * being under tests. This interface must only be inserted in a OurGrid service
 * through the use of aspects.
 * 
 * @author Flavio Vinicius Diniz de Figueiredo - flaviov@lsd.ufcg.edu.br
 */
public interface RemoteTestServices extends Remote {

	/**
	 * This method will call the
	 * <code>org.ourgrid.test.threadServices.ThreadServices#waitUntilWorkIsDone()</code>
	 * and the
	 * <code>org.ourgrid.test.threadServices.ThreadPoolServices#waitUntilWorkIsDone()</code>
	 * to wait until this service has stopped.
	 * 
	 * @throws RemoteException Thrown by RMI standards.
	 * @see org.ourgrid.test.threadServices.ThreadServices#waitUntilWorkIsDone()
	 * @see org.ourgrid.test.threadServices.ThreadPoolServices#waitUntilWorkIsDone()
	 */
	public void waitUntilServiceHasStopped() throws RemoteException;


	/**
	 * This method will call the
	 * <code>org.ourgrid.test.threadServices.ThreadServices#stillIdle()</code>
	 * and the
	 * <code>org.ourgrid.test.threadServices.ThreadPoolServices#stillIdle()</code>
	 * to make sure this service has not started running since the last
	 * <code>waitUntilServiceHasStopped()</code>
	 * 
	 * @throws RemoteException Thrown by RMI standards.
	 * @see org.ourgrid.test.functionalunits.RemoteTestServices#waitUntilServiceHasStopped()
	 */
	public boolean stillIdle() throws RemoteException;


	/**
	 * This method will call <code>java.lang.System#exit(int)</code> forcing
	 * this service to abnormaly stop.
	 * 
	 * @throws RemoteException Thrown by RMI standards.
	 * @see java.lang.System#exit(int)
	 */
	public void killService() throws RemoteException;

}
