/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.functionalunits;

import java.io.File;
import java.rmi.Naming;

import junit.framework.TestCase;

import org.ourgrid.common.spec.GumSpec;
import org.ourgrid.test.FunctionalStressTest;

public class UserAgentUnitTest extends TestCase implements FunctionalStressTest {

	protected void tearDown() throws Exception {

		FunctionalUnitsControl.getInstance().destroyAllRemainingUnits();
		FunctionalUnitsControl.getInstance().reset();
	}


	public void testStopAStoppedUA() throws Exception {

		UserAgentUnit[ ] userAgentUnits = FunctionalUnitsControl.getInstance().createANumberOfUserAgentUnits( 1 );
		assertFalse( userAgentUnits[0].isRunning() );

		try {
			userAgentUnits[0].stop();
			fail();
		} catch ( Exception e ) {
		}

		userAgentUnits[0].destroy();
	}


	public void testTestMyGridExecution() throws Exception {

		UserAgentUnit[ ] userAgentUnits = FunctionalUnitsControl.getInstance().createANumberOfUserAgentUnits( 1 );
		assertFalse( userAgentUnits[0].isRunning() );
		userAgentUnits[0].start();
		try {
			assertNotNull( Naming.lookup( userAgentUnits[0].getAddress() ) );
		} catch ( Exception e ) {
			fail( "The core peer is not initialized. Exception: " + e.getMessage() );
		}
		assertTrue( userAgentUnits[0].isRunning() );
		userAgentUnits[0].stop();
		assertFalse( userAgentUnits[0].isRunning() );
		userAgentUnits[0].destroy();
	}


	public void testDirectoryIsDeleted() throws Exception {

		UserAgentUnit[ ] userAgentUnits = FunctionalUnitsControl.getInstance().createANumberOfUserAgentUnits( 1 );
		assertFalse( userAgentUnits[0].isRunning() );
		userAgentUnits[0].start();

		assertTrue( userAgentUnits[0].isRunning() );
		userAgentUnits[0].stop();
		userAgentUnits[0].destroy();

		GumSpec gumSpec = userAgentUnits[0].getGumSpec();
		File deleted = new File( FunctionalUnitsControl.UAADMIN_DEPLOYDIR + File.separator
				+ gumSpec.getAttribute( GumSpec.ATT_NAME ) + "_" + gumSpec.getAttribute( GumSpec.ATT_PORT ) );
		assertFalse( deleted.exists() );
	}
}
