/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

/**
 * Extends the threadlists to add the stillidle functionality.
 * 
 * @author Flavio Vinicius Diniz de Figueiredo - flaviov@lsd.ufcg.edu.br
 */
public class ExtendedThreadLists extends ThreadLists {

	/**
	 * Boolean that defines if the system is still idle.
	 */
	private boolean stillIdle;


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#waitUntilWorkIsDoneNotifying()
	 */
	public synchronized void waitUntilWorkIsDoneNotifying() {

		setNoLongerIdle();
		super.waitUntilWorkIsDoneNotifying();
		setStillIdle();
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#includeInRunningThreads(java.lang.Thread)
	 */
	public synchronized void includeInRunningThreads( Thread t ) {

		setNoLongerIdle();
		super.includeInRunningThreads( t );
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#includeInStartedThreads(java.lang.Thread)
	 */
	public synchronized void includeInStartedThreads( Thread t ) {

		setNoLongerIdle();
		super.includeInStartedThreads( t );
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#removeFromWaitingThreads(java.lang.Thread)
	 */
	protected synchronized void removeFromWaitingThreads( Thread t ) {

		setNoLongerIdle();
		super.removeFromWaitingThreads( t );
	}


	/**
	 * Verifies if no thread has waken up after waitUntilWorkIsDoneNotifying().
	 * 
	 * @return True if no thread has again started running.
	 */
	public synchronized boolean stillIdle() {

		return this.stillIdle;
	}


	/**
	 * Forces the still idle condition to be false.
	 */
	private void setNoLongerIdle() {

		this.stillIdle = false;
	}


	/**
	 * Forces the still idle condition to be true.
	 */
	private void setStillIdle() {

		this.stillIdle = true;
	}
}
