/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

/**
 * Extends the threadpool services to add the still idle functionality.
 * 
 * @author Flavio Vinicius Diniz de Figueiredo - flaviov@lsd.ufcg.edu.br
 */
public class ExtendedThreadPoolServices extends ThreadPoolServices {

	/**
	 * Verifies if no thread has started running after a waitUntilWorkIsDone().
	 */
	private static boolean stillIdle = false;


	/**
	 * @see org.ourgrid.test.threadServices.ThreadPoolServices#waitUntilWorkIsDone()
	 */
	public static void waitUntilWorkIsDone() {

		setNoLongerIdle();
		ThreadPoolServices.waitUntilWorkIsDone();
		setStillIdle();
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadPoolServices#includePooledExecution(Object)
	 */
	public static void includePooledExecution( Runnable run ) {

		setNoLongerIdle();
		ThreadPoolServices.includePooledExecution( run );
	}


	/**
	 * Verifies that no thread has started running since the last
	 * <code>org.ourgrid.test.threadServices.ThreadServices#waitUntilWorkIsDone()</code>
	 * 
	 * @return True if no thread was started running
	 * @see org.ourgrid.test.threadServices.ThreadServices#waitUntilWorkIsDone()
	 */
	public static boolean stillIdle() {

		isCompiledWithAspects();
		return stillIdle;
	}


	/**
	 * Forces the still idle condition to be false.
	 */
	private static void setNoLongerIdle() {

		stillIdle = false;
	}


	/**
	 * Forces the still idle condition to be true.
	 */
	private static void setStillIdle() {

		stillIdle = true;
	}
}
