/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

import junit.framework.TestCase;

import org.ourgrid.test.StressTest;

/**
 * Tests for the ShutdownnableThread class.
 */
public class ShutdownableThreadTest extends TestCase implements StressTest {

	/**
	 * A runnable to be used on the test.
	 */
	private ShutdownableRunnable myShutdownable;


	/**
	 * Constructor.
	 * 
	 * @param name The Test name.
	 */
	public ShutdownableThreadTest( String name ) {

		super( name );
	}


	/**
	 * Initializes the ShutdownableRunnable to be used in the tests.
	 */
	protected void setUp() throws Exception {

		super.setUp();
		this.myShutdownable = new ShutdownableRunnable( true );
	}


	/**
	 * Shutdowns the ShutdownableRunnable.
	 */
	protected void tearDown() throws Exception {

		super.tearDown();
		if ( this.myShutdownable.getMyThread().getStateValue() != ShutdownableThread.DEAD ) {
			this.myShutdownable.shutdown();
		}
		ThreadServices.waitThreadsDie();
		this.myShutdownable = null;
		// System.gc();
	}


	/**
	 * Some ad-hoc tests.
	 * 
	 * @param args The arguments that can be used inside this method.
	 */
	public static void main( String[ ] args ) {

		System.out.println( "Test" );
		ShutdownableRunnable sr = new ShutdownableRunnable();
		sr.startShutdownable();
		sr.setIsThereAnythingToDo( true );
		sr.shutdown();
		System.out.println( "Will wait until stop" );
		ThreadServices.waitUntilThreadHasFinished( sr.getMyThread() );
		System.out.println( "Enf of test" );

	}


	/**
	 * Tests the run method.
	 */
	public void testRun() {

		this.myShutdownable.startShutdownable();
		this.myShutdownable.setIsThereAnythingToDo( true );
		// Verify it it is possible that in this point the run method has not
		// been called.
		// FakeThreadServices.waitUntilThreadIsRunning(this.myShutdownable.getMyThread());
		ThreadServices.waitUntilWorkIsDone();
		int state = this.myShutdownable.getMyThread().getStateValue();
		super.assertTrue( state == ShutdownableThread.NOT_RUNNABLE_WAIT );
	}


	/**
	 * Another test for the run method.
	 */
	public void testRun2() {

		this.myShutdownable.startShutdownable();
		this.myShutdownable.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadHasStartedRunning( this.myShutdownable.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( this.myShutdownable.getMyThread() );
		int state = this.myShutdownable.getMyThread().getStateValue();
		super.assertTrue( (state == ShutdownableThread.RUNNING) || (state == ShutdownableThread.NOT_RUNNABLE_WAIT) );

	}


	/**
	 * Tests the start method and the ThreadServices.
	 */
	public void testStart() {

		this.myShutdownable.startShutdownable();
		ThreadServices.waitUntilWorkIsDone();
		int state = this.myShutdownable.getMyThread().getStateValue();
		super.assertTrue( state == ShutdownableThread.NOT_RUNNABLE_WAIT );
	}


	/**
	 * Tests the thread state after calling the start method.
	 */
	public void testStartNotWaiting() {

		int state = this.myShutdownable.getMyThread().getStateValue();
		super.assertTrue( state == ShutdownableThread.NEW );
		this.myShutdownable.startShutdownable();
		state = this.myShutdownable.getMyThread().getStateValue();
		super.assertTrue( state != ShutdownableThread.NEW );

	}


	/**
	 * Tests the shutdown method and the thread state after calling it.
	 */
	public void testShutdown() {

		this.myShutdownable.startShutdownable();
		ThreadServices.waitUntilThreadIsWaiting( this.myShutdownable.getMyThread() );
		super.assertEquals( ShutdownableThread.NOT_RUNNABLE_WAIT, this.myShutdownable.getMyThread().getStateValue() );
		this.myShutdownable.shutdown();
		ThreadServices.waitUntilThreadHasFinished( this.myShutdownable.getMyThread() );
		super.assertEquals( ShutdownableThread.DEAD, this.myShutdownable.getMyThread().getStateValue() );
	}
}
