/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

import junit.framework.TestCase;

import org.ourgrid.test.StressTest;

/**
 * Test for the ThreadLists class.
 */
public class ThreadListsTest extends TestCase implements StressTest {

	/**
	 * Constructor.
	 * 
	 * @param str The test name.
	 */
	public ThreadListsTest( String str ) {

		super( str );
	}


	/**
	 * Tests the ThreadServices class, verifying a certain Thread state.
	 */
	public void testSeveralMethodsOneThread() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		ThreadServices.waitUntilThreadHasStartedRunning( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.shutdown();
		ThreadServices.waitUntilThreadHasFinished( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
	}


	/**
	 * Tests the ThreadServices class, verifying two Threads' states.
	 */
	public void testSeveralMethodsTwoThreads() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		ShutdownableRunnable sr2 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		sr2.startShutdownable();
		ThreadServices.waitUntilThreadHasStartedRunning( sr1.getMyThread() );
		ThreadServices.waitUntilThreadHasStartedRunning( sr2.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.setIsThereAnythingToDo( true );
		sr2.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.shutdown();
		sr2.shutdown();
		ThreadServices.waitUntilThreadHasFinished( sr1.getMyThread() );
		ThreadServices.waitUntilThreadHasFinished( sr2.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
	}


	/**
	 * Tests several methods from ThreadServices with two threads.
	 */
	public void testSeveralMethodsTwoThreadsSimilar() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		ShutdownableRunnable sr2 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		ThreadServices.waitUntilThreadHasStartedRunning( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr2.startShutdownable();
		ThreadServices.waitUntilThreadHasStartedRunning( sr2.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr2.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.shutdown();
		ThreadServices.waitUntilThreadHasFinished( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
		sr2.shutdown();
		ThreadServices.waitUntilThreadHasFinished( sr2.getMyThread() );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.DEAD) );

	}


	/**
	 * Tests the waitUntilWorkIsDone method.
	 */
	public void testWaitUntilWorkIsDoneNotifying() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		ShutdownableRunnable sr2 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		sr2.startShutdownable();
		sr2.setIsThereAnythingToDo( true );
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilWorkIsDone();
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.shutdown();
		sr2.shutdown();
	}


	/**
	 * Tests the waitUntilThreadsDie method.
	 */
	public void testWaitUntilAllThreadsDie() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( true );
		ShutdownableRunnable sr2 = new ShutdownableRunnable( true );
		sr1.startShutdownable();
		sr2.startShutdownable();
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.RUNNING)
				|| (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.setIsThereAnythingToDo( true );
		sr2.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr2.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		// Attention: Before a shutdown, we must wait until work is done.
		sr1.shutdown();
		sr2.shutdown();
		ThreadServices.waitThreadsDie();
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
		super.assertTrue( (sr2.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
	}


	/**
	 * Tests the waitUntilThreadHasFinished method.
	 */
	public void testWaitUntilThreadStopsRunning() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( true );
		sr1.startShutdownable();
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilWorkIsDone();
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT) );
		sr1.shutdown();
		ThreadServices.waitUntilThreadHasFinished( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.DEAD) );
	}


	/**
	 * Tests the ThreadServices.waitUntilThreadHasStartedRunning method.
	 */
	public void testWaitUntilThreadIsRunning() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadHasStartedRunning( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING) );
		sr1.shutdown();
	}


	/**
	 * Tests the waitUntilThreadIsWaiting method.
	 */
	public void testWaitUntilThreadIsWaiting() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		sr1.startShutdownable();
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT );
		sr1.shutdown();
	}


	/**
	 * Tests the waitUntilThreadStartedRunning method.
	 */
	public void testIsThreadRunning() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( true );
		sr1.startShutdownable();
		sr1.setIsThereAnythingToDo( true );
		ThreadServices.waitUntilThreadHasStartedRunning( sr1.getMyThread() );
		ThreadServices.waitUntilThreadIsWaiting( sr1.getMyThread() );
		super.assertTrue( (sr1.getMyThread().getStateValue() == ShutdownableThread.NOT_RUNNABLE_WAIT)
				|| (sr1.getMyThread().getStateValue() == ShutdownableThread.RUNNING) );
		sr1.shutdown();
	}


	/**
	 * Verifies if a thread was included in runningThreads after calling the
	 * includeInRunningThreads method.
	 */
	public void testIncludeInRunningThreads() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		ThreadLists tl = new ThreadLists();
		tl.includeInRunningThreads( sr1.getMyThread() );
		super.assertTrue( tl.isThreadRunning( sr1.getMyThread() ) );
		sr1.shutdown();
	}


	/**
	 * Verifies if a thread was included in startedThreads after calling the
	 * includeInRunningThreads method.
	 */
	public void testIncludeInStartedThreads() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( false );
		ThreadLists tl = new ThreadLists();
		tl.includeInStartedThreads( sr1.getMyThread() );
		super.assertTrue( tl.isThreadStarted( sr1.getMyThread() ) );
		sr1.shutdown();
	}


	/**
	 * Tests the isThreadStarted method.
	 */
	public void testIsThreadStarted() {

		ShutdownableRunnable sr1 = new ShutdownableRunnable( true );
		ThreadLists tl = new ThreadLists();
		tl.includeInStartedThreads( sr1.getMyThread() );
		super.assertTrue( tl.isThreadStarted( sr1.getMyThread() ) );
		sr1.shutdown();
		ThreadServices.waitThreadsDie();
		super.assertTrue( !sr1.getMyThread().isAlive() );
	}
}
