/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * This class will control runnables executions at OurGrid. It will receive and
 * control all pooled executions call and will wait until all runnables are
 * finished.
 */
public class ThreadPoolServices {

	/** Set that controls the runnables that are at a thread pool. */
	public static Set<Object> runnablesAtThreadPool = Collections.synchronizedSet( new HashSet<Object>() );


	/** Method to add a runnable at the control set. */
	public static void includePooledExecution( Object run ) {

		runnablesAtThreadPool.add( run );
	}


	/** Method to remove a runnable at the control set. */
	public static void removePooledExecution( Object run ) {

		runnablesAtThreadPool.remove( run );
	}


	/**
	 * This method simply waits the current runnables finish their jobs.
	 */
	public static void waitUntilWorkIsDone() {

		isCompiledWithAspects();
		if ( runnablesAtThreadPool.isEmpty() ) {
			return;
		}

		while ( !runnablesAtThreadPool.isEmpty() ) {
			try {
				Thread.sleep( 100 );
			} catch ( InterruptedException e ) {
				// This error should not happen.
			}
		}
	}


	/**
	 * Throws a RuntimeException indicating that the classes that use this
	 * package should be compiled using AspectJ compiler.
	 */
	public static void isCompiledWithAspects() {

		throw new RuntimeException( "This class should be used only after compiling with aspects." );
	}
}
