/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.test.threadServices;

/**
 * Class that presents fake methods, that will be replaced by correct ones using
 * an aspect (RunningThreadsMonitor) when it is included. If the code is not
 * compiled using the aspects, the methods to wait will simply call sleep.
 * Otherwise, this class methods will be replaced by the ones from
 * threadServices.ThreadLists.
 */
public class ThreadServices {

	/**
	 * This method simply waits the current threads finish their jobs. It can be
	 * redefinied in order to verify if the current application threads have
	 * finished to run, or are under their run method execution but waiting.
	 */
	public static void waitUntilWorkIsDone() {

		throwNotUsingAspectsException();
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#waitUntilThreadHasFinished(Thread)
	 *      if using aspects.
	 */
	public static void waitUntilThreadHasFinished( Thread t ) {

		throwNotUsingAspectsException();
	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#waitUntilThreadHasStartedRunning(Thread)
	 *      if using aspects.
	 */
	public static void waitUntilThreadHasStartedRunning( Thread t ) {

		throwNotUsingAspectsException();
	}


	/**
	 * Throws a RuntimeException indicating that the classes that use this
	 * package should be compiled using AspectJ compiler.
	 */
	public static void throwNotUsingAspectsException() {

		throw new RuntimeException( "This class should be used only after compiling with aspects." );
	}


	/**
	 * @return Without aspects, this method simply returns true. If using
	 *         aspects, see ThreadLists.areAllThreadsWaiting();
	 * @see org.ourgrid.test.threadServices.ThreadLists#areAllRunningThreadsWaiting()
	 */
	public static boolean areAllThreadsWaiting() {

		throwNotUsingAspectsException();
		return true;
	}


	/**
	 * Fake method. Without aspects it waits with a sleep.
	 * 
	 * @see org.ourgrid.test.threadServices.ThreadLists#waitUntilAllThreadsDie()
	 */
	public static void waitThreadsDie() {

		throwNotUsingAspectsException();

	}


	/**
	 * @see org.ourgrid.test.threadServices.ThreadLists#waitUntilThreadIsWaiting(Thread)
	 */
	public static void waitUntilThreadIsWaiting( Thread t ) {

		throwNotUsingAspectsException();
	}


	/**
	 * Prints the waiting threads.
	 * 
	 * @see org.ourgrid.test.threadServices.ThreadLists#printWaitingThreads()
	 */
	public static void printWaitingThreads() {

		throwNotUsingAspectsException();
	}


	/**
	 * Prints the started threads.
	 * 
	 * @see org.ourgrid.test.threadServices.ThreadLists#printStartedThreads()
	 */
	public static void printStartedThreads() {

		throwNotUsingAspectsException();
	}


	/**
	 * Prints the running threads.
	 * 
	 * @see org.ourgrid.test.threadServices.ThreadLists#printRunningThreads()
	 */
	public static void printRunningThreads() {

		throwNotUsingAspectsException();
	}
}
