/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.webstatus;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.ourgrid.common.id.ObjectID;
import org.ourgrid.peer.manager.status.ConsumerStatus;
import org.ourgrid.peer.manager.status.StatusEntry;

/**
 * Description: A data structure that holds all peer status information.
 * 
 * @version 1.0 Date15/10/2004
 */
public class PeerInfoPack implements Serializable {

	/**
	 * Serial identification of the class. It need to be changed only if the
	 * class interface is changed.
	 */
	private static final long serialVersionUID = 33L;

	/**
	 * The peerID information.
	 */
	private ObjectID peerID;

	// New implementations

	/**
	 * Currently version.
	 */
	private String version;

	/**
	 * Currently email.
	 */
	private String email;

	/**
	 * Currently description.
	 */
	private String description;

	/**
	 * Currently uptime
	 */
	private String uptime;

	private String latitude;

	private String longitude;

	private Collection<StatusEntry> localGums;

	private Collection<StatusEntry> remoteGums;

	private Collection<StatusEntry> donatedGums;

	private Collection<StatusEntry> ownerGums;

	private Collection<ConsumerStatus> localConsumers;

	private Collection<ConsumerStatus> remoteConsumers;

	private Map<ObjectID,Double> networkOfFavors;


	public PeerInfoPack( ObjectID peerID, String version, String email, String description, String uptime,
							String latitude, String longitude, Collection<StatusEntry> localGums,
							Collection<StatusEntry> remoteGums, Collection<StatusEntry> donatedGums,
							Collection<StatusEntry> ownerGums, Collection<ConsumerStatus> localConsumers,
							Collection<ConsumerStatus> remoteConsumers, Map<ObjectID,Double> networkOfFavors ) {

		this.peerID = peerID;
		this.version = version;
		this.email = email;
		this.description = description;
		this.uptime = uptime;
		this.latitude = latitude;
		this.longitude = longitude;
		this.localGums = localGums;
		this.remoteGums = remoteGums;
		this.donatedGums = donatedGums;
		this.ownerGums = ownerGums;
		this.localConsumers = localConsumers;
		this.remoteConsumers = remoteConsumers;
		this.networkOfFavors = networkOfFavors;
	}


	/**
	 * @return The peerID information.
	 */
	public ObjectID getPeerID() {

		return peerID;
	}


	/**
	 * @return Currently version
	 */
	public String getVersion() {

		return version;
	}


	/**
	 * @return Currently description
	 */
	public String getDescription() {

		return description;
	}


	/**
	 * @return Currently email
	 */
	public String getEmail() {

		return email;
	}


	/**
	 * @return Currently uptime
	 */
	public String getUptime() {

		return uptime;
	}


	public static long getSerialVersionUID() {

		return serialVersionUID;
	}


	public String getLatitude() {

		return latitude;
	}


	public String getLongitude() {

		return longitude;
	}


	public Collection<StatusEntry> getLocalGums() {

		return localGums;
	}


	public Collection<StatusEntry> getRemoteGums() {

		return remoteGums;
	}


	public Collection<StatusEntry> getDonatedGums() {

		return donatedGums;
	}


	public Collection<ConsumerStatus> getLocalConsumers() {

		return localConsumers;
	}


	public Collection<ConsumerStatus> getRemoteConsumers() {

		return remoteConsumers;
	}


	public Map<ObjectID,Double> getNetworkOfFavors() {

		return networkOfFavors;
	}


	public Collection<StatusEntry> getOwnerGums() {

		return ownerGums;
	}


	public Collection<StatusEntry> getAllGums() {

		Collection<StatusEntry> gums = new ArrayList<StatusEntry>();
		gums.addAll( this.localGums );
		gums.addAll( this.ownerGums );
		return gums;
	}

}
