/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.webstatus;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Date;

/**
 * Description: This is a fake peer, created to join the OurGrid Community and
 * obtain the known peers list from the core peer and then talk to each known
 * peer to get it's status information.
 * 
 * @version 1.0 Date 15/10/2004 (Updated 06/06/2005 by Pergentino)
 */

public interface WebStatus extends Remote {

	/**
	 * Gets all peer status information from all known (by the corepeer) peers.
	 * 
	 * @return A collection of PeerInfoPack, one for each known peer. *
	 * @throws MalformedURLException If the corepeer URL is invalid. This should
	 *         never occurs.
	 * @throws RemoteException If the corepeer could not be conctacted.
	 * @throws NotBoundException If the corepeer is not online
	 */
	public Collection<PeerInfoPack> getLocalPeerInfoList() throws MalformedURLException, RemoteException,
		NotBoundException;


	/**
	 * Get the last date/time of snapshot.
	 * 
	 * @return Last date/time of sanpshot.
	 * @throws MalformedURLException
	 * @throws RemoteException
	 * @throws NotBoundException
	 */
	public Date getLastTimeOfSnapshot() throws MalformedURLException, RemoteException, NotBoundException;
}
