/*
 * Copyright (c) 2002-2006 Universidade Federal de Campina Grande
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ourgrid.webstatus;

import java.io.IOException;
import java.rmi.Naming;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Description: A servlet class designed to generate the WebStatus html page.
 */
public class WebStatusServlet extends HttpServlet implements ServletContextListener {

	private static final long serialVersionUID = 33L;


	/**
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
	 *      javax.servlet.http.HttpServletResponse)
	 */
	public void doGet( HttpServletRequest request, HttpServletResponse response ) throws IOException, ServletException {

		String STATUS_URL = "rmi://127.0.0.1:" + WebStatusImpl.DEFAULT_WEBSTATUS_PORT + "/WebStatusImpl";

		String peerID = request.getParameter( "peerID" );

		WebStatus webStatus = null;
		Collection<PeerInfoPack> peerInfoList = null;

		try {
			webStatus = (WebStatus) Naming.lookup( STATUS_URL );
			peerInfoList = webStatus.getLocalPeerInfoList();
			System.out.println( "info list:  " + peerInfoList );
		} catch ( Exception e ) {
			e.printStackTrace();
		}

		if ( peerID != null ) {

			boolean found = false;

			Iterator<PeerInfoPack> it = peerInfoList.iterator();
			while ( it.hasNext() && !found ) {

				PeerInfoPack p = it.next();

				if ( p.getPeerID().getIdValue().toString().equals( (new Long( peerID )).toString() ) ) {
					found = true;
					request.setAttribute( "pack", p );
				}

			}

		} else {

			if ( peerInfoList != null ) {
				List lista = new LinkedList( peerInfoList );
				Collections.sort( lista, new PeerInfoPackComparator() );
				request.setAttribute( "peerInfoList", lista );
				try {
					request.setAttribute( "timeSnapshot", webStatus.getLastTimeOfSnapshot() );
				} catch ( Exception e ) {
					e.printStackTrace();
				}
			}

		}

		String redirect = "index.jsp";

		if ( request.getParameter( "view" ) != null ) {

			if ( request.getParameter( "view" ).equalsIgnoreCase( "statistics" ) ) {
				redirect = "statistics.jsp";
			} else if ( request.getParameter( "view" ).equalsIgnoreCase( "new" ) ) {
				redirect = "index2.jsp";
			} else if ( request.getParameter( "view" ).equalsIgnoreCase( "rss" ) ) {
				redirect = "rss.jsp";
			} else if ( request.getParameter( "view" ).equalsIgnoreCase( "details" ) ) {
				redirect = "details.jsp";
			}

		}

		try {

			RequestDispatcher dis = request.getRequestDispatcher( redirect );
			dis.forward( request, response );

		} catch ( Exception e ) {
			e.printStackTrace();
			System.out.println( this.getClass().getName() + " ()\n " + e.toString() );
		}

	}


	/**
	 * We are going to perform the same operations for POST requests as for GET
	 * methods, so this method just sends the request to the doGet method.
	 */
	public void doPost( HttpServletRequest request, HttpServletResponse response ) throws IOException, ServletException {

		doGet( request, response );
	}


	public void contextInitialized( ServletContextEvent arg0 ) {

		// TODO Auto-generated method stub

	}


	public void contextDestroyed( ServletContextEvent arg0 ) {

		// TODO Auto-generated method stub

	}

}
