/*
 * Copyright (c) 2002-2005 Universidade Federal de Campina Grande
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ourgrid.threadServicesAspects;

import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Callable;

import org.ourgrid.test.threadServices.*;

/**
 * This aspect is responsible for managing an application's threads at a ThreadPool.
 * Its services are provided by the ThreadPoolServices class on OurGrid.
 *
 */
privileged public aspect ControlThreadPool {
	/* This method will validate the ThreadPoolServices use. */
    void around(): execution(public static void isCompiledWithAspects()){
		// Dont throw any exception.
	}
		
	/* POINTCUT DEFINITIONS. */
	pointcut runnableRunExecutions(Runnable run):execution(public void Runnable+.run()) && target(run);
	pointcut callableCallExecutions(Callable callable):execution(public * Callable+.call()) && target(callable);
	pointcut runnablePooledExecution(Executor pe, Runnable run):call(public void Executor+.execute(*))&& target(pe) && args(run);
	pointcut calledPooledExecution(Executor pe, Callable callable):call(public * ExecutorService+.submit(*))&& target(pe) && args(callable);
	
	/*
	 * ADVICE
	 */
	 before(Executor pe, Runnable run) : runnablePooledExecution(pe, run) {
        ThreadPoolServices.includePooledExecution(run);
     }

	 before(Executor pe, Callable callable) : calledPooledExecution(pe, callable) {
        ThreadPoolServices.includePooledExecution(callable);
     }

     after(Runnable run) : runnableRunExecutions(run) {
        ThreadPoolServices.removePooledExecution(run);
     }
     
      after(Callable callable) : callableCallExecutions(callable) {
        ThreadPoolServices.removePooledExecution(callable);
     }
     
}